@@ -91,7 +91,7 @@ static const GimpActionEntry view_actions[] =
     view_zoom_fit_in_cmd_callback,
     GIMP_HELP_VIEW_ZOOM_FIT_IN },
 
-  { "view-zoom-fill", GIMP_ICON_ZOOM_FIT_BEST,
+  { "view-zoom-fill", GIMP_ICON_VIEW_ZOOM_FILL,
     NC_("view-action", "Fi_ll Window"), NULL, { NULL },
     NC_("view-action", "Adjust the zoom ratio so that the entire window is used"),
     view_zoom_fill_cmd_callback,
@@ -146,9 +146,9 @@ static const GimpActionEntry view_actions[] =
     NC_("view-action",
         "Reset color management to what's configured in preferences"),
     view_color_management_reset_cmd_callback,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_COLOR_MANAGEMENT_RESET },
 
-  { "view-shrink-wrap", GIMP_ICON_ZOOM_FIT_BEST,
+  { "view-shrink-wrap", GIMP_ICON_VIEW_SHRINK_WRAP,
     NC_("view-action", "Shrink _Wrap"), NULL, { "<primary>J", NULL },
     NC_("view-action", "Reduce the image window to the size of the image display"),
     view_shrink_wrap_cmd_callback,
@@ -183,21 +183,21 @@ static const GimpToggleActionEntry view_toggle_actions[] =
     NC_("view-action", "Use color management for this view"),
     view_color_management_enable_cmd_callback,
     TRUE,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_COLOR_MANAGE_VIEW },
 
   { "view-color-management-softproof", NULL,
     NC_("view-action", "_Proof Colors"), NULL, { NULL },
     NC_("view-action", "Use this view for soft-proofing"),
     view_color_management_softproof_cmd_callback,
     FALSE,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_SOFT_PROOF_VIEW },
 
   { "view-display-black-point-compensation", NULL,
     NC_("view-action", "_Black Point Compensation"), NULL, { NULL },
     NC_("view-action", "Use black point compensation for image display"),
     view_display_bpc_cmd_callback,
     TRUE,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_SOFT_PROOF_BLACK_POINT },
 
   { "view-softproof-gamut-check", NULL,
     NC_("view-action", "_Mark Out Of Gamut Colors"), NULL, { NULL },
@@ -205,7 +205,7 @@ static const GimpToggleActionEntry view_toggle_actions[] =
         "be represented in the target color space"),
     view_softproof_gamut_check_cmd_callback,
     FALSE,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_MARK_OUT_OF_GAMUT },
 
   { "view-show-selection", NULL,
     NC_("view-action", "Show _Selection"), NULL, { "<primary>T", NULL },
@@ -273,9 +273,23 @@ static const GimpToggleActionEntry view_toggle_actions[] =
   { "view-snap-to-vectors", NULL,
     NC_("view-action", "Snap t_o Active Path"), NULL, { NULL },
     NC_("view-action", "Tool operations snap to the active path"),
-    view_snap_to_vectors_cmd_callback,
+    view_snap_to_path_cmd_callback,
     FALSE,
-    GIMP_HELP_VIEW_SNAP_TO_VECTORS },
+    GIMP_HELP_VIEW_SNAP_TO_PATH },
+
+  { "view-snap-to-bbox", NULL,
+    NC_("view-action", "Snap to _Bounding Boxes"), NULL, { NULL },
+    NC_("view-action", "Tool operations snap to the bounding boxes"),
+    view_snap_to_bbox_cmd_callback,
+    FALSE,
+    GIMP_HELP_VIEW_SNAP_TO_BBOX },
+
+  { "view-snap-to-equidistance", NULL,
+    NC_("view-action", "Snap to _Equidistance"), NULL, { NULL },
+    NC_("view-action", "Tool operations snap to the equidistance between three bounding boxes"),
+    view_snap_to_equidistance_cmd_callback,
+    FALSE,
+    GIMP_HELP_VIEW_SNAP_TO_EQUIDISTANCE },
 
   { "view-show-menubar", NULL,
     NC_("view-action", "Show _Menubar"), NULL, { NULL },
@@ -353,66 +367,86 @@ static const GimpEnumActionEntry view_zoom_actions[] =
     GIMP_HELP_VIEW_ZOOM_IN }
 };
 
-static const GimpRadioActionEntry view_zoom_explicit_actions[] =
+static const GimpEnumActionEntry view_zoom_explicit_actions[] =
 {
   { "view-zoom-16-1", NULL,
-    NC_("view-zoom-action", "1_6:1  (1600%)"), NULL, { "5", "KP_5", NULL },
+    NC_("view-zoom-action", "Zoom 16:1 (1600%)"),
+    NC_("view-zoom-action", "1_6:1  (1600%)"),
+    { "5", "KP_5", NULL },
     NC_("view-zoom-action", "Zoom 16:1"),
-    160000,
+    160000, FALSE,
     GIMP_HELP_VIEW_ZOOM_IN },
 
   { "view-zoom-8-1", NULL,
-    NC_("view-zoom-action", "_8:1  (800%)"), NULL, { "4", "KP_4", NULL },
+    NC_("view-zoom-action", "Zoom 8:1 (800%)"),
+    NC_("view-zoom-action", "_8:1  (800%)"),
+    { "4", "KP_4", NULL },
     NC_("view-zoom-action", "Zoom 8:1"),
-    80000,
+    80000, FALSE,
     GIMP_HELP_VIEW_ZOOM_IN },
 
   { "view-zoom-4-1", NULL,
-    NC_("view-zoom-action", "_4:1  (400%)"), NULL, { "3", "KP_3", NULL },
+    NC_("view-zoom-action", "Zoom 4:1 (400%)"),
+    NC_("view-zoom-action", "_4:1  (400%)"),
+    { "3", "KP_3", NULL },
     NC_("view-zoom-action", "Zoom 4:1"),
-    40000,
+    40000, FALSE,
     GIMP_HELP_VIEW_ZOOM_IN },
 
   { "view-zoom-2-1", NULL,
-    NC_("view-zoom-action", "_2:1  (200%)"), NULL, { "2", "KP_2", NULL },
+    NC_("view-zoom-action", "Zoom 2:1 (200%)"),
+    NC_("view-zoom-action", "_2:1  (200%)"),
+    { "2", "KP_2", NULL },
     NC_("view-zoom-action", "Zoom 2:1"),
-    20000,
+    20000, FALSE,
     GIMP_HELP_VIEW_ZOOM_IN },
 
   { "view-zoom-1-1", GIMP_ICON_ZOOM_ORIGINAL,
-    NC_("view-zoom-action", "_1:1  (100%)"), NULL, { "1", "KP_1", NULL },
+    NC_("view-zoom-action", "Zoom 1:1 (100%)"),
+    NC_("view-zoom-action", "_1:1  (100%)"),
+    { "1", "KP_1", NULL },
     NC_("view-zoom-action", "Zoom 1:1"),
-    10000,
+    10000, FALSE,
     GIMP_HELP_VIEW_ZOOM_100 },
 
   { "view-zoom-1-2", NULL,
-    NC_("view-zoom-action", "1:_2  (50%)"), NULL, { "<shift>2", "<shift>KP_2", NULL },
+    NC_("view-zoom-action", "Zoom 1:2 (50%)"),
+    NC_("view-zoom-action", "1:_2  (50%)"),
+    { "<shift>2", "<shift>KP_2", NULL },
     NC_("view-zoom-action", "Zoom 1:2"),
-    5000,
+    5000, FALSE,
     GIMP_HELP_VIEW_ZOOM_OUT },
 
   { "view-zoom-1-4", NULL,
-    NC_("view-zoom-action", "1:_4  (25%)"), NULL, { "<shift>3", "<shift>KP_3", NULL },
+    NC_("view-zoom-action", "Zoom 1:4 (25%)"),
+    NC_("view-zoom-action", "1:_4  (25%)"),
+    { "<shift>3", "<shift>KP_3", NULL },
     NC_("view-zoom-action", "Zoom 1:4"),
-    2500,
+    2500, FALSE,
     GIMP_HELP_VIEW_ZOOM_OUT },
 
   { "view-zoom-1-8", NULL,
-    NC_("view-zoom-action", "1:_8  (12.5%)"), NULL, { "<shift>4", "<shift>KP_4", NULL },
+    NC_("view-zoom-action", "Zoom 1:8 (12.5%)"),
+    NC_("view-zoom-action", "1:_8  (12.5%)"),
+    { "<shift>4", "<shift>KP_4", NULL },
     NC_("view-zoom-action", "Zoom 1:8"),
-    1250,
+    1250, FALSE,
     GIMP_HELP_VIEW_ZOOM_OUT },
 
   { "view-zoom-1-16", NULL,
-    NC_("view-zoom-action", "1:1_6  (6.25%)"), NULL, { "<shift>5", "<shift>KP_5", NULL },
+    NC_("view-zoom-action", "Zoom 1:16 (6.25%)"),
+    NC_("view-zoom-action", "1:1_6  (6.25%)"),
+    { "<shift>5", "<shift>KP_5", NULL },
     NC_("view-zoom-action", "Zoom 1:16"),
-    625,
+    625, FALSE,
     GIMP_HELP_VIEW_ZOOM_OUT },
 
   { "view-zoom-other", NULL,
-    NC_("view-zoom-action", "Othe_r zoom factor..."), NULL, { NULL },
     NC_("view-zoom-action", "Set a custom zoom factor"),
-    0,
+    NC_("view-zoom-action", "Othe_r zoom factor..."),
+    { NULL },
+    NC_("view-zoom-action", "Set a custom zoom factor"),
+    0, TRUE,
     GIMP_HELP_VIEW_ZOOM_OTHER }
 };
 
@@ -488,25 +522,25 @@ static const GimpRadioActionEntry view_display_intent_actions[] =
     NC_("view-action", "_Perceptual"), NULL, { NULL },
     NC_("view-action", "Display rendering intent is perceptual"),
     GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_SOFT_PROOF_RENDERING_INTENT },
 
   { "view-display-intent-relative-colorimetric", NULL,
     NC_("view-action", "_Relative Colorimetric"), NULL, { NULL },
     NC_("view-action", "Display rendering intent is relative colorimetric"),
     GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_SOFT_PROOF_RENDERING_INTENT },
 
   { "view-display-intent-saturation", NULL,
     NC_("view-action", "_Saturation"), NULL, { NULL },
     NC_("view-action", "Display rendering intent is saturation"),
     GIMP_COLOR_RENDERING_INTENT_SATURATION,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT },
+    GIMP_HELP_VIEW_SOFT_PROOF_RENDERING_INTENT },
 
   { "view-display-intent-absolute-colorimetric", NULL,
     NC_("view-action", "_Absolute Colorimetric"), NULL, { NULL },
     NC_("view-action", "Display rendering intent is absolute colorimetric"),
     GIMP_COLOR_RENDERING_INTENT_ABSOLUTE_COLORIMETRIC,
-    GIMP_HELP_VIEW_COLOR_MANAGEMENT }
+    GIMP_HELP_VIEW_SOFT_PROOF_RENDERING_INTENT }
 };
 
 static const GimpEnumActionEntry view_padding_color_actions[] =
@@ -634,8 +668,6 @@ static const GimpEnumActionEntry view_scroll_vertical_actions[] =
 void
 view_actions_setup (GimpActionGroup *group)
 {
-  GimpAction *action;
-
   gimp_action_group_add_actions (group, "view-action",
                                  view_actions,
                                  G_N_ELEMENTS (view_actions));
@@ -649,12 +681,10 @@ view_actions_setup (GimpActionGroup *group)
                                       G_N_ELEMENTS (view_zoom_actions),
                                       view_zoom_cmd_callback);
 
-  gimp_action_group_add_radio_actions (group, "view-zoom-action",
-                                       view_zoom_explicit_actions,
-                                       G_N_ELEMENTS (view_zoom_explicit_actions),
-                                       NULL,
-                                       10000,
-                                       view_zoom_explicit_cmd_callback);
+  gimp_action_group_add_enum_actions (group, "view-zoom-action",
+                                      view_zoom_explicit_actions,
+                                      G_N_ELEMENTS (view_zoom_explicit_actions),
+                                      view_zoom_explicit_cmd_callback);
 
   gimp_action_group_add_toggle_actions (group, "view-action",
                                         view_flip_actions,
@@ -696,15 +726,6 @@ view_actions_setup (GimpActionGroup *group)
                                       G_N_ELEMENTS (view_scroll_vertical_actions),
                                       view_scroll_vertical_cmd_callback);
 
-  /*  connect "activate" of view-zoom-other manually so it can be
-   *  selected even if it's the active item of the radio group
-   */
-  action = gimp_action_group_get_action (group, "view-zoom-other");
-
-  g_signal_connect (action, "activate",
-                    G_CALLBACK (view_zoom_other_cmd_callback),
-                    group->user_data);
-
   g_signal_connect_object (group->gimp->config, "notify::check-type",
                            G_CALLBACK (view_actions_check_type_notify),
                            group, 0);
@@ -919,6 +940,10 @@ view_actions_update (GimpActionGroup *group,
   SET_ACTIVE    ("view-snap-to-canvas",       display && options->snap_to_canvas);
   SET_SENSITIVE ("view-snap-to-vectors",      image);
   SET_ACTIVE    ("view-snap-to-vectors",      display && options->snap_to_path);
+  SET_SENSITIVE ("view-snap-to-bbox",         image);
+  SET_ACTIVE    ("view-snap-to-bbox",         display && options->snap_to_bbox);
+  SET_SENSITIVE ("view-snap-to-equidistance", image);
+  SET_ACTIVE    ("view-snap-to-equidistance", display && options->snap_to_equidistance);
 
   SET_SENSITIVE ("view-padding-color-theme",       image);
   SET_SENSITIVE ("view-padding-color-light-check", image);
@@ -965,55 +990,20 @@ static void
 view_actions_set_zoom (GimpActionGroup  *group,
                        GimpDisplayShell *shell)
 {
-  const gchar     *action = NULL;
   GimpImageWindow *window;
   GimpMenuModel   *model;
   gchar           *str;
   gchar           *label;
-  guint            scale;
 
   g_object_get (shell->zoom,
                 "percentage", &str,
                 NULL);
 
-  scale = ROUND (gimp_zoom_model_get_factor (shell->zoom) * 1000);
-
-  switch (scale)
-    {
-    case 16000:  action = "view-zoom-16-1";  break;
-    case  8000:  action = "view-zoom-8-1";   break;
-    case  4000:  action = "view-zoom-4-1";   break;
-    case  2000:  action = "view-zoom-2-1";   break;
-    case  1000:  action = "view-zoom-1-1";   break;
-    case   500:  action = "view-zoom-1-2";   break;
-    case   250:  action = "view-zoom-1-4";   break;
-    case   125:  action = "view-zoom-1-8";   break;
-    case    63:
-    case    62:  action = "view-zoom-1-16";  break;
-    }
-
-  if (! action)
-    {
-      action = "view-zoom-other";
-
-      label = g_strdup_printf (_("Othe_r (%s)..."), str);
-      gimp_action_group_set_action_label (group, action, label);
-      g_free (label);
-
-      shell->other_scale = gimp_zoom_model_get_factor (shell->zoom);
-    }
-
-  gimp_action_group_set_action_active (group, action, TRUE);
-
   window = gimp_display_shell_get_window (shell);
   model  = gimp_image_window_get_menubar_model (window);
   label  = g_strdup_printf (_("_Zoom (%s)"), str);
   gimp_menu_model_set_title (model, "/View/Zoom", label);
   g_free (label);
-
-  /* flag as dirty */
-  shell->other_scale = - fabs (shell->other_scale);
-
   g_free (str);
 }
 
@@ -1064,9 +1054,7 @@ view_actions_check_type_notify (GimpDisplayConfig *config,
                                 GimpActionGroup   *group)
 {
   gimp_action_group_set_action_color (group, "view-padding-color-light-check",
-                                      gimp_render_check_color1 (),
-                                      FALSE);
+                                      (GeglColor *) gimp_render_check_color1 (), FALSE);
   gimp_action_group_set_action_color (group, "view-padding-color-dark-check",
-                                      gimp_render_check_color2 (),
-                                      FALSE);
+                                      (GeglColor *) gimp_render_check_color2 (), FALSE);
 }
