@@ -512,6 +512,9 @@ _("Enable the Seamless Clone tool.")
 #define PLAYGROUND_PAINT_SELECT_TOOL_BLURB \
 _("Enable the Paint Select tool.")
 
+#define PLAYGROUND_USE_LIST_BOX_BLURB \
+_("Use the new GtkListBox widget for simple lists.")
+
 #define SPACE_BAR_ACTION_BLURB \
 _("What to do when the space bar is pressed in the image window.")
 
@@ -558,6 +561,9 @@ _("The name of the icon theme to use.")
 #define OVERRIDE_THEME_ICON_SIZE_BLURB \
 _("Override theme-set icon sizes.")
 
+#define VIEWABLES_FOLLOW_THEME_BLURB \
+_("Use theme colors for preview")
+
 #define ICON_SIZE_BLURB \
 _("The size of the icons to use.")
 
@@ -661,22 +667,22 @@ _("Sets the default channel name for the 'New Channel' dialog.")
 #define CHANNEL_NEW_COLOR_BLURB \
 _("Sets the default color and opacity for the 'New Channel' dialog.")
 
-#define VECTORS_NEW_NAME_BLURB \
+#define PATH_NEW_NAME_BLURB \
 _("Sets the default path name for the 'New Path' dialog.")
 
-#define VECTORS_EXPORT_PATH_BLURB \
+#define PATH_EXPORT_PATH_BLURB \
 _("Sets the default folder path for the 'Export Path' dialog.")
 
-#define VECTORS_EXPORT_ACTIVE_ONLY_BLURB \
+#define PATH_EXPORT_ACTIVE_ONLY_BLURB \
 _("Sets the default 'Export the selected paths' state for the 'Export Path' dialog.")
 
-#define VECTORS_IMPORT_PATH_BLURB \
+#define PATH_IMPORT_PATH_BLURB \
 _("Sets the default folder path for the 'Import Path' dialog.")
 
-#define VECTORS_IMPORT_MERGE_BLURB \
+#define PATH_IMPORT_MERGE_BLURB \
 _("Sets the default 'Merge imported paths' state for the 'Import Path' dialog.")
 
-#define VECTORS_IMPORT_SCALE_BLURB \
+#define PATH_IMPORT_SCALE_BLURB \
 _("Sets the default 'Scale imported paths to fit size' state for the 'Import Path' dialog.")
 
 #define SELECTION_FEATHER_RADIUS_BLURB \
