@@ -254,9 +254,9 @@ gimp_init (Gimp *gimp)
   gimp->drawable_filter_table = gimp_id_table_new ();
 
   gimp->displays = g_object_new (GIMP_TYPE_LIST,
-                                 "children-type", GIMP_TYPE_OBJECT,
-                                 "policy",        GIMP_CONTAINER_POLICY_WEAK,
-                                 "append",        TRUE,
+                                 "child-type", GIMP_TYPE_OBJECT,
+                                 "policy",     GIMP_CONTAINER_POLICY_WEAK,
+                                 "append",     TRUE,
                                  NULL);
   gimp_object_set_static_name (GIMP_OBJECT (gimp->displays), "displays");
   gimp->next_display_id = 1;
@@ -268,15 +268,15 @@ gimp_init (Gimp *gimp)
   gimp_data_factories_init (gimp);
 
   gimp->tool_info_list = g_object_new (GIMP_TYPE_LIST,
-                                       "children-type", GIMP_TYPE_TOOL_INFO,
-                                       "append",        TRUE,
+                                       "child-type", GIMP_TYPE_TOOL_INFO,
+                                       "append",     TRUE,
                                        NULL);
   gimp_object_set_static_name (GIMP_OBJECT (gimp->tool_info_list),
                                "tool infos");
 
   gimp->tool_item_list = g_object_new (GIMP_TYPE_LIST,
-                                       "children-type", GIMP_TYPE_TOOL_ITEM,
-                                       "append",        TRUE,
+                                       "child-type", GIMP_TYPE_TOOL_ITEM,
+                                       "append",     TRUE,
                                        NULL);
   gimp_object_set_static_name (GIMP_OBJECT (gimp->tool_item_list),
                                "tool items");
@@ -540,10 +540,6 @@ gimp_real_initialize (Gimp               *gimp,
 
   status_callback (_("Initialization"), NULL, 0.0);
 
-  /*  set the last values used to default values  */
-  gimp->image_new_last_template =
-    gimp_config_duplicate (GIMP_CONFIG (gimp->config->default_image));
-
   /*  add data objects that need the user context  */
   gimp_data_factories_add_builtin (gimp);
 
@@ -1118,6 +1114,37 @@ gimp_get_tool_info (Gimp        *gimp,
   return (GimpToolInfo *) info;
 }
 
+void
+gimp_set_last_template (Gimp         *gimp,
+                        GimpTemplate *template)
+{
+  GimpTemplate *last_template;
+
+  g_return_if_fail (GIMP_IS_GIMP (gimp));
+  g_return_if_fail (GIMP_IS_TEMPLATE (template));
+
+  last_template = gimp_get_last_template (gimp);
+
+  gimp_config_sync (G_OBJECT (template),
+                    G_OBJECT (last_template), 0);
+}
+
+GimpTemplate *
+gimp_get_last_template (Gimp *gimp)
+{
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
+
+  if (! gimp->image_new_last_template)
+    {
+      /*  set the last values used to default values  */
+      gimp->image_new_last_template =
+        gimp_config_duplicate (GIMP_CONFIG (gimp->config->default_image));
+    }
+
+  return gimp->image_new_last_template;
+}
+
+
 /**
  * gimp_message:
  * @gimp:     a pointer to the %Gimp object
