@@ -95,6 +95,8 @@ struct _Gimp
 
   GList                  *image_windows;
 
+  GObject                *controller_manager;
+
   GimpImage              *clipboard_image;
   GimpBuffer             *clipboard_buffer;
   GimpContainer          *named_buffers;
@@ -225,6 +227,10 @@ GimpContext  * gimp_get_user_context       (Gimp                *gimp);
 GimpToolInfo * gimp_get_tool_info          (Gimp                *gimp,
                                             const gchar         *tool_name);
 
+void           gimp_set_last_template      (Gimp                *gimp,
+                                            GimpTemplate        *_template);
+GimpTemplate * gimp_get_last_template      (Gimp                *gimp);
+
 void           gimp_message                (Gimp                *gimp,
                                             GObject             *handler,
                                             GimpMessageSeverity  severity,
