@@ -43,8 +43,8 @@
 #include "gimppickable-contiguous-region.h"
 #include "gimpscanconvert.h"
 
+#include "vectors/gimppath.h"
 #include "vectors/gimpstroke.h"
-#include "vectors/gimpvectors.h"
 
 #include "gimp-intl.h"
 
@@ -279,24 +279,24 @@ gimp_channel_select_polygon (GimpChannel       *channel,
 }
 
 void
-gimp_channel_select_vectors (GimpChannel    *channel,
-                             const gchar    *undo_desc,
-                             GimpVectors    *vectors,
-                             GimpChannelOps  op,
-                             gboolean        antialias,
-                             gboolean        feather,
-                             gdouble         feather_radius_x,
-                             gdouble         feather_radius_y,
-                             gboolean        push_undo)
+gimp_channel_select_path (GimpChannel    *channel,
+                          const gchar    *undo_desc,
+                          GimpPath       *path,
+                          GimpChannelOps  op,
+                          gboolean        antialias,
+                          gboolean        feather,
+                          gdouble         feather_radius_x,
+                          gdouble         feather_radius_y,
+                          gboolean        push_undo)
 {
   const GimpBezierDesc *bezier;
 
   g_return_if_fail (GIMP_IS_CHANNEL (channel));
   g_return_if_fail (gimp_item_is_attached (GIMP_ITEM (channel)));
   g_return_if_fail (undo_desc != NULL);
-  g_return_if_fail (GIMP_IS_VECTORS (vectors));
+  g_return_if_fail (GIMP_IS_PATH (path));
 
-  bezier = gimp_vectors_get_bezier (vectors);
+  bezier = gimp_path_get_bezier (path);
 
   if (bezier && bezier->num_data > 4)
     {
