@@ -850,7 +850,7 @@ gimp_data_factory_get_data_type (GimpDataFactory *factory)
 {
   g_return_val_if_fail (GIMP_IS_DATA_FACTORY (factory), G_TYPE_NONE);
 
-  return gimp_container_get_children_type (factory->priv->container);
+  return gimp_container_get_child_type (factory->priv->container);
 }
 
 GimpContainer *
