@@ -370,7 +370,7 @@ gimp_drawable_get_line_art_fill_buffer (GimpDrawable      *drawable,
   GeglBuffer *new_mask;
   GeglBuffer *rendered_mask;
   GeglBuffer *fill_buffer   = NULL;
-  GimpRGB     fill_color;
+  GeglColor  *fill_color    = NULL;
   gint        fill_offset_x = 0;
   gint        fill_offset_y = 0;
   gint        x, y, width, height;
@@ -418,9 +418,11 @@ gimp_drawable_get_line_art_fill_buffer (GimpDrawable      *drawable,
       g_return_val_if_fail (GIMP_IS_DRAWABLE (pickable), NULL);
 
       if (gimp_fill_options_get_style (options) == GIMP_FILL_STYLE_FG_COLOR)
-        gimp_context_get_foreground (GIMP_CONTEXT (options), &fill_color);
+        fill_color = gimp_context_get_foreground (GIMP_CONTEXT (options));
       else if (gimp_fill_options_get_style (options) == GIMP_FILL_STYLE_BG_COLOR)
-        gimp_context_get_background (GIMP_CONTEXT (options), &fill_color);
+        fill_color = gimp_context_get_background (GIMP_CONTEXT (options));
+
+      g_return_val_if_fail (fill_color != NULL, NULL);
 
       fill_buffer   = gimp_drawable_get_buffer (drawable);
       fill_offset_x = gimp_item_get_offset_x (GIMP_ITEM (drawable)) -
@@ -430,7 +432,7 @@ gimp_drawable_get_line_art_fill_buffer (GimpDrawable      *drawable,
     }
   new_mask = gimp_pickable_contiguous_region_by_line_art (NULL, line_art,
                                                           fill_buffer,
-                                                          &fill_color,
+                                                          fill_color,
                                                           fill_color_threshold,
                                                           fill_offset_x,
                                                           fill_offset_y,
@@ -468,12 +470,12 @@ gimp_drawable_get_line_art_fill_buffer (GimpDrawable      *drawable,
       GList             *drawables;
       GimpContext       *context = gimp_get_user_context (image->gimp);
       GError            *error   = NULL;
-      const GimpRGB      white   = {1.0, 1.0, 1.0, 1.0};
+      GeglColor         *white   = gegl_color_new ("white");
 
       context = gimp_config_duplicate (GIMP_CONFIG (context));
       /* As we are stroking a mask, we need to set color to white. */
-      gimp_context_set_foreground (GIMP_CONTEXT (context),
-                                   &white);
+      gimp_context_set_foreground (GIMP_CONTEXT (context), white);
+      g_object_unref (white);
 
       channel = gimp_channel_new_from_buffer (image, new_mask, NULL, NULL);
       stroked = gimp_channel_new_from_buffer (image, rendered_mask, NULL, NULL);
