@@ -179,7 +179,7 @@ gimp_drawable_fill_boundary (GimpDrawable       *drawable,
 gboolean
 gimp_drawable_fill_path (GimpDrawable     *drawable,
                          GimpFillOptions  *options,
-                         GimpPath         *vectors,
+                         GimpPath         *path,
                          gboolean          push_undo,
                          GError          **error)
 {
@@ -188,14 +188,14 @@ gimp_drawable_fill_path (GimpDrawable     *drawable,
   g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), FALSE);
   g_return_val_if_fail (gimp_item_is_attached (GIMP_ITEM (drawable)), FALSE);
   g_return_val_if_fail (GIMP_IS_FILL_OPTIONS (options), FALSE);
-  g_return_val_if_fail (GIMP_IS_PATH (vectors), FALSE);
+  g_return_val_if_fail (GIMP_IS_PATH (path), FALSE);
   g_return_val_if_fail (gimp_fill_options_get_style (options) !=
                         GIMP_FILL_STYLE_PATTERN ||
                         gimp_context_get_pattern (GIMP_CONTEXT (options)) != NULL,
                         FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  bezier = gimp_path_get_bezier (vectors);
+  bezier = gimp_path_get_bezier (path);
 
   if (bezier && bezier->num_data > 4)
     {
