@@ -12,13 +12,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 #include "libgimpconfig/gimpconfig.h"
 
@@ -90,9 +92,7 @@ struct _GimpDynamicsOutputPrivate
 };
 
 #define GET_PRIVATE(output) \
-        G_TYPE_INSTANCE_GET_PRIVATE (output, \
-                                     GIMP_TYPE_DYNAMICS_OUTPUT, \
-                                     GimpDynamicsOutputPrivate)
+        ((GimpDynamicsOutputPrivate *) gimp_dynamics_output_get_instance_private ((GimpDynamicsOutput *) (output)))
 
 
 static void   gimp_dynamics_output_finalize     (GObject           *object);
@@ -116,6 +116,7 @@ static void   gimp_dynamics_output_curve_dirty  (GimpCurve          *curve,
 
 G_DEFINE_TYPE_WITH_CODE (GimpDynamicsOutput, gimp_dynamics_output,
                          GIMP_TYPE_OBJECT,
+                         G_ADD_PRIVATE (GimpDynamicsOutput)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_CONFIG, NULL))
 
 #define parent_class gimp_dynamics_output_parent_class
@@ -138,78 +139,89 @@ gimp_dynamics_output_class_init (GimpDynamicsOutputClass *klass)
                                                       GIMP_PARAM_READWRITE |
                                                       G_PARAM_CONSTRUCT));
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_PRESSURE,
-                                    "use-pressure", NULL,
-                                    DEFAULT_USE_PRESSURE,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_PRESSURE,
+                            "use-pressure",
+                            NULL, NULL,
+                            DEFAULT_USE_PRESSURE,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_VELOCITY,
-                                    "use-velocity", NULL,
-                                    DEFAULT_USE_VELOCITY,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_VELOCITY,
+                            "use-velocity",
+                            NULL, NULL,
+                            DEFAULT_USE_VELOCITY,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_DIRECTION,
-                                    "use-direction", NULL,
-                                    DEFAULT_USE_DIRECTION,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_DIRECTION,
+                            "use-direction",
+                            NULL, NULL,
+                            DEFAULT_USE_DIRECTION,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_TILT,
-                                    "use-tilt", NULL,
-                                    DEFAULT_USE_TILT,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_TILT,
+                            "use-tilt",
+                            NULL, NULL,
+                            DEFAULT_USE_TILT,
+                            GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_WHEEL,
+                            "use-wheel",
+                            NULL, NULL,
+                            DEFAULT_USE_TILT,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_WHEEL,
-                                    "use-wheel", NULL,
-                                    DEFAULT_USE_TILT,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_RANDOM,
+                            "use-random",
+                            NULL, NULL,
+                            DEFAULT_USE_RANDOM,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_RANDOM,
-                                    "use-random", NULL,
-                                    DEFAULT_USE_RANDOM,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_USE_FADE,
+                            "use-fade",
+                            NULL, NULL,
+                            DEFAULT_USE_FADE,
+                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_INSTALL_PROP_BOOLEAN (object_class, PROP_USE_FADE,
-                                    "use-fade", NULL,
-                                    DEFAULT_USE_FADE,
-                                    GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_PRESSURE_CURVE,
+                           "pressure-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_PRESSURE_CURVE,
-                                   "pressure-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_VELOCITY_CURVE,
+                           "velocity-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_VELOCITY_CURVE,
-                                   "velocity-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_DIRECTION_CURVE,
+                           "direction-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_DIRECTION_CURVE,
-                                   "direction-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_TILT_CURVE,
+                           "tilt-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_TILT_CURVE,
-                                   "tilt-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_WHEEL_CURVE,
+                           "wheel-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_WHEEL_CURVE,
-                                   "wheel-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_RANDOM_CURVE,
+                           "random-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_RANDOM_CURVE,
-                                   "random-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
-
-  GIMP_CONFIG_INSTALL_PROP_OBJECT (object_class, PROP_FADE_CURVE,
-                                   "fade-curve", NULL,
-                                   GIMP_TYPE_CURVE,
-                                   GIMP_CONFIG_PARAM_AGGREGATE);
-
-  g_type_class_add_private (klass, sizeof (GimpDynamicsOutputPrivate));
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_FADE_CURVE,
+                           "fade-curve",
+                            NULL, NULL,
+                           GIMP_TYPE_CURVE,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 }
 
 static void
@@ -238,13 +250,13 @@ gimp_dynamics_output_finalize (GObject *object)
 {
   GimpDynamicsOutputPrivate *private = GET_PRIVATE (object);
 
-  g_object_unref (private->pressure_curve);
-  g_object_unref (private->velocity_curve);
-  g_object_unref (private->direction_curve);
-  g_object_unref (private->tilt_curve);
-  g_object_unref (private->wheel_curve);
-  g_object_unref (private->random_curve);
-  g_object_unref (private->fade_curve);
+  g_clear_object (&private->pressure_curve);
+  g_clear_object (&private->velocity_curve);
+  g_clear_object (&private->direction_curve);
+  g_clear_object (&private->tilt_curve);
+  g_clear_object (&private->wheel_curve);
+  g_clear_object (&private->random_curve);
+  g_clear_object (&private->fade_curve);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -520,7 +532,7 @@ gimp_dynamics_output_get_angular_value (GimpDynamicsOutput *output,
 {
   GimpDynamicsOutputPrivate *private = GET_PRIVATE (output);
   gdouble                    total   = 0.0;
-  gdouble                    result  = 0.0; /* angles are additive, so we retun zero for no change. */
+  gdouble                    result  = 0.0; /* angles are additive, so we return zero for no change. */
   gint                       factors = 0;
 
   if (private->use_pressure)
@@ -539,8 +551,19 @@ gimp_dynamics_output_get_angular_value (GimpDynamicsOutput *output,
 
   if (private->use_direction)
     {
-      total += gimp_curve_map_value (private->direction_curve,
-                                     coords->direction);
+      gdouble angle = gimp_curve_map_value (private->direction_curve,
+                                            coords->direction);
+
+      if (options->brush_lock_to_view)
+        {
+          if (coords->reflect)
+            angle = 0.5 - angle;
+
+          angle -= coords->angle;
+          angle  = fmod (fmod (angle, 1.0) + 1.0, 1.0);
+        }
+
+      total += angle;
       factors++;
     }
 
@@ -555,12 +578,12 @@ gimp_dynamics_output_get_angular_value (GimpDynamicsOutput *output,
 
       if (tilt_x == 0.0)
         {
-          if (tilt_y >= 0.0)
-            tilt = 0.5;
+          if (tilt_y > 0.0)
+            tilt = 0.25;
           else if (tilt_y < 0.0)
-            tilt = 0.0;
+            tilt = 0.75;
           else
-            tilt = -1.0;
+            tilt = 0.0;
         }
       else
         {
