@@ -20,7 +20,8 @@
 
 #include "config.h"
 
-#include <appstream-glib.h>
+#include <appstream.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -45,11 +46,12 @@ enum
 
 struct _GimpExtensionPrivate
 {
-  gchar    *path;
+  gchar       *path;
 
-  AsApp    *app;
-  gboolean  writable;
-  gboolean  running;
+  AsMetadata  *metadata;
+  AsComponent *component;
+  gboolean     writable;
+  gboolean     running;
 
   /* Extension metadata: directories. */
   GList    *brush_paths;
@@ -144,8 +146,7 @@ gimp_extension_finalize (GObject *object)
   gimp_extension_clean (extension);
 
   g_free (extension->p->path);
-  if (extension->p->app)
-    g_object_unref (extension->p->app);
+  g_clear_object (&extension->p->metadata);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -222,31 +223,26 @@ gimp_extension_new (const gchar *dir,
 const gchar *
 gimp_extension_get_name (GimpExtension *extension)
 {
-  g_return_val_if_fail (extension->p->app != NULL, NULL);
+  g_return_val_if_fail (extension->p->component != NULL, NULL);
 
-  return as_app_get_name (extension->p->app, g_getenv ("LANGUAGE")) ?
-    as_app_get_name (extension->p->app, g_getenv ("LANGUAGE")) :
-    as_app_get_name (extension->p->app, NULL);
+  return as_component_get_name (extension->p->component);
 }
 
 const gchar *
 gimp_extension_get_comment (GimpExtension *extension)
 {
-  g_return_val_if_fail (extension->p->app != NULL, NULL);
 
-  return as_app_get_comment (extension->p->app, g_getenv ("LANGUAGE")) ?
-    as_app_get_comment (extension->p->app, g_getenv ("LANGUAGE")) :
-    as_app_get_comment (extension->p->app, NULL);
+  g_return_val_if_fail (extension->p->component != NULL, NULL);
+
+  return as_component_get_summary (extension->p->component);
 }
 
 const gchar *
 gimp_extension_get_description (GimpExtension *extension)
 {
-  g_return_val_if_fail (extension->p->app != NULL, NULL);
+  g_return_val_if_fail (extension->p->component != NULL, NULL);
 
-  return as_app_get_description (extension->p->app, g_getenv ("LANGUAGE")) ?
-    as_app_get_description (extension->p->app, g_getenv ("LANGUAGE")) :
-    as_app_get_description (extension->p->app, NULL);
+  return as_component_get_description (extension->p->component);
 }
 
 GdkPixbuf *
@@ -255,30 +251,74 @@ gimp_extension_get_screenshot (GimpExtension  *extension,
                                gint            height,
                                const gchar   **caption)
 {
-  GdkPixbuf    *pixbuf = NULL;
-  AsScreenshot *screenshot;
+  GdkPixbuf       *pixbuf       = NULL;
+  AsScreenshot    *screenshot   = NULL;
+  const GPtrArray *screenshots;
 
-  g_return_val_if_fail (extension->p->app != NULL, NULL);
+  g_return_val_if_fail (extension->p->component != NULL, NULL);
+
+#if AS_CHECK_VERSION(1, 0, 0)
+  screenshots = as_component_get_screenshots_all (extension->p->component);
+#else
+  screenshots = as_component_get_screenshots (extension->p->component);
+#endif
+
+  if (screenshots == NULL || screenshots->len == 0)
+    {
+      return pixbuf;
+    }
+
+  /* Look for the screenshot marked as default */
+  for (guint i = 0; i < screenshots->len; i++)
+    {
+      AsScreenshot *ss = g_ptr_array_index (screenshots, i);
+      if (as_screenshot_get_kind (ss) == AS_SCREENSHOT_KIND_DEFAULT)
+        {
+          screenshot = ss;
+          break;
+        }
+    }
+
+  if (screenshot == NULL)
+    {
+      g_printerr (_("Invalid AppStream metadata: failed to find default screenshot for extension \"%s\""),
+                  as_component_get_id (extension->p->component));
+    }
 
-  screenshot = as_app_get_screenshot_default (extension->p->app);
   if (screenshot)
     {
-      AsImage *image;
+      AsImage          *image   = NULL;
+      GFile            *file    = NULL;
+      GFileInputStream *istream = NULL;
+      GError           *error   = NULL;
 
-      image = as_screenshot_get_image_for_locale (screenshot, g_getenv ("LANGUAGE"), width, height);
-      if (! image)
-        image = as_screenshot_get_image_for_locale (screenshot, NULL, width, height);
+#if AS_CHECK_VERSION(1, 0, 0)
+      image = as_screenshot_get_image (screenshot, width, height, 1);
+#else
+      image = as_screenshot_get_image (screenshot, width, height);
+#endif
+
+      file = g_file_new_for_uri (as_image_get_url (image));
+      istream = g_file_read (file, NULL, &error);
+      if (istream != NULL)
+        {
+          pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (istream), NULL, &error);
+        }
+
+      if (error != NULL)
+        {
+          g_printerr (_("Invalid AppStream metadata: Error loading image: \"%s\""), error->message);
+        }
 
-      pixbuf = as_image_get_pixbuf (image);
       if (pixbuf)
         {
           g_object_ref (pixbuf);
         }
       else
         {
-          GFile            *file;
-          GFileInputStream *istream;
-          GError           *error = NULL;
+          GFile            *file    = NULL;
+          GFileInputStream *istream = NULL;
+          GError           *error   = NULL;
 
           file = g_file_new_for_uri (as_image_get_url (image));
           istream = g_file_read (file, NULL, &error);
@@ -298,9 +338,7 @@ gimp_extension_get_screenshot (GimpExtension  *extension,
 
       if (caption)
         {
-          *caption = as_screenshot_get_caption (screenshot, g_getenv ("LANGUAGE"));
-          if (*caption == NULL)
-            *caption = as_screenshot_get_caption (screenshot, NULL);
+          *caption = as_screenshot_get_caption (screenshot);
         }
     }
 
@@ -331,16 +369,29 @@ gboolean
 gimp_extension_load (GimpExtension  *extension,
                      GError        **error)
 {
-  AsApp     *app;
-  GPtrArray *extends;
-  GPtrArray *requires;
-  AsRelease *release;
-  gchar     *appdata_name;
-  gchar     *path;
-  gboolean   success     = FALSE;
-  gboolean   has_require = FALSE;
+  AsMetadata     *metadata;
+  AsComponent    *component;
+#if AS_CHECK_VERSION(1, 0, 0)
+  AsComponentBox *components   = NULL;
+#else
+  GPtrArray      *components   = NULL;
+#endif
+  GPtrArray      *extends;
+  GPtrArray      *relations;
+  AsRelease      *release      = NULL;
+#if AS_CHECK_VERSION(1, 0, 0)
+  AsReleaseList  *rlist        = NULL;
+#else
+  GPtrArray      *rlist        = NULL;
+#endif
+  gchar          *appdata_name;
+  gchar          *path;
+  GFile          *file;
+  gboolean        success      = FALSE;
+  gboolean        has_require  = FALSE;
 
-  g_clear_object (&extension->p->app);
+  g_clear_object (&extension->p->metadata);
+  extension->p->component = NULL;
 
   /* Search in subdirectory if a file with the same name as
    * directory and ending with ".metainfo.xml" exists.
@@ -350,12 +401,32 @@ gimp_extension_load (GimpExtension  *extension,
   path = g_build_filename (extension->p->path, appdata_name, NULL);
   g_free (appdata_name);
 
-  app = as_app_new ();
-  success = as_app_parse_file (app, path,
-                               AS_APP_PARSE_FLAG_USE_HEURISTICS,
-                               error);
+  file = g_file_new_for_path (path);
+
+  metadata = as_metadata_new ();
+  success = as_metadata_parse_file (metadata, file, AS_FORMAT_KIND_XML, error);
+
+  if (success)
+    {
+#if AS_CHECK_VERSION(1, 0, 0)
+      components = as_metadata_get_components (metadata);
+      component  = as_component_box_index (components, 0);
+#else
+      components = as_metadata_get_components (metadata);
+      component  = g_ptr_array_index (components, 0);
+#endif
+    }
+
+  g_object_unref (file);
   g_free (path);
-  if (success && as_app_get_kind (app) != AS_APP_KIND_ADDON)
+
+  if (!success)
+    {
+      return success;
+    }
+
+
+  if (success && as_component_get_kind (component) != AS_COMPONENT_KIND_ADDON)
     {
       /* Properly setting the type will allow extensions to be
        * distributed appropriately through other means.
@@ -364,11 +435,11 @@ gimp_extension_load (GimpExtension  *extension,
         *error = g_error_new (GIMP_EXTENSION_ERROR,
                               GIMP_EXTENSION_BAD_APPDATA,
                               _("Extension AppData must be of type \"addon\", found \"%s\" instead."),
-                              as_app_kind_to_string (as_app_get_kind (app)));
+                              as_component_kind_to_string (as_component_get_kind (component)));
       success = FALSE;
     }
 
-  extends = as_app_get_extends (app);
+  extends = as_component_get_extends (component);
   if (success &&
       ! g_ptr_array_find_with_equal_func (extends, "org.gimp.GIMP",
                                           g_str_equal, NULL))
@@ -384,7 +455,7 @@ gimp_extension_load (GimpExtension  *extension,
     }
 
   if (success &&
-      g_strcmp0 (as_app_get_id (app),
+      g_strcmp0 (as_component_get_id (component),
                  gimp_object_get_name (extension)) != 0)
     {
       /* Extension IDs will be unique and we want therefore the
@@ -394,11 +465,24 @@ gimp_extension_load (GimpExtension  *extension,
         *error = g_error_new (GIMP_EXTENSION_ERROR,
                               GIMP_EXTENSION_FAILED,
                               _("Extension AppData id (\"%s\") and directory (\"%s\") must be the same."),
-                              as_app_get_id (app), gimp_object_get_name (extension));
+                              as_component_get_id (component), gimp_object_get_name (extension));
       success = FALSE;
     }
 
-  release = as_app_get_release_default (app);
+#if AS_CHECK_VERSION(1, 0, 0)
+  rlist = as_component_get_releases_plain (component);
+  if (rlist != NULL && !as_release_list_is_empty (rlist))
+    {
+      release = as_release_list_index (rlist, 0);
+    }
+#else
+  rlist = as_component_get_releases (component);
+  if (rlist != NULL && rlist->len > 0)
+    {
+      release = g_ptr_array_index (rlist, 0);
+    }
+#endif
+
   if (success && (! release || ! as_release_get_version (release)))
     {
       /* We don't need the detail, just to know that the extension has a
@@ -412,24 +496,19 @@ gimp_extension_load (GimpExtension  *extension,
       success = FALSE;
     }
 
-  requires = as_app_get_requires (app);
-  if (success && requires)
+  relations = as_component_get_requires (component);
+  if (success && relations != NULL)
     {
-      gint i;
-
-      /* An extension could set requirements, in particular a range of
-       * supported version of GIMP, but also other extensions.
-       */
-
-      for (i = 0; i < requires->len; i++)
+      for (guint i = 0; i < relations->len; i++)
         {
-          AsRequire *require = g_ptr_array_index (requires, i);
+          AsRelation *relation = g_ptr_array_index(relations, i);
 
-          if (as_require_get_kind (require) == AS_REQUIRE_KIND_ID &&
-              g_strcmp0 (as_require_get_value (require), "org.gimp.GIMP") == 0)
+          if (as_relation_get_item_kind(relation) == AS_RELATION_ITEM_KIND_ID &&
+              g_strcmp0(as_relation_get_value_str(relation), "org.gimp.GIMP") == 0)
             {
               has_require = TRUE;
-              if (! as_require_version_compare (require, GIMP_VERSION, error))
+
+              if (! as_relation_version_compare (relation, GIMP_VERSION, error))
                 {
                   success = FALSE;
                   break;
@@ -437,14 +516,11 @@ gimp_extension_load (GimpExtension  *extension,
             }
           else if (error && *error == NULL)
             {
-              /* Right now we only support requirement relative to GIMP
-               * version.
-               */
               *error = g_error_new (GIMP_EXTENSION_ERROR,
                                     GIMP_EXTENSION_FAILED,
-                                    _("Unsupported <requires> \"%s\" (type %s)."),
-                                    as_require_get_value (require),
-                                    as_require_kind_to_string (as_require_get_kind (require)));
+                                    _("Unsupported <relation> \"%s\" (type %s)."),
+                                    as_relation_get_value_str(relation),
+                                    as_relation_item_kind_to_string(as_relation_get_item_kind(relation)));
               success = FALSE;
               break;
             }
@@ -462,9 +538,14 @@ gimp_extension_load (GimpExtension  *extension,
     }
 
   if (success)
-    extension->p->app = app;
+    {
+      extension->p->metadata  = metadata;
+      extension->p->component = component;
+    }
   else
-    g_object_unref (app);
+    {
+      g_clear_object (&metadata);
+    }
 
   return success;
 }
@@ -476,11 +557,11 @@ gimp_extension_run (GimpExtension  *extension,
   GHashTable *metadata;
   gchar      *value;
 
-  g_return_val_if_fail (extension->p->app != NULL, FALSE);
+  g_return_val_if_fail (extension->p->component != NULL, FALSE);
   g_return_val_if_fail (error && *error == NULL, FALSE);
 
   gimp_extension_clean (extension);
-  metadata = as_app_get_metadata (extension->p->app);
+  metadata = as_component_get_custom (extension->p->component);
 
   value = g_hash_table_lookup (metadata, "GIMP::brush-path");
   extension->p->brush_paths = gimp_extension_validate_paths (extension,
@@ -738,7 +819,7 @@ gimp_extension_validate_paths (GimpExtension  *extension,
 
   for (i = 0; patharray[i]; i++)
     {
-      /* Note: appstream-glib is supposed to return everything as UTF-8,
+      /* Note: appstream is supposed to return everything as UTF-8,
        * so we should not have to bother about this. */
       gchar    *path;
       GFile    *file;
