@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -22,8 +22,8 @@
 #include <errno.h>
 
 #include <cairo.h>
-#include <glib-object.h>
-#include <glib/gstdio.h>
+#include <gegl.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
@@ -32,6 +32,7 @@
 
 #include "config/gimpxmlparser.h"
 
+#include "gimp-utils.h"
 #include "gimpgradient.h"
 #include "gimpgradient-load.h"
 
@@ -39,68 +40,53 @@
 
 
 GList *
-gimp_gradient_load (GimpContext  *context,
-                    const gchar  *filename,
-                    GError      **error)
+gimp_gradient_load (GimpContext   *context,
+                    GFile         *file,
+                    GInputStream  *input,
+                    GError       **error)
 {
-  GimpGradient        *gradient;
+  GimpGradient        *gradient = NULL;
   GimpGradientSegment *prev;
   gint                 num_segments;
   gint                 i;
-  FILE                *file;
-  gchar                line[1024];
+  GDataInputStream    *data_input;
+  gchar               *line;
+  gsize                line_len;
   gint                 linenum;
 
-  g_return_val_if_fail (filename != NULL, NULL);
-  g_return_val_if_fail (g_path_is_absolute (filename), NULL);
+  g_return_val_if_fail (G_IS_FILE (file), NULL);
+  g_return_val_if_fail (G_IS_INPUT_STREAM (input), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  file = g_fopen (filename, "rb");
-
-  if (!file)
-    {
-      g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_OPEN,
-                   _("Could not open '%s' for reading: %s"),
-                   gimp_filename_to_utf8 (filename), g_strerror (errno));
-      return NULL;
-    }
+  data_input = g_data_input_stream_new (input);
 
   linenum = 1;
-  if (! fgets (line, sizeof (line), file))
-    {
-      g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                   _("Fatal parse error in gradient file '%s': "
-                     "Read error in line %d."),
-                   gimp_filename_to_utf8 (filename), linenum);
-      fclose (file);
-      return NULL;
-    }
+  line_len = 1024;
+  line = gimp_data_input_stream_read_line_always (data_input, &line_len,
+                                                  NULL, error);
+  if (! line)
+    goto failed;
 
   if (! g_str_has_prefix (line, "GIMP Gradient"))
     {
       g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                   _("Fatal parse error in gradient file '%s': "
-                     "Not a GIMP gradient file."),
-                   gimp_filename_to_utf8 (filename));
-      fclose (file);
-      return NULL;
+                   _("Not a GIMP gradient file."));
+      g_free (line);
+      goto failed;
     }
 
+  g_free (line);
+
   gradient = g_object_new (GIMP_TYPE_GRADIENT,
                            "mime-type", "application/x-gimp-gradient",
                            NULL);
 
   linenum++;
-  if (! fgets (line, sizeof (line), file))
-    {
-      g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                   _("Fatal parse error in gradient file '%s': "
-                     "Read error in line %d."),
-                   gimp_filename_to_utf8 (filename), linenum);
-      fclose (file);
-      g_object_unref (gradient);
-      return NULL;
-    }
+  line_len = 1024;
+  line = gimp_data_input_stream_read_line_always (data_input, &line_len,
+                                                  NULL, error);
+  if (! line)
+    goto failed;
 
   if (g_str_has_prefix (line, "Name: "))
     {
@@ -108,38 +94,33 @@ gimp_gradient_load (GimpContext  *context,
 
       utf8 = gimp_any_to_utf8 (g_strstrip (line + strlen ("Name: ")), -1,
                                _("Invalid UTF-8 string in gradient file '%s'."),
-                               gimp_filename_to_utf8 (filename));
+                               gimp_file_get_utf8_name (file));
       gimp_object_take_name (GIMP_OBJECT (gradient), utf8);
 
+      g_free (line);
+
       linenum++;
-      if (! fgets (line, sizeof (line), file))
-        {
-          g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                       _("Fatal parse error in gradient file '%s': "
-                         "Read error in line %d."),
-                       gimp_filename_to_utf8 (filename), linenum);
-          fclose (file);
-          g_object_unref (gradient);
-          return NULL;
-        }
+      line_len = 1024;
+      line = gimp_data_input_stream_read_line_always (data_input, &line_len,
+                                                      NULL, error);
+      if (! line)
+        goto failed;
     }
   else /* old gradient format */
     {
       gimp_object_take_name (GIMP_OBJECT (gradient),
-                             g_filename_display_basename (filename));
+                             g_path_get_basename (gimp_file_get_utf8_name (file)));
     }
 
   num_segments = atoi (line);
 
+  g_free (line);
+
   if (num_segments < 1)
     {
       g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                   _("Fatal parse error in gradient file '%s': "
-                     "File is corrupt in line %d."),
-                   gimp_filename_to_utf8 (filename), linenum);
-      g_object_unref (gradient);
-      fclose (file);
-      return NULL;
+                   _("File is corrupt."));
+      goto failed;
     }
 
   prev = NULL;
@@ -152,6 +133,8 @@ gimp_gradient_load (GimpContext  *context,
       gint                 type;
       gint                 left_color_type;
       gint                 right_color_type;
+      gdouble              left_rgba[4];
+      gdouble              right_rgba[4];
 
       seg = gimp_gradient_segment_new ();
 
@@ -163,107 +146,126 @@ gimp_gradient_load (GimpContext  *context,
         gradient->segments = seg;
 
       linenum++;
-      if (! fgets (line, sizeof (line), file))
+      line_len = 1024;
+      line = gimp_data_input_stream_read_line_always (data_input, &line_len,
+                                                      NULL, error);
+      if (! line)
+        goto failed;
+
+      if (! gimp_ascii_strtod (line, &end, &seg->left)     ||
+          ! gimp_ascii_strtod (end,  &end, &seg->middle)   ||
+          ! gimp_ascii_strtod (end,  &end, &seg->right)    ||
+
+          ! gimp_ascii_strtod (end,  &end, &left_rgba[0])  ||
+          ! gimp_ascii_strtod (end,  &end, &left_rgba[1])  ||
+          ! gimp_ascii_strtod (end,  &end, &left_rgba[2])  ||
+          ! gimp_ascii_strtod (end,  &end, &left_rgba[3])  ||
+
+          ! gimp_ascii_strtod (end,  &end, &right_rgba[0]) ||
+          ! gimp_ascii_strtod (end,  &end, &right_rgba[1]) ||
+          ! gimp_ascii_strtod (end,  &end, &right_rgba[2]) ||
+          ! gimp_ascii_strtod (end,  &end, &right_rgba[3]))
         {
           g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                       _("Fatal parse error in gradient file '%s': "
-                         "Read error in line %d."),
-                       gimp_filename_to_utf8 (filename), linenum);
-          fclose (file);
-          g_object_unref (gradient);
-          return NULL;
+                       _("Corrupt segment %d."), i);
+          g_free (line);
+          goto failed;
         }
 
-      seg->left = g_ascii_strtod (line, &end);
-      if (end && errno != ERANGE)
-        seg->middle = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->right = g_ascii_strtod (end, &end);
+      gegl_color_set_pixel (seg->left_color, babl_format ("R'G'B'A double"), left_rgba);
+      gegl_color_set_pixel (seg->right_color, babl_format ("R'G'B'A double"), right_rgba);
 
-      if (end && errno != ERANGE)
-        seg->left_color.r = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->left_color.g = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->left_color.b = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->left_color.a = g_ascii_strtod (end, &end);
-
-      if (end && errno != ERANGE)
-        seg->right_color.r = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->right_color.g = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->right_color.b = g_ascii_strtod (end, &end);
-      if (end && errno != ERANGE)
-        seg->right_color.a = g_ascii_strtod (end, &end);
-
-      if (errno != ERANGE)
+      switch (sscanf (end, "%d %d %d %d",
+                      &type, &color,
+                      &left_color_type, &right_color_type))
         {
-          switch (sscanf (end, "%d %d %d %d",
-                          &type, &color,
-                          &left_color_type, &right_color_type))
+        case 4:
+          seg->left_color_type  = (GimpGradientColor) left_color_type;
+          if (seg->left_color_type < GIMP_GRADIENT_COLOR_FIXED ||
+              seg->left_color_type > GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
             {
-            case 4:
-              seg->left_color_type  = (GimpGradientColor) left_color_type;
-              seg->right_color_type = (GimpGradientColor) right_color_type;
-              /* fall thru */
-
-            case 2:
-              seg->type  = (GimpGradientSegmentType) type;
-              seg->color = (GimpGradientSegmentColor) color;
-              break;
-
-            default:
               g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                           _("Fatal parse error in gradient file '%s': "
-                             "Corrupt segment %d in line %d."),
-                           gimp_filename_to_utf8 (filename), i, linenum);
-              g_object_unref (gradient);
-              fclose (file);
-              return NULL;
+                           _("Corrupt segment %d."), i);
+              g_free (line);
+              goto failed;
             }
-        }
-      else
-        {
+
+          seg->right_color_type = (GimpGradientColor) right_color_type;
+          if (seg->right_color_type < GIMP_GRADIENT_COLOR_FIXED ||
+              seg->right_color_type > GIMP_GRADIENT_COLOR_BACKGROUND_TRANSPARENT)
+            {
+              g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
+                           _("Corrupt segment %d."), i);
+              g_free (line);
+              goto failed;
+            }
+          /* fall thru */
+
+        case 2:
+          seg->type  = (GimpGradientSegmentType) type;
+          if (seg->type < GIMP_GRADIENT_SEGMENT_LINEAR ||
+              seg->type > GIMP_GRADIENT_SEGMENT_STEP)
+            {
+              g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
+                           _("Corrupt segment %d."), i);
+              g_free (line);
+              goto failed;
+            }
+
+          seg->color = (GimpGradientSegmentColor) color;
+          if (seg->color < GIMP_GRADIENT_SEGMENT_RGB ||
+              seg->color > GIMP_GRADIENT_SEGMENT_HSV_CW)
+            {
+              g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
+                           _("Corrupt segment %d."), i);
+              g_free (line);
+              goto failed;
+            }
+          break;
+
+        default:
           g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                       _("Fatal parse error in gradient file '%s': "
-                         "Corrupt segment %d in line %d."),
-                       gimp_filename_to_utf8 (filename), i, linenum);
-          g_object_unref (gradient);
-          fclose (file);
-          return NULL;
+                       _("Corrupt segment %d."), i);
+          g_free (line);
+          goto failed;
         }
 
-      if ( (prev && (prev->right < seg->left))
-           || (!prev && (0. < seg->left) ))
+      g_free (line);
+
+      if (seg->left   > seg->middle              ||
+          seg->middle > seg->right               ||
+          (  prev && (prev->right != seg->left)) ||
+          (! prev && (0.0         != seg->left)))
         {
           g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                       _("Gradient file '%s' is corrupt: "
-                         "Segments do not span the range 0-1."),
-                       gimp_filename_to_utf8 (filename));
-          g_object_unref (gradient);
-          fclose (file);
-          return NULL;
+                       _("Segments do not span the range 0-1."));
+          goto failed;
         }
 
       prev = seg;
     }
 
-  if (prev->right < 1.0)
+  if (prev->right != 1.0)
     {
       g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                   _("Gradient file '%s' is corrupt: "
-                     "Segments do not span the range 0-1."),
-                   gimp_filename_to_utf8 (filename));
-      g_object_unref (gradient);
-      fclose (file);
-      return NULL;
+                   _("Segments do not span the range 0-1."));
+      goto failed;
     }
 
-  fclose (file);
+  g_object_unref (data_input);
 
   return g_list_prepend (NULL, gradient);
+
+ failed:
+
+  g_object_unref (data_input);
+
+  if (gradient)
+    g_object_unref (gradient);
+
+  g_prefix_error (error, _("In line %d of gradient file: "), linenum);
+
+  return NULL;
 }
 
 
@@ -279,7 +281,8 @@ typedef struct
 typedef struct
 {
   gdouble       offset;
-  GimpRGB       color;
+  GeglColor    *color;
+  gdouble       opacity;
 } SvgStop;
 
 
@@ -312,42 +315,32 @@ static const GMarkupParser markup_parser =
 
 
 GList *
-gimp_gradient_load_svg (GimpContext  *context,
-                        const gchar  *filename,
-                        GError      **error)
+gimp_gradient_load_svg (GimpContext   *context,
+                        GFile         *file,
+                        GInputStream  *input,
+                        GError       **error)
 {
   GimpXmlParser *xml_parser;
   SvgParser      parser = { NULL, };
   gboolean       success;
 
-  g_return_val_if_fail (filename != NULL, NULL);
-  g_return_val_if_fail (g_path_is_absolute (filename), NULL);
+  g_return_val_if_fail (G_IS_FILE (file), NULL);
+  g_return_val_if_fail (G_IS_INPUT_STREAM (input), NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
+  /* FIXME input */
+  g_input_stream_close (input, NULL, NULL);
+
   xml_parser = gimp_xml_parser_new (&markup_parser, &parser);
 
-  success = gimp_xml_parser_parse_file (xml_parser, filename, error);
+  success = gimp_xml_parser_parse_gfile (xml_parser, file, error);
 
   gimp_xml_parser_free (xml_parser);
 
   if (success && ! parser.gradients)
     {
       g_set_error (error, GIMP_DATA_ERROR, GIMP_DATA_ERROR_READ,
-                   _("No linear gradients found in '%s'"),
-                   gimp_filename_to_utf8 (filename));
-    }
-  else
-    {
-      if (error && *error) /*  parser reported an error  */
-        {
-          gchar *msg = (*error)->message;
-
-          (*error)->message =
-            g_strdup_printf (_("Failed to import gradients from '%s': %s"),
-                             gimp_filename_to_utf8 (filename), msg);
-
-          g_free (msg);
-        }
+                   _("No linear gradients found."));
     }
 
   if (parser.gradient)
@@ -422,8 +415,21 @@ svg_parser_end_element (GMarkupParseContext  *context,
   if (parser->gradient &&
       strcmp (element_name, "linearGradient") == 0)
     {
+      GList *list;
+
       parser->gradient->segments = svg_parser_gradient_segments (parser->stops);
 
+      for (list = parser->stops; list; list = list->next)
+        {
+          SvgStop *stop = list->data;
+
+          g_clear_object (&stop->color);
+          g_slice_free (SvgStop, stop);
+        }
+
+      g_list_free (parser->stops);
+      parser->stops = NULL;
+
       if (parser->gradient->segments)
         parser->gradients = g_list_prepend (parser->gradients,
                                             parser->gradient);
@@ -448,8 +454,10 @@ svg_parser_gradient_segments (GList *stops)
 
   segment = gimp_gradient_segment_new ();
 
-  segment->left_color  = stop->color;
-  segment->right_color = stop->color;
+  g_clear_object (&segment->left_color);
+  segment->left_color  = gegl_color_duplicate (stop->color);
+  g_clear_object (&segment->right_color);
+  segment->right_color = gegl_color_duplicate (stop->color);
 
   /*  the list of offsets is sorted from largest to smallest  */
   for (list = g_list_next (stops); list; list = g_list_next (list))
@@ -465,17 +473,22 @@ svg_parser_gradient_segments (GList *stops)
       next->prev    = segment;
 
       segment->right       = stop->offset;
-      segment->right_color = stop->color;
+      g_clear_object (&segment->right_color);
+      segment->right_color = gegl_color_duplicate (stop->color);
 
       stop = list->data;
 
-      segment->left_color  = stop->color;
+      g_clear_object (&segment->left_color);
+      segment->left_color = gegl_color_duplicate (stop->color);
     }
 
   segment->middle = (segment->left + segment->right) / 2.0;
 
   if (stop->offset > 0.0)
-    segment->right_color = stop->color;
+    {
+      g_clear_object (&segment->right_color);
+      segment->right_color = gegl_color_duplicate (stop->color);
+    }
 
   /*  FIXME: remove empty segments here or add a GimpGradient API to do that
    */
@@ -490,14 +503,15 @@ svg_parse_gradient_stop_style_prop (SvgStop     *stop,
 {
   if (strcmp (name, "stop-color") == 0)
     {
-      gimp_rgb_parse_css (&stop->color, value, -1);
+      g_clear_object (&stop->color);
+      stop->color = gimp_color_parse_css (value);
     }
   else if (strcmp (name, "stop-opacity") == 0)
     {
       gdouble opacity = g_ascii_strtod (value, NULL);
 
       if (errno != ERANGE)
-        gimp_rgb_set_alpha (&stop->color, CLAMP (opacity, 0.0, 1.0));
+        stop->opacity = CLAMP (opacity, 0.0, 1.0);
     }
 }
 
@@ -527,7 +541,7 @@ svg_parse_gradient_stop_style (SvgStop     *stop,
 
           name = g_strndup (style, sep - style);
           sep++;
-          value = g_strndup (sep, end - sep - (*end == ';' ? 1 : 0));
+          value = g_strndup (sep, end - sep);
 
           svg_parse_gradient_stop_style_prop (stop, name, value);
 
@@ -548,7 +562,8 @@ svg_parse_gradient_stop (const gchar **names,
 {
   SvgStop *stop = g_slice_new0 (SvgStop);
 
-  gimp_rgb_set_alpha (&stop->color, 1.0);
+  stop->color   = NULL;
+  stop->opacity = 1.0;
 
   while (*names && *values)
     {
@@ -576,5 +591,13 @@ svg_parse_gradient_stop (const gchar **names,
       values++;
     }
 
+  if (! stop->color)
+    /* Default stop color is black:
+     * https://svgwg.org/svg2-draft/pservers.html#GradientStops
+     */
+    stop->color = gegl_color_new ("black");
+
+  gimp_color_set_alpha (stop->color, stop->opacity);
+
   return stop;
 }
