@@ -33,9 +33,7 @@
 #include "gegl/gimp-babl.h"
 #include "gegl/gimp-gegl-loops.h"
 
-#include "gimpchannel.h"
 #include "gimpdrawable-filters.h"
-#include "gimpdrawablefilter.h"
 #include "gimpgrouplayer.h"
 #include "gimpgrouplayerundo.h"
 #include "gimpimage.h"
@@ -277,6 +275,7 @@ gimp_group_layer_class_init (GimpGroupLayerClass *klass)
   gimp_object_class->get_memsize         = gimp_group_layer_get_memsize;
 
   viewable_class->default_icon_name      = "gimp-group-layer";
+  viewable_class->default_name           = _("Layer Group");
   viewable_class->ancestry_changed       = gimp_group_layer_ancestry_changed;
   viewable_class->get_size               = gimp_group_layer_get_size;
   viewable_class->get_children           = gimp_group_layer_get_children;
@@ -291,7 +290,6 @@ gimp_group_layer_class_init (GimpGroupLayerClass *klass)
   item_class->resize                     = gimp_group_layer_resize;
   item_class->get_clip                   = gimp_group_layer_get_clip;
 
-  item_class->default_name               = _("Layer Group");
   item_class->rename_desc                = C_("undo-type", "Rename Layer Group");
   item_class->translate_desc             = C_("undo-type", "Move Layer Group");
   item_class->scale_desc                 = C_("undo-type", "Scale Layer Group");
@@ -594,7 +592,6 @@ gimp_group_layer_duplicate (GimpItem *item,
           GimpItem      *child = list->data;
           GimpItem      *new_child;
           GimpLayerMask *mask;
-          GimpContainer *filters;
 
           new_child = gimp_item_duplicate (child, G_TYPE_FROM_INSTANCE (child));
 
@@ -619,36 +616,6 @@ gimp_group_layer_duplicate (GimpItem *item,
           gimp_container_insert (new_private->children,
                                  GIMP_OBJECT (new_child),
                                  position++);
-
-          /* Copy any attached layer effects */
-          filters = gimp_drawable_get_filters (GIMP_DRAWABLE (child));
-          if (gimp_container_get_n_children (filters) > 0)
-            {
-              GList *filter_list;
-
-              for (filter_list = GIMP_LIST (filters)->queue->tail; filter_list;
-                   filter_list = g_list_previous (filter_list))
-                {
-                  if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-                    {
-                      GimpDrawableFilter *old_filter = filter_list->data;
-                      GimpDrawableFilter *filter;
-
-                      filter =
-                        gimp_drawable_filter_duplicate (GIMP_DRAWABLE (new_child),
-                                                        old_filter);
-
-                      if (filter != NULL)
-                        {
-                          gimp_drawable_filter_apply (filter, NULL);
-                          gimp_drawable_filter_commit (filter, TRUE, NULL, FALSE);
-
-                          gimp_drawable_filter_layer_mask_freeze (filter);
-                          g_object_unref (filter);
-                        }
-                    }
-                }
-            }
         }
 
       /*  force the projection to reallocate itself  */
@@ -2002,7 +1969,6 @@ gimp_group_layer_update_size (GimpGroupLayer *group)
   gboolean               size_changed;
   gboolean               resize_mask;
   GList                 *list;
-  GimpContainer         *filters;
 
   old_bounds.x      = gimp_item_get_offset_x (item);
   old_bounds.y      = gimp_item_get_offset_y (item);
@@ -2137,29 +2103,6 @@ gimp_group_layer_update_size (GimpGroupLayer *group)
   if (resize_mask && ! private->transforming)
     gimp_group_layer_update_mask_size (group);
 
-  /* Update the crop of any filters */
-  if (size_changed)
-    {
-      filters = gimp_drawable_get_filters (GIMP_DRAWABLE (group));
-      for (list = GIMP_LIST (filters)->queue->tail;
-           list; list = g_list_previous (list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (list->data))
-            {
-              GimpDrawableFilter *filter = list->data;
-              GimpChannel        *filter_mask;
-
-              filter_mask = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
-
-              /* Don't resize partial layer effects */
-              if (gimp_channel_is_empty (filter_mask))
-                gimp_drawable_filter_refresh_crop (filter, &bounding_box);
-            }
-        }
-      if (list)
-        g_list_free (list);
-    }
-
   /* if we show the mask, invalidate the new mask area */
   if (resize_mask && gimp_layer_get_show_mask (layer))
     {
