@@ -30,8 +30,6 @@
 
 #include "gimp.h"
 #include "gimpchannel.h"
-#include "gimpdrawable-filters.h"
-#include "gimpdrawablefilter.h"
 #include "gimpguide.h"
 #include "gimpimage.h"
 #include "gimpimage-color-profile.h"
@@ -259,9 +257,8 @@ gimp_image_duplicate_layers (GimpImage *image,
        list;
        list = g_list_next (list))
     {
-      GimpLayer     *layer = list->data;
-      GimpLayer     *new_layer;
-      GimpContainer *filters;
+      GimpLayer *layer = list->data;
+      GimpLayer *new_layer;
 
       if (gimp_layer_is_floating_sel (layer))
         continue;
@@ -278,36 +275,6 @@ gimp_image_duplicate_layers (GimpImage *image,
 
       gimp_image_add_layer (new_image, new_layer,
                             NULL, count++, FALSE);
-
-      /* Import any attached layer effects */
-      filters = gimp_drawable_get_filters (GIMP_DRAWABLE (layer));
-      if (gimp_container_get_n_children (filters) > 0)
-        {
-          GList *filter_list;
-
-          for (filter_list = GIMP_LIST (filters)->queue->tail; filter_list;
-               filter_list = g_list_previous (filter_list))
-            {
-              if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-                {
-                  GimpDrawableFilter *old_filter = filter_list->data;
-                  GimpDrawableFilter *filter;
-
-                  filter =
-                    gimp_drawable_filter_duplicate (GIMP_DRAWABLE (new_layer),
-                                                    old_filter);
-
-                  if (filter != NULL)
-                    {
-                      gimp_drawable_filter_apply (filter, NULL);
-                      gimp_drawable_filter_commit (filter, TRUE, NULL, FALSE);
-
-                      gimp_drawable_filter_layer_mask_freeze (filter);
-                      g_object_unref (filter);
-                    }
-                }
-            }
-        }
     }
 
   new_item_stack = GIMP_ITEM_STACK (gimp_image_get_layers (new_image));
