@@ -55,7 +55,7 @@ struct _GimpImageClass
   void (* alpha_changed)                (GimpImage            *image);
   void (* floating_selection_changed)   (GimpImage            *image);
   void (* selected_channels_changed)    (GimpImage            *image);
-  void (* selected_vectors_changed)     (GimpImage            *image);
+  void (* selected_paths_changed)       (GimpImage            *image);
   void (* selected_layers_changed)      (GimpImage            *image);
   void (* component_visibility_changed) (GimpImage            *image,
                                          GimpChannelType       channel);
@@ -103,9 +103,8 @@ struct _GimpImageClass
   void (* undo_event)                   (GimpImage            *image,
                                          GimpUndoEvent         event,
                                          GimpUndo             *undo);
-  void (* layer_sets_changed)           (GimpImage            *image);
-  void (* channel_sets_changed)         (GimpImage            *image);
-  void (* vectors_sets_changed)         (GimpImage            *image);
+  void (* item_sets_changed)            (GimpImage            *image,
+                                         GType                 item_type);
 };
 
 
@@ -201,8 +200,8 @@ void            gimp_image_get_resolution        (GimpImage          *image,
 void            gimp_image_resolution_changed    (GimpImage          *image);
 
 void            gimp_image_set_unit              (GimpImage          *image,
-                                                  GimpUnit            unit);
-GimpUnit        gimp_image_get_unit              (GimpImage          *image);
+                                                  GimpUnit           *unit);
+GimpUnit      * gimp_image_get_unit              (GimpImage          *image);
 void            gimp_image_unit_changed          (GimpImage          *image);
 
 gint            gimp_image_get_width             (GimpImage          *image);
@@ -350,27 +349,29 @@ GimpTattoo      gimp_image_get_tattoo_state      (GimpImage          *image);
 GimpProjection * gimp_image_get_projection       (GimpImage          *image);
 
 
-/*  layers / channels / vectors  */
+/*  layers / channels / paths  */
 
 GimpItemTree  * gimp_image_get_layer_tree        (GimpImage          *image);
 GimpItemTree  * gimp_image_get_channel_tree      (GimpImage          *image);
-GimpItemTree  * gimp_image_get_vectors_tree      (GimpImage          *image);
+GimpItemTree  * gimp_image_get_path_tree         (GimpImage          *image);
 
+GimpContainer * gimp_image_get_items             (GimpImage          *image,
+                                                  GType               item_type);
 GimpContainer * gimp_image_get_layers            (GimpImage          *image);
 GimpContainer * gimp_image_get_channels          (GimpImage          *image);
-GimpContainer * gimp_image_get_vectors           (GimpImage          *image);
+GimpContainer * gimp_image_get_paths             (GimpImage          *image);
 
 gint            gimp_image_get_n_layers          (GimpImage          *image);
 gint            gimp_image_get_n_channels        (GimpImage          *image);
-gint            gimp_image_get_n_vectors         (GimpImage          *image);
+gint            gimp_image_get_n_paths           (GimpImage          *image);
 
 GList         * gimp_image_get_layer_iter        (GimpImage          *image);
 GList         * gimp_image_get_channel_iter      (GimpImage          *image);
-GList         * gimp_image_get_vectors_iter      (GimpImage          *image);
+GList         * gimp_image_get_path_iter         (GimpImage          *image);
 
 GList         * gimp_image_get_layer_list        (GimpImage          *image);
 GList         * gimp_image_get_channel_list      (GimpImage          *image);
-GList         * gimp_image_get_vectors_list      (GimpImage          *image);
+GList         * gimp_image_get_path_list         (GimpImage          *image);
 
 void          gimp_image_unset_selected_channels (GimpImage          *image);
 
@@ -380,29 +381,35 @@ gboolean     gimp_image_equal_selected_drawables (GimpImage          *image,
                                                   GList              *drawables);
 
 GList        * gimp_image_get_selected_drawables (GimpImage          *image);
+
+GList         * gimp_image_get_selected_items    (GimpImage          *image,
+                                                  GType               item_type);
 GList         * gimp_image_get_selected_layers   (GimpImage          *image);
 GList         * gimp_image_get_selected_channels (GimpImage          *image);
-GList         * gimp_image_get_selected_vectors  (GimpImage          *image);
+GList         * gimp_image_get_selected_paths    (GimpImage          *image);
 
+void            gimp_image_set_selected_items    (GimpImage          *image,
+                                                  GType               item_type,
+                                                  GList              *items);
 void            gimp_image_set_selected_layers   (GimpImage          *image,
                                                   GList              *layers);
 void            gimp_image_set_selected_channels (GimpImage          *image,
                                                   GList              *channels);
-void            gimp_image_set_selected_vectors  (GimpImage          *image,
-                                                  GList              *vectors);
+void            gimp_image_set_selected_paths    (GimpImage          *image,
+                                                  GList              *paths);
 
 GimpLayer     * gimp_image_get_layer_by_tattoo   (GimpImage          *image,
                                                   GimpTattoo          tattoo);
 GimpChannel   * gimp_image_get_channel_by_tattoo (GimpImage          *image,
                                                   GimpTattoo          tattoo);
-GimpVectors   * gimp_image_get_vectors_by_tattoo (GimpImage          *image,
+GimpPath      * gimp_image_get_path_by_tattoo    (GimpImage          *image,
                                                   GimpTattoo          tattoo);
 
 GimpLayer     * gimp_image_get_layer_by_name     (GimpImage          *image,
                                                   const gchar        *name);
 GimpChannel   * gimp_image_get_channel_by_name   (GimpImage          *image,
                                                   const gchar        *name);
-GimpVectors   * gimp_image_get_vectors_by_name   (GimpImage          *image,
+GimpPath      * gimp_image_get_path_by_name      (GimpImage          *image,
                                                   const gchar        *name);
 
 gboolean        gimp_image_reorder_item          (GimpImage          *image,
@@ -422,6 +429,16 @@ gboolean        gimp_image_lower_item            (GimpImage          *image,
 gboolean        gimp_image_lower_item_to_bottom  (GimpImage          *image,
                                                   GimpItem           *item);
 
+gboolean        gimp_image_add_item              (GimpImage          *image,
+                                                  GimpItem           *item,
+                                                  GimpItem          *parent,
+                                                  gint                position,
+                                                  gboolean            push_undo);
+void            gimp_image_remove_item           (GimpImage          *image,
+                                                  GimpItem           *item,
+                                                  gboolean            push_undo,
+                                                  GList              *new_selected);
+
 gboolean        gimp_image_add_layer             (GimpImage          *image,
                                                   GimpLayer          *layer,
                                                   GimpLayer          *parent,
@@ -442,6 +459,26 @@ void            gimp_image_add_layers            (GimpImage          *image,
                                                   gint                height,
                                                   const gchar        *undo_desc);
 
+gboolean        gimp_image_add_channel           (GimpImage          *image,
+                                                  GimpChannel        *channel,
+                                                  GimpChannel        *parent,
+                                                  gint                position,
+                                                  gboolean            push_undo);
+void            gimp_image_remove_channel        (GimpImage          *image,
+                                                  GimpChannel        *channel,
+                                                  gboolean            push_undo,
+                                                  GList              *new_selected);
+
+gboolean        gimp_image_add_path              (GimpImage          *image,
+                                                  GimpPath           *path,
+                                                  GimpPath           *parent,
+                                                  gint                position,
+                                                  gboolean            push_undo);
+void            gimp_image_remove_path           (GimpImage          *image,
+                                                  GimpPath           *path,
+                                                  gboolean            push_undo,
+                                                  GList              *new_selected);
+
 void            gimp_image_store_item_set        (GimpImage          *image,
                                                   GimpItemList       *set);
 gboolean        gimp_image_unlink_item_set       (GimpImage          *image,
@@ -457,26 +494,6 @@ void            gimp_image_remove_item_set       (GimpImage          *image,
 void            gimp_image_intersect_item_set    (GimpImage        *image,
                                                   GimpItemList       *set);
 
-gboolean        gimp_image_add_channel           (GimpImage          *image,
-                                                  GimpChannel        *channel,
-                                                  GimpChannel        *parent,
-                                                  gint                position,
-                                                  gboolean            push_undo);
-void            gimp_image_remove_channel        (GimpImage          *image,
-                                                  GimpChannel        *channel,
-                                                  gboolean            push_undo,
-                                                  GList              *new_selected);
-
-gboolean        gimp_image_add_vectors           (GimpImage          *image,
-                                                  GimpVectors        *vectors,
-                                                  GimpVectors        *parent,
-                                                  gint                position,
-                                                  gboolean            push_undo);
-void            gimp_image_remove_vectors        (GimpImage          *image,
-                                                  GimpVectors        *vectors,
-                                                  gboolean            push_undo,
-                                                  GList              *new_selected);
-
 gboolean        gimp_image_add_hidden_item       (GimpImage          *image,
                                                   GimpItem           *item);
 void            gimp_image_remove_hidden_item    (GimpImage          *image,
