@@ -32,8 +32,6 @@
 #include "gimp-parasites.h"
 #include "gimpchannel.h"
 #include "gimpcontainer.h"
-#include "gimpdrawable-filters.h"
-#include "gimpdrawablefilter.h"
 #include "gimpidtable.h"
 #include "gimpimage.h"
 #include "gimpimage-undo.h"
@@ -282,7 +280,6 @@ gimp_item_class_init (GimpItemClass *klass)
   klass->stroke                    = NULL;
   klass->to_selection              = NULL;
 
-  klass->default_name              = NULL;
   klass->rename_desc               = NULL;
   klass->translate_desc            = NULL;
   klass->scale_desc                = NULL;
@@ -663,7 +660,7 @@ gimp_item_real_start_transform (GimpItem *item,
 
 static void
 gimp_item_real_end_transform (GimpItem *item,
-                                gboolean  push_undo)
+                              gboolean  push_undo)
 {
   gimp_item_end_move (item, push_undo);
 }
@@ -777,7 +774,7 @@ gimp_item_new (GType        type,
     gimp_object_set_name (GIMP_OBJECT (item), name);
   else
     gimp_object_set_static_name (GIMP_OBJECT (item),
-                                 GIMP_ITEM_GET_CLASS (item)->default_name);
+                                 GIMP_VIEWABLE_GET_CLASS (item)->default_name);
 
   return item;
 }
@@ -1102,7 +1099,7 @@ gimp_item_rename (GimpItem     *item,
   item_class = GIMP_ITEM_GET_CLASS (item);
 
   if (! new_name || ! *new_name)
-    new_name = item_class->default_name;
+    new_name = GIMP_VIEWABLE_GET_CLASS (item)->default_name;
 
   if (strcmp (new_name, gimp_object_get_name (item)))
     return item_class->rename (item, new_name, item_class->rename_desc, error);
@@ -1786,36 +1783,6 @@ gimp_item_transform (GimpItem               *item,
 
   gimp_item_end_transform (item, push_undo);
 
-  /* Update crop of any filters */
-  if (GIMP_IS_DRAWABLE (item))
-    {
-      GeglRectangle  rect;
-      GimpContainer *filters;
-      GList         *filter_list;
-
-      rect = gimp_drawable_get_bounding_box (GIMP_DRAWABLE (item));
-      gimp_item_mask_intersect (item, &rect.x, &rect.y,
-                                &rect.width, &rect.height);
-
-      filters = gimp_drawable_get_filters (GIMP_DRAWABLE (item));
-
-      for (filter_list = GIMP_LIST (filters)->queue->tail; filter_list;
-           filter_list = g_list_previous (filter_list))
-        {
-          if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
-            {
-              GimpDrawableFilter *filter = filter_list->data;
-              GimpChannel        *mask;
-
-              mask = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
-
-              /* Don't resize partial layer effects */
-              if (! mask || gimp_channel_is_empty (mask))
-                gimp_drawable_filter_refresh_crop (filter, &rect);
-            }
-        }
-    }
-
   if (push_undo)
     gimp_image_undo_group_end (image);
 }
@@ -2514,12 +2481,9 @@ gimp_item_set_lock_content (GimpItem *item,
     {
       if (push_undo && gimp_item_is_attached (item))
         {
-          /* Right now I don't think this should be pushed. */
-#if 0
           GimpImage *image = gimp_item_get_image (item);
 
           gimp_image_undo_push_item_lock_content (image, NULL, item);
-#endif
         }
 
       GET_PRIVATE (item)->lock_content = lock_content;
