@@ -15,15 +15,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <stdlib.h>
-#include <string.h>
-
-#include <glib-object.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
@@ -33,7 +31,7 @@
 
 #include "config/gimpxmlparser.h"
 
-#include "gimp-utils.h"
+#include "gimp-memsize.h"
 #include "gimpcontext.h"
 #include "gimpdata.h"
 #include "gimplist.h"
@@ -70,7 +68,7 @@ typedef struct
   GimpTagCacheRecord  current_record;
 } GimpTagCacheParseData;
 
-struct _GimpTagCachePriv
+struct _GimpTagCachePrivate
 {
   GArray *records;
   GList  *containers;
@@ -112,7 +110,7 @@ static const gchar * gimp_tag_cache_attribute_name_to_value
 static GQuark        gimp_tag_cache_get_error_domain   (void);
 
 
-G_DEFINE_TYPE (GimpTagCache, gimp_tag_cache, GIMP_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpTagCache, gimp_tag_cache, GIMP_TYPE_OBJECT)
 
 #define parent_class gimp_tag_cache_parent_class
 
@@ -120,24 +118,18 @@ G_DEFINE_TYPE (GimpTagCache, gimp_tag_cache, GIMP_TYPE_OBJECT)
 static void
 gimp_tag_cache_class_init (GimpTagCacheClass *klass)
 {
-  GObjectClass      *object_class         = G_OBJECT_CLASS (klass);
-  GimpObjectClass   *gimp_object_class    = GIMP_OBJECT_CLASS (klass);
-  GimpTagCacheClass *gimp_tag_cache_class = GIMP_TAG_CACHE_CLASS (klass);
+  GObjectClass    *object_class      = G_OBJECT_CLASS (klass);
+  GimpObjectClass *gimp_object_class = GIMP_OBJECT_CLASS (klass);
 
   object_class->finalize         = gimp_tag_cache_finalize;
 
   gimp_object_class->get_memsize = gimp_tag_cache_get_memsize;
-
-  g_type_class_add_private (gimp_tag_cache_class,
-                            sizeof (GimpTagCachePriv));
 }
 
 static void
 gimp_tag_cache_init (GimpTagCache *cache)
 {
-  cache->priv = G_TYPE_INSTANCE_GET_PRIVATE (cache,
-                                             GIMP_TYPE_TAG_CACHE,
-                                             GimpTagCachePriv);
+  cache->priv = gimp_tag_cache_get_instance_private (cache);
 
   cache->priv->records    = g_array_new (FALSE, FALSE,
                                          sizeof (GimpTagCacheRecord));
@@ -191,7 +183,7 @@ gimp_tag_cache_get_memsize (GimpObject *object,
 /**
  * gimp_tag_cache_new:
  *
- * Return value: creates new GimpTagCache object.
+ * Returns: creates new GimpTagCache object.
  **/
 GimpTagCache *
 gimp_tag_cache_new (void)
@@ -350,12 +342,13 @@ gimp_tag_cache_tagged_to_cache_record_foreach (GimpTagged  *tagged,
 void
 gimp_tag_cache_save (GimpTagCache *cache)
 {
-  GString *buf;
-  GList   *saved_records;
-  GList   *iterator;
-  gchar   *filename;
-  GError  *error = NULL;
-  gint     i;
+  GString       *buf;
+  GList         *saved_records;
+  GList         *iterator;
+  GFile         *file;
+  GOutputStream *output;
+  GError        *error = NULL;
+  gint           i;
 
   g_return_if_fail (GIMP_IS_TAG_CACHE (cache));
 
@@ -427,16 +420,39 @@ gimp_tag_cache_save (GimpTagCache *cache)
 
   g_string_append (buf, "</tags>\n");
 
-  filename = g_build_filename (gimp_directory (), GIMP_TAG_CACHE_FILE, NULL);
+  file = gimp_directory_file (GIMP_TAG_CACHE_FILE, NULL);
 
-  if (! g_file_set_contents (filename, buf->str, buf->len, &error))
+  output = G_OUTPUT_STREAM (g_file_replace (file,
+                                            NULL, FALSE, G_FILE_CREATE_NONE,
+                                            NULL, &error));
+  if (! output)
     {
-      g_printerr ("Error while saving tag cache: %s\n", error->message);
-      g_error_free (error);
+      g_printerr ("%s\n", error->message);
+    }
+  else if (! g_output_stream_write_all (output, buf->str, buf->len,
+                                        NULL, NULL, &error))
+    {
+      GCancellable *cancellable = g_cancellable_new ();
+
+      g_printerr (_("Error writing '%s': %s\n"),
+                  gimp_file_get_utf8_name (file), error->message);
+
+      /* Cancel the overwrite initiated by g_file_replace(). */
+      g_cancellable_cancel (cancellable);
+      g_output_stream_close (output, cancellable, NULL);
+      g_object_unref (cancellable);
+    }
+  else if (! g_output_stream_close (output, NULL, &error))
+    {
+      g_printerr (_("Error closing '%s': %s\n"),
+                  gimp_file_get_utf8_name (file), error->message);
     }
 
-  g_free (filename);
+  if (output)
+    g_object_unref (output);
 
+  g_clear_error (&error);
+  g_object_unref (file);
   g_string_free (buf, TRUE);
 
   for (iterator = saved_records;
@@ -461,19 +477,17 @@ gimp_tag_cache_save (GimpTagCache *cache)
 void
 gimp_tag_cache_load (GimpTagCache *cache)
 {
-  gchar                 *filename;
-  GError                *error = NULL;
+  GFile                 *file;
   GMarkupParser          markup_parser;
   GimpXmlParser         *xml_parser;
   GimpTagCacheParseData  parse_data;
+  GError                *error = NULL;
 
   g_return_if_fail (GIMP_IS_TAG_CACHE (cache));
 
   /* clear any previous priv->records */
   cache->priv->records = g_array_set_size (cache->priv->records, 0);
 
-  filename = g_build_filename (gimp_directory (), GIMP_TAG_CACHE_FILE, NULL);
-
   parse_data.records = g_array_new (FALSE, FALSE, sizeof (GimpTagCacheRecord));
   memset (&parse_data.current_record, 0, sizeof (GimpTagCacheRecord));
 
@@ -485,7 +499,9 @@ gimp_tag_cache_load (GimpTagCache *cache)
 
   xml_parser = gimp_xml_parser_new (&markup_parser, &parse_data);
 
-  if (gimp_xml_parser_parse_file (xml_parser, filename, &error))
+  file = gimp_directory_file (GIMP_TAG_CACHE_FILE, NULL);
+
+  if (gimp_xml_parser_parse_gfile (xml_parser, file, &error))
     {
       cache->priv->records = g_array_append_vals (cache->priv->records,
                                                   parse_data.records->data,
@@ -494,10 +510,11 @@ gimp_tag_cache_load (GimpTagCache *cache)
   else
     {
       g_printerr ("Failed to parse tag cache: %s\n",
-                  error ? error->message : NULL);
+                  error ? error->message : "WTF unknown error");
+      g_clear_error (&error);
     }
 
-  g_free (filename);
+  g_object_unref (file);
   gimp_xml_parser_free (xml_parser);
   g_array_free (parse_data.records, TRUE);
 }
@@ -601,7 +618,7 @@ gimp_tag_cache_load_error (GMarkupParseContext *context,
                            GError              *error,
                            gpointer             user_data)
 {
-  printf ("Tag cache parse error: %s\n", error->message);
+  g_printerr ("Tag cache parse error: %s\n", error->message);
 }
 
 static const gchar*
