@@ -13,7 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -162,7 +162,7 @@ file_save_dialog_response (GtkWidget *dialog,
 
   if (response_id != GTK_RESPONSE_OK)
     {
-      if (! file_dialog->busy)
+      if (! file_dialog->busy && response_id != GTK_RESPONSE_HELP)
         gtk_widget_destroy (dialog);
 
       return;
@@ -179,23 +179,49 @@ file_save_dialog_response (GtkWidget *dialog,
 
     case CHECK_URI_OK:
       {
-        gboolean xcf_compression = FALSE;
+        GimpImage    *image              = file_dialog->image;
+        GimpProgress *progress           = GIMP_PROGRESS (dialog);
+        GimpDisplay  *display_to_close   = NULL;
+        gboolean      xcf_compression    = FALSE;
+        gboolean      is_save_dialog     = GIMP_IS_SAVE_DIALOG (dialog);
+        gboolean      close_after_saving = FALSE;
+        gboolean      save_a_copy        = FALSE;
+
+        if (is_save_dialog)
+          {
+            close_after_saving = GIMP_SAVE_DIALOG (dialog)->close_after_saving;
+            display_to_close   = GIMP_DISPLAY (GIMP_SAVE_DIALOG (dialog)->display_to_close);
+            save_a_copy        = GIMP_SAVE_DIALOG (dialog)->save_a_copy;
+          }
 
         gimp_file_dialog_set_sensitive (file_dialog, FALSE);
 
         if (GIMP_IS_SAVE_DIALOG (dialog))
+          xcf_compression = GIMP_SAVE_DIALOG (dialog)->compression;
+        else
+          xcf_compression = gimp_image_get_xcf_compression (image);
+
+        /* Hide the file dialog while exporting, avoid dialogs piling
+         * up, even more as some formats have preview features, so the
+         * file dialog is just blocking the view.
+         */
+        if  (GIMP_IS_EXPORT_DIALOG (dialog))
           {
-            xcf_compression = GIMP_SAVE_DIALOG (dialog)->compression;
+            gtk_widget_hide (dialog);
+            progress = GIMP_PROGRESS (GIMP_EXPORT_DIALOG (dialog)->display);
           }
 
-        if (file_save_dialog_save_image (GIMP_PROGRESS (dialog),
+        g_signal_connect (dialog, "destroy",
+                          G_CALLBACK (gtk_widget_destroyed),
+                          &dialog);
+
+        if (file_save_dialog_save_image (progress,
                                          gimp,
-                                         file_dialog->image,
+                                         image,
                                          file,
                                          save_proc,
                                          GIMP_RUN_INTERACTIVE,
-                                         GIMP_IS_SAVE_DIALOG (dialog) &&
-                                         ! GIMP_SAVE_DIALOG (dialog)->save_a_copy,
+                                         is_save_dialog && ! save_a_copy,
                                          FALSE,
                                          GIMP_IS_EXPORT_DIALOG (dialog),
                                          xcf_compression,
@@ -206,47 +232,65 @@ file_save_dialog_response (GtkWidget *dialog,
              * save. Lower-level URI management is handled in
              * file_save()
              */
-            if (GIMP_IS_SAVE_DIALOG (dialog))
+            if (is_save_dialog)
               {
-                if (GIMP_SAVE_DIALOG (dialog)->save_a_copy)
-                  gimp_image_set_save_a_copy_file (file_dialog->image, file);
+                if (save_a_copy)
+                  gimp_image_set_save_a_copy_file (image, file);
 
-                g_object_set_data_full (G_OBJECT (file_dialog->image->gimp),
+                g_object_set_data_full (G_OBJECT (image->gimp),
                                         GIMP_FILE_SAVE_LAST_FILE_KEY,
                                         g_object_ref (file),
                                         (GDestroyNotify) g_object_unref);
               }
             else
               {
-                g_object_set_data_full (G_OBJECT (file_dialog->image->gimp),
+                g_object_set_data_full (G_OBJECT (image->gimp),
                                         GIMP_FILE_EXPORT_LAST_FILE_KEY,
                                         g_object_ref (file),
                                         (GDestroyNotify) g_object_unref);
               }
 
             /*  make sure the menus are updated with the keys we've just set  */
-            gimp_image_flush (file_dialog->image);
+            gimp_image_flush (image);
 
             /* Handle close-after-saving */
-            if (GIMP_IS_SAVE_DIALOG (dialog)                  &&
-                GIMP_SAVE_DIALOG (dialog)->close_after_saving &&
-                GIMP_SAVE_DIALOG (dialog)->display_to_close)
+            if (close_after_saving && display_to_close &&
+                ! gimp_image_is_dirty (gimp_display_get_image (display_to_close)))
               {
-                GimpDisplay *display = GIMP_DISPLAY (GIMP_SAVE_DIALOG (dialog)->display_to_close);
-
-                if (! gimp_image_is_dirty (gimp_display_get_image (display)))
-                  {
-                    gimp_display_close (display);
-                  }
+                gimp_display_close (display_to_close);
               }
 
-            gtk_widget_destroy (dialog);
+            if (dialog)
+              gtk_widget_destroy (dialog);
+          }
+        else
+          {
+            if (dialog)
+              {
+                GFile *parent_dir = g_file_get_parent (file);
+
+                /* XXX Not sure why, but after reshowing the file
+                 * chooser dialog, the displayed name is correct, but
+                 * the parent directory is the current working dir.
+                 * Force it to be the expected folder.
+                 */
+                gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (dialog),
+                                                          parent_dir, NULL);
+                gtk_widget_show (dialog);
+                g_object_unref (parent_dir);
+              }
           }
 
         g_object_unref (file);
         g_free (basename);
 
-        gimp_file_dialog_set_sensitive (file_dialog, TRUE);
+        if (dialog)
+          {
+            gimp_file_dialog_set_sensitive (file_dialog, TRUE);
+            g_signal_handlers_disconnect_by_func (dialog,
+                                                  G_CALLBACK (gtk_widget_destroyed),
+                                                  &dialog);
+          }
       }
       break;
 
@@ -337,6 +381,8 @@ file_save_dialog_check_file (GtkWidget            *dialog,
           if (ext)
             {
               gchar *ext_basename;
+              gchar *dirname;
+              gchar *filename;
               gchar *utf8;
 
               GIMP_LOG (SAVE_DIALOG, "appending .%s to basename", ext);
@@ -346,11 +392,17 @@ file_save_dialog_check_file (GtkWidget            *dialog,
               g_free (basename);
               basename = ext_basename;
 
-              utf8 = g_filename_to_utf8 (basename, -1, NULL, NULL, NULL);
+              dirname  = g_path_get_dirname (gimp_file_get_utf8_name (file));
+              filename = g_build_filename (dirname, basename, NULL);
+              g_free (dirname);
+
+              utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
               gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
                                                  utf8);
               g_free (utf8);
 
+              g_free (filename);
+
               GIMP_LOG (SAVE_DIALOG,
                         "set basename to %s, rerunning response and bailing out",
                         basename);
@@ -708,7 +760,7 @@ file_save_dialog_use_extension (GtkWidget *save_dialog,
 
                                     NULL);
 
-  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
                                            GTK_RESPONSE_OK,
                                            GTK_RESPONSE_CANCEL,
                                            -1);
@@ -763,11 +815,18 @@ file_save_dialog_save_image (GimpProgress        *progress,
        list;
        list = g_list_next (list))
     {
-      gimp_action_group_set_action_sensitive (list->data, "file-quit", FALSE);
+      gimp_action_group_set_action_sensitive (list->data, "file-quit", FALSE, NULL);
     }
 
   gimp_image_set_xcf_compression (image, xcf_compression);
 
+  /* The save may fail and the progress widget be already freed if we
+   * close the main window while the save dialog is running. So add a
+   * weak pointer to avoid sending an error message to an already-freed
+   * GimpProgress. See #11922.
+   */
+  g_object_add_weak_pointer (G_OBJECT (progress), (gpointer *) &progress);
+
   status = file_save (gimp, image, progress, file,
                       save_proc, run_mode,
                       change_saved_state, export_backward, export_forward,
@@ -780,7 +839,7 @@ file_save_dialog_save_image (GimpProgress        *progress,
       break;
 
     case GIMP_PDB_CANCEL:
-      if (verbose_cancel)
+      if (verbose_cancel && progress)
         gimp_message_literal (gimp,
                               G_OBJECT (progress), GIMP_MESSAGE_INFO,
                               _("Saving canceled"));
@@ -788,19 +847,24 @@ file_save_dialog_save_image (GimpProgress        *progress,
 
     default:
       {
-        gimp_message (gimp, G_OBJECT (progress), GIMP_MESSAGE_ERROR,
-                      _("Saving '%s' failed:\n\n%s"),
-                      gimp_file_get_utf8_name (file), error->message);
+        if (progress)
+          gimp_message (gimp, G_OBJECT (progress), GIMP_MESSAGE_ERROR,
+                        _("Saving '%s' failed:\n\n%s"),
+                        gimp_file_get_utf8_name (file),
+                        error ? error->message : _("Unknown error"));
         g_clear_error (&error);
       }
       break;
     }
 
+  if (progress)
+    g_object_remove_weak_pointer (G_OBJECT (progress), (gpointer *) &progress);
+
   for (list = gimp_action_groups_from_name ("file");
        list;
        list = g_list_next (list))
     {
-      gimp_action_group_set_action_sensitive (list->data, "file-quit", TRUE);
+      gimp_action_group_set_action_sensitive (list->data, "file-quit", TRUE, NULL);
     }
 
   return success;
