@@ -94,9 +94,7 @@ static void        quit_close_all_dialog_accel_marshal     (GClosure          *c
 static void        quit_close_all_dialog_container_changed (GimpContainer     *images,
                                                             GimpObject        *image,
                                                             QuitDialog        *private);
-static gboolean    quit_close_all_dialog_images_selected   (GimpContainerView *view,
-                                                            GList             *images,
-                                                            GList             *paths,
+static void        quit_close_all_dialog_images_selected   (GimpContainerView *view,
                                                             QuitDialog        *private);
 static void        quit_close_all_dialog_name_cell_func    (GtkTreeViewColumn *tree_column,
                                                             GtkCellRenderer   *cell,
@@ -266,7 +264,7 @@ quit_close_all_dialog_new (Gimp     *gimp,
   gtk_box_pack_start (GTK_BOX (private->box), view, TRUE, TRUE, 0);
   gtk_widget_show (view);
 
-  g_signal_connect (view, "select-items",
+  g_signal_connect (view, "selection-changed",
                     G_CALLBACK (quit_close_all_dialog_images_selected),
                     private);
 
@@ -443,21 +441,15 @@ quit_close_all_dialog_container_changed (GimpContainer *images,
   g_free (accel_string);
 }
 
-static gboolean
+static void
 quit_close_all_dialog_images_selected (GimpContainerView *view,
-                                       GList             *images,
-                                       GList             *paths,
                                        QuitDialog        *private)
 {
-  /* The signal allows for multiple selection cases, but this specific
-   * dialog only allows one image selected at a time.
-   */
-  g_return_val_if_fail (g_list_length (images) <= 1, FALSE);
+  GimpViewable *image = gimp_container_view_get_1_selected (view);
 
-  if (images)
+  if (image)
     {
-      GimpImage *image = images->data;
-      GList     *list;
+      GList *list;
 
       for (list = gimp_get_display_iter (private->gimp);
            list;
@@ -465,7 +457,7 @@ quit_close_all_dialog_images_selected (GimpContainerView *view,
         {
           GimpDisplay *display = list->data;
 
-          if (gimp_display_get_image (display) == image)
+          if (gimp_display_get_image (display) == GIMP_IMAGE (image))
             {
               gimp_display_shell_present (gimp_display_get_shell (display));
 
@@ -476,8 +468,6 @@ quit_close_all_dialog_images_selected (GimpContainerView *view,
             }
         }
     }
-
-  return TRUE;
 }
 
 static void
