@@ -467,16 +467,16 @@ welcome_dialog_create_welcome_page (Gimp      *gimp,
                     /* "globe with meridians" emoticone in UTF-8. */
                     "\xf0\x9f\x8c\x90",
                     _("GIMP website"), "https://www.gimp.org/");
-  welcome_add_link (GTK_GRID (grid), 0, &row,
-                    /* "graduation cap" emoticone in UTF-8. */
-                    "\xf0\x9f\x8e\x93",
-                    _("Tutorials"),
-                    "https://www.gimp.org/tutorials/");
   welcome_add_link (GTK_GRID (grid), 0, &row,
                     /* "open book" emoticone in UTF-8. */
                     "\xf0\x9f\x93\x96",
                     _("Documentation"),
                     "https://docs.gimp.org/");
+  welcome_add_link (GTK_GRID (grid), 0, &row,
+                    /* "graduation cap" emoticone in UTF-8. */
+                    "\xf0\x9f\x8e\x93",
+                    _("Community Tutorials"),
+                    "https://www.gimp.org/tutorials/");
 
   /* XXX: should we add API docs for plug-in developers once it's
    * properly set up? */
@@ -1007,19 +1007,20 @@ welcome_dialog_create_release_page (Gimp      *gimp,
         {
           GtkWidget *row;
           gchar     *markup;
+          gchar     *text;
+
+          text = g_markup_escape_text (_((gchar *) gimp_welcome_dialog_items[i]), -1);
 
           /* Add a bold dot for pretty listing. */
           if (i < gimp_welcome_dialog_n_items &&
               gimp_welcome_dialog_demos[i] != NULL)
             {
-              markup = g_strdup_printf ("<span weight='ultrabold'>\xe2\x96\xb6</span>  %s",
-                                        _((gchar *) gimp_welcome_dialog_items[i]));
+              markup = g_strdup_printf ("<span weight='ultrabold'>\xe2\x96\xb6</span>  %s", text);
               n_demos++;
             }
           else
             {
-              markup = g_strdup_printf ("<span weight='ultrabold'>\xe2\x80\xa2</span>  %s",
-                                        _((gchar *) gimp_welcome_dialog_items[i]));
+              markup = g_strdup_printf ("<span weight='ultrabold'>\xe2\x80\xa2</span>  %s", text);
             }
 
           row = gtk_list_box_row_new ();
@@ -1036,6 +1037,7 @@ welcome_dialog_create_release_page (Gimp      *gimp,
           gtk_widget_show_all (row);
 
           g_free (markup);
+          g_free (text);
         }
       gtk_container_add (GTK_CONTAINER (scrolled_window), listbox);
       gtk_list_box_set_selection_mode (GTK_LIST_BOX (listbox),
@@ -1083,10 +1085,19 @@ welcome_dialog_create_release_page (Gimp      *gimp,
 
       tmp = g_strdup_printf (GIMP_VERSION);
       if (GIMP_MINOR_VERSION % 2 == 0 && ! strstr (tmp, "RC"))
-        release_link = g_strdup_printf ("https://www.gimp.org/release-notes/gimp-%d.%d.html",
-                                        GIMP_MAJOR_VERSION, GIMP_MINOR_VERSION);
+        {
+          if (GIMP_MICRO_VERSION == 0 && ! strstr (tmp, "RC"))
+            release_link = g_strdup_printf ("https://www.gimp.org/release-notes/gimp-%d.%d.html",
+                                            GIMP_MAJOR_VERSION, GIMP_MINOR_VERSION);
+          else
+            release_link = g_strdup_printf ("https://www.gimp.org/release/%d.%d.%d/",
+                                            GIMP_MAJOR_VERSION, GIMP_MINOR_VERSION,
+                                            GIMP_MICRO_VERSION);
+        }
       else
-        release_link = g_strdup ("https://www.gimp.org/");
+        {
+          release_link = g_strdup ("https://www.gimp.org/");
+        }
       g_free (tmp);
 
       widget = gtk_link_button_new_with_label (release_link, _("Learn more"));
