@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -34,6 +34,8 @@
 #include "core/gimp-transform-utils.h"
 #include "core/gimp-utils.h"
 
+#include "widgets/gimpwidgets-utils.h"
+
 #include "gimpcanvashandle.h"
 #include "gimpcanvastransformguides.h"
 #include "gimpdisplayshell.h"
@@ -57,6 +59,8 @@ enum
   PROP_PIVOT_Y,
   PROP_GUIDE_TYPE,
   PROP_N_GUIDES,
+  PROP_CLIP_GUIDES,
+  PROP_SHOW_GUIDES,
   PROP_INSIDE_FUNCTION,
   PROP_OUTSIDE_FUNCTION,
   PROP_USE_CORNER_HANDLES,
@@ -65,6 +69,7 @@ enum
   PROP_USE_SHEAR_HANDLES,
   PROP_USE_CENTER_HANDLE,
   PROP_USE_PIVOT_HANDLE,
+  PROP_DYNAMIC_HANDLE_SIZE,
   PROP_CONSTRAIN_MOVE,
   PROP_CONSTRAIN_SCALE,
   PROP_CONSTRAIN_ROTATE,
@@ -87,6 +92,8 @@ struct _GimpToolTransformGridPrivate
   gdouble                pivot_y;
   GimpGuidesType         guide_type;
   gint                   n_guides;
+  gboolean               clip_guides;
+  gboolean               show_guides;
   GimpTransformFunction  inside_function;
   GimpTransformFunction  outside_function;
   gboolean               use_corner_handles;
@@ -95,6 +102,7 @@ struct _GimpToolTransformGridPrivate
   gboolean               use_shear_handles;
   gboolean               use_center_handle;
   gboolean               use_pivot_handle;
+  gboolean               dynamic_handle_size;
   gboolean               constrain_move;
   gboolean               constrain_scale;
   gboolean               constrain_rotate;
@@ -109,6 +117,7 @@ struct _GimpToolTransformGridPrivate
   gdouble                curx;         /*  current x coord                    */
   gdouble                cury;         /*  current y coord                    */
 
+  gdouble                button_down;  /*  is the mouse button pressed        */
   gdouble                mousex;       /*  x coord where mouse was clicked    */
   gdouble                mousey;       /*  y coord where mouse was clicked    */
 
@@ -166,10 +175,19 @@ static void     gimp_tool_transform_grid_motion         (GimpToolWidget        *
                                                          const GimpCoords      *coords,
                                                          guint32                time,
                                                          GdkModifierType        state);
+static GimpHit  gimp_tool_transform_grid_hit            (GimpToolWidget        *widget,
+                                                         const GimpCoords      *coords,
+                                                         GdkModifierType        state,
+                                                         gboolean               proximity);
 static void     gimp_tool_transform_grid_hover          (GimpToolWidget        *widget,
                                                          const GimpCoords      *coords,
                                                          GdkModifierType        state,
                                                          gboolean               proximity);
+static void     gimp_tool_transform_grid_leave_notify   (GimpToolWidget        *widget);
+static void     gimp_tool_transform_grid_hover_modifier (GimpToolWidget        *widget,
+                                                         GdkModifierType        key,
+                                                         gboolean               press,
+                                                         GdkModifierType        state);
 static gboolean gimp_tool_transform_grid_get_cursor     (GimpToolWidget        *widget,
                                                          const GimpCoords      *coords,
                                                          GdkModifierType        state,
@@ -177,6 +195,10 @@ static gboolean gimp_tool_transform_grid_get_cursor     (GimpToolWidget        *
                                                          GimpToolCursorType    *tool_cursor,
                                                          GimpCursorModifier    *modifier);
 
+static GimpTransformHandle
+                gimp_tool_transform_grid_get_handle_for_coords
+                                                        (GimpToolTransformGrid *grid,
+                                                         const GimpCoords      *coords);
 static void     gimp_tool_transform_grid_update_hilight (GimpToolTransformGrid *grid);
 static void     gimp_tool_transform_grid_update_box     (GimpToolTransformGrid *grid);
 static void     gimp_tool_transform_grid_update_matrix  (GimpToolTransformGrid *grid);
@@ -185,8 +207,8 @@ static void     gimp_tool_transform_grid_calc_handles   (GimpToolTransformGrid *
                                                          gint                  *handle_h);
 
 
-G_DEFINE_TYPE (GimpToolTransformGrid, gimp_tool_transform_grid,
-               GIMP_TYPE_TOOL_WIDGET)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpToolTransformGrid, gimp_tool_transform_grid,
+                            GIMP_TYPE_TOOL_WIDGET)
 
 #define parent_class gimp_tool_transform_grid_parent_class
 
@@ -205,8 +227,12 @@ gimp_tool_transform_grid_class_init (GimpToolTransformGridClass *klass)
   widget_class->button_press    = gimp_tool_transform_grid_button_press;
   widget_class->button_release  = gimp_tool_transform_grid_button_release;
   widget_class->motion          = gimp_tool_transform_grid_motion;
+  widget_class->hit             = gimp_tool_transform_grid_hit;
   widget_class->hover           = gimp_tool_transform_grid_hover;
+  widget_class->leave_notify    = gimp_tool_transform_grid_leave_notify;
+  widget_class->hover_modifier  = gimp_tool_transform_grid_hover_modifier;
   widget_class->get_cursor      = gimp_tool_transform_grid_get_cursor;
+  widget_class->update_on_scale = TRUE;
 
   g_object_class_install_property (object_class, PROP_TRANSFORM,
                                    gimp_param_spec_matrix3 ("transform",
@@ -282,6 +308,18 @@ gimp_tool_transform_grid_class_init (GimpToolTransformGridClass *klass)
                                                      GIMP_PARAM_READWRITE |
                                                      G_PARAM_CONSTRUCT));
 
+  g_object_class_install_property (object_class, PROP_CLIP_GUIDES,
+                                   g_param_spec_boolean ("clip-guides", NULL, NULL,
+                                                         FALSE,
+                                                         GIMP_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT));
+
+  g_object_class_install_property (object_class, PROP_SHOW_GUIDES,
+                                   g_param_spec_boolean ("show-guides", NULL, NULL,
+                                                         TRUE,
+                                                         GIMP_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (object_class, PROP_INSIDE_FUNCTION,
                                    g_param_spec_enum ("inside-function",
                                                       NULL, NULL,
@@ -340,6 +378,13 @@ gimp_tool_transform_grid_class_init (GimpToolTransformGridClass *klass)
                                                          GIMP_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT));
 
+  g_object_class_install_property (object_class, PROP_DYNAMIC_HANDLE_SIZE,
+                                   g_param_spec_boolean ("dynamic-handle-size",
+                                                         NULL, NULL,
+                                                         TRUE,
+                                                         GIMP_PARAM_READWRITE |
+                                                         G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (object_class, PROP_CONSTRAIN_MOVE,
                                    g_param_spec_boolean ("constrain-move",
                                                          NULL, NULL,
@@ -409,16 +454,12 @@ gimp_tool_transform_grid_class_init (GimpToolTransformGridClass *klass)
                                                          FALSE,
                                                          GIMP_PARAM_READWRITE |
                                                          G_PARAM_CONSTRUCT));
-
-  g_type_class_add_private (klass, sizeof (GimpToolTransformGridPrivate));
 }
 
 static void
 gimp_tool_transform_grid_init (GimpToolTransformGrid *grid)
 {
-  grid->private = G_TYPE_INSTANCE_GET_PRIVATE (grid,
-                                               GIMP_TYPE_TOOL_TRANSFORM_GRID,
-                                               GimpToolTransformGridPrivate);
+  grid->private = gimp_tool_transform_grid_get_instance_private (grid);
 }
 
 static void
@@ -439,7 +480,8 @@ gimp_tool_transform_grid_constructed (GObject *object)
                                                            private->x2,
                                                            private->y2,
                                                            private->guide_type,
-                                                           private->n_guides);
+                                                           private->n_guides,
+                                                           private->clip_guides);
 
   for (i = 0; i < 4; i++)
     {
@@ -570,6 +612,12 @@ gimp_tool_transform_grid_set_property (GObject      *object,
     case PROP_N_GUIDES:
       private->n_guides = g_value_get_int (value);
       break;
+    case PROP_CLIP_GUIDES:
+      private->clip_guides = g_value_get_boolean (value);
+      break;
+    case PROP_SHOW_GUIDES:
+      private->show_guides = g_value_get_boolean (value);
+      break;
 
     case PROP_INSIDE_FUNCTION:
       private->inside_function = g_value_get_enum (value);
@@ -597,6 +645,10 @@ gimp_tool_transform_grid_set_property (GObject      *object,
       private->use_pivot_handle = g_value_get_boolean (value);
       break;
 
+    case PROP_DYNAMIC_HANDLE_SIZE:
+      private->dynamic_handle_size = g_value_get_boolean (value);
+      break;
+
     case PROP_CONSTRAIN_MOVE:
       private->constrain_move = g_value_get_boolean (value);
       break;
@@ -683,6 +735,12 @@ gimp_tool_transform_grid_get_property (GObject    *object,
     case PROP_N_GUIDES:
       g_value_set_int (value, private->n_guides);
       break;
+    case PROP_CLIP_GUIDES:
+      g_value_set_boolean (value, private->clip_guides);
+      break;
+    case PROP_SHOW_GUIDES:
+      g_value_set_boolean (value, private->show_guides);
+      break;
 
     case PROP_INSIDE_FUNCTION:
       g_value_set_enum (value, private->inside_function);
@@ -710,6 +768,10 @@ gimp_tool_transform_grid_get_property (GObject    *object,
       g_value_set_boolean (value, private->use_pivot_handle);
       break;
 
+    case PROP_DYNAMIC_HANDLE_SIZE:
+      g_value_set_boolean (value, private->dynamic_handle_size);
+      break;
+
     case PROP_CONSTRAIN_MOVE:
       g_value_set_boolean (value, private->constrain_move);
       break;
@@ -758,6 +820,17 @@ transform_is_convex (GimpVector2 *pos)
                                            pos[3].x, pos[3].y);
 }
 
+static gboolean
+transform_grid_is_convex (GimpToolTransformGrid *grid)
+{
+  GimpToolTransformGridPrivate *private = grid->private;
+
+  return gimp_transform_polygon_is_convex (private->tx1, private->ty1,
+                                           private->tx2, private->ty2,
+                                           private->tx3, private->ty3,
+                                           private->tx4, private->ty4);
+}
+
 static inline gboolean
 vectorisnull (GimpVector2 v)
 {
@@ -833,11 +906,11 @@ calcangle (GimpVector2 a,
 
   length = norm (a) * norm (b);
 
-  angle = acos (dotprod (a, b)/length);
+  angle = acos (SAFE_CLAMP (dotprod (a, b) / length, -1.0, +1.0));
   angle2 = b.y;
   b.y = -b.x;
   b.x = angle2;
-  angle2 = acos (dotprod (a, b)/length);
+  angle2 = acos (SAFE_CLAMP (dotprod (a, b) / length, -1.0, +1.0));
 
   return ((angle2 > G_PI / 2.0) ? angle : 2.0 * G_PI - angle);
 }
@@ -1011,7 +1084,9 @@ gimp_tool_transform_grid_changed (GimpToolWidget *widget)
                                     private->x2,
                                     private->y2,
                                     private->guide_type,
-                                    private->n_guides);
+                                    private->n_guides,
+                                    private->clip_guides);
+  gimp_canvas_item_set_visible (private->guides, private->show_guides);
 
   get_handle_geometry (grid, o, angle);
   gimp_tool_transform_grid_calc_handles (grid, &handle_w, &handle_h);
@@ -1027,21 +1102,29 @@ gimp_tool_transform_grid_changed (GimpToolWidget *widget)
         factor = 1.5;
 
       h = private->handles[GIMP_TRANSFORM_HANDLE_NW + i];
-      gimp_canvas_handle_set_position (h, o[i].x, o[i].y);
-      gimp_canvas_handle_set_size (h, handle_w * factor, handle_h * factor);
-      gimp_canvas_handle_set_angles (h, angle[i + 4], 0.0);
       gimp_canvas_item_set_visible (h, private->use_corner_handles);
 
+      if (private->use_corner_handles)
+        {
+          gimp_canvas_handle_set_position (h, o[i].x, o[i].y);
+          gimp_canvas_handle_set_size (h, handle_w * factor, handle_h * factor);
+          gimp_canvas_handle_set_angles (h, angle[i + 4], 0.0);
+        }
+
       /*  the perspective handles  */
       factor = 1.0;
       if (private->use_corner_handles)
         factor = 0.8;
 
       h = private->handles[GIMP_TRANSFORM_HANDLE_NW_P + i];
-      gimp_canvas_handle_set_position (h, o[i].x, o[i].y);
-      gimp_canvas_handle_set_size (h, handle_w * factor, handle_h * factor);
-      gimp_canvas_handle_set_angles (h, angle[i + 4], 0.0);
       gimp_canvas_item_set_visible (h, private->use_perspective_handles);
+
+      if (private->use_perspective_handles)
+        {
+          gimp_canvas_handle_set_position (h, o[i].x, o[i].y);
+          gimp_canvas_handle_set_size (h, handle_w * factor, handle_h * factor);
+          gimp_canvas_handle_set_angles (h, angle[i + 4], 0.0);
+        }
     }
 
   /*  draw the side handles  */
@@ -1055,10 +1138,14 @@ gimp_tool_transform_grid_changed (GimpToolWidget *widget)
       GimpCanvasItem *h;
 
       h = private->handles[GIMP_TRANSFORM_HANDLE_N + i];
-      gimp_canvas_handle_set_position (h, t[i].x, t[i].y);
-      gimp_canvas_handle_set_size (h, handle_w, handle_h);
-      gimp_canvas_handle_set_angles (h, angle[i], 0.0);
       gimp_canvas_item_set_visible (h, private->use_side_handles);
+
+      if (private->use_side_handles)
+        {
+          gimp_canvas_handle_set_position (h, t[i].x, t[i].y);
+          gimp_canvas_handle_set_size (h, handle_w, handle_h);
+          gimp_canvas_handle_set_angles (h, angle[i], 0.0);
+        }
     }
 
   /*  draw the shear handles  */
@@ -1068,7 +1155,7 @@ gimp_tool_transform_grid_changed (GimpToolWidget *widget)
                     0.25);
   t[2] = scalemult (vectoradd (           o[1]      , scalemult (o[3], 3.0)),
                     0.25);
-  t[3] = scalemult (vectoradd (scalemult (o[2], 3.0),            o[0]      ),
+  t[3] = scalemult (vectoradd (scalemult (o[0], 3.0),            o[2]      ),
                     0.25);
 
   for (i = 0; i < 4; i++)
@@ -1076,10 +1163,14 @@ gimp_tool_transform_grid_changed (GimpToolWidget *widget)
       GimpCanvasItem *h;
 
       h = private->handles[GIMP_TRANSFORM_HANDLE_N_S + i];
-      gimp_canvas_handle_set_position (h, t[i].x, t[i].y);
-      gimp_canvas_handle_set_size (h, handle_w, handle_h);
-      gimp_canvas_handle_set_angles (h, angle[i], 0.0);
       gimp_canvas_item_set_visible (h, private->use_shear_handles);
+
+      if (private->use_shear_handles)
+        {
+          gimp_canvas_handle_set_position (h, t[i].x, t[i].y);
+          gimp_canvas_handle_set_size (h, handle_w, handle_h);
+          gimp_canvas_handle_set_angles (h, angle[i], 0.0);
+        }
     }
 
   d = MIN (handle_w, handle_h);
@@ -1089,28 +1180,34 @@ gimp_tool_transform_grid_changed (GimpToolWidget *widget)
   gimp_canvas_item_set_visible (private->handles[GIMP_TRANSFORM_HANDLE_PIVOT],
                                 private->use_pivot_handle);
 
-  gimp_canvas_handle_set_position (private->pivot_items[0],
-                                   private->tpx, private->tpy);
-  gimp_canvas_handle_set_size (private->pivot_items[0], d, d);
+  if (private->use_pivot_handle)
+    {
+      gimp_canvas_handle_set_position (private->pivot_items[0],
+                                       private->tpx, private->tpy);
+      gimp_canvas_handle_set_size (private->pivot_items[0], d, d);
 
-  gimp_canvas_handle_set_position (private->pivot_items[1],
-                                   private->tpx, private->tpy);
-  gimp_canvas_handle_set_size (private->pivot_items[1], d, d);
+      gimp_canvas_handle_set_position (private->pivot_items[1],
+                                       private->tpx, private->tpy);
+      gimp_canvas_handle_set_size (private->pivot_items[1], d, d);
+    }
 
   d = MIN (handle_w, handle_h);
 
   gimp_canvas_item_set_visible (private->handles[GIMP_TRANSFORM_HANDLE_CENTER],
                                 private->use_center_handle);
 
-  gimp_canvas_handle_set_position (private->center_items[0],
-                                   private->tcx, private->tcy);
-  gimp_canvas_handle_set_size (private->center_items[0], d, d);
-  gimp_canvas_handle_set_angles (private->center_items[0], angle[8], 0.0);
+  if (private->use_center_handle)
+    {
+      gimp_canvas_handle_set_position (private->center_items[0],
+                                       private->tcx, private->tcy);
+      gimp_canvas_handle_set_size (private->center_items[0], d, d);
+      gimp_canvas_handle_set_angles (private->center_items[0], angle[8], 0.0);
 
-  gimp_canvas_handle_set_position (private->center_items[1],
-                                   private->tcx, private->tcy);
-  gimp_canvas_handle_set_size (private->center_items[1], d, d);
-  gimp_canvas_handle_set_angles (private->center_items[1], angle[8], 0.0);
+      gimp_canvas_handle_set_position (private->center_items[1],
+                                       private->tcx, private->tcy);
+      gimp_canvas_handle_set_size (private->center_items[1], d, d);
+      gimp_canvas_handle_set_angles (private->center_items[1], angle[8], 0.0);
+    }
 
   gimp_tool_transform_grid_update_hilight (grid);
 }
@@ -1125,8 +1222,9 @@ gimp_tool_transform_grid_button_press (GimpToolWidget      *widget,
   GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
   GimpToolTransformGridPrivate *private = grid->private;
 
-  private->mousex = coords->x;
-  private->mousey = coords->y;
+  private->button_down = TRUE;
+  private->mousex      = coords->x;
+  private->mousey      = coords->y;
 
   if (private->handle != GIMP_TRANSFORM_HANDLE_NONE)
     {
@@ -1190,6 +1288,10 @@ gimp_tool_transform_grid_button_release (GimpToolWidget        *widget,
                                          GdkModifierType        state,
                                          GimpButtonReleaseType  release_type)
 {
+  GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
+  GimpToolTransformGridPrivate *private = grid->private;
+
+  private->button_down = FALSE;
 }
 
 void
@@ -1237,16 +1339,29 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
   newpos[3].y = oldpos[3].y = private->prev_ty4;
 
   /* put center point in this array too */
-  oldpos[4].x = (oldpos[0].x + oldpos[1].x + oldpos[2].x + oldpos[3].x) / 4.;
-  oldpos[4].y = (oldpos[0].y + oldpos[1].y + oldpos[2].y + oldpos[3].y) / 4.;
+  oldpos[4].x = private->prev_tcx;
+  oldpos[4].y = private->prev_tcy;
 
   d = vectorsubtract (cur, mouse);
 
   newpivot_x = &private->tpx;
   newpivot_y = &private->tpy;
 
-  pivot.x = private->prev_tpx;
-  pivot.y = private->prev_tpy;
+  if (private->use_pivot_handle)
+    {
+      pivot.x = private->prev_tpx;
+      pivot.y = private->prev_tpy;
+    }
+  else
+    {
+      /* when the transform grid doesn't use a pivot handle, use the center
+       * point as the pivot instead.
+       */
+      pivot.x = private->prev_tcx;
+      pivot.y = private->prev_tcy;
+
+      fixedpivot = TRUE;
+    }
 
   /* move */
   if (handle == GIMP_TRANSFORM_HANDLE_CENTER)
@@ -1363,11 +1478,11 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
           this = 3; left = 2; right = 1; opposite = 0;
         }
       else
-        g_assert_not_reached ();
+        gimp_assert_not_reached ();
 
       /* when the keep aspect transformation constraint is enabled,
        * the translation shall only be along the diagonal that runs
-       * trough this corner point.
+       * through this corner point.
        */
       if (private->constrain_scale)
         {
@@ -1467,7 +1582,7 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
           this_l = 0; this_r = 2;
         }
       else
-        g_assert_not_reached ();
+        gimp_assert_not_reached ();
 
       opp_l = 3 - this_r; opp_r = 3 - this_l;
 
@@ -1553,7 +1668,7 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
           this_l = 3; this_r = 1;
         }
       else
-        g_assert_not_reached ();
+        gimp_assert_not_reached ();
 
       if (private->constrain_shear)
         {
@@ -1604,14 +1719,14 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
           this = 3; left = 2; right = 1; opposite = 0;
         }
       else
-        g_assert_not_reached ();
+        gimp_assert_not_reached ();
 
       if (private->constrain_perspective)
         {
           /* when the constrain transformation constraint is enabled,
            * the translation shall only be either along the side
            * angles of the two sides that run to this corner point, or
-           * along the diagonal that runs trough this corner point.
+           * along the diagonal that runs through this corner point.
            */
           GimpVector2 proj[4];
           gdouble     rej[4];
@@ -1654,12 +1769,6 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
         }
     }
 
-  for (i = 0; i < 4; i++)
-    {
-      *x[i] = newpos[i].x;
-      *y[i] = newpos[i].y;
-    }
-
   /* this will have been set to TRUE if an operation used the pivot in
    * addition to being a user option
    */
@@ -1672,6 +1781,22 @@ gimp_tool_transform_grid_motion (GimpToolWidget   *widget,
       pivot = vectoradd (pivot, delta);
     }
 
+  /* make sure the new coordinates are valid */
+  for (i = 0; i < 4; i++)
+    {
+      if (! isfinite (newpos[i].x) || ! isfinite (newpos[i].y))
+        return;
+    }
+
+  if (! isfinite (pivot.x) || ! isfinite (pivot.y))
+    return;
+
+  for (i = 0; i < 4; i++)
+    {
+      *x[i] = newpos[i].x;
+      *y[i] = newpos[i].y;
+    }
+
   /* set unconditionally: if options get toggled during operation, we
    * have to move pivot back
    */
@@ -1715,7 +1840,7 @@ get_friendly_operation_name (GimpTransformHandle handle)
     case GIMP_TRANSFORM_HANDLE_ROTATION:
       return _("Click-Drag to rotate");
     default:
-      g_assert_not_reached ();
+      gimp_assert_not_reached ();
     }
 }
 
@@ -1729,6 +1854,9 @@ gimp_tool_transform_get_area_handle (GimpToolTransformGrid *grid,
 
   switch (function)
     {
+    case GIMP_TRANSFORM_FUNCTION_NONE:
+      break;
+
     case GIMP_TRANSFORM_FUNCTION_MOVE:
       handle = GIMP_TRANSFORM_HANDLE_CENTER;
       break;
@@ -1848,6 +1976,23 @@ gimp_tool_transform_get_area_handle (GimpToolTransformGrid *grid,
   return handle;
 }
 
+GimpHit
+gimp_tool_transform_grid_hit (GimpToolWidget   *widget,
+                              const GimpCoords *coords,
+                              GdkModifierType   state,
+                              gboolean          proximity)
+{
+  GimpToolTransformGrid *grid = GIMP_TOOL_TRANSFORM_GRID (widget);
+  GimpTransformHandle    handle;
+
+  handle = gimp_tool_transform_grid_get_handle_for_coords (grid, coords);
+
+  if (handle != GIMP_TRANSFORM_HANDLE_NONE)
+    return GIMP_HIT_DIRECT;
+
+  return GIMP_HIT_INDIRECT;
+}
+
 void
 gimp_tool_transform_grid_hover (GimpToolWidget   *widget,
                                 const GimpCoords *coords,
@@ -1856,18 +2001,9 @@ gimp_tool_transform_grid_hover (GimpToolWidget   *widget,
 {
   GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
   GimpToolTransformGridPrivate *private = grid->private;
-  GimpTransformHandle           handle  = GIMP_TRANSFORM_HANDLE_NONE;
-  GimpTransformHandle           i;
+  GimpTransformHandle           handle;
 
-  for (i = GIMP_TRANSFORM_HANDLE_NONE + 1; i < GIMP_N_TRANSFORM_HANDLES; i++)
-    {
-      if (private->handles[i] &&
-          gimp_canvas_item_hit (private->handles[i], coords->x, coords->y))
-        {
-          handle = i;
-          break;
-        }
-    }
+  handle = gimp_tool_transform_grid_get_handle_for_coords (grid, coords);
 
   if (handle == GIMP_TRANSFORM_HANDLE_NONE)
     {
@@ -1904,6 +2040,68 @@ gimp_tool_transform_grid_hover (GimpToolWidget   *widget,
   gimp_tool_transform_grid_update_hilight (grid);
 }
 
+void
+gimp_tool_transform_grid_leave_notify (GimpToolWidget *widget)
+{
+  GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
+  GimpToolTransformGridPrivate *private = grid->private;
+
+  private->handle = GIMP_TRANSFORM_HANDLE_NONE;
+
+  gimp_tool_transform_grid_update_hilight (grid);
+
+  GIMP_TOOL_WIDGET_CLASS (parent_class)->leave_notify (widget);
+}
+
+static void
+gimp_tool_transform_grid_modifier (GimpToolWidget  *widget,
+                                   GdkModifierType  key)
+{
+  GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
+  GimpToolTransformGridPrivate *private = grid->private;
+
+  if (key == gimp_get_constrain_behavior_mask ())
+    {
+      g_object_set (widget,
+                    "frompivot-scale",       ! private->frompivot_scale,
+                    "frompivot-shear",       ! private->frompivot_shear,
+                    "frompivot-perspective", ! private->frompivot_perspective,
+                    NULL);
+    }
+  else if (key == gimp_get_extend_selection_mask ())
+    {
+      g_object_set (widget,
+                    "cornersnap",            ! private->cornersnap,
+                    "constrain-move",        ! private->constrain_move,
+                    "constrain-scale",       ! private->constrain_scale,
+                    "constrain-rotate",      ! private->constrain_rotate,
+                    "constrain-shear",       ! private->constrain_shear,
+                    "constrain-perspective", ! private->constrain_perspective,
+                    NULL);
+    }
+}
+
+static void
+gimp_tool_transform_grid_hover_modifier (GimpToolWidget  *widget,
+                                         GdkModifierType  key,
+                                         gboolean         press,
+                                         GdkModifierType  state)
+{
+  GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
+  GimpToolTransformGridPrivate *private = grid->private;
+  GimpCoords                    coords  = { 0.0, };
+
+  gimp_tool_transform_grid_modifier (widget, key);
+
+  if (private->button_down)
+    {
+      /*  send a non-motion to update the grid with the new constraints  */
+      coords.x = private->curx;
+      coords.y = private->cury;
+      gimp_tool_transform_grid_motion (widget, &coords, 0, state);
+    }
+}
+
 static gboolean
 gimp_tool_transform_grid_get_cursor (GimpToolWidget     *widget,
                                      const GimpCoords   *coords,
@@ -1914,7 +2112,7 @@ gimp_tool_transform_grid_get_cursor (GimpToolWidget     *widget,
 {
   GimpToolTransformGrid        *grid    = GIMP_TOOL_TRANSFORM_GRID (widget);
   GimpToolTransformGridPrivate *private = grid->private;
-  gdouble                       angle[8];
+  gdouble                       angle[9];
   gint                          i;
   GimpCursorType                map[8];
   GimpVector2                   pos[4], this, that;
@@ -2042,7 +2240,7 @@ gimp_tool_transform_grid_get_cursor (GimpToolWidget     *widget,
             flip = TRUE;
           break;
         default:
-          g_assert_not_reached ();
+          gimp_assert_not_reached ();
         }
 
       if (flip)
@@ -2106,6 +2304,25 @@ gimp_tool_transform_grid_get_cursor (GimpToolWidget     *widget,
   return TRUE;
 }
 
+static GimpTransformHandle
+gimp_tool_transform_grid_get_handle_for_coords (GimpToolTransformGrid *grid,
+                                                const GimpCoords      *coords)
+{
+  GimpToolTransformGridPrivate *private = grid->private;
+  GimpTransformHandle           i;
+
+  for (i = GIMP_TRANSFORM_HANDLE_NONE + 1; i < GIMP_N_TRANSFORM_HANDLES; i++)
+    {
+      if (private->handles[i] &&
+          gimp_canvas_item_hit (private->handles[i], coords->x, coords->y))
+        {
+          return i;
+        }
+    }
+
+  return GIMP_TRANSFORM_HANDLE_NONE;
+}
+
 static void
 gimp_tool_transform_grid_update_hilight (GimpToolTransformGrid *grid)
 {
@@ -2146,14 +2363,24 @@ gimp_tool_transform_grid_update_box (GimpToolTransformGrid  *grid)
   private->tpx = private->pivot_x;
   private->tpy = private->pivot_y;
 
-  private->tcx = (private->tx1 +
-                  private->tx2 +
-                  private->tx3 +
-                  private->tx4) / 4.0;
-  private->tcy = (private->ty1 +
-                  private->ty2 +
-                  private->ty3 +
-                  private->ty4) / 4.0;
+  if (transform_grid_is_convex (grid))
+    {
+      gimp_matrix3_transform_point (&private->transform,
+                                    (private->x1 + private->x2) / 2.0,
+                                    (private->y1 + private->y2) / 2.0,
+                                    &private->tcx, &private->tcy);
+    }
+  else
+    {
+      private->tcx = (private->tx1 +
+                      private->tx2 +
+                      private->tx3 +
+                      private->tx4) / 4.0;
+      private->tcy = (private->ty1 +
+                      private->ty2 +
+                      private->ty3 +
+                      private->ty4) / 4.0;
+    }
 }
 
 static void
@@ -2199,6 +2426,14 @@ gimp_tool_transform_grid_calc_handles (GimpToolTransformGrid *grid,
   gint                          x1, y1;
   gint                          x2, y2;
 
+  if (! private->dynamic_handle_size)
+    {
+      *handle_w = GIMP_CANVAS_HANDLE_SIZE_LARGE;
+      *handle_h = GIMP_CANVAS_HANDLE_SIZE_LARGE;
+
+      return;
+    }
+
   gimp_canvas_item_transform_xy (private->guides,
                                  private->tx1, private->ty1,
                                  &dx1, &dy1);
@@ -2245,3 +2480,15 @@ gimp_tool_transform_grid_new (GimpDisplayShell  *shell,
                        "y2",         y2,
                        NULL);
 }
+
+
+/*  protected functions  */
+
+GimpTransformHandle
+gimp_tool_transform_grid_get_handle (GimpToolTransformGrid *grid)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL_TRANSFORM_GRID (grid),
+                        GIMP_TRANSFORM_HANDLE_NONE);
+
+  return grid->private->handle;
+}
