@@ -351,12 +351,9 @@ gimp_check_updates_process (const gchar    *source,
   parser = json_parser_new ();
   if (! json_parser_load_from_data (parser, file_contents, file_length, &error))
     {
-      gchar *uri = g_file_get_uri (G_FILE (source));
-
       g_printerr ("%s: parsing of %s failed: %s\n", G_STRFUNC,
-                  uri, error->message);
+                  source, error->message);
 
-      g_free (uri);
       g_free (file_contents);
       g_clear_object (&parser);
       g_clear_error (&error);
@@ -425,8 +422,30 @@ gimp_check_updates_callback (GObject      *source,
     {
       gchar *uri = g_file_get_uri (G_FILE (source));
 
+#ifndef GIMP_CONSOLE_COMPILATION
+      if (error->domain == G_IO_ERROR &&
+          error->code == G_IO_ERROR_NOT_SUPPORTED)
+        {
+#ifndef G_OS_WIN32
+          g_message ("%s: loading of %s failed: %s\n\n%s",
+                     G_STRFUNC, uri, error->message,
+                     _("Perhaps you are missing GIO backends and need "
+                       "to install GVFS?"));
+#else
+          g_message ("%s: loading of %s failed: %s\n\n%s",
+                     G_STRFUNC, uri, error->message,
+                     _("Perhaps you are missing GIO backends."));
+#endif /* G_OS_WIN32 */
+        }
+      else
+        {
+          g_printerr ("%s: loading of %s failed: %s\n", G_STRFUNC,
+                      uri, error->message);
+        }
+#else
       g_printerr ("%s: loading of %s failed: %s\n", G_STRFUNC,
                   uri, error->message);
+#endif /* GIMP_CONSOLE_COMPILATION */
 
       g_free (uri);
       g_clear_error (&error);
@@ -600,7 +619,8 @@ gimp_get_version_url (void)
 #if defined(GIMP_RELEASE) && ! defined(GIMP_RC_VERSION)
   return "https://www.gimp.org/gimp_versions.json";
 #else
-  if (g_getenv ("GIMP_DEV_VERSIONS_JSON"))
+  if (g_getenv ("GIMP_DEV_VERSIONS_JSON") &&
+      g_strcmp0 (g_getenv ("GIMP_DEV_VERSIONS_JSON"), "") != 0)
     return g_getenv ("GIMP_DEV_VERSIONS_JSON");
   else
     return "https://testing.gimp.org/gimp_versions.json";
