@@ -15,13 +15,17 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
+#include "libgimpwidgets/gimpwidgets.h"
+
 #include "gui-types.h"
 
 #include "core/gimp.h"
@@ -33,14 +37,13 @@
 #include "widgets/gimpdockcontainer.h"
 #include "widgets/gimpdockwindow.h"
 #include "widgets/gimptoolbox.h"
+#include "widgets/gimpwidgets-utils.h"
 
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplayshell.h"
 #include "display/gimpdisplayshell-appearance.h"
 #include "display/gimpimagewindow.h"
 
-#include "menus/menus.h"
-
 #include "gimpuiconfigurer.h"
 
 
@@ -74,7 +77,7 @@ static void              gimp_ui_configurer_separate_docks              (GimpUIC
                                                                          GimpImageWindow   *source_image_window);
 static void              gimp_ui_configurer_move_docks_to_window        (GimpUIConfigurer  *ui_configurer,
                                                                          GimpDockColumns   *dock_columns,
-                                                                         GimpAlignmentType  screen_side_destination);
+                                                                         GimpAlignmentType  screen_side);
 static void              gimp_ui_configurer_separate_shells             (GimpUIConfigurer  *ui_configurer,
                                                                          GimpImageWindow   *source_image_window);
 static void              gimp_ui_configurer_configure_for_single_window (GimpUIConfigurer  *ui_configurer);
@@ -82,7 +85,8 @@ static void              gimp_ui_configurer_configure_for_multi_window  (GimpUIC
 static GimpImageWindow * gimp_ui_configurer_get_uber_window             (GimpUIConfigurer  *ui_configurer);
 
 
-G_DEFINE_TYPE (GimpUIConfigurer, gimp_ui_configurer, GIMP_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpUIConfigurer, gimp_ui_configurer,
+                            GIMP_TYPE_OBJECT)
 
 #define parent_class gimp_ui_configurer_parent_class
 
@@ -100,16 +104,12 @@ gimp_ui_configurer_class_init (GimpUIConfigurerClass *klass)
                                                         GIMP_TYPE_GIMP,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
-  g_type_class_add_private (klass,
-                            sizeof (GimpUIConfigurerPrivate));
 }
 
 static void
 gimp_ui_configurer_init (GimpUIConfigurer *ui_configurer)
 {
-  ui_configurer->p = G_TYPE_INSTANCE_GET_PRIVATE (ui_configurer,
-                                                  GIMP_TYPE_UI_CONFIGURER,
-                                                  GimpUIConfigurerPrivate);
+  ui_configurer->p = gimp_ui_configurer_get_instance_private (ui_configurer);
 }
 
 static void
@@ -223,6 +223,7 @@ gimp_ui_configurer_move_docks_to_columns (GimpUIConfigurer *ui_configurer,
         dock_columns = gimp_image_window_get_right_docks (uber_image_window);
 
       dock_container = GIMP_DOCK_CONTAINER (dock_window);
+
       g_object_add_weak_pointer (G_OBJECT (dock_window),
                                  (gpointer) &dock_window);
 
@@ -319,8 +320,7 @@ gimp_ui_configurer_separate_docks (GimpUIConfigurer  *ui_configurer,
 /**
  * gimp_ui_configurer_move_docks_to_window:
  * @dock_columns:
- * @screen_side_destination: At what side of the screen the dock window
- *                           should be put.
+ * @screen_side: At what side of the screen the dock window should be put.
  *
  * Moves docks in @dock_columns into a new #GimpDockWindow and
  * position it on the screen in a non-overlapping manner.
@@ -328,26 +328,33 @@ gimp_ui_configurer_separate_docks (GimpUIConfigurer  *ui_configurer,
 static void
 gimp_ui_configurer_move_docks_to_window (GimpUIConfigurer  *ui_configurer,
                                          GimpDockColumns   *dock_columns,
-                                         GimpAlignmentType  screen_side_destination)
+                                         GimpAlignmentType  screen_side)
 {
-  GdkScreen    *screen           = gtk_widget_get_screen (GTK_WIDGET (dock_columns));
-  GList        *docks            = g_list_copy (gimp_dock_columns_get_docks (dock_columns));
-  GList        *iter             = NULL;
-  gboolean      contains_toolbox = FALSE;
-  GtkWidget    *dock_window      = NULL;
-  GtkAllocation original_size    = { 0, 0, 0, 0 };
+  GdkMonitor    *monitor;
+  GdkRectangle   monitor_rect;
+  GdkGravity     gravity;
+  GList         *docks;
+  GList         *iter;
+  gboolean       contains_toolbox = FALSE;
+  GtkWidget     *dock_window;
+  GtkAllocation  original_size;
+  gint           x, y;
 
-  /* Are there docks to move at all? */
+  docks = g_list_copy (gimp_dock_columns_get_docks (dock_columns));
   if (! docks)
     return;
 
+  monitor = gimp_widget_get_monitor (GTK_WIDGET (dock_columns));
+
+  gdk_monitor_get_workarea (monitor, &monitor_rect);
+
   /* Remember the size so we can set the new dock window to the same
    * size
    */
   gtk_widget_get_allocation (GTK_WIDGET (dock_columns), &original_size);
 
   /* Do we need a toolbox window? */
-  for (iter = docks; iter; iter = iter->next)
+  for (iter = docks; iter; iter = g_list_next (iter))
     {
       GimpDock *dock = GIMP_DOCK (iter->data);
 
@@ -365,15 +372,16 @@ gimp_ui_configurer_move_docks_to_window (GimpUIConfigurer  *ui_configurer,
    */
   dock_window =
     gimp_dialog_factory_dialog_new (gimp_dialog_factory_get_singleton (),
-                                    screen,
+                                    monitor,
                                     NULL /*ui_manager*/,
+                                    GTK_WIDGET (dock_columns),
                                     (contains_toolbox ?
                                      "gimp-toolbox-window" :
                                      "gimp-dock-window"),
                                     -1 /*view_size*/,
                                     FALSE /*present*/);
 
-  for (iter = docks; iter; iter = iter->next)
+  for (iter = docks; iter; iter = g_list_next (iter))
     {
       GimpDock *dock = GIMP_DOCK (iter->data);
 
@@ -385,12 +393,27 @@ gimp_ui_configurer_move_docks_to_window (GimpUIConfigurer  *ui_configurer,
     }
 
   /* Position the window */
-  if (screen_side_destination == GIMP_ALIGN_LEFT)
-    gtk_window_parse_geometry (GTK_WINDOW (dock_window), "+0+0");
-  else if (screen_side_destination == GIMP_ALIGN_RIGHT)
-    gtk_window_parse_geometry (GTK_WINDOW (dock_window), "-0+0");
+  if (screen_side == GIMP_ALIGN_LEFT)
+    {
+      gravity = GDK_GRAVITY_NORTH_WEST;
+
+      x = monitor_rect.x;
+      y = monitor_rect.y;
+    }
+  else if (screen_side == GIMP_ALIGN_RIGHT)
+    {
+      gravity = GDK_GRAVITY_NORTH_EAST;
+
+      x = monitor_rect.x + monitor_rect.width - original_size.width;
+      y = monitor_rect.y;
+    }
   else
-    g_assert_not_reached ();
+    {
+      gimp_assert_not_reached ();
+    }
+
+  gtk_window_set_gravity (GTK_WINDOW (dock_window), gravity);
+  gtk_window_move (GTK_WINDOW (dock_window), x, y);
 
   /* Try to keep the same size */
   gtk_window_set_default_size (GTK_WINDOW (dock_window),
@@ -426,8 +449,8 @@ gimp_ui_configurer_separate_shells (GimpUIConfigurer *ui_configurer,
       /* Create a new image window */
       new_image_window = gimp_image_window_new (ui_configurer->p->gimp,
                                                 NULL,
-                                                global_menu_factory,
-                                                gimp_dialog_factory_get_singleton ());
+                                                gimp_dialog_factory_get_singleton (),
+                                                gimp_widget_get_monitor (GTK_WIDGET (source_image_window)));
       /* Move the shell there */
       shell = gimp_image_window_get_shell (source_image_window, 1);
 
@@ -552,7 +575,7 @@ gimp_ui_configurer_get_uber_window (GimpUIConfigurer *ui_configurer)
  * @ui_configurer:
  *
  * Updates the appearance of all shells in all image windows, so they
- * do whatever they deem neccessary to fit the new UI mode mode.
+ * do whatever they deem necessary to fit the new UI mode mode.
  **/
 static void
 gimp_ui_configurer_update_appearance (GimpUIConfigurer *ui_configurer)
