@@ -58,6 +58,8 @@
 #include "core/gimpcancelable.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpcontext.h"
+#include "core/gimpdatafactory.h"
+#include "core/gimpdrawable.h"
 #include "core/gimpgradient.h"
 #include "core/gimpimage.h"
 #include "core/gimpimagefile.h"
@@ -85,6 +87,7 @@
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimppaletteselect.h"
 #include "widgets/gimppatternselect.h"
+#include "widgets/gimppickableselect.h"
 #include "widgets/gimpprogressdialog.h"
 #include "widgets/gimpuimanager.h"
 #include "widgets/gimpwidgets-utils.h"
@@ -132,10 +135,10 @@ static GFile        * gui_get_theme_dir          (Gimp                *gimp);
 static GFile        * gui_get_icon_theme_dir     (Gimp                *gimp);
 static GimpObject   * gui_get_window_strategy    (Gimp                *gimp);
 static GimpDisplay  * gui_get_empty_display      (Gimp                *gimp);
-static guint32        gui_display_get_window_id  (GimpDisplay         *display);
+static GBytes       * gui_display_get_window_id  (GimpDisplay         *display);
 static GimpDisplay  * gui_display_create         (Gimp                *gimp,
                                                   GimpImage           *image,
-                                                  GimpUnit             unit,
+                                                  GimpUnit            *unit,
                                                   gdouble              scale,
                                                   GObject             *monitor);
 static void           gui_display_delete         (GimpDisplay         *display);
@@ -152,18 +155,19 @@ static void           gui_free_progress          (Gimp                *gimp,
 static gboolean       gui_pdb_dialog_new         (Gimp                *gimp,
                                                   GimpContext         *context,
                                                   GimpProgress        *progress,
-                                                  GimpContainer       *container,
+                                                  GType                object_type,
+                                                  GBytes              *parent_handle,
                                                   const gchar         *title,
                                                   const gchar         *callback_name,
-                                                  const gchar         *object_name,
+                                                  GimpObject          *object,
                                                   va_list              args);
 static gboolean       gui_pdb_dialog_set         (Gimp                *gimp,
-                                                  GimpContainer       *container,
+                                                  GType                contents_type,
                                                   const gchar         *callback_name,
-                                                  const gchar         *object_name,
+                                                  GimpObject          *object,
                                                   va_list              args);
 static gboolean       gui_pdb_dialog_close       (Gimp                *gimp,
-                                                  GimpContainer       *container,
+                                                  GType                contents_type,
                                                   const gchar         *callback_name);
 static gboolean       gui_recent_list_add_file   (Gimp                *gimp,
                                                   GFile               *file,
@@ -374,7 +378,7 @@ gui_get_empty_display (Gimp *gimp)
   return display;
 }
 
-static guint32
+static GBytes *
 gui_display_get_window_id (GimpDisplay *display)
 {
   GimpDisplay      *disp  = GIMP_DISPLAY (display);
@@ -382,19 +386,17 @@ gui_display_get_window_id (GimpDisplay *display)
 
   if (shell)
     {
-      GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (shell));
-
-      if (GTK_IS_WINDOW (toplevel))
-        return gimp_window_get_native_id (GTK_WINDOW (toplevel));
+      if (shell)
+        return g_bytes_ref (shell->window_handle);
     }
 
-  return 0;
+  return NULL;
 }
 
 static GimpDisplay *
 gui_display_create (Gimp      *gimp,
                     GimpImage *image,
-                    GimpUnit   unit,
+                    GimpUnit  *unit,
                     gdouble    scale,
                     GObject   *monitor)
 {
@@ -512,16 +514,16 @@ gui_wait (Gimp         *gimp,
   args = gimp_procedure_get_arguments (procedure);
   gimp_value_array_truncate (args, 5);
 
-  g_value_set_enum   (gimp_value_array_index (args, 0),
-                      GIMP_RUN_INTERACTIVE);
-  g_value_set_int    (gimp_value_array_index (args, 1),
-                      output_pipe[0]);
-  g_value_set_int    (gimp_value_array_index (args, 2),
-                      input_pipe[1]);
-  g_value_set_string (gimp_value_array_index (args, 3),
-                      message);
-  g_value_set_int    (gimp_value_array_index (args, 4),
-                      GIMP_IS_CANCELABLE (waitable));
+  g_value_set_enum    (gimp_value_array_index (args, 0),
+                       GIMP_RUN_INTERACTIVE);
+  g_value_set_int     (gimp_value_array_index (args, 1),
+                       output_pipe[0]);
+  g_value_set_int     (gimp_value_array_index (args, 2),
+                       input_pipe[1]);
+  g_value_set_string  (gimp_value_array_index (args, 3),
+                       message);
+  g_value_set_boolean (gimp_value_array_index (args, 4),
+                       GIMP_IS_CANCELABLE (waitable));
 
   gimp_procedure_execute_async (procedure, gimp,
                                 gimp_get_user_context (gimp),
@@ -612,65 +614,74 @@ static gboolean
 gui_pdb_dialog_new (Gimp          *gimp,
                     GimpContext   *context,
                     GimpProgress  *progress,
-                    GimpContainer *container,
+                    GType          contents_type,
+                    GBytes        *parent_handle,
                     const gchar   *title,
                     const gchar   *callback_name,
-                    const gchar   *object_name,
+                    GimpObject    *object,
                     va_list        args)
 {
   GType        dialog_type = G_TYPE_NONE;
   const gchar *dialog_role = NULL;
   const gchar *help_id     = NULL;
 
-  if (gimp_container_get_children_type (container) == GIMP_TYPE_BRUSH)
+  if (contents_type == GIMP_TYPE_BRUSH)
     {
       dialog_type = GIMP_TYPE_BRUSH_SELECT;
       dialog_role = "gimp-brush-selection";
       help_id     = GIMP_HELP_BRUSH_DIALOG;
     }
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_FONT)
+  else if (contents_type == GIMP_TYPE_FONT)
     {
       dialog_type = GIMP_TYPE_FONT_SELECT;
       dialog_role = "gimp-font-selection";
       help_id     = GIMP_HELP_FONT_DIALOG;
     }
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_GRADIENT)
+  else if (contents_type == GIMP_TYPE_GRADIENT)
     {
       dialog_type = GIMP_TYPE_GRADIENT_SELECT;
       dialog_role = "gimp-gradient-selection";
       help_id     = GIMP_HELP_GRADIENT_DIALOG;
     }
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_PALETTE)
+  else if (contents_type == GIMP_TYPE_PALETTE)
     {
       dialog_type = GIMP_TYPE_PALETTE_SELECT;
       dialog_role = "gimp-palette-selection";
       help_id     = GIMP_HELP_PALETTE_DIALOG;
     }
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_PATTERN)
+  else if (contents_type == GIMP_TYPE_PATTERN)
     {
       dialog_type = GIMP_TYPE_PATTERN_SELECT;
       dialog_role = "gimp-pattern-selection";
       help_id     = GIMP_HELP_PATTERN_DIALOG;
     }
+  else if (g_type_is_a (contents_type, GIMP_TYPE_DRAWABLE))
+    {
+      dialog_type = GIMP_TYPE_PICKABLE_SELECT;
+      dialog_role = "gimp-pickable-selection";
+    }
+  else
+    {
+      g_return_val_if_reached (FALSE);
+    }
 
   if (dialog_type != G_TYPE_NONE)
     {
-      GimpObject *object = NULL;
+      if (! object && ! g_type_is_a (contents_type, GIMP_TYPE_DRAWABLE))
+        object = gimp_context_get_by_type (context, contents_type);
 
-      if (object_name && strlen (object_name))
-        object = gimp_container_get_child_by_name (container, object_name);
-
-      if (! object)
-        object = gimp_context_get_by_type (context,
-                                           gimp_container_get_children_type (container));
-
-      if (object)
+      if (object || g_type_is_a (contents_type, GIMP_TYPE_DRAWABLE))
         {
           gint        n_properties = 0;
           gchar     **names        = NULL;
           GValue     *values       = NULL;
           GtkWidget  *dialog;
           GtkWidget  *view;
+          gboolean    use_header_bar;
+
+          g_object_get (gtk_settings_get_default (),
+                        "gtk-dialogs-use-header", &use_header_bar,
+                        NULL);
 
           names = gimp_properties_append (dialog_type,
                                           &n_properties, names, &values,
@@ -680,10 +691,11 @@ gui_pdb_dialog_new (Gimp          *gimp,
                                           "help-id",        help_id,
                                           "pdb",            gimp->pdb,
                                           "context",        context,
-                                          "select-type",    gimp_container_get_children_type (container),
+                                          "select-type",    contents_type,
                                           "initial-object", object,
                                           "callback-name",  callback_name,
                                           "menu-factory",   menus_get_global_menu_factory (gimp),
+                                          "use-header-bar", use_header_bar,
                                           NULL);
 
           names = gimp_properties_append_valist (dialog_type,
@@ -703,27 +715,25 @@ gui_pdb_dialog_new (Gimp          *gimp,
             gimp_docked_set_show_button_bar (GIMP_DOCKED (view), FALSE);
 
           if (progress)
-            {
-              guint32 window_id = gimp_progress_get_window_id (progress);
-
-              if (window_id)
-                gimp_window_set_transient_for (GTK_WINDOW (dialog), window_id);
-            }
+            gimp_window_set_transient_for (GTK_WINDOW (dialog), progress);
 
           gtk_widget_show (dialog);
 
           /*  workaround for bug #360106  */
-          {
-            GSource  *source = g_timeout_source_new (100);
-            GClosure *closure;
+            {
+              GSource  *source = g_timeout_source_new (100);
+              GClosure *closure;
 
-            closure = g_cclosure_new_object (G_CALLBACK (gui_pdb_dialog_present),
-                                             G_OBJECT (dialog));
+              closure = g_cclosure_new_object (G_CALLBACK (gui_pdb_dialog_present),
+                                               G_OBJECT (dialog));
 
-            g_source_set_closure (source, closure);
-            g_source_attach (source, NULL);
-            g_source_unref (source);
-          }
+              g_source_set_closure (source, closure);
+              g_source_attach (source, NULL);
+              g_source_unref (source);
+            }
+
+          if (parent_handle != NULL)
+            gimp_window_set_transient_for_handle (GTK_WINDOW (dialog), parent_handle);
 
           return TRUE;
         }
@@ -734,73 +744,96 @@ gui_pdb_dialog_new (Gimp          *gimp,
 
 static gboolean
 gui_pdb_dialog_set (Gimp          *gimp,
-                    GimpContainer *container,
+                    GType          contents_type,
                     const gchar   *callback_name,
-                    const gchar   *object_name,
+                    GimpObject    *object,
                     va_list        args)
 {
-  GimpPdbDialogClass *klass = NULL;
+  GimpPdbDialogClass *klass     = NULL;
+  GimpContainer      *container = NULL;
+  GimpPdbDialog      *dialog;
 
-  if (gimp_container_get_children_type (container) == GIMP_TYPE_BRUSH)
-    klass = g_type_class_peek (GIMP_TYPE_BRUSH_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_FONT)
-    klass = g_type_class_peek (GIMP_TYPE_FONT_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_GRADIENT)
-    klass = g_type_class_peek (GIMP_TYPE_GRADIENT_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_PALETTE)
-    klass = g_type_class_peek (GIMP_TYPE_PALETTE_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_PATTERN)
-    klass = g_type_class_peek (GIMP_TYPE_PATTERN_SELECT);
-
-  if (klass)
+  if (contents_type == GIMP_TYPE_BRUSH)
     {
-      GimpPdbDialog *dialog;
+      klass = g_type_class_peek (GIMP_TYPE_BRUSH_SELECT);
+      container = gimp_data_factory_get_container (gimp->brush_factory);
+    }
+  else if (contents_type == GIMP_TYPE_FONT)
+    {
+      klass = g_type_class_peek (GIMP_TYPE_FONT_SELECT);
+      container = gimp_data_factory_get_container (gimp->font_factory);
+    }
+  else if (contents_type == GIMP_TYPE_GRADIENT)
+    {
+      klass = g_type_class_peek (GIMP_TYPE_GRADIENT_SELECT);
+      container = gimp_data_factory_get_container (gimp->gradient_factory);
+    }
+  else if (contents_type == GIMP_TYPE_PALETTE)
+    {
+      klass = g_type_class_peek (GIMP_TYPE_PALETTE_SELECT);
+      container = gimp_data_factory_get_container (gimp->palette_factory);
+    }
+  else if (contents_type == GIMP_TYPE_PATTERN)
+    {
+      klass = g_type_class_peek (GIMP_TYPE_PATTERN_SELECT);
+      container = gimp_data_factory_get_container (gimp->pattern_factory);
+    }
+  else if (contents_type == GIMP_TYPE_DRAWABLE)
+    {
+      klass = g_type_class_peek (GIMP_TYPE_PICKABLE_SELECT);
+    }
 
-      dialog = gimp_pdb_dialog_get_by_callback (klass, callback_name);
+  g_return_val_if_fail (klass != NULL, FALSE);
 
-      if (dialog && dialog->select_type == gimp_container_get_children_type (container))
+  dialog = gimp_pdb_dialog_get_by_callback (klass, callback_name);
+
+  if (dialog != NULL                       &&
+      dialog->select_type == contents_type &&
+      (container == NULL || gimp_container_get_child_index (container, object) != -1))
+    {
+      const gchar *prop_name = va_arg (args, const gchar *);
+
+      if (g_type_is_a (contents_type, GIMP_TYPE_RESOURCE))
         {
-          GimpObject *object;
-
-          object = gimp_container_get_child_by_name (container, object_name);
-
-          if (object)
-            {
-              const gchar *prop_name = va_arg (args, const gchar *);
-
-              gimp_context_set_by_type (dialog->context, dialog->select_type,
-                                        object);
-
-              if (prop_name)
-                g_object_set_valist (G_OBJECT (dialog), prop_name, args);
-
-              gtk_window_present (GTK_WINDOW (dialog));
-
-              return TRUE;
-            }
+          g_return_val_if_fail (container != NULL, FALSE);
+          gimp_context_set_by_type (dialog->context, dialog->select_type, object);
         }
+      else
+        {
+          g_return_val_if_fail (klass->set_object != NULL, FALSE);
+          klass->set_object (dialog, object);
+        }
+
+      if (prop_name)
+        g_object_set_valist (G_OBJECT (dialog), prop_name, args);
+
+      gtk_window_present (GTK_WINDOW (dialog));
+
+      return TRUE;
     }
 
   return FALSE;
 }
 
 static gboolean
-gui_pdb_dialog_close (Gimp          *gimp,
-                      GimpContainer *container,
-                      const gchar   *callback_name)
+gui_pdb_dialog_close (Gimp        *gimp,
+                      GType        contents_type,
+                      const gchar *callback_name)
 {
   GimpPdbDialogClass *klass = NULL;
 
-  if (gimp_container_get_children_type (container) == GIMP_TYPE_BRUSH)
+  if (contents_type == GIMP_TYPE_BRUSH)
     klass = g_type_class_peek (GIMP_TYPE_BRUSH_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_FONT)
+  else if (contents_type == GIMP_TYPE_FONT)
     klass = g_type_class_peek (GIMP_TYPE_FONT_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_GRADIENT)
+  else if (contents_type == GIMP_TYPE_GRADIENT)
     klass = g_type_class_peek (GIMP_TYPE_GRADIENT_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_PALETTE)
+  else if (contents_type == GIMP_TYPE_PALETTE)
     klass = g_type_class_peek (GIMP_TYPE_PALETTE_SELECT);
-  else if (gimp_container_get_children_type (container) == GIMP_TYPE_PATTERN)
+  else if (contents_type == GIMP_TYPE_PATTERN)
     klass = g_type_class_peek (GIMP_TYPE_PATTERN_SELECT);
+  else if (contents_type == GIMP_TYPE_DRAWABLE)
+    klass = g_type_class_peek (GIMP_TYPE_PICKABLE_SELECT);
 
   if (klass)
     {
@@ -808,7 +841,7 @@ gui_pdb_dialog_close (Gimp          *gimp,
 
       dialog = gimp_pdb_dialog_get_by_callback (klass, callback_name);
 
-      if (dialog && dialog->select_type == gimp_container_get_children_type (container))
+      if (dialog && dialog->select_type == contents_type)
         {
           gtk_widget_destroy (GTK_WIDGET (dialog));
           return TRUE;
@@ -968,6 +1001,11 @@ gui_inhibit (Gimp *gimp)
     }
 
   app = GTK_APPLICATION (g_application_get_default ());
+  if (app == NULL)
+    /* This may happen when quitting. The GtkApplication is finalized
+     * faster than the images without display.
+     */
+    return;
 
   if (gimp_displays_dirty (gimp))
     {
@@ -985,7 +1023,7 @@ gui_inhibit (Gimp *gimp)
       n_images = n_dirty_images;
     }
 
-  if (gtk_application_is_inhibited (app, GTK_APPLICATION_INHIBIT_LOGOUT))
+  if (cookie != 0)
     {
       gtk_application_uninhibit (app, cookie);
       cookie = 0;
