@@ -25,7 +25,8 @@ void               gui_abort     (const gchar    *abort_message);
 GimpInitStatusFunc gui_init      (Gimp           *gimp,
                                   gboolean        no_splash,
                                   GimpApp        *app,
-                                  const gchar    *test_base_dir);
+                                  const gchar    *test_base_dir,
+                                  const gchar    *system_lang_l10n);
 
 gboolean           gui_recover   (gint            n_recoveries);
 
