@@ -15,37 +15,28 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <errno.h>
-#include <string.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <glib/gstdio.h>
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpconfig/gimpconfig.h"
 
-#ifdef G_OS_WIN32
-#include "libgimpbase/gimpwin32-io.h"
-#endif
-
 #include "gui-types.h"
 
 #include "config/gimpconfig-file.h"
 #include "config/gimpguiconfig.h"
 
 #include "core/gimp.h"
+#include "core/gimperror.h"
 
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpsessioninfo.h"
+#include "widgets/gimpwidgets-utils.h"
 
 #include "dialogs/dialogs.h"
 
@@ -60,11 +51,13 @@ enum
   SESSION_INFO = 1,
   HIDE_DOCKS,
   SINGLE_WINDOW_MODE,
+  SHOW_TABS,
+  TABS_POSITION,
   LAST_TIP_SHOWN
 };
 
 
-static gchar * session_filename (Gimp *gimp);
+static GFile * session_file (Gimp *gimp);
 
 
 /*  private variables  */
@@ -77,36 +70,36 @@ static gboolean   sessionrc_deleted = FALSE;
 void
 session_init (Gimp *gimp)
 {
-  gchar      *filename;
+  GFile      *file;
   GScanner   *scanner;
   GTokenType  token;
   GError     *error = NULL;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
 
-  filename = session_filename (gimp);
+  file = session_file (gimp);
 
-  scanner = gimp_scanner_new_file (filename, &error);
+  scanner = gimp_scanner_new_file (file, &error);
 
   if (! scanner && error->code == GIMP_CONFIG_ERROR_OPEN_ENOENT)
     {
       g_clear_error (&error);
-      g_free (filename);
+      g_object_unref (file);
 
-      filename = g_build_filename (gimp_sysconf_directory (),
-                                   "sessionrc", NULL);
-      scanner = gimp_scanner_new_file (filename, NULL);
+      file = gimp_sysconf_directory_file ("sessionrc", NULL);
+
+      scanner = gimp_scanner_new_file (file, NULL);
     }
 
   if (! scanner)
     {
       g_clear_error (&error);
-      g_free (filename);
+      g_object_unref (file);
       return;
     }
 
   if (gimp->be_verbose)
-    g_print ("Parsing '%s'\n", gimp_filename_to_utf8 (filename));
+    g_print ("Parsing '%s'\n", gimp_file_get_utf8_name (file));
 
   g_scanner_scope_add_symbol (scanner, 0, "session-info",
                               GINT_TO_POINTER (SESSION_INFO));
@@ -114,6 +107,10 @@ session_init (Gimp *gimp)
                               GINT_TO_POINTER (HIDE_DOCKS));
   g_scanner_scope_add_symbol (scanner, 0,  "single-window-mode",
                               GINT_TO_POINTER (SINGLE_WINDOW_MODE));
+  g_scanner_scope_add_symbol (scanner, 0,  "show-tabs",
+                              GINT_TO_POINTER (SHOW_TABS));
+  g_scanner_scope_add_symbol (scanner, 0,  "tabs-position",
+                              GINT_TO_POINTER (TABS_POSITION));
   g_scanner_scope_add_symbol (scanner, 0,  "last-tip-shown",
                               GINT_TO_POINTER (LAST_TIP_SHOWN));
 
@@ -219,7 +216,7 @@ session_init (Gimp *gimp)
                 {
                   g_object_unref (info);
 
-                  /* set token to left paren to we won't set another
+                  /* set token to left paren so we won't set another
                    * error below, gimp_config_deserialize() already did
                    */
                   token = G_TOKEN_LEFT_PAREN;
@@ -252,6 +249,32 @@ session_init (Gimp *gimp)
                             "single-window-mode", single_window_mode,
                             NULL);
             }
+          else if (scanner->value.v_symbol == GINT_TO_POINTER (SHOW_TABS))
+          {
+            gboolean show_tabs;
+
+            token = G_TOKEN_IDENTIFIER;
+
+            if (! gimp_scanner_parse_boolean (scanner, &show_tabs))
+              break;
+
+            g_object_set (gimp->config,
+                          "show-tabs", show_tabs,
+                          NULL);
+          }
+          else if (scanner->value.v_symbol == GINT_TO_POINTER (TABS_POSITION))
+            {
+              gint tabs_position;
+
+              token = G_TOKEN_INT;
+
+              if (! gimp_scanner_parse_int (scanner, &tabs_position))
+                break;
+
+              g_object_set (gimp->config,
+                            "tabs-position", tabs_position,
+                            NULL);
+            }
           else if (scanner->value.v_symbol == GINT_TO_POINTER (LAST_TIP_SHOWN))
             {
               gint last_tip_shown;
@@ -291,11 +314,11 @@ session_init (Gimp *gimp)
       gimp_message_literal (gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
       g_clear_error (&error);
 
-      gimp_config_file_backup_on_error (filename, "sessionrc", NULL);
+      gimp_config_file_backup_on_error (file, "sessionrc", NULL);
     }
 
-  gimp_scanner_destroy (scanner);
-  g_free (filename);
+  gimp_scanner_unref (scanner);
+  g_object_unref (file);
 
   dialogs_load_recent_docks (gimp);
 }
@@ -307,11 +330,14 @@ session_exit (Gimp *gimp)
 }
 
 void
-session_restore (Gimp *gimp)
+session_restore (Gimp       *gimp,
+                 GdkMonitor *monitor)
 {
   g_return_if_fail (GIMP_IS_GIMP (gimp));
+  g_return_if_fail (GDK_IS_MONITOR (monitor));
 
-  gimp_dialog_factory_restore (gimp_dialog_factory_get_singleton ());
+  gimp_dialog_factory_restore (gimp_dialog_factory_get_singleton (),
+                               monitor);
 
   /* make sure GimpImageWindow acts upon hide-docks at the right time,
    * see bug #678043.
@@ -328,7 +354,7 @@ session_save (Gimp     *gimp,
               gboolean  always_save)
 {
   GimpConfigWriter *writer;
-  gchar            *filename;
+  GFile            *file;
   GError           *error = NULL;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
@@ -336,25 +362,25 @@ session_save (Gimp     *gimp,
   if (sessionrc_deleted && ! always_save)
     return;
 
-  filename = session_filename (gimp);
+  file = session_file (gimp);
 
   if (gimp->be_verbose)
-    g_print ("Writing '%s'\n", gimp_filename_to_utf8 (filename));
+    g_print ("Writing '%s'\n", gimp_file_get_utf8_name (file));
 
   writer =
-    gimp_config_writer_new_file (filename,
-                                 TRUE,
-                                 "GIMP sessionrc\n\n"
-                                 "This file takes session-specific info "
-                                 "(that is info, you want to keep between "
-                                 "two GIMP sessions).  You are not supposed "
-                                 "to edit it manually, but of course you "
-                                 "can do.  The sessionrc will be entirely "
-                                 "rewritten every time you quit GIMP.  "
-                                 "If this file isn't found, defaults are "
-                                 "used.",
-                                 NULL);
-  g_free (filename);
+    gimp_config_writer_new_from_file (file,
+                                      TRUE,
+                                      "GIMP sessionrc\n\n"
+                                      "This file takes session-specific info "
+                                      "(that is info, you want to keep between "
+                                      "two GIMP sessions).  You are not supposed "
+                                      "to edit it manually, but of course you "
+                                      "can do.  The sessionrc will be entirely "
+                                      "rewritten every time you quit GIMP.  "
+                                      "If this file isn't found, defaults are "
+                                      "used.",
+                                      NULL);
+  g_object_unref (file);
 
   if (!writer)
     return;
@@ -374,6 +400,17 @@ session_save (Gimp     *gimp,
                                  "yes" : "no");
   gimp_config_writer_close (writer);
 
+  gimp_config_writer_open (writer, "show-tabs");
+  gimp_config_writer_printf (writer,
+                             GIMP_GUI_CONFIG (gimp->config)->show_tabs ?
+                             "yes" : "no");
+  gimp_config_writer_close (writer);
+
+  gimp_config_writer_open (writer, "tabs-position");
+  gimp_config_writer_printf (writer, "%d",
+                             GIMP_GUI_CONFIG (gimp->config)->tabs_position);
+  gimp_config_writer_close (writer);
+
   gimp_config_writer_open (writer, "last-tip-shown");
   gimp_config_writer_printf (writer, "%d",
                              GIMP_GUI_CONFIG (gimp->config)->last_tip_shown);
@@ -394,51 +431,55 @@ gboolean
 session_clear (Gimp    *gimp,
                GError **error)
 {
-  gchar    *filename;
-  gboolean  success = TRUE;
+  GFile    *file;
+  GError   *my_error = NULL;
+  gboolean  success  = TRUE;
 
   g_return_val_if_fail (GIMP_IS_GIMP (gimp), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  filename = session_filename (gimp);
+  file = session_file (gimp);
 
-  if (g_unlink (filename) != 0 && errno != ENOENT)
+  if (! g_file_delete (file, NULL, &my_error) &&
+      my_error->code != G_IO_ERROR_NOT_FOUND)
     {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-		   _("Deleting \"%s\" failed: %s"),
-                   gimp_filename_to_utf8 (filename), g_strerror (errno));
       success = FALSE;
+
+      g_set_error (error, GIMP_ERROR, GIMP_FAILED,
+                   _("Deleting \"%s\" failed: %s"),
+                   gimp_file_get_utf8_name (file), my_error->message);
     }
   else
     {
       sessionrc_deleted = TRUE;
     }
 
-  g_free (filename);
+  g_clear_error (&my_error);
+  g_object_unref (file);
 
   return success;
 }
 
 
-static gchar *
-session_filename (Gimp *gimp)
+static GFile *
+session_file (Gimp *gimp)
 {
   const gchar *basename;
   gchar       *filename;
+  GFile       *file;
 
   basename = g_getenv ("GIMP_TESTING_SESSIONRC_NAME");
   if (! basename)
     basename = "sessionrc";
 
-  filename = gimp_personal_rc_file (basename);
-
   if (gimp->session_name)
-    {
-      gchar *tmp = g_strconcat (filename, ".", gimp->session_name, NULL);
+    filename = g_strconcat (basename, ".", gimp->session_name, NULL);
+  else
+    filename = g_strdup (basename);
 
-      g_free (filename);
-      filename = tmp;
-    }
+  file = gimp_directory_file (filename, NULL);
 
-  return filename;
+  g_free (filename);
+
+  return file;
 }
