@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __LANGUAGE_H__
@@ -23,7 +23,8 @@
 #endif
 
 
-void language_init (const gchar *language);
+const gchar * language_init (const gchar  *language,
+                             const gchar **system_lang_l10n);
 
 
 #endif /* __LANGUAGE_H__ */
