@@ -65,12 +65,17 @@ struct _GimpPaintCore
   GeglBuffer     *paint_buffer;      /*  the buffer to paint pixels to       */
   gint            paint_buffer_x;
   gint            paint_buffer_y;
+  GHashTable     *original_bounds;   /* the original bounds of drawables     */
 
   GeglBuffer     *mask_buffer;       /*  the target drawable's mask          */
 
   GHashTable     *applicators;
 
   GArray         *stroke_buffer;
+
+  GimpSymmetry   *sym;
+  GimpPaintLockBlinkState
+                  lock_blink_state;
 };
 
 struct _GimpPaintCoreClass
@@ -152,6 +157,16 @@ void      gimp_paint_core_set_show_all              (GimpPaintCore    *core,
                                                      gboolean          show_all);
 gboolean  gimp_paint_core_get_show_all              (GimpPaintCore    *core);
 
+gboolean  gimp_paint_core_expand_drawable           (GimpPaintCore    *paint_core,
+                                                     GimpDrawable     *drawable,
+                                                     GimpPaintOptions *paint_options,
+                                                     gint              x1,
+                                                     gint              x2,
+                                                     gint              y1,
+                                                     gint              y2,
+                                                     gint             *x,
+                                                     gint             *y);
+
 void      gimp_paint_core_set_current_coords        (GimpPaintCore    *core,
                                                      const GimpCoords *coords);
 void      gimp_paint_core_get_current_coords        (GimpPaintCore    *core,
