@@ -67,7 +67,7 @@
 #define DEFAULT_FADE_LENGTH             100.0
 #define DEFAULT_FADE_REVERSE            FALSE
 #define DEFAULT_FADE_REPEAT             GIMP_REPEAT_NONE
-#define DEFAULT_FADE_UNIT               GIMP_UNIT_PIXEL
+#define DEFAULT_FADE_UNIT               gimp_unit_pixel ()
 
 #define DEFAULT_GRADIENT_REVERSE        FALSE
 #define DEFAULT_GRADIENT_BLEND_SPACE    GIMP_GRADIENT_BLEND_RGB_PERCEPTUAL
@@ -79,6 +79,11 @@
 #define DEFAULT_SMOOTHING_QUALITY       20
 #define DEFAULT_SMOOTHING_FACTOR        50
 
+#define DEFAULT_EXPAND_USE              FALSE
+#define DEFAULT_EXPAND_AMOUNT           100.0
+#define DEFAULT_EXPAND_FILL_TYPE        GIMP_FILL_TRANSPARENT
+#define DEFAULT_EXPAND_MASK_FILL_TYPE   GIMP_ADD_MASK_WHITE
+
 enum
 {
   PROP_0,
@@ -130,7 +135,12 @@ enum
 
   PROP_USE_SMOOTHING,
   PROP_SMOOTHING_QUALITY,
-  PROP_SMOOTHING_FACTOR
+  PROP_SMOOTHING_FACTOR,
+
+  PROP_EXPAND_USE,
+  PROP_EXPAND_AMOUNT,
+  PROP_EXPAND_FILL_TYPE,
+  PROP_EXPAND_MASK_FILL_TYPE
 };
 
 
@@ -304,6 +314,36 @@ gimp_paint_options_class_init (GimpPaintOptionsClass *klass)
                             DEFAULT_USE_JITTER,
                             GIMP_PARAM_STATIC_STRINGS);
 
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_EXPAND_USE,
+                            "expand-use",
+                            _("Expand Layers"),
+                            _("Expand active layer as you paint"),
+                            DEFAULT_EXPAND_USE,
+                            GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_EXPAND_AMOUNT,
+                           "expand-amount",
+                           _("Amount"),
+                           _("Amount of expansion"),
+                           1.0, 1000.0, DEFAULT_EXPAND_AMOUNT,
+                           GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_EXPAND_FILL_TYPE,
+                         "expand-fill-type",
+                         _("Fill With"),
+                         _("Fill layer with"),
+                         GIMP_TYPE_FILL_TYPE,
+                         DEFAULT_EXPAND_FILL_TYPE,
+                         GIMP_PARAM_STATIC_STRINGS);
+
+  GIMP_CONFIG_PROP_ENUM (object_class, PROP_EXPAND_MASK_FILL_TYPE,
+                         "expand-mask-fill-type",
+                         _("Fill Mask With"),
+                         _("Fill layer mask with"),
+                         GIMP_TYPE_ADD_MASK_TYPE,
+                         DEFAULT_EXPAND_MASK_FILL_TYPE,
+                         GIMP_PARAM_STATIC_STRINGS);
+
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_DYNAMICS_ENABLED,
                             "dynamics-enabled",
                             _("Enable dynamics"),
@@ -587,7 +627,7 @@ gimp_paint_options_set_property (GObject      *object,
       fade_options->fade_repeat = g_value_get_enum (value);
       break;
     case PROP_FADE_UNIT:
-      fade_options->fade_unit = g_value_get_int (value);
+      fade_options->fade_unit = g_value_get_object (value);
       break;
 
     case PROP_GRADIENT_REVERSE:
@@ -638,6 +678,19 @@ gimp_paint_options_set_property (GObject      *object,
       smoothing_options->smoothing_factor = g_value_get_double (value);
       break;
 
+    case PROP_EXPAND_USE:
+      options->expand_use = g_value_get_boolean (value);
+      break;
+    case PROP_EXPAND_AMOUNT:
+      options->expand_amount = g_value_get_double (value);
+      break;
+    case PROP_EXPAND_FILL_TYPE:
+      options->expand_fill_type = g_value_get_enum (value);
+      break;
+    case PROP_EXPAND_MASK_FILL_TYPE:
+      options->expand_mask_fill_type = g_value_get_enum (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -733,7 +786,7 @@ gimp_paint_options_get_property (GObject    *object,
       g_value_set_enum (value, fade_options->fade_repeat);
       break;
     case PROP_FADE_UNIT:
-      g_value_set_int (value, fade_options->fade_unit);
+      g_value_set_object (value, fade_options->fade_unit);
       break;
 
     case PROP_GRADIENT_REVERSE:
@@ -784,6 +837,19 @@ gimp_paint_options_get_property (GObject    *object,
       g_value_set_double (value, smoothing_options->smoothing_factor);
       break;
 
+    case PROP_EXPAND_USE:
+      g_value_set_boolean (value, options->expand_use);
+      break;
+    case PROP_EXPAND_AMOUNT:
+      g_value_set_double (value, options->expand_amount);
+      break;
+    case PROP_EXPAND_FILL_TYPE:
+      g_value_set_enum (value, options->expand_fill_type);
+      break;
+    case PROP_EXPAND_MASK_FILL_TYPE:
+      g_value_set_enum (value, options->expand_mask_fill_type);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -917,30 +983,26 @@ gimp_paint_options_get_fade (GimpPaintOptions *paint_options,
 
   fade_options = paint_options->fade_options;
 
-  switch (fade_options->fade_unit)
+  if (fade_options->fade_unit == gimp_unit_pixel ())
     {
-    case GIMP_UNIT_PIXEL:
       fade_out = fade_options->fade_length;
-      break;
-
-    case GIMP_UNIT_PERCENT:
+    }
+  else if (fade_options->fade_unit == gimp_unit_percent ())
+    {
       fade_out = (MAX (gimp_image_get_width  (image),
                        gimp_image_get_height (image)) *
                   fade_options->fade_length / 100);
-      break;
+    }
+  else
+    {
+      gdouble xres;
+      gdouble yres;
 
-    default:
-      {
-        gdouble xres;
-        gdouble yres;
+      gimp_image_get_resolution (image, &xres, &yres);
 
-        gimp_image_get_resolution (image, &xres, &yres);
-
-        unit_factor = gimp_unit_get_factor (fade_options->fade_unit);
-        fade_out    = (fade_options->fade_length *
-                       MAX (xres, yres) / unit_factor);
-      }
-      break;
+      unit_factor = gimp_unit_get_factor (fade_options->fade_unit);
+      fade_out    = (fade_options->fade_length *
+                     MAX (xres, yres) / unit_factor);
     }
 
   /*  factor in the fade out value  */
@@ -988,17 +1050,17 @@ gimp_paint_options_get_jitter (GimpPaintOptions *paint_options,
 }
 
 gboolean
-gimp_paint_options_get_gradient_color (GimpPaintOptions *paint_options,
-                                       GimpImage        *image,
-                                       gdouble           grad_point,
-                                       gdouble           pixel_dist,
-                                       GimpRGB          *color)
+gimp_paint_options_get_gradient_color (GimpPaintOptions  *paint_options,
+                                       GimpImage         *image,
+                                       gdouble            grad_point,
+                                       gdouble            pixel_dist,
+                                       GeglColor        **color)
 {
   GimpDynamics *dynamics;
 
   g_return_val_if_fail (GIMP_IS_PAINT_OPTIONS (paint_options), FALSE);
   g_return_val_if_fail (GIMP_IS_IMAGE (image), FALSE);
-  g_return_val_if_fail (color != NULL, FALSE);
+  g_return_val_if_fail (color != NULL && *color == NULL, FALSE);
 
   dynamics = gimp_context_get_dynamics (GIMP_CONTEXT (paint_options));
 
@@ -1217,9 +1279,18 @@ static const gchar *gradient_props[] =
   "gradient-repeat"
 };
 
+static const gchar *expand_props[] =
+{
+  "expand-use",
+  "expand-amount",
+  "expand-fill-type",
+  "expand-mask-fill-type",
+};
+
 static const gint max_n_props = (G_N_ELEMENTS (brush_props) +
                                  G_N_ELEMENTS (dynamics_props) +
-                                 G_N_ELEMENTS (gradient_props));
+                                 G_N_ELEMENTS (gradient_props) +
+                                 G_N_ELEMENTS (expand_props));
 
 gboolean
 gimp_paint_options_is_prop (const gchar         *prop_name,
@@ -1250,6 +1321,13 @@ gimp_paint_options_is_prop (const gchar         *prop_name,
           return TRUE;
     }
 
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_EXPAND)
+    {
+      for (i = 0; i < G_N_ELEMENTS (expand_props); i++)
+        if (! strcmp (prop_name, expand_props[i]))
+          return TRUE;
+    }
+
   return FALSE;
 }
 
@@ -1284,6 +1362,12 @@ gimp_paint_options_copy_props (GimpPaintOptions    *src,
         names[n_props++] = gradient_props[i];
     }
 
+  if (prop_mask & GIMP_CONTEXT_PROP_MASK_EXPAND)
+    {
+      for (i = 0; i < G_N_ELEMENTS (expand_props); i++)
+        names[n_props++] = expand_props[i];
+    }
+
   if (n_props > 0)
     {
       g_object_getv (G_OBJECT (src), n_props, names, values);
