@@ -14,23 +14,26 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpbase/gimpprotocol.h"
 #include "libgimpbase/gimpwire.h"
 
+#include "libgimp/gimpgpparams.h"
+
 #include "plug-in-types.h"
 
-#include "base/tile.h"
-#include "base/tile-manager.h"
+#include "gegl/gimp-babl.h"
+#include "gegl/gimp-gegl-tile-compat.h"
 
 #include "core/gimp.h"
 #include "core/gimpdrawable.h"
@@ -47,7 +50,6 @@
 #include "gimpplugindef.h"
 #include "gimppluginshm.h"
 #include "gimptemporaryprocedure.h"
-#include "plug-in-params.h"
 
 #include "gimp-intl.h"
 
@@ -93,10 +95,10 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
 
     case GP_CONFIG:
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent a CONFIG message.  This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
       break;
 
@@ -106,19 +108,19 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
 
     case GP_TILE_ACK:
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent a TILE_ACK message.  This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
       break;
 
     case GP_TILE_DATA:
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent a TILE_DATA message.  This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
       break;
 
@@ -132,10 +134,10 @@ gimp_plug_in_handle_message (GimpPlugIn      *plug_in,
 
     case GP_TEMP_PROC_RUN:
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent a TEMP_PROC_RUN message.  This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
       break;
 
@@ -176,7 +178,7 @@ gimp_plug_in_handle_tile_request (GimpPlugIn *plug_in,
 {
   g_return_if_fail (request != NULL);
 
-  if (request->drawable_ID == -1)
+  if (request->drawable_id == -1)
     gimp_plug_in_handle_tile_put (plug_in, request);
   else
     gimp_plug_in_handle_tile_get (plug_in, request);
@@ -190,10 +192,11 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
   GPTileData      *tile_info;
   GimpWireMessage  msg;
   GimpDrawable    *drawable;
-  TileManager     *tm;
-  Tile            *tile;
+  GeglBuffer      *buffer;
+  const Babl      *format;
+  GeglRectangle    tile_rect;
 
-  tile_data.drawable_ID = -1;
+  tile_data.drawable_id = -1;
   tile_data.tile_num    = 0;
   tile_data.shadow      = 0;
   tile_data.bpp         = 0;
@@ -228,95 +231,104 @@ gimp_plug_in_handle_tile_put (GimpPlugIn *plug_in,
 
   tile_info = msg.data;
 
-  drawable = (GimpDrawable *) gimp_item_get_by_ID (plug_in->manager->gimp,
-                                                   tile_info->drawable_ID);
+  drawable = (GimpDrawable *) gimp_item_get_by_id (plug_in->manager->gimp,
+                                                   tile_info->drawable_id);
 
   if (! GIMP_IS_DRAWABLE (drawable))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "tried writing to invalid drawable %d (killing)",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    tile_info->drawable_ID);
+                    gimp_file_get_utf8_name (plug_in->file),
+                    tile_info->drawable_id);
       gimp_plug_in_close (plug_in, TRUE);
       return;
     }
   else if (gimp_item_is_removed (GIMP_ITEM (drawable)))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "tried writing to drawable %d which was removed "
                     "from the image (killing)",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    tile_info->drawable_ID);
+                    gimp_file_get_utf8_name (plug_in->file),
+                    tile_info->drawable_id);
       gimp_plug_in_close (plug_in, TRUE);
       return;
     }
 
   if (tile_info->shadow)
     {
+
       /*  don't check whether the drawable is a group or locked here,
        *  the plugin will get a proper error message when it tries to
        *  merge the shadow tiles, which is much better than just
        *  killing it.
        */
-      tm = gimp_drawable_get_shadow_tiles (drawable);
+      buffer = gimp_drawable_get_shadow_buffer (drawable);
 
       gimp_plug_in_cleanup_add_shadow (plug_in, drawable);
     }
   else
     {
-      if (gimp_item_is_content_locked (GIMP_ITEM (drawable)))
+      if (gimp_item_is_content_locked (GIMP_ITEM (drawable), NULL))
         {
           gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                        "Plug-In \"%s\"\n(%s)\n\n"
+                        "Plug-in \"%s\"\n(%s)\n\n"
                         "tried writing to a locked drawable %d (killing)",
                         gimp_object_get_name (plug_in),
-                        gimp_filename_to_utf8 (plug_in->prog),
-                        tile_info->drawable_ID);
+                        gimp_file_get_utf8_name (plug_in->file),
+                        tile_info->drawable_id);
           gimp_plug_in_close (plug_in, TRUE);
           return;
         }
       else if (gimp_viewable_get_children (GIMP_VIEWABLE (drawable)))
         {
           gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                        "Plug-In \"%s\"\n(%s)\n\n"
+                        "Plug-in \"%s\"\n(%s)\n\n"
                         "tried writing to a group layer %d (killing)",
                         gimp_object_get_name (plug_in),
-                        gimp_filename_to_utf8 (plug_in->prog),
-                        tile_info->drawable_ID);
+                        gimp_file_get_utf8_name (plug_in->file),
+                        tile_info->drawable_id);
           gimp_plug_in_close (plug_in, TRUE);
           return;
         }
 
-      tm = gimp_drawable_get_tiles (drawable);
+      buffer = gimp_drawable_get_buffer (drawable);
     }
 
-  tile = tile_manager_get (tm, tile_info->tile_num, TRUE, TRUE);
-
-  if (! tile)
+  if (! gimp_gegl_buffer_get_tile_rect (buffer,
+                                        GIMP_PLUG_IN_TILE_WIDTH,
+                                        GIMP_PLUG_IN_TILE_HEIGHT,
+                                        tile_info->tile_num,
+                                        &tile_rect))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
-                    "requested invalid tile (killing)",
+                    "Plug-in \"%s\"\n(%s)\n\n"
+                    "requested invalid tile #%d for writing (killing)",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file),
+                    tile_info->tile_num);
       gimp_plug_in_close (plug_in, TRUE);
       return;
     }
 
+  format = gegl_buffer_get_format (buffer);
+
   if (tile_data.use_shm)
-    memcpy (tile_data_pointer (tile, 0, 0),
-            gimp_plug_in_shm_get_addr (plug_in->manager->shm),
-            tile_size (tile));
+    {
+      gegl_buffer_set (buffer, &tile_rect, 0, format,
+                       gimp_plug_in_shm_get_addr (plug_in->manager->shm),
+                       GEGL_AUTO_ROWSTRIDE);
+    }
   else
-    memcpy (tile_data_pointer (tile, 0, 0),
-            tile_info->data,
-            tile_size (tile));
+    {
+      gegl_buffer_set (buffer, &tile_rect, 0, format,
+                       tile_info->data,
+                       GEGL_AUTO_ROWSTRIDE);
+    }
 
-  tile_release (tile, TRUE);
   gimp_wire_destroy (&msg);
 
   if (! gp_tile_ack_write (plug_in->my_write, plug_in))
@@ -335,74 +347,92 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
   GPTileData       tile_data;
   GimpWireMessage  msg;
   GimpDrawable    *drawable;
-  TileManager     *tm;
-  Tile            *tile;
+  GeglBuffer      *buffer;
+  const Babl      *format;
+  GeglRectangle    tile_rect;
+  gint             tile_size;
 
-  drawable = (GimpDrawable *) gimp_item_get_by_ID (plug_in->manager->gimp,
-                                                   request->drawable_ID);
+  drawable = (GimpDrawable *) gimp_item_get_by_id (plug_in->manager->gimp,
+                                                   request->drawable_id);
 
   if (! GIMP_IS_DRAWABLE (drawable))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "tried reading from invalid drawable %d (killing)",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    request->drawable_ID);
+                    gimp_file_get_utf8_name (plug_in->file),
+                    request->drawable_id);
       gimp_plug_in_close (plug_in, TRUE);
       return;
     }
   else if (gimp_item_is_removed (GIMP_ITEM (drawable)))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "tried reading from drawable %d which was removed "
                     "from the image (killing)",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    request->drawable_ID);
+                    gimp_file_get_utf8_name (plug_in->file),
+                    request->drawable_id);
       gimp_plug_in_close (plug_in, TRUE);
       return;
     }
 
   if (request->shadow)
     {
-      tm = gimp_drawable_get_shadow_tiles (drawable);
+      buffer = gimp_drawable_get_shadow_buffer (drawable);
 
       gimp_plug_in_cleanup_add_shadow (plug_in, drawable);
     }
   else
     {
-      tm = gimp_drawable_get_tiles (drawable);
+      buffer = gimp_drawable_get_buffer (drawable);
     }
 
-  tile = tile_manager_get (tm, request->tile_num, TRUE, FALSE);
-
-  if (! tile)
+  if (! gimp_gegl_buffer_get_tile_rect (buffer,
+                                        GIMP_PLUG_IN_TILE_WIDTH,
+                                        GIMP_PLUG_IN_TILE_HEIGHT,
+                                        request->tile_num,
+                                        &tile_rect))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
-                    "requested invalid tile (killing)",
+                    "Plug-in \"%s\"\n(%s)\n\n"
+                    "requested invalid tile #%d for reading (killing)",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file),
+                    request->tile_num);
       gimp_plug_in_close (plug_in, TRUE);
       return;
     }
 
-  tile_data.drawable_ID = request->drawable_ID;
+  format = gegl_buffer_get_format (buffer);
+
+  tile_size = (babl_format_get_bytes_per_pixel (format) *
+               tile_rect.width * tile_rect.height);
+
+  tile_data.drawable_id = request->drawable_id;
   tile_data.tile_num    = request->tile_num;
   tile_data.shadow      = request->shadow;
-  tile_data.bpp         = tile_bpp (tile);
-  tile_data.width       = tile_ewidth (tile);
-  tile_data.height      = tile_eheight (tile);
+  tile_data.bpp         = babl_format_get_bytes_per_pixel (format);
+  tile_data.width       = tile_rect.width;
+  tile_data.height      = tile_rect.height;
   tile_data.use_shm     = (plug_in->manager->shm != NULL);
 
   if (tile_data.use_shm)
-    memcpy (gimp_plug_in_shm_get_addr (plug_in->manager->shm),
-            tile_data_pointer (tile, 0, 0),
-            tile_size (tile));
+    {
+      gegl_buffer_get (buffer, &tile_rect, 1.0, format,
+                       gimp_plug_in_shm_get_addr (plug_in->manager->shm),
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+    }
   else
-    tile_data.data = tile_data_pointer (tile, 0, 0);
+    {
+      tile_data.data = g_malloc (tile_size);
+
+      gegl_buffer_get (buffer, &tile_rect, 1.0, format,
+                       tile_data.data,
+                       GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+    }
 
   if (! gp_tile_data_write (plug_in->my_write, &tile_data, plug_in))
     {
@@ -412,8 +442,6 @@ gimp_plug_in_handle_tile_get (GimpPlugIn *plug_in,
       return;
     }
 
-  tile_release (tile, FALSE);
-
   if (! gimp_wire_read_msg (plug_in->my_read, &msg, plug_in))
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
@@ -476,8 +504,8 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
   gchar               *canonical;
   const gchar         *proc_name   = NULL;
   GimpProcedure       *procedure;
-  GValueArray         *args        = NULL;
-  GValueArray         *return_vals = NULL;
+  GimpValueArray      *args        = NULL;
+  GimpValueArray      *return_vals = NULL;
   GError              *error       = NULL;
 
   g_return_if_fail (proc_run != NULL);
@@ -503,11 +531,11 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
           if (plug_in->manager->gimp->pdb_compat_mode == GIMP_PDB_COMPAT_WARN)
             {
               gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_WARNING,
-                            "Plug-In \"%s\"\n(%s)\n"
+                            "Plug-in \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.\n"
                             "It should call '%s' instead!",
                             gimp_object_get_name (plug_in),
-                            gimp_filename_to_utf8 (plug_in->prog),
+                            gimp_file_get_utf8_name (plug_in->file),
                             canonical, proc_name);
             }
         }
@@ -519,20 +547,20 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
           if (! strcmp (procedure->deprecated, "NONE"))
             {
               gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_WARNING,
-                            "Plug-In \"%s\"\n(%s)\n"
+                            "Plug-in \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.",
                             gimp_object_get_name (plug_in),
-                            gimp_filename_to_utf8 (plug_in->prog),
+                            gimp_file_get_utf8_name (plug_in->file),
                             canonical);
             }
           else
             {
               gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_WARNING,
-                            "WARNING: Plug-In \"%s\"\n(%s)\n"
+                            "WARNING: Plug-in \"%s\"\n(%s)\n"
                             "called deprecated procedure '%s'.\n"
                             "It should call '%s' instead!",
                             gimp_object_get_name (plug_in),
-                            gimp_filename_to_utf8 (plug_in->prog),
+                            gimp_file_get_utf8_name (plug_in->file),
                             canonical, procedure->deprecated);
             }
         }
@@ -541,10 +569,12 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
   if (! proc_name)
     proc_name = canonical;
 
-  args = plug_in_params_to_args (procedure ? procedure->args     : NULL,
-                                 procedure ? procedure->num_args : 0,
-                                 proc_run->params, proc_run->nparams,
-                                 FALSE, FALSE);
+  args = _gimp_gp_params_to_value_array (plug_in->manager->gimp,
+                                         procedure ? procedure->args     : NULL,
+                                         procedure ? procedure->num_args : 0,
+                                         proc_run->params,
+                                         proc_run->n_params,
+                                         FALSE);
 
   /*  Execute the procedure even if gimp_pdb_lookup_procedure()
    *  returned NULL, gimp_pdb_execute_procedure_by_name_args() will
@@ -561,7 +591,7 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
                                                          args);
   gimp_plug_in_manager_plug_in_pop (plug_in->manager);
 
-  g_value_array_free (args);
+  gimp_value_array_unref (args);
 
   if (error)
     {
@@ -583,9 +613,9 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
        *  since proc_name may have been remapped by gimp->procedural_compat_ht
        *  and canonical may be different too.
        */
-      proc_return.name    = proc_run->name;
-      proc_return.nparams = return_vals->n_values;
-      proc_return.params  = plug_in_args_to_params (return_vals, FALSE);
+      proc_return.name     = proc_run->name;
+      proc_return.n_params = gimp_value_array_length (return_vals);
+      proc_return.params   = _gimp_value_array_to_gp_params (return_vals, FALSE);
 
       if (! gp_proc_return_write (plug_in->my_write, &proc_return, plug_in))
         {
@@ -594,10 +624,10 @@ gimp_plug_in_handle_proc_run (GimpPlugIn *plug_in,
           gimp_plug_in_close (plug_in, TRUE);
         }
 
-      g_free (proc_return.params);
+      _gimp_gp_params_free (proc_return.params, proc_return.n_params, FALSE);
     }
 
-  g_value_array_free (return_vals);
+  gimp_value_array_unref (return_vals);
 }
 
 static void
@@ -609,11 +639,12 @@ gimp_plug_in_handle_proc_return (GimpPlugIn   *plug_in,
   g_return_if_fail (proc_return != NULL);
 
   proc_frame->return_vals =
-    plug_in_params_to_args (proc_frame->procedure->values,
-                            proc_frame->procedure->num_values,
-                            proc_return->params,
-                            proc_return->nparams,
-                            TRUE, TRUE);
+    _gimp_gp_params_to_value_array (plug_in->manager->gimp,
+                                    proc_frame->procedure->values,
+                                    proc_frame->procedure->num_values,
+                                    proc_return->params,
+                                    proc_return->n_params,
+                                    TRUE);
 
   if (proc_frame->main_loop)
     {
@@ -644,11 +675,12 @@ gimp_plug_in_handle_temp_proc_return (GimpPlugIn   *plug_in,
       GimpPlugInProcFrame *proc_frame = plug_in->temp_proc_frames->data;
 
       proc_frame->return_vals =
-        plug_in_params_to_args (proc_frame->procedure->values,
-                                proc_frame->procedure->num_values,
-                                proc_return->params,
-                                proc_return->nparams,
-                                TRUE, TRUE);
+        _gimp_gp_params_to_value_array (plug_in->manager->gimp,
+                                        proc_frame->procedure->values,
+                                        proc_frame->procedure->num_values,
+                                        proc_return->params,
+                                        proc_return->n_params,
+                                        TRUE);
 
       gimp_plug_in_main_loop_quit (plug_in);
       gimp_plug_in_proc_frame_pop (plug_in);
@@ -656,11 +688,11 @@ gimp_plug_in_handle_temp_proc_return (GimpPlugIn   *plug_in,
   else
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent a TEMP_PROC_RETURN message while not running "
                     "a temporary procedure.  This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
     }
 }
@@ -671,106 +703,80 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
 {
   GimpPlugInProcedure *proc       = NULL;
   GimpProcedure       *procedure  = NULL;
-  gchar               *canonical;
   gboolean             null_name  = FALSE;
-  gboolean             valid_utf8 = FALSE;
+  gboolean             valid_utf8 = TRUE;
   gint                 i;
 
   g_return_if_fail (proc_install != NULL);
   g_return_if_fail (proc_install->name != NULL);
 
-  canonical = gimp_canonicalize_identifier (proc_install->name);
-
-  /*  Sanity check for array arguments  */
-
-  for (i = 1; i < proc_install->nparams; i++)
+  if (! gimp_is_canonical_identifier (proc_install->name))
     {
-      if ((proc_install->params[i].type == GIMP_PDB_INT32ARRAY ||
-           proc_install->params[i].type == GIMP_PDB_INT8ARRAY  ||
-           proc_install->params[i].type == GIMP_PDB_FLOATARRAY ||
-           proc_install->params[i].type == GIMP_PDB_STRINGARRAY ||
-           proc_install->params[i].type == GIMP_PDB_COLORARRAY)
-          &&
-          proc_install->params[i - 1].type != GIMP_PDB_INT32)
-        {
-          gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                        "Plug-In \"%s\"\n(%s)\n\n"
-                        "attempted to install procedure \"%s\" "
-                        "which fails to comply with the array parameter "
-                        "passing standard.  Argument %d is noncompliant.",
-                        gimp_object_get_name (plug_in),
-                        gimp_filename_to_utf8 (plug_in->prog),
-                        canonical, i);
-          g_free (canonical);
-          return;
-        }
+      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                    "Plug-in \"%s\"\n(%s)\n\n"
+                    "attempted to install procedure \"%s\" with a "
+                    "non-canonical name.",
+                    gimp_object_get_name (plug_in),
+                    gimp_file_get_utf8_name (plug_in->file),
+                    proc_install->name);
+      return;
     }
 
   /*  Sanity check strings for UTF-8 validity  */
 
-#define VALIDATE(str)         (g_utf8_validate ((str), -1, NULL))
-#define VALIDATE_OR_NULL(str) ((str) == NULL || g_utf8_validate ((str), -1, NULL))
+#define VALIDATE(str) ((str) == NULL || g_utf8_validate ((str), -1, NULL))
 
-  if (VALIDATE_OR_NULL (proc_install->menu_path) &&
-      VALIDATE         (canonical)               &&
-      VALIDATE_OR_NULL (proc_install->blurb)     &&
-      VALIDATE_OR_NULL (proc_install->help)      &&
-      VALIDATE_OR_NULL (proc_install->author)    &&
-      VALIDATE_OR_NULL (proc_install->copyright) &&
-      VALIDATE_OR_NULL (proc_install->date))
+  for (i = 0; i < proc_install->n_params && valid_utf8 && ! null_name; i++)
     {
-      null_name  = FALSE;
-      valid_utf8 = TRUE;
-
-      for (i = 0; i < proc_install->nparams && valid_utf8 && !null_name; i++)
+      if (! proc_install->params[i].name)
         {
-          if (! proc_install->params[i].name)
-            {
-              null_name = TRUE;
-            }
-          else if (! (VALIDATE         (proc_install->params[i].name) &&
-                      VALIDATE_OR_NULL (proc_install->params[i].description)))
-            {
-              valid_utf8 = FALSE;
-            }
+          null_name = TRUE;
         }
-
-      for (i = 0; i < proc_install->nreturn_vals && valid_utf8 && !null_name; i++)
+      else if (! (VALIDATE (proc_install->params[i].name) &&
+                  VALIDATE (proc_install->params[i].nick) &&
+                  VALIDATE (proc_install->params[i].blurb)))
         {
-          if (! proc_install->return_vals[i].name)
-            {
-              null_name = TRUE;
-            }
-          else if (! (VALIDATE         (proc_install->return_vals[i].name) &&
-                      VALIDATE_OR_NULL (proc_install->return_vals[i].description)))
-            {
-              valid_utf8 = FALSE;
-            }
+          valid_utf8 = FALSE;
+        }
+    }
+
+  for (i = 0; i < proc_install->n_return_vals && valid_utf8 && !null_name; i++)
+    {
+      if (! proc_install->return_vals[i].name)
+        {
+          null_name = TRUE;
+        }
+      else if (! (VALIDATE (proc_install->return_vals[i].name) &&
+                  VALIDATE (proc_install->return_vals[i].nick) &&
+                  VALIDATE (proc_install->return_vals[i].blurb)))
+        {
+          valid_utf8 = FALSE;
         }
     }
 
 #undef VALIDATE
-#undef VALIDATE_OR_NULL
 
   if (null_name)
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
-                    "attempted to install a procedure NULL parameter name.",
+                    "Plug-in \"%s\"\n(%s)\n\n"
+                    "attempted to install procedure \"%s\" with a "
+                    "NULL parameter name.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
-      g_free (canonical);
+                    gimp_file_get_utf8_name (plug_in->file),
+                    proc_install->name);
       return;
     }
 
   if (! valid_utf8)
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
-                    "attempted to install a procedure with invalid UTF-8 strings.",
+                    "Plug-in \"%s\"\n(%s)\n\n"
+                    "attempted to install procedure \"%s\" with "
+                    "invalid UTF-8 strings.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
-      g_free (canonical);
+                    gimp_file_get_utf8_name (plug_in->file),
+                    proc_install->name);
       return;
     }
 
@@ -778,13 +784,13 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
 
   switch (proc_install->type)
     {
-    case GIMP_PLUGIN:
-    case GIMP_EXTENSION:
+    case GIMP_PDB_PROC_TYPE_PLUGIN:
+    case GIMP_PDB_PROC_TYPE_PERSISTENT:
       procedure = gimp_plug_in_procedure_new (proc_install->type,
-                                              plug_in->prog);
+                                              plug_in->file);
       break;
 
-    case GIMP_TEMPORARY:
+    case GIMP_PDB_PROC_TYPE_TEMPORARY:
       procedure = gimp_temporary_procedure_new (plug_in);
       break;
     }
@@ -794,74 +800,36 @@ gimp_plug_in_handle_proc_install (GimpPlugIn    *plug_in,
   proc->mtime                 = time (NULL);
   proc->installed_during_init = (plug_in->call_mode == GIMP_PLUG_IN_CALL_INIT);
 
-  gimp_object_take_name (GIMP_OBJECT (procedure), canonical);
-  gimp_procedure_set_strings (procedure,
-                              proc_install->name,
-                              proc_install->blurb,
-                              proc_install->help,
-                              proc_install->author,
-                              proc_install->copyright,
-                              proc_install->date,
-                              NULL);
+  gimp_object_set_name (GIMP_OBJECT (procedure), proc_install->name);
 
-  gimp_plug_in_procedure_set_image_types (proc, proc_install->image_types);
-
-  for (i = 0; i < proc_install->nparams; i++)
+  for (i = 0; i < proc_install->n_params; i++)
     {
       GParamSpec *pspec =
-        gimp_pdb_compat_param_spec (plug_in->manager->gimp,
-                                    proc_install->params[i].type,
-                                    proc_install->params[i].name,
-                                    proc_install->params[i].description);
+        _gimp_gp_param_def_to_param_spec (&proc_install->params[i]);
 
-      gimp_procedure_add_argument (procedure, pspec);
+      if (pspec)
+        gimp_procedure_add_argument (procedure, pspec);
     }
 
-  for (i = 0; i < proc_install->nreturn_vals; i++)
+  for (i = 0; i < proc_install->n_return_vals; i++)
     {
       GParamSpec *pspec =
-        gimp_pdb_compat_param_spec (plug_in->manager->gimp,
-                                    proc_install->return_vals[i].type,
-                                    proc_install->return_vals[i].name,
-                                    proc_install->return_vals[i].description);
+        _gimp_gp_param_def_to_param_spec (&proc_install->return_vals[i]);
 
-      gimp_procedure_add_return_value (procedure, pspec);
-    }
-
-  /*  Sanity check menu path  */
-
-  if (proc_install->menu_path && strlen (proc_install->menu_path))
-    {
-      if (proc_install->menu_path[0] == '<')
-        {
-          GError *error = NULL;
-
-          if (! gimp_plug_in_procedure_add_menu_path (proc,
-                                                      proc_install->menu_path,
-                                                      &error))
-            {
-              gimp_message_literal (plug_in->manager->gimp,
-				    NULL, GIMP_MESSAGE_WARNING,
-				    error->message);
-              g_clear_error (&error);
-            }
-        }
-      else
-        {
-          proc->menu_label = g_strdup (proc_install->menu_path);
-        }
+      if (pspec)
+        gimp_procedure_add_return_value (procedure, pspec);
     }
 
   /*  Install the procedure  */
 
   switch (proc_install->type)
     {
-    case GIMP_PLUGIN:
-    case GIMP_EXTENSION:
+    case GIMP_PDB_PROC_TYPE_PLUGIN:
+    case GIMP_PDB_PROC_TYPE_PERSISTENT:
       gimp_plug_in_def_add_procedure (plug_in->plug_in_def, proc);
       break;
 
-    case GIMP_TEMPORARY:
+    case GIMP_PDB_PROC_TYPE_TEMPORARY:
       gimp_plug_in_add_temp_proc (plug_in, GIMP_TEMPORARY_PROCEDURE (proc));
       break;
     }
@@ -874,19 +842,27 @@ gimp_plug_in_handle_proc_uninstall (GimpPlugIn      *plug_in,
                                     GPProcUninstall *proc_uninstall)
 {
   GimpPlugInProcedure *proc;
-  gchar               *canonical;
 
   g_return_if_fail (proc_uninstall != NULL);
   g_return_if_fail (proc_uninstall->name != NULL);
 
-  canonical = gimp_canonicalize_identifier (proc_uninstall->name);
+  if (! gimp_is_canonical_identifier (proc_uninstall->name))
+    {
+      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                    "Plug-in \"%s\"\n(%s)\n\n"
+                    "attempted to uninstall procedure \"%s\" with a "
+                    "non-canonical name.",
+                    gimp_object_get_name (plug_in),
+                    gimp_file_get_utf8_name (plug_in->file),
+                    proc_uninstall->name);
+      return;
+    }
 
-  proc = gimp_plug_in_procedure_find (plug_in->temp_procedures, canonical);
+  proc = gimp_plug_in_procedure_find (plug_in->temp_procedures,
+                                      proc_uninstall->name);
 
   if (proc)
     gimp_plug_in_remove_temp_proc (plug_in, GIMP_TEMPORARY_PROCEDURE (proc));
-
-  g_free (canonical);
 }
 
 static void
@@ -899,11 +875,11 @@ gimp_plug_in_handle_extension_ack (GimpPlugIn *plug_in)
   else
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent an EXTENSION_ACK message while not being started "
                     "as an extension.  This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
     }
 }
@@ -918,11 +894,11 @@ gimp_plug_in_handle_has_init (GimpPlugIn *plug_in)
   else
     {
       gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-In \"%s\"\n(%s)\n\n"
+                    "Plug-in \"%s\"\n(%s)\n\n"
                     "sent an HAS_INIT message while not in query().  "
                     "This should not happen.",
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
       gimp_plug_in_close (plug_in, TRUE);
     }
 }
