@@ -263,9 +263,9 @@ file_proc_in_group (GimpPlugInProcedure    *file_proc,
 
   is_xcf_save = (strcmp (name, "gimp-xcf-save") == 0);
 
-  is_filter   = (strcmp (name, "file-gz-save")  == 0 ||
-                 strcmp (name, "file-bz2-save") == 0 ||
-                 strcmp (name, "file-xz-save")  == 0);
+  is_filter   = (strcmp (name, "file-gz-export")  == 0 ||
+                 strcmp (name, "file-bz2-export") == 0 ||
+                 strcmp (name, "file-xz-export")  == 0);
 
   switch (group)
     {
@@ -683,7 +683,7 @@ file_check_single_magic (const gchar  *offset,
           if (! info)
             return FILE_MATCH_NONE;
 
-          fileval = g_file_info_get_size (info);
+          fileval = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
           g_object_unref (info);
         }
       else if (offs >= 0 &&
