@@ -269,7 +269,7 @@ gimp_plug_in_manager_search_directory (GimpPlugInManager *manager,
         {
           GFile *child;
 
-          if (g_file_info_get_is_hidden (info))
+          if (g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN))
             {
               g_object_unref (info);
               continue;
@@ -305,7 +305,7 @@ gimp_plug_in_manager_search_directory (GimpPlugInManager *manager,
                       gchar *file_name;
                       char  *ext;
 
-                      if (g_file_info_get_is_hidden (info2))
+                      if (g_file_info_get_attribute_boolean (info2, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN))
                         {
                           g_object_unref (info2);
                           continue;
@@ -556,7 +556,7 @@ gimp_plug_in_manager_run_extensions (GimpPlugInManager  *manager,
       GimpPlugInProcedure *proc = list->data;
 
       if (proc->file                                                       &&
-          GIMP_PROCEDURE (proc)->proc_type == GIMP_PDB_PROC_TYPE_EXTENSION &&
+          GIMP_PROCEDURE (proc)->proc_type == GIMP_PDB_PROC_TYPE_PERSISTENT &&
           GIMP_PROCEDURE (proc)->num_args  == 0)
         {
           extensions = g_list_prepend (extensions, proc);
@@ -661,10 +661,11 @@ gimp_plug_in_manager_add_from_file (GimpPlugInManager *manager,
   basename = g_path_get_basename (filename);
   g_free (filename);
 
-  /* When we scan build dirs for plug-ins, there will be some
+  /* When we scan build dirs for plug-ins, there might be some
    * executable files that are not plug-ins that we want to ignore,
-   * for example plug-ins/common/mkgen.pl if
-   * GIMP_TESTING_PLUGINDIRS=plug-ins/common
+   * for example back with autotools, there used to be a file
+   * plug-ins/common/mkgen.pl if GIMP_TESTING_PLUGINDIRS=plug-ins/common
+   * Such files should be listed in GIMP_TESTING_PLUGINDIRS_BASENAME_IGNORES
    */
   if (gimp_plug_in_manager_ignore_plugin_basename (basename))
     {
