@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_PLUG_IN_PROC_FRAME_H__
@@ -31,7 +31,7 @@ struct _GimpPlugInProcFrame
   GimpProcedure       *procedure;
   GMainLoop           *main_loop;
 
-  GValueArray         *return_vals;
+  GimpValueArray      *return_vals;
 
   GimpProgress        *progress;
   gboolean             progress_created;
@@ -60,7 +60,7 @@ GimpPlugInProcFrame * gimp_plug_in_proc_frame_ref     (GimpPlugInProcFrame *proc
 void                  gimp_plug_in_proc_frame_unref   (GimpPlugInProcFrame *proc_frame,
                                                        GimpPlugIn          *plug_in);
 
-GValueArray         * gimp_plug_in_proc_frame_get_return_values
+GimpValueArray      * gimp_plug_in_proc_frame_get_return_values
                                                       (GimpPlugInProcFrame *proc_frame);
 
 
