@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __SANITY_H__
@@ -23,7 +23,8 @@
 #endif
 
 
-const gchar * sanity_check (void);
+const gchar * sanity_check_early (void);
+const gchar * sanity_check_late  (void);
 
 
 #endif /* __SANITY_H__ */
