@@ -232,7 +232,8 @@ gimp_text_tool_editor_start (GimpTextTool *text_tool)
     }
 
   gimp_text_tool_editor_position (text_tool);
-  gtk_widget_show (text_tool->style_overlay);
+  if (options->show_on_canvas)
+    gtk_widget_show (text_tool->style_overlay);
 }
 
 void
@@ -1337,6 +1338,8 @@ gimp_text_tool_options_notify (GimpTextOptions *options,
     {
       gtk_widget_set_visible (text_tool->style_editor,
                               options->show_on_canvas);
+      gtk_widget_set_visible (text_tool->style_overlay,
+                              options->show_on_canvas);
     }
 }
 
@@ -1399,21 +1402,14 @@ static void
 gimp_text_tool_enter_text (GimpTextTool *text_tool,
                            const gchar  *str)
 {
-  GtkTextBuffer *buffer      = GTK_TEXT_BUFFER (text_tool->buffer);
-  GList         *insert_tags = NULL;
-  GList         *remove_tags = NULL;
+  GtkTextBuffer *buffer;
   gboolean       had_selection;
 
+  buffer        = GTK_TEXT_BUFFER (text_tool->buffer);
   had_selection = gtk_text_buffer_get_has_selection (buffer);
 
   gtk_text_buffer_begin_user_action (buffer);
 
-  if (had_selection && text_tool->style_editor)
-    insert_tags = gimp_text_style_editor_list_tags (GIMP_TEXT_STYLE_EDITOR (text_tool->style_editor),
-                                                    &remove_tags);
-
-  gimp_text_tool_delete_selection (text_tool);
-
   if (! had_selection && text_tool->overwrite_mode && strcmp (str, "\n"))
     {
       GtkTextIter cursor;
@@ -1425,9 +1421,23 @@ gimp_text_tool_enter_text (GimpTextTool *text_tool,
         gimp_text_tool_delete_from_cursor (text_tool, GTK_DELETE_CHARS, 1);
     }
 
-  if (had_selection && text_tool->style_editor)
-    gimp_text_buffer_set_insert_tags (text_tool->buffer,
-                                      insert_tags, remove_tags);
+  if (had_selection)
+    {
+      GtkTextIter start, end;
+      GList      *insert_tags = NULL;
+      GList      *remove_tags = NULL;
+
+      gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
+      gtk_text_iter_order (&start, &end);
+
+      insert_tags = gimp_text_buffer_get_tags_on_iter (text_tool->buffer, &start);
+
+      gimp_text_tool_delete_selection (text_tool);
+
+      remove_tags = gimp_text_buffer_get_all_tags (text_tool->buffer);
+      gimp_text_buffer_set_insert_tags (text_tool->buffer,
+                                        insert_tags, remove_tags);
+    }
 
   gimp_text_buffer_insert (text_tool->buffer, str);
 
@@ -1587,24 +1597,21 @@ gimp_text_tool_im_preedit_changed (GtkIMContext *context,
                         case PANGO_ATTR_FOREGROUND:
                             {
                               PangoAttrColor *color_attr = (PangoAttrColor *) attr;
-                              GimpRGB         color;
+                              GeglColor      *color;
 
-                              color.r = (gdouble) color_attr->color.red / 65535.0;
-                              color.g = (gdouble) color_attr->color.green / 65535.0;
-                              color.b = (gdouble) color_attr->color.blue / 65535.0;
+                              color = gegl_color_new (NULL);
+                              gegl_color_set_pixel (color, babl_format ("R'G'B' u16"), &color_attr->color);
 
                               if (attr->klass->type == PANGO_ATTR_BACKGROUND)
-                                {
-                                  gimp_text_buffer_set_preedit_bg_color (text_tool->buffer,
-                                                                         &start, &end,
-                                                                         &color);
-                                }
+                                gimp_text_buffer_set_preedit_bg_color (text_tool->buffer,
+                                                                       &start, &end,
+                                                                       color);
                               else
-                                {
-                                  gimp_text_buffer_set_preedit_color (text_tool->buffer,
-                                                                      &start, &end,
-                                                                      &color);
-                                }
+                                gimp_text_buffer_set_preedit_color (text_tool->buffer,
+                                                                    &start, &end,
+                                                                    color);
+
+                              g_object_unref (color);
                             }
                           break;
                         default:
