@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -20,6 +20,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "tools-types.h"
@@ -43,6 +44,9 @@
 #include "gimp-intl.h"
 
 
+/* #define DEBUG_ACTIVE_STATE 1 */
+
+
 enum
 {
   PROP_0,
@@ -114,9 +118,9 @@ static void            gimp_tool_real_cursor_update  (GimpTool         *tool,
                                                       const GimpCoords *coords,
                                                       GdkModifierType   state,
                                                       GimpDisplay      *display);
-static const gchar   * gimp_tool_real_get_undo_desc  (GimpTool         *tool,
+static const gchar   * gimp_tool_real_can_undo       (GimpTool         *tool,
                                                       GimpDisplay      *display);
-static const gchar   * gimp_tool_real_get_redo_desc  (GimpTool         *tool,
+static const gchar   * gimp_tool_real_can_redo       (GimpTool         *tool,
                                                       GimpDisplay      *display);
 static gboolean        gimp_tool_real_undo           (GimpTool         *tool,
                                                       GimpDisplay      *display);
@@ -135,6 +139,7 @@ static void            gimp_tool_options_notify      (GimpToolOptions  *options,
                                                       const GParamSpec *pspec,
                                                       GimpTool         *tool);
 static void            gimp_tool_clear_status        (GimpTool         *tool);
+static void            gimp_tool_release             (GimpTool         *tool);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpTool, gimp_tool, GIMP_TYPE_OBJECT,
@@ -170,8 +175,8 @@ gimp_tool_class_init (GimpToolClass *klass)
   klass->active_modifier_key = gimp_tool_real_active_modifier_key;
   klass->oper_update         = gimp_tool_real_oper_update;
   klass->cursor_update       = gimp_tool_real_cursor_update;
-  klass->get_undo_desc       = gimp_tool_real_get_undo_desc;
-  klass->get_redo_desc       = gimp_tool_real_get_redo_desc;
+  klass->can_undo            = gimp_tool_real_can_undo;
+  klass->can_redo            = gimp_tool_real_can_redo;
   klass->undo                = gimp_tool_real_undo;
   klass->redo                = gimp_tool_real_redo;
   klass->get_popup           = gimp_tool_real_get_popup;
@@ -192,7 +197,7 @@ gimp_tool_init (GimpTool *tool)
   tool->ID                    = global_tool_ID++;
   tool->control               = g_object_new (GIMP_TYPE_TOOL_CONTROL, NULL);
   tool->display               = NULL;
-  tool->drawable              = NULL;
+  tool->drawables             = NULL;
   tool->focus_display         = NULL;
   tool->modifier_state        = 0;
   tool->active_modifier_state = 0;
@@ -206,7 +211,7 @@ gimp_tool_constructed (GObject *object)
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_assert (GIMP_IS_TOOL_INFO (tool->tool_info));
+  gimp_assert (GIMP_IS_TOOL_INFO (tool->tool_info));
 
   g_signal_connect_object (gimp_tool_get_options (tool), "notify",
                            G_CALLBACK (gimp_tool_options_notify),
@@ -228,17 +233,12 @@ gimp_tool_finalize (GObject *object)
 {
   GimpTool *tool = GIMP_TOOL (object);
 
-  if (tool->tool_info)
-    {
-      g_object_unref (tool->tool_info);
-      tool->tool_info = NULL;
-    }
-
-  if (tool->control)
-    {
-      g_object_unref (tool->control);
-      tool->control = NULL;
-    }
+  g_clear_object (&tool->tool_info);
+  g_clear_object (&tool->control);
+  g_clear_pointer (&tool->label,     g_free);
+  g_clear_pointer (&tool->undo_desc, g_free);
+  g_clear_pointer (&tool->icon_name, g_free);
+  g_clear_pointer (&tool->help_id,   g_free);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -331,7 +331,9 @@ gimp_tool_real_control (GimpTool       *tool,
       break;
 
     case GIMP_TOOL_ACTION_HALT:
-      tool->display = NULL;
+      tool->display   = NULL;
+      g_list_free (tool->drawables);
+      tool->drawables = NULL;
       break;
 
     case GIMP_TOOL_ACTION_COMMIT:
@@ -351,8 +353,9 @@ gimp_tool_real_button_press (GimpTool            *tool,
     {
       GimpImage *image = gimp_display_get_image (display);
 
-      tool->display  = display;
-      tool->drawable = gimp_image_get_active_drawable (image);
+      tool->display   = display;
+      g_list_free (tool->drawables);
+      tool->drawables = gimp_image_get_selected_drawables (image);
 
       gimp_tool_control_activate (tool->control);
     }
@@ -434,15 +437,15 @@ gimp_tool_real_cursor_update (GimpTool         *tool,
 }
 
 static const gchar *
-gimp_tool_real_get_undo_desc (GimpTool    *tool,
-                              GimpDisplay *display)
+gimp_tool_real_can_undo (GimpTool    *tool,
+                         GimpDisplay *display)
 {
   return NULL;
 }
 
 static const gchar *
-gimp_tool_real_get_redo_desc (GimpTool    *tool,
-                              GimpDisplay *display)
+gimp_tool_real_can_redo (GimpTool    *tool,
+                         GimpDisplay *display)
 {
   return NULL;
 }
@@ -492,6 +495,90 @@ gimp_tool_get_options (GimpTool *tool)
   return tool->tool_info->tool_options;
 }
 
+void
+gimp_tool_set_label (GimpTool    *tool,
+                     const gchar *label)
+{
+  g_return_if_fail (GIMP_IS_TOOL (tool));
+
+  g_free (tool->label);
+  tool->label = g_strdup (label);
+}
+
+const gchar *
+gimp_tool_get_label (GimpTool *tool)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL (tool), NULL);
+
+  if (tool->label)
+    return tool->label;
+
+  return tool->tool_info->label;
+}
+
+void
+gimp_tool_set_undo_desc (GimpTool    *tool,
+                         const gchar *undo_desc)
+{
+  g_return_if_fail (GIMP_IS_TOOL (tool));
+
+  g_free (tool->undo_desc);
+  tool->undo_desc = g_strdup (undo_desc);
+}
+
+const gchar *
+gimp_tool_get_undo_desc (GimpTool *tool)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL (tool), NULL);
+
+  if (tool->undo_desc)
+    return tool->undo_desc;
+
+  return tool->tool_info->label;
+}
+
+void
+gimp_tool_set_icon_name (GimpTool    *tool,
+                         const gchar *icon_name)
+{
+  g_return_if_fail (GIMP_IS_TOOL (tool));
+
+  g_free (tool->icon_name);
+  tool->icon_name = g_strdup (icon_name);
+}
+
+const gchar *
+gimp_tool_get_icon_name (GimpTool *tool)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL (tool), NULL);
+
+  if (tool->icon_name)
+    return tool->icon_name;
+
+  return gimp_viewable_get_icon_name (GIMP_VIEWABLE (tool->tool_info));
+}
+
+void
+gimp_tool_set_help_id (GimpTool    *tool,
+                       const gchar *help_id)
+{
+  g_return_if_fail (GIMP_IS_TOOL (tool));
+
+  g_free (tool->help_id);
+  tool->help_id = g_strdup (help_id);
+}
+
+const gchar *
+gimp_tool_get_help_id (GimpTool *tool)
+{
+  g_return_val_if_fail (GIMP_IS_TOOL (tool), NULL);
+
+  if (tool->help_id)
+    return tool->help_id;
+
+  return tool->tool_info->help_id;
+}
+
 gboolean
 gimp_tool_has_display (GimpTool    *tool,
                        GimpDisplay *display)
@@ -566,6 +653,8 @@ gimp_tool_control (GimpTool       *tool,
 {
   g_return_if_fail (GIMP_IS_TOOL (tool));
 
+  g_object_ref (tool);
+
   switch (action)
     {
     case GIMP_TOOL_ACTION_PAUSE:
@@ -590,7 +679,22 @@ gimp_tool_control (GimpTool       *tool,
         }
       break;
 
+    case GIMP_TOOL_ACTION_COMMIT:
+      gimp_tool_release (tool);
+
+      GIMP_TOOL_GET_CLASS (tool)->control (tool, action, display);
+
+      /*  always HALT after COMMIT here and not in each tool individually;
+       *  some tools interact with their subclasses (e.g. filter tool
+       *  and operation tool), and it's essential that COMMIT runs
+       *  through the entire class hierarchy before HALT
+       */
+      action = GIMP_TOOL_ACTION_HALT;
+
+      /* pass through */
     case GIMP_TOOL_ACTION_HALT:
+      gimp_tool_release (tool);
+
       GIMP_TOOL_GET_CLASS (tool)->control (tool, action, display);
 
       if (gimp_tool_control_is_active (tool->control))
@@ -598,11 +702,9 @@ gimp_tool_control (GimpTool       *tool,
 
       gimp_tool_clear_status (tool);
       break;
-
-    case GIMP_TOOL_ACTION_COMMIT:
-      GIMP_TOOL_GET_CLASS (tool)->control (tool, action, display);
-      break;
     }
+
+  g_object_unref (tool);
 }
 
 void
@@ -626,6 +728,10 @@ gimp_tool_button_press (GimpTool            *tool,
       tool->button_press_state    = state;
       tool->active_modifier_state = state;
 
+      tool->last_pointer_coords = *coords;
+      tool->last_pointer_time   = time - g_get_monotonic_time () / 1000;
+      tool->last_pointer_state  = state;
+
       if (gimp_tool_control_get_wants_click (tool->control))
         {
           tool->in_click_distance   = TRUE;
@@ -699,6 +805,8 @@ gimp_tool_button_release (GimpTool         *tool,
 
   g_object_ref (tool);
 
+  tool->last_pointer_state = 0;
+
   my_coords = *coords;
 
   if (state & GDK_BUTTON3_MASK)
@@ -734,7 +842,9 @@ gimp_tool_button_release (GimpTool         *tool,
 
   g_warn_if_fail (gimp_tool_control_is_active (tool->control) == FALSE);
 
-  if (tool->active_modifier_state != 0)
+  if (tool->active_modifier_state                            != 0 &&
+      gimp_tool_control_get_active_modifiers (tool->control) !=
+      GIMP_TOOL_ACTIVE_MODIFIERS_SAME)
     {
       gimp_tool_control_activate (tool->control);
 
@@ -743,7 +853,8 @@ gimp_tool_button_release (GimpTool         *tool,
       gimp_tool_control_halt (tool->control);
     }
 
-  tool->button_press_state = 0;
+  tool->button_press_state    = 0;
+  tool->active_modifier_state = 0;
 
   g_object_unref (tool);
 }
@@ -762,6 +873,10 @@ gimp_tool_motion (GimpTool         *tool,
 
   tool->got_motion_event = TRUE;
 
+  tool->last_pointer_coords = *coords;
+  tool->last_pointer_time   = time - g_get_monotonic_time () / 1000;
+  tool->last_pointer_state  = state;
+
   GIMP_TOOL_GET_CLASS (tool)->motion (tool, coords, time, state, display);
 }
 
@@ -922,7 +1037,8 @@ gimp_tool_set_active_modifier_state (GimpTool        *tool,
                                      GdkModifierType  state,
                                      GimpDisplay     *display)
 {
-  gboolean press;
+  GimpToolActiveModifiers active_modifiers;
+  gboolean                press;
 
   g_return_if_fail (GIMP_IS_TOOL (tool));
   g_return_if_fail (GIMP_IS_DISPLAY (display));
@@ -933,6 +1049,8 @@ gimp_tool_set_active_modifier_state (GimpTool        *tool,
 
   g_return_if_fail (display == tool->focus_display);
 
+  active_modifiers = gimp_tool_control_get_active_modifiers (tool->control);
+
   if (state_changed (tool->active_modifier_state, state, GDK_SHIFT_MASK,
                      &press))
     {
@@ -941,15 +1059,29 @@ gimp_tool_set_active_modifier_state (GimpTool        *tool,
                   press ? "pressed" : "released");
 #endif
 
-      if (! press && (tool->button_press_state & GDK_SHIFT_MASK))
+      switch (active_modifiers)
         {
-          tool->button_press_state &= ~GDK_SHIFT_MASK;
-        }
-      else
-        {
-          gimp_tool_active_modifier_key (tool, GDK_SHIFT_MASK,
-                                         press, state,
-                                         display);
+        case GIMP_TOOL_ACTIVE_MODIFIERS_OFF:
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SAME:
+          gimp_tool_modifier_key (tool, GDK_SHIFT_MASK,
+                                  press, state,
+                                  display);
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SEPARATE:
+          if (! press && (tool->button_press_state & GDK_SHIFT_MASK))
+            {
+              tool->button_press_state &= ~GDK_SHIFT_MASK;
+            }
+          else
+            {
+              gimp_tool_active_modifier_key (tool, GDK_SHIFT_MASK,
+                                             press, state,
+                                             display);
+            }
+          break;
         }
     }
 
@@ -961,15 +1093,29 @@ gimp_tool_set_active_modifier_state (GimpTool        *tool,
                   press ? "pressed" : "released");
 #endif
 
-      if (! press && (tool->button_press_state & GDK_CONTROL_MASK))
+      switch (active_modifiers)
         {
-          tool->button_press_state &= ~GDK_CONTROL_MASK;
-        }
-      else
-        {
-          gimp_tool_active_modifier_key (tool, GDK_CONTROL_MASK,
-                                         press, state,
-                                         display);
+        case GIMP_TOOL_ACTIVE_MODIFIERS_OFF:
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SAME:
+          gimp_tool_modifier_key (tool, GDK_CONTROL_MASK,
+                                  press, state,
+                                  display);
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SEPARATE:
+          if (! press && (tool->button_press_state & GDK_CONTROL_MASK))
+            {
+              tool->button_press_state &= ~GDK_CONTROL_MASK;
+            }
+          else
+            {
+              gimp_tool_active_modifier_key (tool, GDK_CONTROL_MASK,
+                                             press, state,
+                                             display);
+            }
+          break;
         }
     }
 
@@ -981,15 +1127,29 @@ gimp_tool_set_active_modifier_state (GimpTool        *tool,
                   press ? "pressed" : "released");
 #endif
 
-      if (! press && (tool->button_press_state & GDK_MOD1_MASK))
+      switch (active_modifiers)
         {
-          tool->button_press_state &= ~GDK_MOD1_MASK;
-        }
-      else
-        {
-          gimp_tool_active_modifier_key (tool, GDK_MOD1_MASK,
-                                         press, state,
-                                         display);
+        case GIMP_TOOL_ACTIVE_MODIFIERS_OFF:
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SAME:
+          gimp_tool_modifier_key (tool, GDK_MOD1_MASK,
+                                  press, state,
+                                  display);
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SEPARATE:
+          if (! press && (tool->button_press_state & GDK_MOD1_MASK))
+            {
+              tool->button_press_state &= ~GDK_MOD1_MASK;
+            }
+          else
+            {
+              gimp_tool_active_modifier_key (tool, GDK_MOD1_MASK,
+                                             press, state,
+                                             display);
+            }
+          break;
         }
     }
 
@@ -1001,19 +1161,36 @@ gimp_tool_set_active_modifier_state (GimpTool        *tool,
                   press ? "pressed" : "released");
 #endif
 
-      if (! press && (tool->button_press_state & GDK_MOD2_MASK))
+      switch (active_modifiers)
         {
-          tool->button_press_state &= ~GDK_MOD2_MASK;
-        }
-      else
-        {
-          gimp_tool_active_modifier_key (tool, GDK_MOD2_MASK,
-                                         press, state,
-                                         display);
+        case GIMP_TOOL_ACTIVE_MODIFIERS_OFF:
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SAME:
+          gimp_tool_modifier_key (tool, GDK_MOD2_MASK,
+                                  press, state,
+                                  display);
+          break;
+
+        case GIMP_TOOL_ACTIVE_MODIFIERS_SEPARATE:
+          if (! press && (tool->button_press_state & GDK_MOD2_MASK))
+            {
+              tool->button_press_state &= ~GDK_MOD2_MASK;
+            }
+          else
+            {
+              gimp_tool_active_modifier_key (tool, GDK_MOD2_MASK,
+                                             press, state,
+                                             display);
+            }
+          break;
         }
     }
 
   tool->active_modifier_state = state;
+
+  if (active_modifiers == GIMP_TOOL_ACTIVE_MODIFIERS_SAME)
+    tool->modifier_state = state;
 }
 
 void
@@ -1056,27 +1233,27 @@ gimp_tool_cursor_update (GimpTool         *tool,
 }
 
 const gchar *
-gimp_tool_get_undo_desc (GimpTool    *tool,
-                         GimpDisplay *display)
+gimp_tool_can_undo (GimpTool    *tool,
+                    GimpDisplay *display)
 {
   g_return_val_if_fail (GIMP_IS_TOOL (tool), NULL);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), NULL);
 
   if (display == tool->display)
-    return GIMP_TOOL_GET_CLASS (tool)->get_undo_desc (tool, display);
+    return GIMP_TOOL_GET_CLASS (tool)->can_undo (tool, display);
 
   return NULL;
 }
 
 const gchar *
-gimp_tool_get_redo_desc (GimpTool    *tool,
-                         GimpDisplay *display)
+gimp_tool_can_redo (GimpTool    *tool,
+                    GimpDisplay *display)
 {
   g_return_val_if_fail (GIMP_IS_TOOL (tool), NULL);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), NULL);
 
   if (display == tool->display)
-    return GIMP_TOOL_GET_CLASS (tool)->get_redo_desc (tool, display);
+    return GIMP_TOOL_GET_CLASS (tool)->can_redo (tool, display);
 
   return NULL;
 }
@@ -1088,7 +1265,7 @@ gimp_tool_undo (GimpTool    *tool,
   g_return_val_if_fail (GIMP_IS_TOOL (tool), FALSE);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), FALSE);
 
-  if (display == tool->display)
+  if (gimp_tool_can_undo (tool, display))
     return GIMP_TOOL_GET_CLASS (tool)->undo (tool, display);
 
   return FALSE;
@@ -1101,7 +1278,7 @@ gimp_tool_redo (GimpTool    *tool,
   g_return_val_if_fail (GIMP_IS_TOOL (tool), FALSE);
   g_return_val_if_fail (GIMP_IS_DISPLAY (display), FALSE);
 
-  if (display == tool->display)
+  if (gimp_tool_can_redo (tool, display))
     return GIMP_TOOL_GET_CLASS (tool)->redo (tool, display);
 
   return FALSE;
@@ -1123,6 +1300,26 @@ gimp_tool_get_popup (GimpTool         *tool,
                                                 ui_path);
 }
 
+/*
+ * gimp_tool_push_status:
+ * @tool:
+ * @display:
+ * @format:
+ *
+ * Push a new status message for the context of the @tool, in the shell
+ * associated to @display. Note that the message will replace any
+ * message for the same context (i.e. same tool) while also ensuring it
+ * is in the front of the message list (except for any progress or
+ * temporary messages which might be going on and are always front).
+ * This is the main difference with gimp_tool_replace_status() which
+ * does not necessarily try to make it a front message.
+ *
+ * In particular, it means you don't have to call gimp_tool_pop_status()
+ * first before calling gimp_tool_push_status(). Even more, you should
+ * not pop the status if you were planning to push a new status for the
+ * same context because you are triggering non necessary redraws of the
+ * status bar.
+ */
 void
 gimp_tool_push_status (GimpTool    *tool,
                        GimpDisplay *display,
@@ -1208,6 +1405,26 @@ gimp_tool_push_status_length (GimpTool            *tool,
   tool->status_displays = g_list_prepend (tool->status_displays, display);
 }
 
+/*
+ * gimp_tool_replace_status:
+ * @tool:
+ * @display:
+ * @format:
+ *
+ * Push a new status message for the context of the @tool, in the shell
+ * associated to @display. If any message for the same context (i.e.
+ * same tool) is already lined up, it will be replaced without changing
+ * the appearance order. In other words, it doesn't try to prioritize
+ * this new status message. Yet if no message for the same context
+ * existed already, the new status would end up front (except for any
+ * progress or temporary messages which might be going on and are always
+ * front).
+ * See also gimp_tool_push_status().
+ *
+ * Therefore you should not call gimp_tool_pop_status() first before
+ * calling gimp_tool_replace_status() as it would not be the same
+ * behavior and could trigger unnecessary redraws of the status bar.
+ */
 void
 gimp_tool_replace_status (GimpTool    *tool,
                           GimpDisplay *display,
@@ -1321,3 +1538,18 @@ gimp_tool_clear_status (GimpTool *tool)
   while (tool->status_displays)
     gimp_tool_pop_status (tool, tool->status_displays->data);
 }
+
+static void
+gimp_tool_release (GimpTool *tool)
+{
+  if (tool->last_pointer_state &&
+      gimp_tool_control_is_active (tool->control))
+    {
+      gimp_tool_button_release (
+        tool,
+        &tool->last_pointer_coords,
+        tool->last_pointer_time + g_get_monotonic_time () / 1000,
+        tool->last_pointer_state,
+        tool->display);
+    }
+}
