@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_TOOL_H__
@@ -40,12 +40,17 @@ struct _GimpTool
 
   GimpToolInfo    *tool_info;
 
+  gchar           *label;
+  gchar           *undo_desc;
+  gchar           *icon_name;
+  gchar           *help_id;
+
   gint             ID;          /*  unique tool ID                         */
 
   GimpToolControl *control;
 
   GimpDisplay     *display;     /*  pointer to currently active display    */
-  GimpDrawable    *drawable;    /*  pointer to the tool's current drawable */
+  GList           *drawables;   /*  list of the tool's current drawables   */
 
   /*  private state of gimp_tool_set_focus_display() and
    *  gimp_tool_set_[active_]modifier_state()
@@ -55,6 +60,12 @@ struct _GimpTool
   GdkModifierType  button_press_state;
   GdkModifierType  active_modifier_state;
 
+  /*  private state for synthesizing button_release() events
+   */
+  GimpCoords       last_pointer_coords;
+  guint32          last_pointer_time;
+  GdkModifierType  last_pointer_state;
+
   /*  private state for click detection
    */
   gboolean         in_click_distance;
@@ -69,6 +80,8 @@ struct _GimpTool
   /*  on-canvas progress  */
   GimpCanvasItem  *progress;
   GimpDisplay     *progress_display;
+  GtkWidget       *progress_grab_widget;
+  gboolean         progress_cancelable;
 };
 
 struct _GimpToolClass
@@ -134,6 +147,15 @@ struct _GimpToolClass
                                            GdkModifierType        state,
                                            GimpDisplay           *display);
 
+  const gchar   * (* can_undo)            (GimpTool              *tool,
+                                           GimpDisplay           *display);
+  const gchar   * (* can_redo)            (GimpTool              *tool,
+                                           GimpDisplay           *display);
+  gboolean        (* undo)                (GimpTool              *tool,
+                                           GimpDisplay           *display);
+  gboolean        (* redo)                (GimpTool              *tool,
+                                           GimpDisplay           *display);
+
   GimpUIManager * (* get_popup)           (GimpTool              *tool,
                                            const GimpCoords      *coords,
                                            GdkModifierType        state,
@@ -150,6 +172,22 @@ GType             gimp_tool_get_type            (void) G_GNUC_CONST;
 
 GimpToolOptions * gimp_tool_get_options         (GimpTool            *tool);
 
+void              gimp_tool_set_label           (GimpTool            *tool,
+                                                 const gchar         *label);
+const gchar     * gimp_tool_get_label           (GimpTool            *tool);
+
+void              gimp_tool_set_undo_desc       (GimpTool            *tool,
+                                                 const gchar         *undo_desc);
+const gchar     * gimp_tool_get_undo_desc       (GimpTool            *tool);
+
+void              gimp_tool_set_icon_name       (GimpTool            *tool,
+                                                 const gchar         *icon_name);
+const gchar     * gimp_tool_get_icon_name       (GimpTool            *tool);
+
+void              gimp_tool_set_help_id         (GimpTool            *tool,
+                                                 const gchar         *help_id);
+const gchar     * gimp_tool_get_help_id         (GimpTool            *tool);
+
 gboolean          gimp_tool_has_display         (GimpTool            *tool,
                                                  GimpDisplay         *display);
 GimpDisplay     * gimp_tool_has_image           (GimpTool            *tool,
@@ -204,6 +242,15 @@ void              gimp_tool_cursor_update       (GimpTool            *tool,
                                                  GdkModifierType      state,
                                                  GimpDisplay         *display);
 
+const gchar     * gimp_tool_can_undo            (GimpTool            *tool,
+                                                 GimpDisplay         *display);
+const gchar     * gimp_tool_can_redo            (GimpTool            *tool,
+                                                 GimpDisplay         *display);
+gboolean          gimp_tool_undo                (GimpTool            *tool,
+                                                 GimpDisplay         *display);
+gboolean          gimp_tool_redo                (GimpTool            *tool,
+                                                 GimpDisplay         *display);
+
 GimpUIManager   * gimp_tool_get_popup           (GimpTool            *tool,
                                                  const GimpCoords    *coords,
                                                  GdkModifierType      state,
