@@ -23,6 +23,7 @@
 #include <gtk/gtk.h>
 
 #include "libgimpconfig/gimpconfig.h"
+#include "libgimpwidgets/gimpwidgets.h"
 
 #include "tools-types.h"
 
@@ -184,6 +185,11 @@ tool_manager_exit (Gimp *gimp)
                                         tool_manager_image_changed,
                                         tool_manager);
 
+  if (tool_manager->image)
+    g_signal_handlers_disconnect_by_func (tool_manager->image,
+                                          tool_manager_selected_layers_changed,
+                                          tool_manager);
+
   gimp_container_remove_handler (gimp->images,
                                  tool_manager->image_clean_handler_id);
   gimp_container_remove_handler (gimp->images,
@@ -715,14 +721,14 @@ tool_manager_tool_changed (GimpContext     *user_context,
   /* FIXME: gimp_busy HACK */
   if (user_context->gimp->busy)
     {
-      /*  there may be contexts waiting for the user_context's "tool-changed"
-       *  signal, so stop emitting it.
-       */
-      g_signal_stop_emission_by_name (user_context, "tool-changed");
-
       if (G_TYPE_FROM_INSTANCE (tool_manager->active_tool) !=
           tool_info->tool_type)
         {
+          /*  there may be contexts waiting for the user_context's "tool-changed"
+           *  signal, so stop emitting it.
+           */
+          g_signal_stop_emission_by_name (user_context, "tool-changed");
+
           g_signal_handlers_block_by_func (user_context,
                                            tool_manager_tool_changed,
                                            tool_manager);
@@ -771,8 +777,11 @@ tool_manager_tool_changed (GimpContext     *user_context,
 
   g_object_unref (new_tool);
 
-  /* ??? */
-  tool_manager_cast_spell (tool_info);
+  if (gimp_widget_animation_enabled ())
+    {
+      /* ??? */
+      tool_manager_cast_spell (tool_info);
+    }
 }
 
 static void
