@@ -26,7 +26,8 @@
 GimpTempBuf * gimp_path_get_new_preview (GimpViewable *viewable,
                                          GimpContext  *context,
                                          gint          width,
-                                         gint          height);
+                                         gint          height,
+                                         GeglColor    *fg_color);
 
 
 #endif /* __GIMP_PATH_PREVIEW_H__ */
