@@ -29,6 +29,7 @@
 
 #include "widgets-types.h"
 
+#include "core/gimp.h"
 #include "core/gimpcontainer.h"
 #include "core/gimpcontext.h"
 #include "core/gimpviewable.h"
@@ -37,6 +38,7 @@
 #include "gimpcontainertreestore.h"
 #include "gimpcontainericonview.h"
 #include "gimpcontainerview.h"
+#include "gimpcontainerview-cruft.h"
 #include "gimpdnd.h"
 #include "gimpviewrenderer.h"
 #include "gimpwidgets-utils.h"
@@ -45,6 +47,8 @@
 struct _GimpContainerIconViewPrivate
 {
   GimpViewRenderer *dnd_renderer;
+
+  gulong            color_scheme_handler_id;
 };
 
 
@@ -62,6 +66,11 @@ static void          gimp_container_icon_view_set_context       (GimpContainerVi
                                                                  GimpContext                 *context);
 static void          gimp_container_icon_view_set_selection_mode(GimpContainerView           *view,
                                                                  GtkSelectionMode             mode);
+static void          gimp_container_icon_view_set_view_size     (GimpContainerView           *view);
+static gboolean      gimp_container_icon_view_set_selected      (GimpContainerView           *view,
+                                                                 GList                       *items);
+static gint          gimp_container_icon_view_get_selected        (GimpContainerView      *view,
+                                                                   GList                 **items);
 
 static gpointer      gimp_container_icon_view_insert_item       (GimpContainerView           *view,
                                                                  GimpViewable                *viewable,
@@ -77,11 +86,9 @@ static void          gimp_container_icon_view_reorder_item      (GimpContainerVi
 static void          gimp_container_icon_view_rename_item       (GimpContainerView           *view,
                                                                  GimpViewable                *viewable,
                                                                  gpointer                     insert_data);
-static gboolean      gimp_container_icon_view_select_items      (GimpContainerView           *view,
-                                                                 GList                       *items,
-                                                                 GList                       *paths);
 static void          gimp_container_icon_view_clear_items       (GimpContainerView           *view);
-static void          gimp_container_icon_view_set_view_size     (GimpContainerView           *view);
+
+static void          gimp_container_icon_view_invalidate        (GimpContainerIconView       *view);
 
 static void          gimp_container_icon_view_selection_changed (GtkIconView                 *view,
                                                                  GimpContainerIconView       *icon_view);
@@ -101,13 +108,12 @@ static gboolean      gimp_container_icon_view_tooltip           (GtkWidget
 static GimpViewable * gimp_container_icon_view_drag_viewable    (GtkWidget    *widget,
                                                                  GimpContext **context,
                                                                  gpointer      data);
-static GdkPixbuf    * gimp_container_icon_view_drag_pixbuf        (GtkWidget *widget,
-                                                                   gpointer   data);
-static gboolean      gimp_container_icon_view_get_selected_single (GimpContainerIconView  *icon_view,
-                                                                   GtkTreeIter            *iter);
-static gint          gimp_container_icon_view_get_selected        (GimpContainerView      *view,
-                                                                   GList                 **items,
-                                                                   GList                 **paths);
+static GdkPixbuf   * gimp_container_icon_view_drag_pixbuf       (GtkWidget *widget,
+                                                                 gpointer   data);
+static gboolean      gimp_container_icon_view_get_1_selected    (GimpContainerIconView  *icon_view,
+                                                                 GtkTreeIter            *iter);
+
+static void          gimp_container_icon_view_trigger_redraw    (GimpContainerIconView *view);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpContainerIconView, gimp_container_icon_view,
@@ -132,6 +138,8 @@ gimp_container_icon_view_class_init (GimpContainerIconViewClass *klass)
 
   widget_class->unmap       = gimp_container_icon_view_unmap;
   widget_class->popup_menu  = gimp_container_icon_view_popup_menu;
+
+  gtk_widget_class_set_css_name (widget_class, "GimpContainerIconView");
 }
 
 static void
@@ -145,14 +153,15 @@ gimp_container_icon_view_view_iface_init (GimpContainerViewInterface *iface)
   iface->set_container      = gimp_container_icon_view_set_container;
   iface->set_context        = gimp_container_icon_view_set_context;
   iface->set_selection_mode = gimp_container_icon_view_set_selection_mode;
+  iface->set_view_size      = gimp_container_icon_view_set_view_size;
+  iface->set_selected       = gimp_container_icon_view_set_selected;
+  iface->get_selected       = gimp_container_icon_view_get_selected;
+
   iface->insert_item        = gimp_container_icon_view_insert_item;
   iface->remove_item        = gimp_container_icon_view_remove_item;
   iface->reorder_item       = gimp_container_icon_view_reorder_item;
   iface->rename_item        = gimp_container_icon_view_rename_item;
-  iface->select_items       = gimp_container_icon_view_select_items;
   iface->clear_items        = gimp_container_icon_view_clear_items;
-  iface->set_view_size      = gimp_container_icon_view_set_view_size;
-  iface->get_selected       = gimp_container_icon_view_get_selected;
 
   iface->insert_data_free = (GDestroyNotify) gtk_tree_iter_free;
 }
@@ -164,6 +173,8 @@ gimp_container_icon_view_init (GimpContainerIconView *icon_view)
 
   icon_view->priv = gimp_container_icon_view_get_instance_private (icon_view);
 
+  icon_view->priv->color_scheme_handler_id = 0;
+
   gimp_container_tree_store_columns_init (icon_view->model_columns,
                                           &icon_view->n_model_columns);
 
@@ -249,7 +260,7 @@ gimp_container_icon_view_popup_menu (GtkWidget *widget)
   GtkTreePath           *path;
   GdkRectangle           rect;
 
-  if (!gimp_container_icon_view_get_selected_single (icon_view, &iter))
+  if (! gimp_container_icon_view_get_1_selected (icon_view, &iter))
     return FALSE;
 
   path = gtk_tree_model_get_path (icon_view->model, &iter);
@@ -286,12 +297,12 @@ gimp_container_icon_view_new (GimpContainer *container,
 
   gimp_container_view_set_view_size (view, view_size, 0 /* ignore border */);
 
-  if (container)
-    gimp_container_view_set_container (view, container);
-
   if (context)
     gimp_container_view_set_context (view, context);
 
+  if (container)
+    gimp_container_view_set_container (view, container);
+
   return GTK_WIDGET (icon_view);
 }
 
@@ -312,9 +323,9 @@ gimp_container_icon_view_set_container (GimpContainerView *view,
       if (! container)
         {
           if (gimp_dnd_viewable_source_remove (GTK_WIDGET (icon_view->view),
-                                               gimp_container_get_children_type (old_container)))
+                                               gimp_container_get_child_type (old_container)))
             {
-              if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_children_type (old_container)))->get_size)
+              if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_child_type (old_container)))->get_size)
                 gimp_dnd_pixbuf_source_remove (GTK_WIDGET (icon_view->view));
 
               gtk_drag_source_unset (GTK_WIDGET (icon_view->view));
@@ -329,15 +340,15 @@ gimp_container_icon_view_set_container (GimpContainerView *view,
     {
       if (gimp_dnd_drag_source_set_by_type (GTK_WIDGET (icon_view->view),
                                             GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
-                                            gimp_container_get_children_type (container),
+                                            gimp_container_get_child_type (container),
                                             GDK_ACTION_COPY))
         {
           gimp_dnd_viewable_source_add (GTK_WIDGET (icon_view->view),
-                                        gimp_container_get_children_type (container),
+                                        gimp_container_get_child_type (container),
                                         gimp_container_icon_view_drag_viewable,
                                         icon_view);
 
-          if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_children_type (container)))->get_size)
+          if (GIMP_VIEWABLE_CLASS (g_type_class_peek (gimp_container_get_child_type (container)))->get_size)
             gimp_dnd_pixbuf_source_add (GTK_WIDGET (icon_view->view),
                                         gimp_container_icon_view_drag_pixbuf,
                                         icon_view);
@@ -363,6 +374,16 @@ gimp_container_icon_view_set_context (GimpContainerView *view,
   if (icon_view->model)
     gimp_container_tree_store_set_context (GIMP_CONTAINER_TREE_STORE (icon_view->model),
                                            context);
+
+  if (context)
+    {
+      if (! icon_view->priv->color_scheme_handler_id)
+        icon_view->priv->color_scheme_handler_id =
+          g_signal_connect_object (context->gimp->config,
+                                   "notify::theme-color-scheme",
+                                   G_CALLBACK (gimp_container_icon_view_trigger_redraw),
+                                   icon_view, G_CONNECT_SWAPPED);
+    }
 }
 
 static void
@@ -376,6 +397,136 @@ gimp_container_icon_view_set_selection_mode (GimpContainerView *view,
   parent_view_iface->set_selection_mode (view, mode);
 }
 
+static void
+gimp_container_icon_view_set_view_size (GimpContainerView *view)
+{
+  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+
+  if (icon_view->model)
+    gimp_container_tree_store_set_view_size (GIMP_CONTAINER_TREE_STORE (icon_view->model));
+
+  if (icon_view->view)
+    {
+      gtk_icon_view_set_columns (icon_view->view, -1);
+      gtk_icon_view_set_item_width (icon_view->view, -1);
+
+      /* ugly workaround to force the icon view to invalidate all its
+       * cached icon sizes
+       */
+      gtk_icon_view_set_item_orientation (icon_view->view,
+                                          GTK_ORIENTATION_VERTICAL);
+      gtk_icon_view_set_item_orientation (icon_view->view,
+                                          GTK_ORIENTATION_HORIZONTAL);
+    }
+}
+
+static gboolean
+gimp_container_icon_view_set_selected (GimpContainerView *view,
+                                       GList             *items)
+{
+  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+  GList                 *list;
+
+  if (items)
+    {
+      GList *paths = NULL;
+
+      for (list = items; list; list = g_list_next (list))
+        {
+          GtkTreeIter *iter;
+          GtkTreePath *path;
+
+          iter = _gimp_container_view_lookup (GIMP_CONTAINER_VIEW (view),
+                                              list->data);
+          if (! iter)
+            /* This may happen when the GimpContainerIconView only
+             * shows a subpart of the whole icons. We don't select
+             * what is not shown.
+             */
+            continue;
+
+          path = gtk_tree_model_get_path (icon_view->model, iter);
+
+          paths = g_list_prepend (paths, path);
+        }
+
+      paths = g_list_reverse (paths);
+
+      g_signal_handlers_block_by_func (icon_view->view,
+                                       gimp_container_icon_view_selection_changed,
+                                       icon_view);
+
+      gtk_icon_view_unselect_all (icon_view->view);
+
+      for (list = paths; list; list = g_list_next (list))
+        {
+          gtk_icon_view_select_path (icon_view->view, list->data);
+        }
+
+      if (paths)
+        {
+          gtk_icon_view_set_cursor (icon_view->view, paths->data, NULL, FALSE);
+          gtk_icon_view_scroll_to_path (icon_view->view, paths->data,
+                                        FALSE, 0.0, 0.0);
+        }
+
+      g_signal_handlers_unblock_by_func (icon_view->view,
+                                         gimp_container_icon_view_selection_changed,
+                                         icon_view);
+
+      g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
+
+      _gimp_container_view_selection_changed (GIMP_CONTAINER_VIEW (icon_view));
+    }
+  else
+    {
+      gtk_icon_view_unselect_all (icon_view->view);
+    }
+
+  return TRUE;
+}
+
+static gint
+gimp_container_icon_view_get_selected (GimpContainerView  *view,
+                                       GList             **items)
+{
+  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+  GList                 *paths;
+  gint                   n_selected;
+
+  paths = gtk_icon_view_get_selected_items (icon_view->view);
+  n_selected = g_list_length (paths);
+
+  if (items)
+    {
+      GList *list;
+
+      for (list = paths; list; list = g_list_next (list))
+        {
+          GtkTreeIter       iter;
+          GimpViewRenderer *renderer;
+
+          gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), &iter,
+                                   (GtkTreePath *) list->data);
+
+          gtk_tree_model_get (icon_view->model, &iter,
+                              GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                              -1);
+
+          if (renderer->viewable)
+            *items = g_list_prepend (*items, renderer->viewable);
+
+          g_object_unref (renderer);
+        }
+
+      *items = g_list_reverse (*items);
+    }
+
+  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
+
+  return n_selected;
+}
+
 static gpointer
 gimp_container_icon_view_insert_item (GimpContainerView *view,
                                       GimpViewable      *viewable,
@@ -430,8 +581,8 @@ gimp_container_icon_view_reorder_item (GimpContainerView *view,
     {
       GtkTreeIter selected_iter;
 
-      selected = gimp_container_icon_view_get_selected_single (icon_view,
-                                                               &selected_iter);
+      selected = gimp_container_icon_view_get_1_selected (icon_view,
+                                                          &selected_iter);
 
       if (selected)
         {
@@ -454,7 +605,7 @@ gimp_container_icon_view_reorder_item (GimpContainerView *view,
                                           iter);
 
   if (selected)
-    gimp_container_view_select_item (view, viewable);
+    gimp_container_view_set_1_selected (view, viewable);
 }
 
 static void
@@ -470,75 +621,6 @@ gimp_container_icon_view_rename_item (GimpContainerView *view,
                                          iter);
 }
 
-static gboolean
-gimp_container_icon_view_select_items (GimpContainerView *view,
-                                       GList             *viewables,
-                                       GList             *paths)
-{
-  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
-  GList                 *list;
-
-  if (viewables)
-    {
-      gboolean free_paths = FALSE;
-
-      if (g_list_length (paths) != g_list_length (viewables))
-        {
-          free_paths = TRUE;
-          paths = NULL;
-          for (list = viewables; list; list = list->next)
-            {
-              GtkTreeIter *iter;
-              GtkTreePath *path;
-
-              iter = gimp_container_view_lookup (GIMP_CONTAINER_VIEW (view),
-                                                 list->data);
-              if (! iter)
-                /* This may happen when the GimpContainerIconView only
-                 * shows a subpart of the whole icons. We don't select
-                 * what is not shown.
-                 */
-                continue;
-
-              path = gtk_tree_model_get_path (icon_view->model, iter);
-
-              paths = g_list_prepend (paths, path);
-            }
-          paths = g_list_reverse (paths);
-        }
-
-      g_signal_handlers_block_by_func (icon_view->view,
-                                       gimp_container_icon_view_selection_changed,
-                                       icon_view);
-
-      gtk_icon_view_unselect_all (icon_view->view);
-
-      for (list = paths; list; list = list->next)
-        {
-          gtk_icon_view_select_path (icon_view->view, list->data);
-        }
-
-      if (list)
-        {
-          gtk_icon_view_set_cursor (icon_view->view, list->data, NULL, FALSE);
-          gtk_icon_view_scroll_to_path (icon_view->view, list->data, FALSE, 0.0, 0.0);
-        }
-
-      g_signal_handlers_unblock_by_func (icon_view->view,
-                                         gimp_container_icon_view_selection_changed,
-                                         icon_view);
-
-      if (free_paths)
-        g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-    }
-  else
-    {
-      gtk_icon_view_unselect_all (icon_view->view);
-    }
-
-  return TRUE;
-}
-
 static void
 gimp_container_icon_view_clear_items (GimpContainerView *view)
 {
@@ -550,25 +632,24 @@ gimp_container_icon_view_clear_items (GimpContainerView *view)
 }
 
 static void
-gimp_container_icon_view_set_view_size (GimpContainerView *view)
+gimp_container_icon_view_invalidate (GimpContainerIconView *view)
 {
-  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
+  GtkTreeIter iter;
+  gboolean    iter_valid;
 
-  if (icon_view->model)
-    gimp_container_tree_store_set_view_size (GIMP_CONTAINER_TREE_STORE (icon_view->model));
-
-  if (icon_view->view)
+  for (iter_valid = gtk_tree_model_get_iter_first (view->model, &iter);
+       iter_valid;
+       iter_valid = gtk_tree_model_iter_next (view->model, &iter))
     {
-      gtk_icon_view_set_columns (icon_view->view, -1);
-      gtk_icon_view_set_item_width (icon_view->view, -1);
+      GimpViewRenderer *renderer;
 
-      /* ugly workaround to force the icon view to invalidate all its
-       * cached icon sizes
-       */
-      gtk_icon_view_set_item_orientation (icon_view->view,
-                                          GTK_ORIENTATION_VERTICAL);
-      gtk_icon_view_set_item_orientation (icon_view->view,
-                                          GTK_ORIENTATION_HORIZONTAL);
+      gtk_tree_model_get (view->model, &iter,
+                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
+                          -1);
+
+      gimp_view_renderer_invalidate (renderer);
+
+      g_object_unref (renderer);
     }
 }
 
@@ -579,35 +660,7 @@ static void
 gimp_container_icon_view_selection_changed (GtkIconView           *gtk_icon_view,
                                             GimpContainerIconView *icon_view)
 {
-  GimpContainerView *view = GIMP_CONTAINER_VIEW (icon_view);
-  GList             *items = NULL;
-  GList             *paths;
-  GList             *list;
-
-  paths = gtk_icon_view_get_selected_items (icon_view->view);
-
-  for (list = paths; list; list = list->next)
-    {
-      GtkTreeIter       iter;
-      GimpViewRenderer *renderer;
-
-      gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), &iter,
-                               (GtkTreePath *) list->data);
-
-      gtk_tree_model_get (icon_view->model, &iter,
-                          GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
-                          -1);
-
-      if (renderer->viewable)
-        items = g_list_prepend (items, renderer->viewable);
-
-      g_object_unref (renderer);
-    }
-
-  gimp_container_view_multi_selected (view, items, paths);
-
-  g_list_free_full (paths, (GDestroyNotify) gtk_tree_path_free);
-  g_list_free (items);
+  _gimp_container_view_selection_changed (GIMP_CONTAINER_VIEW (icon_view));
 }
 
 static void
@@ -624,8 +677,8 @@ gimp_container_icon_view_item_activated (GtkIconView           *view,
                       GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
                       -1);
 
-  gimp_container_view_item_activated (GIMP_CONTAINER_VIEW (icon_view),
-                                      renderer->viewable);
+  _gimp_container_view_item_activated (GIMP_CONTAINER_VIEW (icon_view),
+                                       renderer->viewable);
 
   g_object_unref (renderer);
 }
@@ -663,11 +716,16 @@ gimp_container_icon_view_button_press (GtkWidget             *widget,
            * selection. Otherwise, we use the current selection. This
            * allows to not break multiple selection when right-clicking.
            */
-          if (! gimp_container_view_is_item_selected (container_view, renderer->viewable))
-            gimp_container_view_item_selected (container_view, renderer->viewable);
+          if (! gimp_container_view_is_item_selected (container_view,
+                                                      renderer->viewable))
+            {
+              gimp_container_view_set_1_selected (container_view,
+                                                  renderer->viewable);
+            }
 
           /* Show the context menu. */
-          handled = gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view), (GdkEvent *) bevent);
+          handled = gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view),
+                                                       (GdkEvent *) bevent);
         }
       /* Else LMB down or similar.  Propagate. */
 
@@ -680,7 +738,8 @@ gimp_container_icon_view_button_press (GtkWidget             *widget,
       /* Button down outside any item. */
       if (gdk_event_triggers_context_menu ((GdkEvent *) bevent))
         {
-          (void) gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view), (GdkEvent *) bevent);
+          (void) gimp_editor_popup_menu_at_pointer (GIMP_EDITOR (icon_view),
+                                                    (GdkEvent *) bevent);
           /* Discard result.  Does not actually popup menu except for managed views, e.g. dockable. */
         }
 
@@ -771,14 +830,14 @@ gimp_container_icon_view_drag_pixbuf (GtkWidget *widget,
   if (renderer && gimp_viewable_get_size (renderer->viewable, &width, &height))
     return gimp_viewable_get_new_pixbuf (renderer->viewable,
                                          renderer->context,
-                                         width, height);
+                                         width, height, NULL);
 
   return NULL;
 }
 
 static gboolean
-gimp_container_icon_view_get_selected_single (GimpContainerIconView  *icon_view,
-                                              GtkTreeIter            *iter)
+gimp_container_icon_view_get_1_selected (GimpContainerIconView *icon_view,
+                                         GtkTreeIter           *iter)
 {
   GList    *selected_items;
   gboolean  retval;
@@ -788,7 +847,7 @@ gimp_container_icon_view_get_selected_single (GimpContainerIconView  *icon_view,
   if (g_list_length (selected_items) == 1)
     {
       gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), iter,
-                               (GtkTreePath *) selected_items->data);
+                               selected_items->data);
 
       retval = TRUE;
     }
@@ -802,102 +861,9 @@ gimp_container_icon_view_get_selected_single (GimpContainerIconView  *icon_view,
   return retval;
 }
 
-static gint
-gimp_container_icon_view_get_selected (GimpContainerView    *view,
-                                       GList               **items,
-                                       GList               **paths)
+static void
+gimp_container_icon_view_trigger_redraw (GimpContainerIconView *view)
 {
-  GimpContainerIconView *icon_view = GIMP_CONTAINER_ICON_VIEW (view);
-  GList                 *selected_paths;
-  gint                   selected_count;
-  GimpContainer         *container;
-
-  container = gimp_container_view_get_container (view);
-
-  if (container)
-    {
-      const gchar *signal_name;
-      GimpContext *context;
-      GType        children_type;
-
-      context       = gimp_container_view_get_context (view);
-      children_type = gimp_container_get_children_type (container);
-      signal_name   = gimp_context_type_to_signal_name (children_type);
-
-      /* As a special case, for containers tied to a context object, we
-       * look up this object as being selected.
-       * */
-      if (signal_name && context)
-        {
-          GimpObject  *object;
-
-          object  = gimp_context_get_by_type (context, children_type);
-
-          selected_count = object ? 1 : 0;
-          if (items)
-            {
-              if (object)
-                *items = g_list_prepend (NULL, object);
-              else
-                *items = NULL;
-            }
-          if (paths)
-            *paths = NULL;
-
-          return selected_count;
-        }
-    }
-
-  selected_paths = gtk_icon_view_get_selected_items (icon_view->view);
-  selected_count = g_list_length (selected_paths);
-
-  if (items)
-    {
-      GList *removed_paths = NULL;
-      GList *list;
-
-      *items = NULL;
-
-      for (list = selected_paths;
-           list;
-           list = g_list_next (list))
-        {
-          GtkTreeIter       iter;
-          GimpViewRenderer *renderer;
-
-          gtk_tree_model_get_iter (GTK_TREE_MODEL (icon_view->model), &iter,
-                                   (GtkTreePath *) list->data);
-
-          gtk_tree_model_get (icon_view->model, &iter,
-                              GIMP_CONTAINER_TREE_STORE_COLUMN_RENDERER, &renderer,
-                              -1);
-
-          if (renderer->viewable)
-            *items = g_list_prepend (*items, renderer->viewable);
-          else
-            /* Remove from the selected_paths list but at the end, in order not
-             * to break the for loop.
-             */
-            removed_paths = g_list_prepend (removed_paths, list);
-
-          g_object_unref (renderer);
-        }
-      *items = g_list_reverse (*items);
-
-      for (list = removed_paths; list; list = list->next)
-        {
-          GList *remove_list = list->data;
-
-          selected_paths = g_list_remove_link (selected_paths, remove_list);
-          gtk_tree_path_free (remove_list->data);
-        }
-      g_list_free_full (removed_paths, (GDestroyNotify) g_list_free);
-    }
-
-  if (paths)
-    *paths = selected_paths;
-  else
-    g_list_free_full (selected_paths, (GDestroyNotify) gtk_tree_path_free);
-
-  return selected_count;
+  gimp_container_icon_view_invalidate (view);
+  gtk_widget_queue_draw (GTK_WIDGET (view));
 }
