@@ -41,10 +41,6 @@ struct _GimpContainerTreeViewPrivate
   GdkScrollDirection  scroll_dir;
 
   gboolean            dnd_drop_to_empty;
-
-  gdouble             zoom_accumulated_scroll_delta;
-  GtkGesture         *zoom_gesture;
-  gdouble             zoom_gesture_last_set_value;
 };
 
 
