@@ -36,6 +36,7 @@
 #include "core/gimpcontext.h"
 #include "core/gimpdatafactory.h"
 #include "core/gimpgradient.h"
+#include "core/gimpimage.h"
 #include "core/gimplist.h"
 #include "core/gimppattern.h"
 #include "core/gimptoolinfo.h"
@@ -106,6 +107,9 @@ static void gimp_device_status_notify_device   (GimpDeviceManager     *manager,
 static void gimp_device_status_config_notify   (GimpGuiConfig         *config,
                                                 const GParamSpec      *pspec,
                                                 GimpDeviceStatus      *status);
+static void gimp_device_status_image_changed   (GimpContext           *user_context,
+                                                GimpImage             *image,
+                                                GimpDeviceStatus      *status);
 static void gimp_device_status_notify_info     (GimpDeviceInfo        *device_info,
                                                 const GParamSpec      *pspec,
                                                 GimpDeviceStatusEntry *entry);
@@ -115,6 +119,10 @@ static void gimp_device_status_view_clicked    (GtkWidget             *widget,
                                                 GdkModifierType        state,
                                                 const gchar           *identifier);
 
+static void
+        gimp_device_status_set_color_help_data (GimpDeviceStatusEntry *entry,
+                                                GimpContext           *user_context);
+
 
 G_DEFINE_TYPE (GimpDeviceStatus, gimp_device_status, GIMP_TYPE_EDITOR)
 
@@ -124,7 +132,8 @@ G_DEFINE_TYPE (GimpDeviceStatus, gimp_device_status, GIMP_TYPE_EDITOR)
 static void
 gimp_device_status_class_init (GimpDeviceStatusClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->constructed  = gimp_device_status_constructed;
   object_class->dispose      = gimp_device_status_dispose;
@@ -135,6 +144,8 @@ gimp_device_status_class_init (GimpDeviceStatusClass *klass)
                                                         GIMP_TYPE_GIMP,
                                                         GIMP_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT_ONLY));
+
+  gtk_widget_class_set_css_name (widget_class, "GimpDeviceStatus");
 }
 
 static void
@@ -191,6 +202,10 @@ gimp_device_status_constructed (GObject *object)
 
   gimp_device_status_config_notify (GIMP_GUI_CONFIG (status->gimp->config),
                                     NULL, status);
+
+  g_signal_connect_object (gimp_get_user_context (status->gimp), "image-changed",
+                           G_CALLBACK (gimp_device_status_image_changed),
+                           status, 0);
 }
 
 static void
@@ -492,6 +507,21 @@ gimp_device_status_config_notify (GimpGuiConfig    *config,
     }
 }
 
+static void
+gimp_device_status_image_changed (GimpContext      *user_context,
+                                  GimpImage        *image,
+                                  GimpDeviceStatus *status)
+{
+  GList *list;
+
+  for (list = status->devices; list; list = list->next)
+    {
+      GimpDeviceStatusEntry *entry = list->data;
+
+      gimp_device_status_set_color_help_data (entry, user_context);
+    }
+}
+
 static void
 toggle_prop_visible (GtkWidget *widget,
                      GtkWidget *widget_none,
@@ -550,21 +580,8 @@ gimp_device_status_notify_info (GimpDeviceInfo        *device_info,
     }
 
   if (! strcmp (pspec->name, "tool-options"))
-    {
-      GimpRGB color;
-      guchar  r, g, b;
-      gchar   buf[64];
-
-      gimp_context_get_foreground (entry->context, &color);
-      gimp_rgb_get_uchar (&color, &r, &g, &b);
-      g_snprintf (buf, sizeof (buf), _("Foreground: %d, %d, %d"), r, g, b);
-      gimp_help_set_help_data (entry->foreground, buf, NULL);
-
-      gimp_context_get_background (entry->context, &color);
-      gimp_rgb_get_uchar (&color, &r, &g, &b);
-      g_snprintf (buf, sizeof (buf), _("Background: %d, %d, %d"), r, g, b);
-      gimp_help_set_help_data (entry->background, buf, NULL);
-    }
+    gimp_device_status_set_color_help_data (entry,
+                                            gimp_get_user_context (entry->context->gimp));
 }
 
 static void
@@ -592,3 +609,42 @@ gimp_device_status_view_clicked (GtkWidget       *widget,
                                              gimp_widget_get_monitor (widget),
                                              identifier);
 }
+
+static void
+gimp_device_status_set_color_help_data (GimpDeviceStatusEntry *entry,
+                                        GimpContext           *user_context)
+{
+  const Babl *format;
+  GimpImage  *image = NULL;
+  GeglColor  *color;
+  guchar      rgb[4];
+  gchar      *buf;
+
+  /* Note: we don't set the GeglColor on purpose. Whatever is the specific
+   * color space is too disconnected to this GUI usage and showing info in
+   * this space would only be confusing.
+   */
+  format = gimp_context_get_rgba_format (user_context, NULL, "u8", &image);
+
+  color = gimp_context_get_foreground (entry->context);
+  gegl_color_get_pixel (color, format, rgb);
+  if (image != NULL)
+    buf = g_strdup_printf (_("Foreground: %d, %d, %d (in color space of \"%s\")"),
+                           rgb[0], rgb[1], rgb[2],
+                           gimp_image_get_display_name (image));
+  else
+    buf = g_strdup_printf (_("Foreground: %d, %d, %d (in sRGB)"), rgb[0], rgb[1], rgb[2]);
+  gimp_help_set_help_data (entry->foreground, buf, NULL);
+  g_free (buf);
+
+  color = gimp_context_get_background (entry->context);
+  gegl_color_get_pixel (color, format, rgb);
+  if (image != NULL)
+    buf = g_strdup_printf (_("Background: %d, %d, %d (in color space of \"%s\")"),
+                           rgb[0], rgb[1], rgb[2],
+                           gimp_image_get_display_name (image));
+  else
+    buf = g_strdup_printf (_("Background: %d, %d, %d (in sRGB)"), rgb[0], rgb[1], rgb[2]);
+  gimp_help_set_help_data (entry->background, buf, NULL);
+  g_free (buf);
+}
