@@ -29,9 +29,11 @@ typedef struct _GimpMenuFactoryEntry GimpMenuFactoryEntry;
 
 struct _GimpMenuFactoryEntry
 {
-  gchar *identifier;
-  GList *action_groups;
-  GList *managed_uis;
+  gchar        *identifier;
+  GList        *action_groups;
+  GList        *managed_uis;
+
+  GHashTable   *managers;
 };
 
 
@@ -67,7 +69,10 @@ void              gimp_menu_factory_manager_register     (GimpMenuFactory   *fac
                                                           const gchar       *first_group,
                                                           ...)  G_GNUC_NULL_TERMINATED;
 GList           * gimp_menu_factory_get_registered_menus (GimpMenuFactory   *factory);
-GimpUIManager   * gimp_menu_factory_manager_new          (GimpMenuFactory   *factory,
+GimpUIManager   * gimp_menu_factory_get_manager          (GimpMenuFactory   *factory,
+                                                          const gchar       *identifier,
+                                                          gpointer           callback_data);
+void              gimp_menu_factory_delete_manager       (GimpMenuFactory   *factory,
                                                           const gchar       *identifier,
                                                           gpointer           callback_data);
 
