@@ -15,28 +15,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <stdlib.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <string.h>
-#include <sys/types.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <glib/gstdio.h>
-
-#include <glib.h>
-
-#ifdef G_OS_WIN32
-#include "libgimpbase/gimpwin32-io.h"
-#endif
 
 #include <gegl.h>
 #include <gtk/gtk.h>
@@ -49,11 +33,17 @@
 #include "gimptextbuffer.h"
 #include "gimptextbuffer-serialize.h"
 #include "gimptexttag.h"
-#include "gimpwidgets-utils.h"
 
 #include "gimp-intl.h"
 
 
+enum
+{
+  COLOR_APPLIED,
+  LAST_SIGNAL
+};
+
+
 /*  local function prototypes  */
 
 static void   gimp_text_buffer_constructed (GObject           *object);
@@ -69,6 +59,8 @@ G_DEFINE_TYPE (GimpTextBuffer, gimp_text_buffer, GTK_TYPE_TEXT_BUFFER)
 
 #define parent_class gimp_text_buffer_parent_class
 
+static guint buffer_signals[LAST_SIGNAL] = { 0, };
+
 
 static void
 gimp_text_buffer_class_init (GimpTextBufferClass *klass)
@@ -81,6 +73,15 @@ gimp_text_buffer_class_init (GimpTextBufferClass *klass)
   object_class->finalize    = gimp_text_buffer_finalize;
 
   buffer_class->mark_set    = gimp_text_buffer_mark_set;
+
+  buffer_signals[COLOR_APPLIED] =
+    g_signal_new ("color-applied",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpTextBufferClass, color_applied),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 1,
+                  GEGL_TYPE_COLOR);
 }
 
 static void
@@ -103,8 +104,7 @@ gimp_text_buffer_constructed (GObject *object)
 {
   GimpTextBuffer *buffer = GIMP_TEXT_BUFFER (object);
 
-  if (G_OBJECT_CLASS (parent_class)->constructed)
-    G_OBJECT_CLASS (parent_class)->constructed (object);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
   gtk_text_buffer_set_text (GTK_TEXT_BUFFER (buffer), "", -1);
 
@@ -123,6 +123,11 @@ gimp_text_buffer_constructed (GObject *object)
                                                       "underline", PANGO_UNDERLINE_SINGLE,
                                                       NULL);
 
+  buffer->preedit_underline_tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer),
+                                                              "preedit-underline",
+                                                              "underline", PANGO_UNDERLINE_SINGLE,
+                                                              NULL);
+
   buffer->strikethrough_tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer),
                                                           "strikethrough",
                                                           "strikethrough", TRUE,
@@ -172,6 +177,18 @@ gimp_text_buffer_finalize (GObject *object)
       buffer->color_tags = NULL;
     }
 
+  if (buffer->preedit_color_tags)
+    {
+      g_list_free (buffer->preedit_color_tags);
+      buffer->preedit_color_tags = NULL;
+    }
+
+  if (buffer->preedit_bg_color_tags)
+    {
+      g_list_free (buffer->preedit_bg_color_tags);
+      buffer->preedit_bg_color_tags = NULL;
+    }
+
   gimp_text_buffer_clear_insert_tags (buffer);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -357,7 +374,7 @@ gimp_text_buffer_get_iter_size (GimpTextBuffer    *buffer,
   return NULL;
 }
 
-static GtkTextTag *
+GtkTextTag *
 gimp_text_buffer_get_size_tag (GimpTextBuffer *buffer,
                                gint            size)
 {
@@ -837,13 +854,17 @@ gimp_text_buffer_get_iter_font (GimpTextBuffer     *buffer,
   return NULL;
 }
 
-static GtkTextTag *
+GtkTextTag *
 gimp_text_buffer_get_font_tag (GimpTextBuffer *buffer,
                                const gchar    *font)
 {
   GList      *list;
   GtkTextTag *tag;
   gchar       name[256];
+  PangoFontDescription *pfd = pango_font_description_from_string (font);
+  char *description = pango_font_description_to_string (pfd);
+
+  pango_font_description_free (pfd);
 
   for (list = buffer->font_tags; list; list = g_list_next (list))
     {
@@ -853,22 +874,24 @@ gimp_text_buffer_get_font_tag (GimpTextBuffer *buffer,
 
       tag_font = gimp_text_tag_get_font (tag);
 
-      if (! strcmp (font, tag_font))
+      if (! strcmp (description, tag_font))
         {
           g_free (tag_font);
+          g_free (description);
           return tag;
         }
 
       g_free (tag_font);
     }
 
-  g_snprintf (name, sizeof (name), "font-%s", font);
+  g_snprintf (name, sizeof (name), "font-%s", description);
 
   tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer),
                                     name,
-                                    "font", font,
+                                    "font", description,
                                     NULL);
-
+  gtk_text_tag_set_priority (tag, 0);
+  g_free (description);
   buffer->font_tags = g_list_prepend (buffer->font_tags, tag);
 
   return tag;
@@ -909,9 +932,9 @@ gimp_text_buffer_set_font (GimpTextBuffer    *buffer,
 }
 
 GtkTextTag *
-gimp_text_buffer_get_iter_color (GimpTextBuffer    *buffer,
-                                 const GtkTextIter *iter,
-                                 GimpRGB           *color)
+gimp_text_buffer_get_iter_color (GimpTextBuffer     *buffer,
+                                 const GtkTextIter  *iter,
+                                 GeglColor         **color)
 {
   GList *list;
 
@@ -922,7 +945,7 @@ gimp_text_buffer_get_iter_color (GimpTextBuffer    *buffer,
       if (gtk_text_iter_has_tag (iter, tag))
         {
           if (color)
-            gimp_text_tag_get_color (tag, color);
+            gimp_text_tag_get_fg_color (tag, color);
 
           return tag;
         }
@@ -931,47 +954,36 @@ gimp_text_buffer_get_iter_color (GimpTextBuffer    *buffer,
   return NULL;
 }
 
-static GtkTextTag *
+GtkTextTag *
 gimp_text_buffer_get_color_tag (GimpTextBuffer *buffer,
-                                const GimpRGB  *color)
+                                GeglColor      *color)
 {
   GList      *list;
   GtkTextTag *tag;
   gchar       name[256];
-  GdkColor    gdk_color;
-  guchar      r, g, b;
-
-  gimp_rgb_get_uchar (color, &r, &g, &b);
+  gdouble     rgba[4];
 
   for (list = buffer->color_tags; list; list = g_list_next (list))
     {
-      GimpRGB tag_color;
-      guchar  tag_r, tag_g, tag_b;
+      GeglColor *tag_color;
 
       tag = list->data;
 
-      gimp_text_tag_get_color (tag, &tag_color);
-
-      gimp_rgb_get_uchar (&tag_color, &tag_r, &tag_g, &tag_b);
+      gimp_text_tag_get_fg_color (tag, &tag_color);
 
       /* Do not compare the alpha channel, since it's unused */
-      if (tag_r == r &&
-          tag_g == g &&
-          tag_b == b)
-        {
-          return tag;
-        }
+      if (gimp_color_is_perceptually_identical (color, tag_color))
+        return tag;
     }
 
+  gegl_color_get_pixel (color, babl_format ("R'G'B'A double"), rgba);
   g_snprintf (name, sizeof (name), "color-#%02x%02x%02x",
-              r, g, b);
-
-  gimp_rgb_get_gdk_color (color, &gdk_color);
+              (guint) (rgba[0] * 255), (guint) (rgba[1] * 255), (guint) (rgba[2] * 255));
 
   tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer),
                                     name,
-                                    "foreground-gdk", &gdk_color,
-                                    "foreground-set", TRUE,
+                                    "foreground-rgba", (GdkRGBA *) rgba,
+                                    "foreground-set",  TRUE,
                                     NULL);
 
   buffer->color_tags = g_list_prepend (buffer->color_tags, tag);
@@ -983,7 +995,7 @@ void
 gimp_text_buffer_set_color (GimpTextBuffer    *buffer,
                             const GtkTextIter *start,
                             const GtkTextIter *end,
-                            const GimpRGB     *color)
+                            GeglColor         *color)
 {
   GList *list;
 
@@ -1006,6 +1018,151 @@ gimp_text_buffer_set_color (GimpTextBuffer    *buffer,
     {
       GtkTextTag *tag = gimp_text_buffer_get_color_tag (buffer, color);
 
+      gtk_text_buffer_apply_tag (GTK_TEXT_BUFFER (buffer), tag,
+                                 start, end);
+
+      g_signal_emit (buffer, buffer_signals[COLOR_APPLIED], 0, color);
+    }
+
+  gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
+}
+
+GtkTextTag *
+gimp_text_buffer_get_preedit_color_tag (GimpTextBuffer *buffer,
+                                        GeglColor      *color)
+{
+  GList      *list;
+  GtkTextTag *tag;
+  gchar       name[256];
+  gdouble     rgba[4];
+
+  for (list = buffer->preedit_color_tags; list; list = g_list_next (list))
+    {
+      GeglColor *tag_color;
+
+      tag = list->data;
+
+      gimp_text_tag_get_fg_color (tag, &tag_color);
+
+      /* Do not compare the alpha channel, since it's unused */
+      if (gimp_color_is_perceptually_identical (color, tag_color))
+        return tag;
+    }
+
+  /* We save the color as unbounded sRGB. Is it right? TODO */
+  gegl_color_get_pixel (color, babl_format ("R'G'B'A double"), rgba);
+  g_snprintf (name, sizeof (name), "preedit-color-#%02x%02x%02x",
+              (guint) (rgba[0] * 255), (guint) (rgba[1] * 255), (guint) (rgba[2] * 255));
+
+  tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer),
+                                    name,
+                                    "foreground-rgba", (GdkRGBA *) rgba,
+                                    "foreground-set",  TRUE,
+                                    NULL);
+
+  buffer->preedit_color_tags = g_list_prepend (buffer->preedit_color_tags, tag);
+
+  return tag;
+}
+
+void
+gimp_text_buffer_set_preedit_color (GimpTextBuffer    *buffer,
+                                    const GtkTextIter *start,
+                                    const GtkTextIter *end,
+                                    GeglColor         *color)
+{
+  GList *list;
+
+  g_return_if_fail (GIMP_IS_TEXT_BUFFER (buffer));
+  g_return_if_fail (start != NULL);
+  g_return_if_fail (end != NULL);
+
+  if (gtk_text_iter_equal (start, end))
+    return;
+
+  gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (buffer));
+
+  for (list = buffer->preedit_color_tags; list; list = g_list_next (list))
+    {
+      gtk_text_buffer_remove_tag (GTK_TEXT_BUFFER (buffer), list->data,
+                                  start, end);
+    }
+
+  if (color)
+    {
+      GtkTextTag *tag = gimp_text_buffer_get_preedit_color_tag (buffer, color);
+
+      gtk_text_buffer_apply_tag (GTK_TEXT_BUFFER (buffer), tag,
+                                 start, end);
+    }
+
+  gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
+}
+
+GtkTextTag *
+gimp_text_buffer_get_preedit_bg_color_tag (GimpTextBuffer *buffer,
+                                           GeglColor      *color)
+{
+  GList      *list;
+  GtkTextTag *tag;
+  gchar       name[256];
+  gdouble     rgba[4];
+
+  for (list = buffer->preedit_bg_color_tags; list; list = g_list_next (list))
+    {
+      GeglColor *tag_color;
+
+      tag = list->data;
+
+      gimp_text_tag_get_bg_color (tag, &tag_color);
+
+      /* Do not compare the alpha channel, since it's unused */
+      if (gimp_color_is_perceptually_identical (color, tag_color))
+        return tag;
+    }
+
+  gegl_color_get_pixel (color, babl_format ("R'G'B'A double"), rgba);
+  g_snprintf (name, sizeof (name), "bg-color-#%02x%02x%02x",
+              (guint) (rgba[0] * 255), (guint) (rgba[1] * 255), (guint) (rgba[2] * 255));
+
+  tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (buffer),
+                                    name,
+                                    "background-rgba", (GdkRGBA *) rgba,
+                                    "background-set",  TRUE,
+                                    NULL);
+
+  buffer->preedit_bg_color_tags = g_list_prepend (buffer->preedit_bg_color_tags, tag);
+
+  return tag;
+}
+
+void
+gimp_text_buffer_set_preedit_bg_color (GimpTextBuffer    *buffer,
+                                       const GtkTextIter *start,
+                                       const GtkTextIter *end,
+                                       GeglColor         *color)
+{
+  GList *list;
+
+  g_return_if_fail (GIMP_IS_TEXT_BUFFER (buffer));
+  g_return_if_fail (start != NULL);
+  g_return_if_fail (end != NULL);
+
+  if (gtk_text_iter_equal (start, end))
+    return;
+
+  gtk_text_buffer_begin_user_action (GTK_TEXT_BUFFER (buffer));
+
+  for (list = buffer->preedit_bg_color_tags; list; list = g_list_next (list))
+    {
+      gtk_text_buffer_remove_tag (GTK_TEXT_BUFFER (buffer), list->data,
+                                  start, end);
+    }
+
+  if (color)
+    {
+      GtkTextTag *tag = gimp_text_buffer_get_preedit_bg_color_tag (buffer, color);
+
       gtk_text_buffer_apply_tag (GTK_TEXT_BUFFER (buffer), tag,
                                  start, end);
     }
@@ -1015,11 +1172,15 @@ gimp_text_buffer_set_color (GimpTextBuffer    *buffer,
 
 /*  Pango markup attribute names  */
 
-#define GIMP_TEXT_ATTR_NAME_SIZE     "size"
-#define GIMP_TEXT_ATTR_NAME_BASELINE "rise"
-#define GIMP_TEXT_ATTR_NAME_KERNING  "letter_spacing"
-#define GIMP_TEXT_ATTR_NAME_FONT     "font"
-#define GIMP_TEXT_ATTR_NAME_COLOR    "foreground"
+#define GIMP_TEXT_ATTR_NAME_SIZE      "size"
+#define GIMP_TEXT_ATTR_NAME_BASELINE  "rise"
+#define GIMP_TEXT_ATTR_NAME_KERNING   "letter_spacing"
+#define GIMP_TEXT_ATTR_NAME_FONT      "font"
+#define GIMP_TEXT_ATTR_NAME_STYLE     "style"
+#define GIMP_TEXT_ATTR_NAME_COLOR     "foreground"
+#define GIMP_TEXT_ATTR_NAME_FG_COLOR  "fgcolor"
+#define GIMP_TEXT_ATTR_NAME_BG_COLOR  "background"
+#define GIMP_TEXT_ATTR_NAME_UNDERLINE "underline"
 
 const gchar *
 gimp_text_buffer_tag_to_name (GimpTextBuffer  *buffer,
@@ -1099,17 +1260,66 @@ gimp_text_buffer_tag_to_name (GimpTextBuffer  *buffer,
 
       if (value)
         {
-          GimpRGB color;
-          guchar  r, g, b;
+          GeglColor *color;
+          guchar     rgb[3];
 
-          gimp_text_tag_get_color (tag, &color);
-          gimp_rgb_get_uchar (&color, &r, &g, &b);
-
-          *value = g_strdup_printf ("#%02x%02x%02x", r, g, b);
+          gimp_text_tag_get_fg_color (tag, &color);
+          gegl_color_get_pixel (color, babl_format ("R'G'B' u8"), rgb);
+          *value = g_strdup_printf ("#%02x%02x%02x", rgb[0], rgb[1], rgb[2]);
+          g_object_unref (color);
         }
 
       return "span";
     }
+  else if (g_list_find (buffer->preedit_color_tags, tag))
+    {
+      /* "foreground" and "fgcolor" attributes are similar, but I use
+       * one or the other as a trick to differentiate the color chosen
+       * from the user and a display color for preedit. */
+      if (attribute)
+        *attribute = GIMP_TEXT_ATTR_NAME_FG_COLOR;
+
+      if (value)
+        {
+          GeglColor *color;
+          guchar     rgb[3];
+
+          gimp_text_tag_get_fg_color (tag, &color);
+          gegl_color_get_pixel (color, babl_format ("R'G'B' u8"), rgb);
+          *value = g_strdup_printf ("#%02x%02x%02x", rgb[0], rgb[1], rgb[2]);
+          g_object_unref (color);
+        }
+
+      return "span";
+    }
+  else if (g_list_find (buffer->preedit_bg_color_tags, tag))
+    {
+      if (attribute)
+        *attribute = GIMP_TEXT_ATTR_NAME_BG_COLOR;
+
+      if (value)
+        {
+          GeglColor *color;
+          guchar     rgb[3];
+
+          gimp_text_tag_get_bg_color (tag, &color);
+          gegl_color_get_pixel (color, babl_format ("R'G'B' u8"), rgb);
+          *value = g_strdup_printf ("#%02x%02x%02x", rgb[0], rgb[1], rgb[2]);
+          g_object_unref (color);
+        }
+
+      return "span";
+    }
+  else if (tag == buffer->preedit_underline_tag)
+    {
+      if (attribute)
+        *attribute = GIMP_TEXT_ATTR_NAME_UNDERLINE;
+
+      if (value)
+        *value = g_strdup ("single");
+
+      return "span";
+    }
 
   return NULL;
 }
@@ -1161,14 +1371,23 @@ gimp_text_buffer_name_to_tag (GimpTextBuffer *buffer,
         }
       else if (! strcmp (attribute, GIMP_TEXT_ATTR_NAME_COLOR))
         {
-          GimpRGB color;
-          guint   r, g, b;
+          GtkTextTag *tag;
+          GeglColor  *color;
+          guint       r, g, b;
+          guchar      rgb[3];
 
           sscanf (value, "#%02x%02x%02x", &r, &g, &b);
+          rgb[0] = r;
+          rgb[1] = g;
+          rgb[2] = b;
 
-          gimp_rgb_set_uchar (&color, r, g, b);
+          color = gegl_color_new (NULL);
+          gegl_color_set_pixel (color, babl_format ("R'G'B' u8"), rgb);
 
-          return gimp_text_buffer_get_color_tag (buffer, &color);
+          tag = gimp_text_buffer_get_color_tag (buffer, color);
+          g_object_unref (color);
+
+          return tag;
         }
     }
 
@@ -1213,6 +1432,7 @@ gimp_text_buffer_insert (GimpTextBuffer *buffer,
   GList       *insert_tags;
   GList       *remove_tags;
   GSList      *tags_off = NULL;
+  GeglColor   *color;
 
   g_return_if_fail (GIMP_IS_TEXT_BUFFER (buffer));
 
@@ -1281,6 +1501,12 @@ gimp_text_buffer_insert (GimpTextBuffer *buffer,
   g_list_free (remove_tags);
   g_list_free (insert_tags);
 
+  if (gimp_text_buffer_get_iter_color (buffer, &start, &color))
+    {
+      g_signal_emit (buffer, buffer_signals[COLOR_APPLIED], 0, color);
+      g_object_unref (color);
+    }
+
   gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
 }
 
@@ -1401,25 +1627,29 @@ gimp_text_buffer_get_iter_at_index (GimpTextBuffer *buffer,
 
 gboolean
 gimp_text_buffer_load (GimpTextBuffer *buffer,
-                       const gchar    *filename,
+                       GFile          *file,
                        GError        **error)
 {
-  FILE        *file;
-  gchar        buf[2048];
-  gint         remaining = 0;
-  GtkTextIter  iter;
+  GInputStream *input;
+  gchar         buf[2048];
+  gint          to_read;
+  gsize         bytes_read;
+  gsize         total_read = 0;
+  gint          remaining  = 0;
+  GtkTextIter   iter;
+  GError       *my_error = NULL;
 
   g_return_val_if_fail (GIMP_IS_TEXT_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
+  g_return_val_if_fail (G_IS_FILE (file), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  file = g_fopen (filename, "r");
-
-  if (! file)
+  input = G_INPUT_STREAM (g_file_read (file, NULL, &my_error));
+  if (! input)
     {
-      g_set_error_literal (error, G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
-                           g_strerror (errno));
+      g_set_error (error, my_error->domain, my_error->code,
+                   _("Could not open '%s' for reading: %s"),
+                   gimp_file_get_utf8_name (file), my_error->message);
+      g_clear_error (&my_error);
       return FALSE;
     }
 
@@ -1428,63 +1658,80 @@ gimp_text_buffer_load (GimpTextBuffer *buffer,
   gimp_text_buffer_set_text (buffer, NULL);
   gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (buffer), &iter);
 
-  while (! feof (file))
+  do
     {
+      gboolean    success;
       const char *leftover;
-      gint        count;
-      gint        to_read = sizeof (buf) - remaining - 1;
 
-      count = fread (buf + remaining, 1, to_read, file);
-      buf[count + remaining] = '\0';
+      to_read = sizeof (buf) - remaining - 1;
 
-      g_utf8_validate (buf, count + remaining, &leftover);
+      success = g_input_stream_read_all (input, buf + remaining, to_read,
+                                         &bytes_read, NULL, &my_error);
+
+      total_read += bytes_read;
+      buf[bytes_read + remaining] = '\0';
+
+      g_utf8_validate (buf, bytes_read + remaining, &leftover);
 
       gtk_text_buffer_insert (GTK_TEXT_BUFFER (buffer), &iter,
                               buf, leftover - buf);
       gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (buffer), &iter);
 
-      remaining = (buf + remaining + count) - leftover;
-      g_memmove (buf, leftover, remaining);
+      remaining = (buf + remaining + bytes_read) - leftover;
+      memmove (buf, leftover, remaining);
 
-      if (remaining > 6 || count < to_read)
-        break;
+      if (! success)
+        {
+          if (total_read > 0)
+            {
+              g_message (_("Input file '%s' appears truncated: %s"),
+                         gimp_file_get_utf8_name (file),
+                         my_error->message);
+              g_clear_error (&my_error);
+              break;
+            }
+
+          gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
+          g_object_unref (input);
+
+          g_propagate_error (error, my_error);
+
+          return FALSE;
+        }
     }
+  while (remaining <= 6 && bytes_read == to_read);
 
   if (remaining)
     g_message (_("Invalid UTF-8 data in file '%s'."),
-               gimp_filename_to_utf8 (filename));
-
-  fclose (file);
+               gimp_file_get_utf8_name (file));
 
   gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (buffer));
+  g_object_unref (input);
 
   return TRUE;
 }
 
 gboolean
 gimp_text_buffer_save (GimpTextBuffer *buffer,
-                       const gchar    *filename,
+                       GFile          *file,
                        gboolean        selection_only,
                        GError        **error)
 {
-  GtkTextIter  start_iter;
-  GtkTextIter  end_iter;
-  gint         fd;
-  gchar       *text_contents;
+  GOutputStream *output;
+  GtkTextIter    start_iter;
+  GtkTextIter    end_iter;
+  gchar         *text_contents;
+  GError        *my_error = NULL;
 
   g_return_val_if_fail (GIMP_IS_TEXT_BUFFER (buffer), FALSE);
-  g_return_val_if_fail (filename != NULL, FALSE);
+  g_return_val_if_fail (G_IS_FILE (file), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  fd = g_open (filename, O_WRONLY | O_CREAT | O_APPEND, 0666);
-
-  if (fd == -1)
-    {
-      g_set_error_literal (error, G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
-                           g_strerror (errno));
-      return FALSE;
-    }
+  output = G_OUTPUT_STREAM (g_file_replace (file,
+                                            NULL, FALSE, G_FILE_CREATE_NONE,
+                                            NULL, error));
+  if (! output)
+    return FALSE;
 
   if (selection_only)
     gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (buffer),
@@ -1500,27 +1747,96 @@ gimp_text_buffer_save (GimpTextBuffer *buffer,
     {
       gint text_length = strlen (text_contents);
 
-      if (text_length > 0)
+      if (! g_output_stream_write_all (output, text_contents, text_length,
+                                       NULL, NULL, &my_error))
         {
-          gint bytes_written;
+          GCancellable *cancellable = g_cancellable_new ();
 
-          bytes_written = write (fd, text_contents, text_length);
+          g_set_error (error, my_error->domain, my_error->code,
+                       _("Writing text file '%s' failed: %s"),
+                       gimp_file_get_utf8_name (file), my_error->message);
+          g_clear_error (&my_error);
+          g_free (text_contents);
 
-          if (bytes_written != text_length)
-            {
-              g_free (text_contents);
-              close (fd);
-              g_set_error_literal (error, G_FILE_ERROR,
-                                   g_file_error_from_errno (errno),
-                                   g_strerror (errno));
-              return FALSE;
-            }
+          /* Cancel the overwrite initiated by g_file_replace(). */
+          g_cancellable_cancel (cancellable);
+          g_output_stream_close (output, cancellable, NULL);
+          g_object_unref (cancellable);
+          g_object_unref (output);
+
+          return FALSE;
         }
 
       g_free (text_contents);
     }
 
-  close (fd);
+  g_object_unref (output);
 
   return TRUE;
 }
+
+/**
+ * gimp_text_buffer_get_tags_on_iter:
+ * @buffer: a #GimpTextBuffer
+ * @iter: a position in @buffer
+ *
+ * Returns a list of tags that apply to @iter, in ascending order
+ * of priority (highest-priority tags are last). The GtkTextTag in
+ * the list don’t have a reference added, but you have to free the
+ * list itself.
+ *
+ * Returns: (element-type GtkTextTag) (transfer container): GList of #GtkTextTag
+ */
+GList *
+gimp_text_buffer_get_tags_on_iter (GimpTextBuffer    *buffer,
+                                   const GtkTextIter *iter)
+{
+  GList  *result    = NULL;
+  GSList *tag       = NULL;
+  GSList *tags_list = NULL;
+
+  g_return_val_if_fail (GIMP_IS_TEXT_BUFFER (buffer), NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+  g_return_val_if_fail (gtk_text_iter_get_buffer (iter) == GTK_TEXT_BUFFER (buffer), NULL);
+
+  tags_list = gtk_text_iter_get_tags (iter);
+  for (tag = tags_list; tag != NULL; tag = g_slist_next (tag))
+    {
+      result = g_list_prepend (result, tag->data);
+    }
+  g_slist_free (tags_list);
+
+  result = g_list_reverse (result);
+  return result;
+}
+
+/**
+ * gimp_text_buffer_get_all_tags:
+ * @buffer: a #GimpTextBuffer
+ *
+ * Returns a list of all tags for a @buffer, The GtkTextTag
+ * in the list don’t have a reference added, but you have to
+ * free the list itself.
+ *
+ * Returns: (element-type GtkTextTag) (transfer container): GList of #GtkTextTag
+ */
+GList *
+gimp_text_buffer_get_all_tags (GimpTextBuffer *buffer)
+{
+  GList *result = NULL;
+
+  g_return_val_if_fail (GIMP_IS_TEXT_BUFFER (buffer), NULL);
+
+  result = g_list_prepend (result, buffer->bold_tag);
+  result = g_list_prepend (result, buffer->italic_tag);
+  result = g_list_prepend (result, buffer->underline_tag);
+  result = g_list_prepend (result, buffer->strikethrough_tag);
+
+  result = g_list_concat (result, g_list_copy (buffer->size_tags));
+  result = g_list_concat (result, g_list_copy (buffer->baseline_tags));
+  result = g_list_concat (result, g_list_copy (buffer->kerning_tags));
+  result = g_list_concat (result, g_list_copy (buffer->font_tags));
+  result = g_list_concat (result, g_list_copy (buffer->color_tags));
+
+  return result;
+}
