@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -23,6 +23,7 @@
 #include <gegl.h>
 #include <gtk/gtk.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
@@ -105,6 +106,7 @@ static void        gimp_tool_options_editor_presets_update    (GimpToolOptionsEd
 
 G_DEFINE_TYPE_WITH_CODE (GimpToolOptionsEditor, gimp_tool_options_editor,
                          GIMP_TYPE_EDITOR,
+                         G_ADD_PRIVATE (GimpToolOptionsEditor)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_DOCKED,
                                                 gimp_tool_options_editor_docked_iface_init))
 
@@ -127,8 +129,6 @@ gimp_tool_options_editor_class_init (GimpToolOptionsEditorClass *klass)
                                                         GIMP_TYPE_GIMP,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
-
-  g_type_class_add_private (klass, sizeof (GimpToolOptionsEditorPrivate));
 }
 
 static void
@@ -143,11 +143,8 @@ static void
 gimp_tool_options_editor_init (GimpToolOptionsEditor *editor)
 {
   GtkScrolledWindow *scrolled_window;
-  GtkWidget         *viewport;
 
-  editor->p = G_TYPE_INSTANCE_GET_PRIVATE (editor,
-                                           GIMP_TYPE_TOOL_OPTIONS_EDITOR,
-                                           GimpToolOptionsEditorPrivate);
+  editor->p = gimp_tool_options_editor_get_instance_private (editor);
 
   gtk_widget_set_size_request (GTK_WIDGET (editor), -1, 200);
 
@@ -172,20 +169,15 @@ gimp_tool_options_editor_init (GimpToolOptionsEditor *editor)
   gtk_scrolled_window_set_policy (scrolled_window,
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_overlay_scrolling (scrolled_window, FALSE);
 
   gtk_box_pack_start (GTK_BOX (editor), editor->p->scrolled_window,
                       TRUE, TRUE, 0);
   gtk_widget_show (editor->p->scrolled_window);
 
-  viewport = gtk_viewport_new (gtk_scrolled_window_get_hadjustment (scrolled_window),
-                               gtk_scrolled_window_get_vadjustment (scrolled_window));
-  gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
-  gtk_container_add (GTK_CONTAINER (scrolled_window), viewport);
-  gtk_widget_show (viewport);
-
   /*  The vbox containing the tool options  */
   editor->p->options_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-  gtk_container_add (GTK_CONTAINER (viewport), editor->p->options_vbox);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), editor->p->options_vbox);
   gtk_widget_show (editor->p->options_vbox);
 }
 
@@ -204,7 +196,7 @@ gimp_tool_options_editor_constructed (GObject *object)
                             GIMP_HELP_TOOL_OPTIONS_SAVE,
                             G_CALLBACK (gimp_tool_options_editor_save_clicked),
                             NULL,
-                            editor);
+                            G_OBJECT (editor));
 
   editor->p->restore_button =
     gimp_editor_add_button (GIMP_EDITOR (editor),
@@ -213,7 +205,7 @@ gimp_tool_options_editor_constructed (GObject *object)
                             GIMP_HELP_TOOL_OPTIONS_RESTORE,
                             G_CALLBACK (gimp_tool_options_editor_restore_clicked),
                             NULL,
-                            editor);
+                            G_OBJECT (editor));
 
   editor->p->delete_button =
     gimp_editor_add_button (GIMP_EDITOR (editor),
@@ -222,7 +214,7 @@ gimp_tool_options_editor_constructed (GObject *object)
                             GIMP_HELP_TOOL_OPTIONS_DELETE,
                             G_CALLBACK (gimp_tool_options_editor_delete_clicked),
                             NULL,
-                            editor);
+                            G_OBJECT (editor));
 
   editor->p->reset_button =
     gimp_editor_add_action_button (GIMP_EDITOR (editor), "tool-options",
@@ -314,13 +306,11 @@ gimp_tool_options_editor_get_preview (GimpDocked   *docked,
                                       GimpContext  *context,
                                       GtkIconSize   size)
 {
-  GtkSettings *settings = gtk_widget_get_settings (GTK_WIDGET (docked));
-  GtkWidget   *view;
-  gint         width;
-  gint         height;
-
-  gtk_icon_size_lookup_for_settings (settings, size, &width, &height);
+  GtkWidget *view;
+  gint       width;
+  gint       height;
 
+  gtk_icon_size_lookup (size, &width, &height);
   view = gimp_prop_view_new (G_OBJECT (context), "tool", context, height);
   GIMP_VIEW (view)->renderer->size = -1;
   gimp_view_renderer_set_size_full (GIMP_VIEW (view)->renderer,
@@ -340,7 +330,7 @@ gimp_tool_options_editor_get_title (GimpDocked *docked)
 
   tool_info = gimp_context_get_tool (context);
 
-  return tool_info ? g_strdup (tool_info->blurb) : NULL;
+  return tool_info ? g_strdup (tool_info->label) : NULL;
 }
 
 static gboolean
@@ -380,15 +370,6 @@ gimp_tool_options_editor_get_tool_options (GimpToolOptionsEditor *editor)
 
 /*  private functions  */
 
-static void
-gimp_tool_options_editor_menu_pos (GtkMenu  *menu,
-                                   gint     *x,
-                                   gint     *y,
-                                   gpointer  data)
-{
-  gimp_button_menu_position (GTK_WIDGET (data), menu, GTK_POS_RIGHT, x, y);
-}
-
 static void
 gimp_tool_options_editor_menu_popup (GimpToolOptionsEditor *editor,
                                      GtkWidget             *button,
@@ -396,15 +377,16 @@ gimp_tool_options_editor_menu_popup (GimpToolOptionsEditor *editor,
 {
   GimpEditor *gimp_editor = GIMP_EDITOR (editor);
 
-  gtk_ui_manager_get_widget (GTK_UI_MANAGER (gimp_editor_get_ui_manager (gimp_editor)),
-                             gimp_editor_get_ui_path (gimp_editor));
   gimp_ui_manager_update (gimp_editor_get_ui_manager (gimp_editor),
                           gimp_editor_get_popup_data (gimp_editor));
 
-  gimp_ui_manager_ui_popup (gimp_editor_get_ui_manager (gimp_editor), path,
-                            button,
-                            gimp_tool_options_editor_menu_pos, button,
-                            NULL, NULL);
+  gimp_ui_manager_ui_popup_at_widget (gimp_editor_get_ui_manager (gimp_editor),
+                                      path, NULL, NULL,
+                                      button,
+                                      GDK_GRAVITY_WEST,
+                                      GDK_GRAVITY_NORTH_EAST,
+                                      NULL,
+                                      NULL, NULL);
 }
 
 static void
@@ -414,7 +396,7 @@ gimp_tool_options_editor_save_clicked (GtkWidget             *widget,
   if (gtk_widget_get_sensitive (editor->p->restore_button) /* evil but correct */)
     {
       gimp_tool_options_editor_menu_popup (editor, widget,
-                                           "/tool-options-popup/Save");
+                                           "/tool-options-popup/Tool Options Menu/Save Tool Preset");
     }
   else
     {
@@ -429,7 +411,7 @@ gimp_tool_options_editor_restore_clicked (GtkWidget             *widget,
                                           GimpToolOptionsEditor *editor)
 {
   gimp_tool_options_editor_menu_popup (editor, widget,
-                                       "/tool-options-popup/Restore");
+                                       "/tool-options-popup/Tool Options Menu/Restore Tool Preset");
 }
 
 static void
@@ -437,7 +419,7 @@ gimp_tool_options_editor_delete_clicked (GtkWidget             *widget,
                                          GimpToolOptionsEditor *editor)
 {
   gimp_tool_options_editor_menu_popup (editor, widget,
-                                       "/tool-options-popup/Delete");
+                                       "/tool-options-popup/Tool Options Menu/Delete Tool Preset");
 }
 
 static void
