@@ -17,11 +17,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpwidgets/gimpwidgets.h"
@@ -32,6 +33,7 @@
 
 #include "gimpviewrenderer.h"
 #include "gimpviewrenderer-frame.h"
+#include "gimpwidgets-utils.h"
 
 
 /* utility to stretch a frame to the desired size */
@@ -196,9 +198,7 @@ gimp_view_renderer_ensure_frame (GimpViewRenderer *renderer,
 
   if (! class->frame)
     {
-      class->frame = gtk_widget_render_icon (widget,
-                                             GIMP_STOCK_FRAME,
-                                             GTK_ICON_SIZE_DIALOG, NULL);
+      class->frame = gimp_widget_load_icon (widget, GIMP_ICON_FRAME, 48);
 
       /*  FIXME: shouldn't be hardcoded  */
       class->frame_left   = 2;
@@ -234,7 +234,7 @@ gimp_view_renderer_get_frame_pixbuf (GimpViewRenderer *renderer,
     {
       pixbuf = gimp_viewable_get_pixbuf (renderer->viewable,
                                          renderer->context,
-                                         w, h);
+                                         w, h, NULL);
       if (!pixbuf)
         return NULL;
 
@@ -251,7 +251,8 @@ gimp_view_renderer_get_frame_pixbuf (GimpViewRenderer *renderer,
     {
       pixbuf = gimp_viewable_get_pixbuf (renderer->viewable,
                                          renderer->context,
-                                         width - 2, height - 2);
+                                         width - 2, height - 2,
+                                         NULL);
       if (!pixbuf)
         return NULL;
 
