@@ -12,15 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <stdio.h>
-#include <errno.h>
-
-#include <glib-object.h>
+#include <gio/gio.h>
 
 #include "core/core-types.h"
 
@@ -29,49 +26,27 @@
 
 #include "gimp-intl.h"
 
+
 gboolean
 xcf_seek_pos (XcfInfo  *info,
-              guint     pos,
+              goffset   pos,
               GError  **error)
 {
   if (info->cp != pos)
     {
-      info->cp = pos;
-      if (fseek (info->fp, info->cp, SEEK_SET) == -1)
-        {
-          g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-                       _("Could not seek in XCF file: %s"),
-                       g_strerror (errno));
+      GError *my_error = NULL;
 
+      info->cp = pos;
+
+      if (! g_seekable_seek (info->seekable, info->cp, G_SEEK_SET,
+                             NULL, &my_error))
+        {
+          g_propagate_prefixed_error (error, my_error,
+                                      _("Could not seek in XCF file: "));
           return FALSE;
         }
-    }
 
-  return TRUE;
-}
-
-gboolean
-xcf_seek_end (XcfInfo  *info,
-              GError  **error)
-{
-  if (fseek (info->fp, 0, SEEK_END) == -1)
-    {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-                   _("Could not seek in XCF file: %s"),
-                   g_strerror (errno));
-
-      return FALSE;
-    }
-
-  info->cp = ftell (info->fp);
-
-  if (fseek (info->fp, 0, SEEK_END) == -1)
-    {
-      g_set_error (error, G_FILE_ERROR, g_file_error_from_errno (errno),
-                   _("Could not seek in XCF file: %s"),
-                   g_strerror (errno));
-
-      return FALSE;
+      gimp_assert (info->cp == g_seekable_tell (info->seekable));
     }
 
   return TRUE;
