@@ -80,6 +80,10 @@
 #  undef RGB
 #endif
 
+#ifdef GDK_WINDOWING_QUARTZ
+#include <Cocoa/Cocoa.h>
+#endif
+
 #include <locale.h>
 
 #include "gimp.h"
@@ -226,10 +230,34 @@ gimp_main (GType  plug_in_type,
   /* Use Dr. Mingw (dumps backtrace on crash) if it is available. */
   {
     time_t   t;
+#ifdef ENABLE_RELOCATABLE_RESOURCES
+    gchar   *plugin_dir;
+    size_t   codeview_path_len;
+    gchar   *codeview_path;
+#endif
     gchar   *filename;
     gchar   *dir;
     wchar_t *plug_in_backtrace_path_utf16;
 
+#ifdef ENABLE_RELOCATABLE_RESOURCES
+    /* FIXME: https://github.com/jrfonseca/drmingw/issues/91
+     * FIXME: This needs to take into account the plugin path
+    plugin_dir = ???
+    codeview_path_len = strlen (g_getenv ("_NT_SYMBOL_PATH") ? g_getenv ("_NT_SYMBOL_PATH") : "") + strlen (plugin_dir) + 2;
+    codeview_path = g_try_malloc (codeview_path_len);
+    if (codeview_path == NULL)
+      {
+        g_warning ("Failed to allocate memory");
+      }
+    if (g_getenv ("_NT_SYMBOL_PATH"))
+      g_snprintf (codeview_path, codeview_path_len, "%s;%s", plugin_dir, g_getenv ("_NT_SYMBOL_PATH"));
+    else
+      g_snprintf (codeview_path, codeview_path_len, "%s", plugin_dir);
+    g_setenv ("_NT_SYMBOL_PATH", codeview_path, TRUE);
+    g_free (codeview_path);
+    g_free (plugin_dir);*/
+#endif
+
     /* This has to be the non-roaming directory (i.e., the local
      * directory) as backtraces correspond to the binaries on this
      * system.
@@ -270,8 +298,8 @@ gimp_main (GType  plug_in_type,
     typedef BOOL (WINAPI *t_SetProcessDEPPolicy) (DWORD dwFlags);
     t_SetProcessDEPPolicy p_SetProcessDEPPolicy;
 
-    p_SetProcessDEPPolicy = GetProcAddress (GetModuleHandleW (L"kernel32.dll"),
-                                            "SetProcessDEPPolicy");
+    p_SetProcessDEPPolicy = (t_SetProcessDEPPolicy) GetProcAddress (GetModuleHandleW (L"kernel32.dll"),
+                                                                    "SetProcessDEPPolicy");
     if (p_SetProcessDEPPolicy)
       (*p_SetProcessDEPPolicy) (PROCESS_DEP_ENABLE|PROCESS_DEP_DISABLE_ATL_THUNK_EMULATION);
   }
@@ -329,6 +357,14 @@ gimp_main (GType  plug_in_type,
 
 #endif /* G_OS_WIN32 */
 
+#ifdef GDK_WINDOWING_QUARTZ
+  /* Sets activation policy to prevent plugins from appearing as separate apps
+   * in Dock.
+   * Makes plugins behave as helper processes of GIMP on macOS.
+   */
+  [NSApp setActivationPolicy:NSApplicationActivationPolicyAccessory];
+#endif
+
   g_assert (plug_in_type != G_TYPE_NONE);
 
   if ((argc != N_ARGS) || (strcmp (argv[ARG_GIMP], "-gimp") != 0))
@@ -1124,9 +1160,12 @@ _gimp_config (GPConfig *config)
   format = babl_format_with_space (config->check_custom_encoding1, space);
   if (bpp != babl_format_get_bytes_per_pixel (format))
     {
-      g_warning ("%s: checker board color 1's format expects %d bpp but %" G_GSIZE_FORMAT " bytes were passed.",
+      g_warning ("%s: checker board color 1's format expects %d bpp but %"
+                 G_GSIZE_FORMAT " bytes were passed.",
                  G_STRFUNC, babl_format_get_bytes_per_pixel (format), bpp);
-      gegl_color_set_pixel (_check_custom_color1, babl_format ("R'G'B'A double"), GIMP_CHECKS_CUSTOM_COLOR1);
+      gegl_color_set_pixel (_check_custom_color1,
+                            babl_format ("R'G'B'A double"),
+                            GIMP_CHECKS_CUSTOM_COLOR1);
     }
   else
     {
@@ -1143,9 +1182,12 @@ _gimp_config (GPConfig *config)
   format = babl_format_with_space (config->check_custom_encoding2, space);
   if (bpp != babl_format_get_bytes_per_pixel (format))
     {
-      g_warning ("%s: checker board color 2's format expects %d bpp but %" G_GSIZE_FORMAT " bytes were passed.",
+      g_warning ("%s: checker board color 2's format expects %d bpp but %"
+                 G_GSIZE_FORMAT " bytes were passed.",
                  G_STRFUNC, babl_format_get_bytes_per_pixel (format), bpp);
-      gegl_color_set_pixel (_check_custom_color2, babl_format ("R'G'B'A double"), GIMP_CHECKS_CUSTOM_COLOR2);
+      gegl_color_set_pixel (_check_custom_color2,
+                            babl_format ("R'G'B'A double"),
+                            GIMP_CHECKS_CUSTOM_COLOR2);
     }
   else
     {
