@@ -32,6 +32,10 @@
 #include <Cocoa/Cocoa.h>
 #endif
 
+#ifdef GDK_WINDOWING_WAYLAND
+#include <gdk/gdkwayland.h>
+#endif
+
 #include "gimp.h"
 #include "gimpui.h"
 
@@ -55,25 +59,26 @@
 
 /*  local function prototypes  */
 
-static void      gimp_ui_help_func              (const gchar       *help_id,
-                                                 gpointer           help_data);
-static void      gimp_ui_theme_changed          (GFileMonitor      *monitor,
-                                                 GFile             *file,
-                                                 GFile             *other_file,
-                                                 GFileMonitorEvent  event_type,
-                                                 GtkCssProvider    *css_provider);
-static void      gimp_ensure_modules            (void);
-#ifndef GDK_WINDOWING_WIN32
-static void      gimp_window_transient_realized (GtkWidget         *window,
-                                                 GdkWindow         *parent);
-#endif
-static gboolean  gimp_window_set_transient_for  (GtkWindow         *window,
-                                                 GdkWindow         *parent);
+static void        gimp_ui_help_func               (const gchar       *help_id,
+                                                    gpointer           help_data);
+static void        gimp_ui_theme_changed           (GFileMonitor      *monitor,
+                                                    GFile             *file,
+                                                    GFile             *other_file,
+                                                    GFileMonitorEvent  event_type,
+                                                    GtkCssProvider    *css_provider);
+static void        gimp_ensure_modules             (void);
 
 #ifdef GDK_WINDOWING_QUARTZ
-static gboolean  gimp_osx_focus_window          (gpointer);
+static gboolean    gimp_osx_focus_window           (gpointer);
 #endif
 
+#ifndef GDK_WINDOWING_WIN32
+static GdkWindow * gimp_ui_get_foreign_window      (gpointer           window);
+#endif
+static gboolean    gimp_window_transient_on_mapped (GtkWidget         *window,
+                                                    GdkEventAny       *event,
+                                                    GBytes            *handle);
+
 
 static gboolean gimp_ui_initialized = FALSE;
 
@@ -87,12 +92,16 @@ static gboolean gimp_ui_initialized = FALSE;
  *             executable and _not_ the PDB procedure name.
  *
  * This function initializes GTK+ with gtk_init().
+ * It also initializes Gegl and Babl.
  *
  * It also sets up various other things so that the plug-in user looks
  * and behaves like the GIMP core. This includes selecting the GTK+
  * theme and setting up the help system as chosen in the GIMP
  * preferences. Any plug-in that provides a user interface should call
  * this function.
+ *
+ * It can safely be called more than once.
+ * Calls after the first return quickly with no effect.
  **/
 void
 gimp_ui_init (const gchar *prog_name)
@@ -120,6 +129,14 @@ gimp_ui_init (const gchar *prog_name)
 #endif
     }
 
+#ifdef GDK_WINDOWING_QUARTZ
+  /* Sets activation policy to prevent plugins from appearing as separate apps
+   * in Dock.
+   * Makes plugins behave as helper processes of GIMP on macOS.
+   */
+  [NSApp setActivationPolicy:NSApplicationActivationPolicyAccessory];
+#endif
+
   if (gimp_user_time ())
     {
       /* Construct a fake startup ID as we only want to pass the
@@ -171,87 +188,12 @@ gimp_ui_init (const gchar *prog_name)
   g_idle_add (gimp_osx_focus_window, NULL);
 #endif
 
+  /* Some widgets use GEGL buffers for thumbnails, previews, etc. */
+  gegl_init (NULL, NULL);
+
   gimp_ui_initialized = TRUE;
 }
 
-static GdkWindow *
-gimp_ui_get_foreign_window (guint32 window)
-{
-#ifdef GDK_WINDOWING_X11
-  if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
-    return gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
-                                                   window);
-#endif
-
-#ifdef GDK_WINDOWING_WIN32
-  return gdk_win32_window_foreign_new_for_display (gdk_display_get_default (),
-                                                   (HWND) (uintptr_t) window);
-#endif
-
-  return NULL;
-}
-
-/**
- * gimp_ui_get_display_window:
- * @display: a #GimpDisplay.
- *
- * Returns the #GdkWindow of a display window. The purpose is to allow
- * to make plug-in dialogs transient to the image display as explained
- * with gdk_window_set_transient_for().
- *
- * You shouldn't have to call this function directly. Use
- * gimp_window_set_transient_for_display() instead.
- *
- * Returns: (nullable) (transfer full): A reference to a #GdkWindow or %NULL.
- *               You should unref the window using g_object_unref() as
- *               soon as you don't need it any longer.
- *
- * Since: 2.4
- */
-GdkWindow *
-gimp_ui_get_display_window (GimpDisplay *display)
-{
-  guint32 window;
-
-  g_return_val_if_fail (gimp_ui_initialized, NULL);
-
-  window = gimp_display_get_window_handle (display);
-  if (window)
-    return gimp_ui_get_foreign_window (window);
-
-  return NULL;
-}
-
-/**
- * gimp_ui_get_progress_window:
- *
- * Returns the #GdkWindow of the window this plug-in's progress bar is
- * shown in. Use it to make plug-in dialogs transient to this window
- * as explained with gdk_window_set_transient_for().
- *
- * You shouldn't have to call this function directly. Use
- * gimp_window_set_transient() instead.
- *
- * Returns: (transfer full): A reference to a #GdkWindow or %NULL.
- *               You should unref the window using g_object_unref() as
- *               soon as you don't need it any longer.
- *
- * Since: 2.4
- */
-GdkWindow *
-gimp_ui_get_progress_window (void)
-{
-  guint32  window;
-
-  g_return_val_if_fail (gimp_ui_initialized, NULL);
-
-  window = gimp_progress_get_window_handle ();
-  if (window)
-     return gimp_ui_get_foreign_window (window);
-
-  return NULL;
-}
-
 #ifdef GDK_WINDOWING_QUARTZ
 static void
 gimp_window_transient_show (GtkWidget *window)
@@ -263,17 +205,60 @@ gimp_window_transient_show (GtkWidget *window)
 }
 #endif
 
+/**
+ * gimp_window_set_transient_for:
+ * @window: the #GtkWindow that should become transient
+ * @handle: handle of the window that should become the parent
+ *
+ * Indicates to the window manager that @window is a transient dialog
+ * to the window identified by @handle.
+ *
+ * Note that @handle is an opaque data, which you should not try to
+ * construct yourself or make sense of. It may be different things
+ * depending on the OS or even the display server. You should only use
+ * a handle returned by [func@Gimp.progress_get_window_handle],
+ * [method@Gimp.Display.get_window_handle] or
+ * [method@GimpUi.Dialog.get_native_handle].
+ *
+ * Most of the time you will want to use the convenience function
+ * [func@GimpUi.window_set_transient].
+ *
+ * Since: 3.0
+ */
+void
+gimp_window_set_transient_for (GtkWindow *window,
+                               GBytes    *handle)
+{
+  g_return_if_fail (gimp_ui_initialized);
+  g_return_if_fail (GTK_IS_WINDOW (window));
+  g_return_if_fail (handle != NULL);
+
+  g_signal_handlers_disconnect_matched (window, G_SIGNAL_MATCH_FUNC,
+                                        0, 0, NULL,
+                                        gimp_window_transient_on_mapped,
+                                        NULL);
+
+  g_signal_connect_data (window, "map-event",
+                         G_CALLBACK (gimp_window_transient_on_mapped),
+                         g_bytes_ref (handle),
+                         (GClosureNotify) g_bytes_unref,
+                         G_CONNECT_AFTER);
+
+  if (gtk_widget_get_mapped (GTK_WIDGET (window)))
+    gimp_window_transient_on_mapped (GTK_WIDGET (window), NULL, handle);
+}
+
 /**
  * gimp_window_set_transient_for_display:
  * @window:  the #GtkWindow that should become transient
  * @display: display of the image window that should become the parent
  *
  * Indicates to the window manager that @window is a transient dialog
- * associated with the GIMP image window that is identified by it's
- * display ID. See gdk_window_set_transient_for () for more information.
+ * associated with the GIMP image window that is identified by its
+ * display. See [method@Gdk.Window.set_transient_for] for more information.
  *
  * Most of the time you will want to use the convenience function
- * gimp_window_set_transient().
+ * [func@GimpUi.window_set_transient].
  *
  * Since: 2.4
  */
@@ -281,24 +266,26 @@ void
 gimp_window_set_transient_for_display (GtkWindow   *window,
                                        GimpDisplay *display)
 {
+  GBytes *handle;
+
   g_return_if_fail (gimp_ui_initialized);
   g_return_if_fail (GTK_IS_WINDOW (window));
+  g_return_if_fail (GIMP_IS_DISPLAY (display));
 
-  if (! gimp_window_set_transient_for (window,
-                                       gimp_ui_get_display_window (display)))
-    {
-      /*  if setting the window transient failed, at least set
-       *  WIN_POS_CENTER, which will center the window on the screen
-       *  where the mouse is (see bug #684003).
-       */
-      gtk_window_set_position (window, GTK_WIN_POS_CENTER);
+  g_signal_handlers_disconnect_matched (window, G_SIGNAL_MATCH_FUNC,
+                                        0, 0, NULL,
+                                        gimp_window_transient_on_mapped,
+                                        NULL);
 
-#ifdef GDK_WINDOWING_QUARTZ
-      g_signal_connect (window, "show",
-                        G_CALLBACK (gimp_window_transient_show),
-                        NULL);
-#endif
-    }
+  handle = gimp_display_get_window_handle (display);
+  g_signal_connect_data (window, "map-event",
+                         G_CALLBACK (gimp_window_transient_on_mapped),
+                         handle,
+                         (GClosureNotify) g_bytes_unref,
+                         G_CONNECT_AFTER);
+
+  if (gtk_widget_get_mapped (GTK_WIDGET (window)))
+    gimp_window_transient_on_mapped (GTK_WIDGET (window), NULL, handle);
 }
 
 /**
@@ -314,20 +301,25 @@ gimp_window_set_transient_for_display (GtkWindow   *window,
 void
 gimp_window_set_transient (GtkWindow *window)
 {
+  GBytes *handle;
+
   g_return_if_fail (gimp_ui_initialized);
   g_return_if_fail (GTK_IS_WINDOW (window));
 
-  if (! gimp_window_set_transient_for (window, gimp_ui_get_progress_window ()))
-    {
-      /*  see above  */
-      gtk_window_set_position (window, GTK_WIN_POS_CENTER);
+  g_signal_handlers_disconnect_matched (window, G_SIGNAL_MATCH_FUNC,
+                                        0, 0, NULL,
+                                        gimp_window_transient_on_mapped,
+                                        NULL);
 
-#ifdef GDK_WINDOWING_QUARTZ
-      g_signal_connect (window, "show",
-                        G_CALLBACK (gimp_window_transient_show),
-                        NULL);
-#endif
-    }
+  handle = gimp_progress_get_window_handle ();
+  g_signal_connect_data (window, "map-event",
+                         G_CALLBACK (gimp_window_transient_on_mapped),
+                         handle,
+                         (GClosureNotify) g_bytes_unref,
+                         G_CONNECT_AFTER);
+
+  if (gtk_widget_get_mapped (GTK_WIDGET (window)))
+    gimp_window_transient_on_mapped (GTK_WIDGET (window), NULL, handle);
 }
 
 
@@ -401,49 +393,6 @@ gimp_ensure_modules (void)
     }
 }
 
-#ifndef GDK_WINDOWING_WIN32
-static void
-gimp_window_transient_realized (GtkWidget *window,
-                                GdkWindow *parent)
-{
-  if (gtk_widget_get_realized (window))
-    gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
-}
-#endif
-
-static gboolean
-gimp_window_set_transient_for (GtkWindow *window,
-                               GdkWindow *parent)
-{
-  gtk_window_set_transient_for (window, NULL);
-
-  /* To know why it is disabled on Win32, see
-   * gimp_window_set_transient_for() in app/widgets/gimpwidgets-utils.c.
-   */
-#ifndef GDK_WINDOWING_WIN32
-  g_signal_handlers_disconnect_matched (window, G_SIGNAL_MATCH_FUNC,
-                                        0, 0, NULL,
-                                        gimp_window_transient_realized,
-                                        NULL);
-
-  if (! parent)
-    return FALSE;
-
-  if (gtk_widget_get_realized (GTK_WIDGET (window)))
-    gdk_window_set_transient_for (gtk_widget_get_window (GTK_WIDGET (window)),
-                                  parent);
-
-  g_signal_connect_object (window, "realize",
-                           G_CALLBACK (gimp_window_transient_realized),
-                           parent, 0);
-  g_object_unref (parent);
-
-  return TRUE;
-#endif
-
-  return FALSE;
-}
-
 #ifdef GDK_WINDOWING_QUARTZ
 static gboolean
 gimp_osx_focus_window (gpointer user_data)
@@ -452,3 +401,109 @@ gimp_osx_focus_window (gpointer user_data)
   return FALSE;
 }
 #endif
+
+/* Currently broken on Win32 so avoiding a "defined but not used"
+ * warning when building on Windows.
+ */
+#ifndef GDK_WINDOWING_WIN32
+static GdkWindow *
+gimp_ui_get_foreign_window (gpointer window)
+{
+#ifdef GDK_WINDOWING_X11
+  if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    return gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
+                                                   (Window) window);
+#endif
+
+#ifdef GDK_WINDOWING_WIN32
+  return gdk_win32_window_foreign_new_for_display (gdk_display_get_default (),
+                                                   (HWND) window);
+#endif
+
+  return NULL;
+}
+#endif
+
+static gboolean
+gimp_window_transient_on_mapped (GtkWidget   *window,
+                                 GdkEventAny *event,
+                                 GBytes      *handle)
+{
+  gboolean transient_set = FALSE;
+
+  if (handle == NULL)
+    return FALSE;
+
+#ifdef GDK_WINDOWING_WAYLAND
+  if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()))
+    {
+      char *wayland_handle;
+
+      wayland_handle = (char *) g_bytes_get_data (handle, NULL);
+      gdk_wayland_window_set_transient_for_exported (gtk_widget_get_window (window),
+                                                     wayland_handle);
+      transient_set = TRUE;
+    }
+#endif
+
+#ifdef GDK_WINDOWING_X11
+  if (! transient_set && GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    {
+      GdkWindow *parent;
+      Window    *handle_data;
+      Window     parent_ID;
+      gsize      handle_size;
+
+      handle_data = (Window *) g_bytes_get_data (handle, &handle_size);
+      g_return_val_if_fail (handle_size == sizeof (Window), FALSE);
+      parent_ID = *handle_data;
+
+      parent = gimp_ui_get_foreign_window ((gpointer) parent_ID);
+
+      if (parent)
+        gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
+
+      transient_set = TRUE;
+    }
+#endif
+  /* To know why it is disabled on Win32, see gimp_window_set_transient_cb() in
+   * app/widgets/gimpwidgets-utils.c.
+   */
+#if 0 && defined (GDK_WINDOWING_WIN32)
+  if (! transient_set)
+    {
+      GdkWindow *parent;
+      HANDLE    *handle_data;
+      HANDLE     parent_ID;
+      gsize      handle_size;
+
+      handle_data = (HANDLE *) g_bytes_get_data (handle, &handle_size);
+      g_return_val_if_fail (handle_size == sizeof (HANDLE), FALSE);
+      parent_ID = *handle_data;
+
+      parent = gimp_ui_get_foreign_window ((gpointer) parent_ID);
+
+      if (parent)
+        gdk_window_set_transient_for (gtk_widget_get_window (window), parent);
+
+      transient_set = TRUE;
+    }
+#endif
+
+  if (! transient_set)
+    {
+      /*  if setting the window transient failed, at least set
+       *  WIN_POS_CENTER, which will center the window on the screen
+       *  where the mouse is (see bug #684003).
+       */
+      gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
+
+#ifdef GDK_WINDOWING_QUARTZ
+      g_signal_connect (window, "show",
+                        G_CALLBACK (gimp_window_transient_show),
+                        NULL);
+#endif
+    }
+
+  return FALSE;
+}
