@@ -1755,7 +1755,7 @@ gimp_metadata_set_creation_date (GimpMetadata *metadata,
 
   /* XMP: Xmp.photoshop.DateCreated = date when the original image was
    *   taken, this can be before Xmp.xmp.CreateDate. */
-  datetime_buf = g_date_time_format (datetime, "%Y:%m:%dT%T\%:z");
+  datetime_buf = g_date_time_format (datetime, "%Y-%m-%dT%T\%:z");
 
   gexiv2_metadata_try_set_tag_string (g2metadata,
                                       "Xmp.xmp.CreateDate",
