@@ -18,10 +18,13 @@
 
 #include "config.h"
 
+#include <gegl.h>
+#include <gio/gio.h>
 #include <glib-object.h>
 
 #include "gimpbasetypes.h"
 
+#include "gimpchoice.h"
 #include "gimpparamspecs.h"
 #include "gimpparasite.h"
 #include "gimpprotocol.h"
@@ -493,11 +496,17 @@ _gp_config_read (GIOChannel      *channel,
   if (! _gimp_wire_read_int8 (channel,
                               (guint8 *) &config->check_type, 1, user_data))
     goto cleanup;
-  if (! _gimp_wire_read_color (channel, &config->check_custom_color1,
-                               1, user_data))
+  if (! _gimp_wire_read_gegl_color (channel,
+                                    &config->check_custom_color1,
+                                    &config->check_custom_icc1,
+                                    &config->check_custom_encoding1,
+                                    1, user_data))
     goto cleanup;
-  if (! _gimp_wire_read_color (channel, &config->check_custom_color2,
-                               1, user_data))
+  if (! _gimp_wire_read_gegl_color (channel,
+                                    &config->check_custom_color2,
+                                    &config->check_custom_icc2,
+                                    &config->check_custom_encoding2,
+                                    1, user_data))
     goto cleanup;
   if (! _gimp_wire_read_int8 (channel,
                               (guint8 *) &config->show_help_button, 1,
@@ -573,6 +582,13 @@ _gp_config_read (GIOChannel      *channel,
   return;
 
  cleanup:
+  g_bytes_unref (config->check_custom_color1);
+  g_bytes_unref (config->check_custom_icc1);
+  g_free (config->check_custom_encoding1);
+  g_bytes_unref (config->check_custom_color2);
+  g_bytes_unref (config->check_custom_icc2);
+  g_free (config->check_custom_encoding2);
+
   g_free (config->app_name);
   g_free (config->wm_class);
   g_free (config->display_name);
@@ -607,11 +623,17 @@ _gp_config_write (GIOChannel      *channel,
                                (const guint8 *) &config->check_type, 1,
                                user_data))
     return;
-  if (! _gimp_wire_write_color (channel, &config->check_custom_color1,
-                                1, user_data))
+  if (! _gimp_wire_write_gegl_color (channel,
+                                     &config->check_custom_color1,
+                                     &config->check_custom_icc1,
+                                     &config->check_custom_encoding1,
+                                     1, user_data))
     return;
-  if (! _gimp_wire_write_color (channel, &config->check_custom_color2,
-                                1, user_data))
+  if (! _gimp_wire_write_gegl_color (channel,
+                                     &config->check_custom_color2,
+                                     &config->check_custom_icc2,
+                                     &config->check_custom_encoding2,
+                                     1, user_data))
     return;
   if (! _gimp_wire_write_int8 (channel,
                                (const guint8 *) &config->show_help_button, 1,
@@ -691,6 +713,13 @@ _gp_config_destroy (GimpWireMessage *msg)
 
   if (config)
     {
+      g_bytes_unref (config->check_custom_color1);
+      g_bytes_unref (config->check_custom_icc1);
+      g_free (config->check_custom_encoding1);
+      g_bytes_unref (config->check_custom_color2);
+      g_bytes_unref (config->check_custom_icc2);
+      g_free (config->check_custom_encoding2);
+
       g_free (config->app_name);
       g_free (config->wm_class);
       g_free (config->display_name);
@@ -1085,6 +1114,7 @@ _gp_param_def_read (GIOChannel *channel,
   switch (param_def->param_def_type)
     {
     case GP_PARAM_DEF_TYPE_DEFAULT:
+    case GP_PARAM_DEF_TYPE_EXPORT_OPTIONS:
       break;
 
     case GP_PARAM_DEF_TYPE_INT:
@@ -1100,6 +1130,47 @@ _gp_param_def_read (GIOChannel *channel,
         return FALSE;
       break;
 
+    case GP_PARAM_DEF_TYPE_CHOICE:
+        {
+          GimpChoice *choice;
+          guint32     size;
+          gchar      *nick;
+
+          if (! _gimp_wire_read_string (channel, &nick, (int) 1, user_data))
+            return FALSE;
+
+          param_def->meta.m_choice.default_val = g_strdup (nick);
+
+          if (! _gimp_wire_read_int32 (channel, &size, 1, user_data))
+            return FALSE;
+
+          choice = gimp_choice_new ();
+
+          for (gint i = 0; i < size; i++)
+            {
+              gchar *label;
+              gchar *help;
+              gint   id;
+
+              if (! _gimp_wire_read_string (channel, &nick,
+                                           (int) 1, user_data)  ||
+                  ! _gimp_wire_read_int32 (channel, (guint32 *) &id,
+                                           1, user_data)        ||
+                  ! _gimp_wire_read_string (channel, &label,
+                                            (int) 1, user_data) ||
+                  ! _gimp_wire_read_string (channel, &help,
+                                            (int) 1, user_data))
+                {
+                  g_object_unref (choice);
+                  return FALSE;
+                }
+
+              gimp_choice_add (choice, nick, id, label, help);
+            }
+          param_def->meta.m_choice.choice = choice;
+        }
+      break;
+
     case GP_PARAM_DEF_TYPE_UNIT:
       if (! _gimp_wire_read_int32 (channel,
                                    (guint32 *) &param_def->meta.m_unit.allow_pixels, 1,
@@ -1127,15 +1198,15 @@ _gp_param_def_read (GIOChannel *channel,
         return FALSE;
       break;
 
-    case GP_PARAM_DEF_TYPE_FLOAT:
+    case GP_PARAM_DEF_TYPE_DOUBLE:
       if (! _gimp_wire_read_double (channel,
-                                    &param_def->meta.m_float.min_val, 1,
+                                    &param_def->meta.m_double.min_val, 1,
                                     user_data) ||
           ! _gimp_wire_read_double (channel,
-                                    &param_def->meta.m_float.max_val, 1,
+                                    &param_def->meta.m_double.max_val, 1,
                                     user_data) ||
           ! _gimp_wire_read_double (channel,
-                                    &param_def->meta.m_float.default_val, 1,
+                                    &param_def->meta.m_double.default_val, 1,
                                     user_data))
         return FALSE;
       break;
@@ -1147,14 +1218,52 @@ _gp_param_def_read (GIOChannel *channel,
         return FALSE;
       break;
 
-    case GP_PARAM_DEF_TYPE_COLOR:
-      if (! _gimp_wire_read_int32 (channel,
-                                   (guint32 *) &param_def->meta.m_color.has_alpha, 1,
-                                   user_data) ||
-          ! _gimp_wire_read_color (channel,
-                                   &param_def->meta.m_color.default_val, 1,
-                                   user_data))
-        return FALSE;
+    case GP_PARAM_DEF_TYPE_GEGL_COLOR:
+        {
+          GPParamColor *default_val = NULL;
+          GBytes       *pixel_data = NULL;
+          GBytes       *icc_data   = NULL;
+          gchar        *encoding   = NULL;
+
+          if (! _gimp_wire_read_int32 (channel,
+                                       (guint32 *) &param_def->meta.m_gegl_color.has_alpha, 1,
+                                       user_data) ||
+              ! _gimp_wire_read_gegl_color (channel, &pixel_data, &icc_data, &encoding,
+                                            1, user_data))
+            return FALSE;
+
+          if (pixel_data != NULL)
+            {
+              default_val = g_new0 (GPParamColor, 1);
+
+              default_val->format.encoding = encoding;
+              default_val->size            = g_bytes_get_size (pixel_data);
+              if (default_val->size > 40)
+                {
+                  g_free (default_val);
+                  g_free (encoding);
+                  g_bytes_unref (pixel_data);
+                  g_bytes_unref (icc_data);
+                  return FALSE;
+                }
+              memcpy (default_val->data, g_bytes_get_data (pixel_data, NULL),
+                      default_val->size);
+              g_bytes_unref (pixel_data);
+              if (icc_data)
+                {
+                  gsize profile_size;
+
+                  default_val->format.profile_data = g_bytes_unref_to_data (icc_data, &profile_size);
+                  default_val->format.profile_size = (guint32) profile_size;
+                }
+              else
+                {
+                  default_val->format.profile_data = NULL;
+                  default_val->format.profile_size = 0;
+                }
+            }
+          param_def->meta.m_gegl_color.default_val = default_val;
+        }
       break;
 
     case GP_PARAM_DEF_TYPE_ID:
@@ -1170,6 +1279,36 @@ _gp_param_def_read (GIOChannel *channel,
                                     user_data))
         return FALSE;
       break;
+
+    case GP_PARAM_DEF_TYPE_RESOURCE:
+      if (! _gimp_wire_read_int32 (channel,
+                                   (guint32 *) &param_def->meta.m_resource.none_ok, 1,
+                                   user_data))
+        return FALSE;
+      if (! _gimp_wire_read_int32 (channel,
+                                   (guint32 *) &param_def->meta.m_resource.default_to_context, 1,
+                                   user_data))
+        return FALSE;
+      if (! _gimp_wire_read_int32 (channel,
+                                   (guint32 *) &param_def->meta.m_resource.default_resource_id, 1,
+                                   user_data))
+        return FALSE;
+      break;
+
+    case GP_PARAM_DEF_TYPE_FILE:
+      if (! _gimp_wire_read_int32 (channel,
+                                   (guint32 *) &param_def->meta.m_file.action, 1,
+                                   user_data))
+        return FALSE;
+      if (! _gimp_wire_read_int32 (channel,
+                                   (guint32 *) &param_def->meta.m_file.none_ok, 1,
+                                   user_data))
+        return FALSE;
+      if (! _gimp_wire_read_string (channel,
+                                    &param_def->meta.m_file.default_uri, 1,
+                                    user_data))
+        return FALSE;
+      break;
     }
 
   return TRUE;
@@ -1193,20 +1332,43 @@ _gp_param_def_destroy (GPParamDef *param_def)
       break;
 
     case GP_PARAM_DEF_TYPE_BOOLEAN:
-    case GP_PARAM_DEF_TYPE_FLOAT:
+    case GP_PARAM_DEF_TYPE_DOUBLE:
+      break;
+
+    case GP_PARAM_DEF_TYPE_CHOICE:
+      g_free (param_def->meta.m_choice.default_val);
+      g_object_unref (param_def->meta.m_choice.choice);
       break;
 
     case GP_PARAM_DEF_TYPE_STRING:
       g_free (param_def->meta.m_string.default_val);
       break;
 
-    case GP_PARAM_DEF_TYPE_COLOR:
+    case GP_PARAM_DEF_TYPE_GEGL_COLOR:
+      if (param_def->meta.m_gegl_color.default_val)
+        {
+          g_free (param_def->meta.m_gegl_color.default_val->format.encoding);
+          g_free (param_def->meta.m_gegl_color.default_val->format.profile_data);
+        }
+      g_free (param_def->meta.m_gegl_color.default_val);
+      break;
+
     case GP_PARAM_DEF_TYPE_ID:
       break;
 
     case GP_PARAM_DEF_TYPE_ID_ARRAY:
       g_free (param_def->meta.m_id_array.type_name);
       break;
+
+    case GP_PARAM_DEF_TYPE_EXPORT_OPTIONS:
+      break;
+
+    case GP_PARAM_DEF_TYPE_RESOURCE:
+      break;
+
+    case GP_PARAM_DEF_TYPE_FILE:
+      g_free (param_def->meta.m_file.default_uri);
+      break;
     }
 }
 
@@ -1327,6 +1489,7 @@ _gp_param_def_write (GIOChannel *channel,
   switch (param_def->param_def_type)
     {
     case GP_PARAM_DEF_TYPE_DEFAULT:
+    case GP_PARAM_DEF_TYPE_EXPORT_OPTIONS:
       break;
 
     case GP_PARAM_DEF_TYPE_INT:
@@ -1342,6 +1505,51 @@ _gp_param_def_write (GIOChannel *channel,
         return FALSE;
       break;
 
+    case GP_PARAM_DEF_TYPE_CHOICE:
+        {
+          GList *values;
+          gint   size;
+
+          if (! _gimp_wire_write_string (channel,
+                                         &param_def->meta.m_choice.default_val,
+                                         1, user_data))
+            return FALSE;
+
+          values = gimp_choice_list_nicks (param_def->meta.m_choice.choice);
+          size   = g_list_length (values);
+
+          if (! _gimp_wire_write_int32 (channel,
+                                        (guint32 *) &size, 1,
+                                        user_data))
+            return FALSE;
+
+          for (GList *iter = values; iter; iter = iter->next)
+            {
+              const gchar *label;
+              const gchar *help;
+              gint         id;
+
+              gimp_choice_get_documentation (param_def->meta.m_choice.choice,
+                                             iter->data, &label, &help);
+              id = gimp_choice_get_id (param_def->meta.m_choice.choice,
+                                       iter->data);
+              if (! _gimp_wire_write_string (channel,
+                                             (gchar **) &iter->data,
+                                             1, user_data)  ||
+                  ! _gimp_wire_write_int32 (channel,
+                                            (guint32 *) &id, 1,
+                                            user_data)      ||
+                  ! _gimp_wire_write_string (channel,
+                                             (gchar **) &label,
+                                             1, user_data)  ||
+                  ! _gimp_wire_write_string (channel,
+                                             (gchar **) &help,
+                                             1, user_data))
+                return FALSE;
+            }
+        }
+      break;
+
     case GP_PARAM_DEF_TYPE_UNIT:
       if (! _gimp_wire_write_int32 (channel,
                                     (guint32 *) &param_def->meta.m_unit.allow_pixels, 1,
@@ -1369,15 +1577,15 @@ _gp_param_def_write (GIOChannel *channel,
         return FALSE;
       break;
 
-    case GP_PARAM_DEF_TYPE_FLOAT:
+    case GP_PARAM_DEF_TYPE_DOUBLE:
       if (! _gimp_wire_write_double (channel,
-                                     &param_def->meta.m_float.min_val, 1,
+                                     &param_def->meta.m_double.min_val, 1,
                                      user_data) ||
           ! _gimp_wire_write_double (channel,
-                                     &param_def->meta.m_float.max_val, 1,
+                                     &param_def->meta.m_double.max_val, 1,
                                      user_data) ||
           ! _gimp_wire_write_double (channel,
-                                     &param_def->meta.m_float.default_val, 1,
+                                     &param_def->meta.m_double.default_val, 1,
                                      user_data))
         return FALSE;
       break;
@@ -1389,14 +1597,40 @@ _gp_param_def_write (GIOChannel *channel,
         return FALSE;
       break;
 
-    case GP_PARAM_DEF_TYPE_COLOR:
-      if (! _gimp_wire_write_int32 (channel,
-                                    (guint32 *) &param_def->meta.m_color.has_alpha, 1,
-                                    user_data) ||
-          ! _gimp_wire_write_color (channel,
-                                    &param_def->meta.m_color.default_val, 1,
-                                    user_data))
-        return FALSE;
+    case GP_PARAM_DEF_TYPE_GEGL_COLOR:
+        {
+          GBytes *pixel_data = NULL;
+          GBytes *icc_data   = NULL;
+          gchar  *encoding   = "";
+
+          if (! _gimp_wire_write_int32 (channel,
+                                        (guint32 *) &param_def->meta.m_gegl_color.has_alpha, 1,
+                                        user_data))
+            return FALSE;
+
+          if (param_def->meta.m_gegl_color.default_val)
+            {
+              pixel_data = g_bytes_new_static (param_def->meta.m_gegl_color.default_val->data,
+                                               param_def->meta.m_gegl_color.default_val->size);
+              icc_data   = g_bytes_new_static (param_def->meta.m_gegl_color.default_val->format.profile_data,
+                                               param_def->meta.m_gegl_color.default_val->format.profile_size);
+              encoding   = param_def->meta.m_gegl_color.default_val->format.encoding;
+            }
+
+          if (! _gimp_wire_write_gegl_color (channel,
+                                             &pixel_data,
+                                             &icc_data,
+                                             &encoding,
+                                             1, user_data))
+            {
+              g_bytes_unref (pixel_data);
+              g_bytes_unref (icc_data);
+              return FALSE;
+            }
+
+          g_bytes_unref (pixel_data);
+          g_bytes_unref (icc_data);
+        }
       break;
 
     case GP_PARAM_DEF_TYPE_ID:
@@ -1412,6 +1646,36 @@ _gp_param_def_write (GIOChannel *channel,
                                      user_data))
         return FALSE;
       break;
+
+    case GP_PARAM_DEF_TYPE_RESOURCE:
+      if (! _gimp_wire_write_int32 (channel,
+                                    (guint32 *) &param_def->meta.m_resource.none_ok, 1,
+                                    user_data))
+        return FALSE;
+      if (! _gimp_wire_write_int32 (channel,
+                                    (guint32 *) &param_def->meta.m_resource.default_to_context, 1,
+                                    user_data))
+        return FALSE;
+      if (! _gimp_wire_write_int32 (channel,
+                                    (guint32 *) &param_def->meta.m_resource.default_resource_id, 1,
+                                    user_data))
+        return FALSE;
+      break;
+
+    case GP_PARAM_DEF_TYPE_FILE:
+      if (! _gimp_wire_write_int32 (channel,
+                                    (guint32 *) &param_def->meta.m_file.action, 1,
+                                    user_data))
+        return FALSE;
+      if (! _gimp_wire_write_int32 (channel,
+                                    (guint32 *) &param_def->meta.m_file.none_ok, 1,
+                                    user_data))
+        return FALSE;
+      if (! _gimp_wire_write_string (channel,
+                                     &param_def->meta.m_file.default_uri, 1,
+                                     user_data))
+        return FALSE;
+      break;
     }
 
   return TRUE;
@@ -1596,9 +1860,9 @@ _gp_params_read (GIOChannel  *channel,
             goto cleanup;
           break;
 
-        case GP_PARAM_TYPE_FLOAT:
+        case GP_PARAM_TYPE_DOUBLE:
           if (! _gimp_wire_read_double (channel,
-                                        &(*params)[i].data.d_float, 1,
+                                        &(*params)[i].data.d_double, 1,
                                         user_data))
             goto cleanup;
           break;
@@ -1611,11 +1875,166 @@ _gp_params_read (GIOChannel  *channel,
             goto cleanup;
           break;
 
-        case GP_PARAM_TYPE_COLOR:
-          if (! _gimp_wire_read_color (channel,
-                                       &(*params)[i].data.d_color, 1,
+        case GP_PARAM_TYPE_BABL_FORMAT:
+          /* Read encoding. */
+          if (! _gimp_wire_read_string (channel,
+                                        &(*params)[i].data.d_format.encoding, 1,
+                                        user_data))
+            goto cleanup;
+
+          /* Read space (profile data). */
+          if (! _gimp_wire_read_int32 (channel,
+                                       &(*params)[i].data.d_format.profile_size, 1,
+                                       user_data))
+            {
+              g_clear_pointer (&(*params)[i].data.d_format.encoding, g_free);
+              goto cleanup;
+            }
+
+          (*params)[i].data.d_format.profile_data = g_new0 (guint8, (*params)[i].data.d_format.profile_size);
+          if (! _gimp_wire_read_int8 (channel,
+                                      (*params)[i].data.d_format.profile_data,
+                                      (*params)[i].data.d_format.profile_size,
+                                      user_data))
+            {
+              g_clear_pointer (&(*params)[i].data.d_format.encoding, g_free);
+              g_clear_pointer (&(*params)[i].data.d_format.profile_data, g_free);
+              goto cleanup;
+            }
+
+          break;
+
+        case GP_PARAM_TYPE_GEGL_COLOR:
+          /* Read the color data. */
+          if (! _gimp_wire_read_int32 (channel,
+                                       &(*params)[i].data.d_gegl_color.size, 1,
                                        user_data))
             goto cleanup;
+
+          if ((*params)[i].data.d_gegl_color.size > 40 ||
+              ! _gimp_wire_read_int8 (channel,
+                                      (*params)[i].data.d_gegl_color.data,
+                                      (*params)[i].data.d_gegl_color.size,
+                                      user_data))
+            goto cleanup;
+
+          /* Read encoding. */
+          if (! _gimp_wire_read_string (channel,
+                                        &(*params)[i].data.d_gegl_color.format.encoding, 1,
+                                        user_data))
+            goto cleanup;
+
+          /* Read space (profile data). */
+          if (! _gimp_wire_read_int32 (channel,
+                                       &(*params)[i].data.d_gegl_color.format.profile_size, 1,
+                                       user_data))
+            {
+              g_clear_pointer (&(*params)[i].data.d_gegl_color.format.encoding, g_free);
+              goto cleanup;
+            }
+
+          (*params)[i].data.d_gegl_color.format.profile_data = g_new0 (guint8, (*params)[i].data.d_gegl_color.format.profile_size);
+          if (! _gimp_wire_read_int8 (channel,
+                                      (*params)[i].data.d_gegl_color.format.profile_data,
+                                      (*params)[i].data.d_gegl_color.format.profile_size,
+                                      user_data))
+            {
+              g_clear_pointer (&(*params)[i].data.d_gegl_color.format.encoding, g_free);
+              g_clear_pointer (&(*params)[i].data.d_gegl_color.format.profile_data, g_free);
+              goto cleanup;
+            }
+
+          break;
+
+        case GP_PARAM_TYPE_COLOR_ARRAY:
+          if (! _gimp_wire_read_int32 (channel,
+                                       &(*params)[i].data.d_color_array.size, 1,
+                                       user_data))
+            goto cleanup;
+
+          (*params)[i].data.d_color_array.colors = g_new0 (GPParamColor,
+                                                           (*params)[i].data.d_color_array.size);
+
+          for (gint j = 0; j < (*params)[i].data.d_color_array.size; j++)
+            {
+              /* Read the color data. */
+              if (! _gimp_wire_read_int32 (channel,
+                                           &(*params)[i].data.d_color_array.colors[j].size, 1,
+                                           user_data))
+                {
+                  for (gint k = 0; k < j; k++)
+                    {
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.encoding);
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.profile_data);
+                    }
+                  g_clear_pointer (&(*params)[i].data.d_color_array.colors, g_free);
+                  goto cleanup;
+                }
+
+              if ((*params)[i].data.d_color_array.colors[j].size > 40 ||
+                  ! _gimp_wire_read_int8 (channel,
+                                          (*params)[i].data.d_color_array.colors[j].data,
+                                          (*params)[i].data.d_color_array.colors[j].size,
+                                          user_data))
+                {
+                  for (gint k = 0; k < j; k++)
+                    {
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.encoding);
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.profile_data);
+                    }
+                  g_clear_pointer (&(*params)[i].data.d_color_array.colors, g_free);
+                  goto cleanup;
+                }
+
+              /* Read encoding. */
+              if (! _gimp_wire_read_string (channel,
+                                            &(*params)[i].data.d_color_array.colors[j].format.encoding, 1,
+                                            user_data))
+                {
+                  for (gint k = 0; k < j; k++)
+                    {
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.encoding);
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.profile_data);
+                    }
+                  g_clear_pointer (&(*params)[i].data.d_color_array.colors, g_free);
+                  goto cleanup;
+                }
+
+              /* Read space (profile data). */
+              if (! _gimp_wire_read_int32 (channel,
+                                           &(*params)[i].data.d_color_array.colors[j].format.profile_size, 1,
+                                           user_data))
+                {
+                  for (gint k = 0; k < j; k++)
+                    {
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.encoding);
+                      g_free ((*params)[i].data.d_color_array.colors[k].format.profile_data);
+                    }
+                  g_clear_pointer (&(*params)[i].data.d_color_array.colors[j].format.encoding, g_free);
+                  g_clear_pointer (&(*params)[i].data.d_color_array.colors, g_free);
+                  goto cleanup;
+                }
+
+              if ((*params)[i].data.d_color_array.colors[j].format.profile_size > 0)
+                {
+                  (*params)[i].data.d_color_array.colors[j].format.profile_data = g_new0 (guint8, (*params)[i].data.d_color_array.colors[j].format.profile_size);
+                  if (! _gimp_wire_read_int8 (channel,
+                                              (*params)[i].data.d_color_array.colors[j].format.profile_data,
+                                              (*params)[i].data.d_color_array.colors[j].format.profile_size,
+                                              user_data))
+                    {
+                      for (gint k = 0; k < j; k++)
+                        {
+                          g_free ((*params)[i].data.d_color_array.colors[k].format.encoding);
+                          g_free ((*params)[i].data.d_color_array.colors[k].format.profile_data);
+                        }
+                      g_clear_pointer (&(*params)[i].data.d_color_array.colors[j].format.encoding, g_free);
+                      g_clear_pointer (&(*params)[i].data.d_color_array.colors[j].format.profile_data, g_free);
+                      g_clear_pointer (&(*params)[i].data.d_color_array.colors, g_free);
+                      goto cleanup;
+                    }
+                }
+            }
           break;
 
         case GP_PARAM_TYPE_ARRAY:
@@ -1740,12 +2159,29 @@ _gp_params_read (GIOChannel  *channel,
             (*params)[i].data.d_parasite.data = NULL;
           break;
 
+        case GP_PARAM_TYPE_EXPORT_OPTIONS:
+          /* XXX: reading export options when we'll have any. */
+          break;
+
         case GP_PARAM_TYPE_PARAM_DEF:
           if (! _gp_param_def_read (channel,
                                     &(*params)[i].data.d_param_def,
                                     user_data))
             goto cleanup;
           break;
+
+        case GP_PARAM_TYPE_VALUE_ARRAY:
+          {
+            guint n_values = 0;
+
+            (*params)[i].data.d_value_array.values = NULL;
+            _gp_params_read (channel,
+                             &(*params)[i].data.d_value_array.values,
+                             &n_values,
+                             user_data);
+            (*params)[i].data.d_value_array.n_values = (guint32) n_values;
+            break;
+          }
         }
     }
 
@@ -1790,9 +2226,9 @@ _gp_params_write (GIOChannel *channel,
             return;
           break;
 
-        case GP_PARAM_TYPE_FLOAT:
+        case GP_PARAM_TYPE_DOUBLE:
           if (! _gimp_wire_write_double (channel,
-                                         (const gdouble *) &params[i].data.d_float, 1,
+                                         (const gdouble *) &params[i].data.d_double, 1,
                                          user_data))
             return;
           break;
@@ -1805,11 +2241,68 @@ _gp_params_write (GIOChannel *channel,
             return;
           break;
 
-        case GP_PARAM_TYPE_COLOR:
-          if (! _gimp_wire_write_color (channel,
-                                        &params[i].data.d_color, 1,
+        case GP_PARAM_TYPE_BABL_FORMAT:
+          if (! _gimp_wire_write_string (channel,
+                                         &params[i].data.d_format.encoding, 1,
+                                         user_data) ||
+              ! _gimp_wire_write_int32 (channel,
+                                        (const guint32 *) &params[i].data.d_format.profile_size, 1,
+                                        user_data)  ||
+              ! _gimp_wire_write_int8 (channel,
+                                       (const guint8 *) params[i].data.d_format.profile_data,
+                                       params[i].data.d_format.profile_size,
+                                       user_data))
+            return;
+          break;
+
+        case GP_PARAM_TYPE_GEGL_COLOR:
+          if (! _gimp_wire_write_int32 (channel,
+                                        (const guint32 *) &params[i].data.d_gegl_color.size, 1,
+                                        user_data)  ||
+              ! _gimp_wire_write_int8 (channel,
+                                       (const guint8 *) params[i].data.d_gegl_color.data,
+                                       params[i].data.d_gegl_color.size,
+                                       user_data)   ||
+              ! _gimp_wire_write_string (channel,
+                                         &params[i].data.d_gegl_color.format.encoding, 1,
+                                         user_data) ||
+              ! _gimp_wire_write_int32 (channel,
+                                        (const guint32 *) &params[i].data.d_gegl_color.format.profile_size, 1,
+                                        user_data)  ||
+              ! _gimp_wire_write_int8 (channel,
+                                       (const guint8 *) params[i].data.d_gegl_color.format.profile_data,
+                                       params[i].data.d_gegl_color.format.profile_size,
+                                       user_data))
+            return;
+          break;
+
+        case GP_PARAM_TYPE_COLOR_ARRAY:
+          if (! _gimp_wire_write_int32 (channel,
+                                        (const guint32 *) &params[i].data.d_color_array.size, 1,
                                         user_data))
             return;
+
+          for (gint j = 0; j < params[i].data.d_color_array.size; j++)
+            {
+              if (! _gimp_wire_write_int32 (channel,
+                                            (const guint32 *) &params[i].data.d_color_array.colors[j].size, 1,
+                                            user_data)  ||
+                  ! _gimp_wire_write_int8 (channel,
+                                           (const guint8 *) params[i].data.d_color_array.colors[j].data,
+                                           params[i].data.d_color_array.colors[j].size,
+                                           user_data)   ||
+                  ! _gimp_wire_write_string (channel,
+                                             &params[i].data.d_color_array.colors[j].format.encoding, 1,
+                                             user_data) ||
+                  ! _gimp_wire_write_int32 (channel,
+                                            (const guint32 *) &params[i].data.d_color_array.colors[j].format.profile_size, 1,
+                                            user_data)  ||
+                  ! _gimp_wire_write_int8 (channel,
+                                           (const guint8 *) params[i].data.d_color_array.colors[j].format.profile_data,
+                                           params[i].data.d_color_array.colors[j].format.profile_size,
+                                           user_data))
+                return;
+            }
           break;
 
         case GP_PARAM_TYPE_ARRAY:
@@ -1900,12 +2393,26 @@ _gp_params_write (GIOChannel *channel,
           }
           break;
 
+        case GP_PARAM_TYPE_EXPORT_OPTIONS:
+          /* XXX When we'll have actual export options, this is where
+           * we'll want to pass them through the wire.
+           */
+          break;
+
         case GP_PARAM_TYPE_PARAM_DEF:
           if (! _gp_param_def_write (channel,
                                      &params[i].data.d_param_def,
                                      user_data))
             return;
           break;
+
+        case GP_PARAM_TYPE_VALUE_ARRAY:
+          _gp_params_write (channel,
+                            params[i].data.d_value_array.values,
+                            params[i].data.d_value_array.n_values,
+                            user_data);
+          break;
+
         }
     }
 }
@@ -1923,7 +2430,7 @@ _gp_params_destroy (GPParam *params,
       switch (params[i].param_type)
         {
         case GP_PARAM_TYPE_INT:
-        case GP_PARAM_TYPE_FLOAT:
+        case GP_PARAM_TYPE_DOUBLE:
           break;
 
         case GP_PARAM_TYPE_STRING:
@@ -1931,7 +2438,22 @@ _gp_params_destroy (GPParam *params,
           g_free (params[i].data.d_string);
           break;
 
-        case GP_PARAM_TYPE_COLOR:
+        case GP_PARAM_TYPE_BABL_FORMAT:
+          g_free (params[i].data.d_format.profile_data);
+          break;
+
+        case GP_PARAM_TYPE_GEGL_COLOR:
+          g_free (params[i].data.d_gegl_color.format.encoding);
+          g_free (params[i].data.d_gegl_color.format.profile_data);
+          break;
+
+        case GP_PARAM_TYPE_COLOR_ARRAY:
+         for (gint j = 0; j < params[i].data.d_color_array.size; j++)
+           {
+             g_free (params[i].data.d_color_array.colors[j].format.encoding);
+             g_free (params[i].data.d_color_array.colors[j].format.profile_data);
+           }
+          g_free (params[i].data.d_color_array.colors);
           break;
 
         case GP_PARAM_TYPE_ARRAY:
@@ -1958,9 +2480,17 @@ _gp_params_destroy (GPParam *params,
             g_free (params[i].data.d_parasite.data);
           break;
 
+        case GP_PARAM_TYPE_EXPORT_OPTIONS:
+          break;
+
         case GP_PARAM_TYPE_PARAM_DEF:
           _gp_param_def_destroy (&params[i].data.d_param_def);
           break;
+
+        case GP_PARAM_TYPE_VALUE_ARRAY:
+          _gp_params_destroy (params[i].data.d_value_array.values,
+                              params[i].data.d_value_array.n_values);
+          break;
         }
     }
 
