@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 /* Introducing eevl eva, the evaluator. A straightforward recursive
@@ -26,13 +26,13 @@
  * expression as it goes along, and does not create a parse tree or
  * anything, and will not optimize anything. It uses doubles for
  * precision, with the given use case, that's enough to combat any
- * rounding errors (as opposed to optimizing the evalutation).
+ * rounding errors (as opposed to optimizing the evaluation).
  *
  * It relies on external unit resolving through a callback and does
  * elementary dimensionality constraint check (e.g. "2 mm + 3 px * 4
- * in" is an error, as L + L^2 is a missmatch). It uses setjmp/longjmp
+ * in" is an error, as L + L^2 is a mismatch). It uses setjmp/longjmp
  * for try/catch like pattern on error, it uses g_strtod() for numeric
- * conversions and it's non-destructive in terms of the paramters, and
+ * conversions and it's non-destructive in terms of the parameters, and
  * it's reentrant.
  *
  * EBNF:
@@ -40,17 +40,21 @@
  *   expression    ::= term { ('+' | '-') term }*  |
  *                     <empty string> ;
  *
- *   term          ::= signed factor { ( '*' | '/' ) signed factor }* ;
+ *   term          ::= ratio { ( '*' | '/' ) ratio }* ;
+ *
+ *   ratio         ::= signed factor { ':' signed factor }* ;
  *
  *   signed factor ::= ( '+' | '-' )? factor ;
  *
- *   unit factor   ::= factor unit? ;
+ *   factor        ::= quantity ( '^' signed factor )? ;
  *
- *   factor        ::= number | '(' expression ')' ;
+ *   quantity      ::= number unit? | '(' expression ')' ;
  *
  *   number        ::= ? what g_strtod() consumes ? ;
  *
- *   unit          ::= ? what not g_strtod() consumes and not whitespace ? ;
+ *   unit          ::= simple unit ( '^' signed factor )? ;
+ *
+ *   simple unit   ::= ? what not g_strtod() consumes and not whitespace ? ;
  *
  * The code should match the EBNF rather closely (except for the
  * non-terminal unit factor, which is inlined into factor) for
@@ -68,6 +72,8 @@
 
 #include <glib-object.h>
 
+#include "libgimpmath/gimpmath.h"
+
 #include "gimpeevl.h"
 #include "gimpwidgets-error.h"
 
@@ -105,62 +111,60 @@ typedef struct
 
 typedef struct
 {
-  const gchar             *string;
-  GimpEevlUnitResolverProc unit_resolver_proc;
-  gpointer                 data;
+  const gchar     *string;
+  GimpEevlOptions  options;
 
-  GimpEevlToken            current_token;
-  const gchar             *start_of_current_token;
-  
+  GimpEevlToken    current_token;
+  const gchar     *start_of_current_token;
 
-  jmp_buf                  catcher;
-  const gchar             *error_message;
+  jmp_buf          catcher;
+  const gchar     *error_message;
 
 } GimpEevl;
 
 
-static void             gimp_eevl_init                     (GimpEevl                 *eva,
-                                                            const gchar              *string,
-                                                            GimpEevlUnitResolverProc  unit_resolver_proc,
-                                                            gpointer                  data);
-static GimpEevlQuantity gimp_eevl_complete                 (GimpEevl                 *eva);
-static GimpEevlQuantity gimp_eevl_expression               (GimpEevl                 *eva);
-static GimpEevlQuantity gimp_eevl_term                     (GimpEevl                 *eva);
-static GimpEevlQuantity gimp_eevl_signed_factor            (GimpEevl                 *eva);
-static GimpEevlQuantity gimp_eevl_factor                   (GimpEevl                 *eva);
-static gboolean         gimp_eevl_accept                   (GimpEevl                 *eva,
-                                                            GimpEevlTokenType         token_type,
-                                                            GimpEevlToken            *consumed_token);
-static void             gimp_eevl_lex                      (GimpEevl                 *eva);
-static void             gimp_eevl_lex_accept_count          (GimpEevl                 *eva,
-                                                            gint                      count,
-                                                            GimpEevlTokenType         token_type);
-static void             gimp_eevl_lex_accept_to             (GimpEevl                 *eva,
-                                                            gchar                    *to,
-                                                            GimpEevlTokenType         token_type);
-static void             gimp_eevl_move_past_whitespace     (GimpEevl                 *eva);
-static gboolean         gimp_eevl_unit_identifier_start    (gunichar                  c);
-static gboolean         gimp_eevl_unit_identifier_continue (gunichar                  c);
-static gint             gimp_eevl_unit_identifier_size     (const gchar              *s,
-                                                            gint                      start);
-static void             gimp_eevl_expect                   (GimpEevl                 *eva,
-                                                            GimpEevlTokenType         token_type,
-                                                            GimpEevlToken            *value);
-static void             gimp_eevl_error                    (GimpEevl                 *eva,
-                                                            gchar                    *msg);
+static void             gimp_eevl_init                     (GimpEevl              *eva,
+                                                            const gchar           *string,
+                                                            const GimpEevlOptions *options);
+static GimpEevlQuantity gimp_eevl_complete                 (GimpEevl              *eva);
+static GimpEevlQuantity gimp_eevl_expression               (GimpEevl              *eva);
+static GimpEevlQuantity gimp_eevl_term                     (GimpEevl              *eva);
+static GimpEevlQuantity gimp_eevl_ratio                    (GimpEevl              *eva);
+static GimpEevlQuantity gimp_eevl_signed_factor            (GimpEevl              *eva);
+static GimpEevlQuantity gimp_eevl_factor                   (GimpEevl              *eva);
+static GimpEevlQuantity gimp_eevl_quantity                 (GimpEevl              *eva);
+static gboolean         gimp_eevl_accept                   (GimpEevl              *eva,
+                                                            GimpEevlTokenType      token_type,
+                                                            GimpEevlToken         *consumed_token);
+static void             gimp_eevl_lex                      (GimpEevl              *eva);
+static void             gimp_eevl_lex_accept_count          (GimpEevl              *eva,
+                                                            gint                   count,
+                                                            GimpEevlTokenType      token_type);
+static void             gimp_eevl_lex_accept_to             (GimpEevl              *eva,
+                                                            gchar                 *to,
+                                                            GimpEevlTokenType      token_type);
+static void             gimp_eevl_move_past_whitespace     (GimpEevl              *eva);
+static gboolean         gimp_eevl_unit_identifier_start    (gunichar               c);
+static gboolean         gimp_eevl_unit_identifier_continue (gunichar               c);
+static gint             gimp_eevl_unit_identifier_size     (const gchar           *s,
+                                                            gint                   start);
+static void             gimp_eevl_expect                   (GimpEevl              *eva,
+                                                            GimpEevlTokenType      token_type,
+                                                            GimpEevlToken         *value);
+static void             gimp_eevl_error                    (GimpEevl              *eva,
+                                                            gchar                 *msg);
 
 
 /**
  * gimp_eevl_evaluate:
- * @string:             The NULL-terminated string to be evaluated.
- * @unit_resolver_proc: Unit resolver callback.
- * @result:             Result of evaluation.
- * @data:               Data passed to unit resolver.
- * @error_pos:          Will point to the poisiton within the string,
- *                      before which the parse / evaluation error
- *                      occured. Will be set to null of no error occured.
- * @error_message:      Will point to a static string with a semi-descriptive
- *                      error message if parsing / evaluation failed.
+ * @string:        The NULL-terminated string to be evaluated.
+ * @options:       Evaluations options.
+ * @result:        Result of evaluation.
+ * @error_pos:     Will point to the position within the string,
+ *                 before which the parse / evaluation error
+ *                 occurred. Will be set to null if no error occurred.
+ * @error_message: Will point to a static string with a semi-descriptive
+ *                 error message if parsing / evaluation failed.
  *
  * Evaluates the given arithmetic expression, along with an optional dimension
  * analysis, and basic unit conversions.
@@ -171,27 +175,26 @@ static void             gimp_eevl_error                    (GimpEevl
  *
  * Returns: A #GimpEevlQuantity with a value given in the base unit along with
  * the order of the dimension (i.e. if the base unit is inches, a dimension
- * order of two menas in^2).
+ * order of two means in^2).
  **/
 gboolean
-gimp_eevl_evaluate (const gchar               *string,
-                    GimpEevlUnitResolverProc   unit_resolver_proc,
-                    GimpEevlQuantity          *result,
-                    gpointer                   data,
-                    const gchar              **error_pos,
-                    GError                   **error)
+gimp_eevl_evaluate (const gchar            *string,
+                    const GimpEevlOptions  *options,
+                    GimpEevlQuantity       *result,
+                    const gchar           **error_pos,
+                    GError                **error)
 {
   GimpEevl eva;
 
   g_return_val_if_fail (g_utf8_validate (string, -1, NULL), FALSE);
-  g_return_val_if_fail (unit_resolver_proc != NULL, FALSE);
+  g_return_val_if_fail (options != NULL, FALSE);
+  g_return_val_if_fail (options->unit_resolver_proc != NULL, FALSE);
   g_return_val_if_fail (result != NULL, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   gimp_eevl_init (&eva,
                   string,
-                  unit_resolver_proc,
-                  data);
+                  options);
 
   if (!setjmp (eva.catcher))  /* try... */
     {
@@ -214,14 +217,12 @@ gimp_eevl_evaluate (const gchar               *string,
 }
 
 static void
-gimp_eevl_init (GimpEevl                  *eva,
-                const gchar               *string,
-                GimpEevlUnitResolverProc   unit_resolver_proc,
-                gpointer                   data)
+gimp_eevl_init (GimpEevl              *eva,
+                const gchar           *string,
+                const GimpEevlOptions *options)
 {
-  eva->string              = string;
-  eva->unit_resolver_proc  = unit_resolver_proc;
-  eva->data                = data;
+  eva->string  = string;
+  eva->options = *options;
 
   eva->current_token.type  = GIMP_EEVL_TOKEN_END;
 
@@ -236,6 +237,7 @@ gimp_eevl_complete (GimpEevl *eva)
 {
   GimpEevlQuantity result = {0, 0};
   GimpEevlQuantity default_unit_factor;
+  gdouble          default_unit_offset;
 
   /* Empty expression evaluates to 0 */
   if (gimp_eevl_accept (eva, GIMP_EEVL_TOKEN_END, NULL))
@@ -246,9 +248,10 @@ gimp_eevl_complete (GimpEevl *eva)
   /* There should be nothing left to parse by now */
   gimp_eevl_expect (eva, GIMP_EEVL_TOKEN_END, 0);
 
-  eva->unit_resolver_proc (NULL,
-                           &default_unit_factor,
-                           eva->data);
+  eva->options.unit_resolver_proc (NULL,
+                                   &default_unit_factor,
+                                   &default_unit_offset,
+                                   eva->options.data);
 
   /* Entire expression is dimensionless, apply default unit if
    * applicable
@@ -256,6 +259,7 @@ gimp_eevl_complete (GimpEevl *eva)
   if (result.dimension == 0 && default_unit_factor.dimension != 0)
     {
       result.value     /= default_unit_factor.value;
+      result.value     += default_unit_offset;
       result.dimension  = default_unit_factor.dimension;
     }
   return result;
@@ -277,30 +281,34 @@ gimp_eevl_expression (GimpEevl *eva)
     {
       GimpEevlQuantity new_term = gimp_eevl_term (eva);
 
-      /* If dimensions missmatch, attempt default unit assignent */
+      /* If dimensions mismatch, attempt default unit assignment */
       if (new_term.dimension != evaluated_terms.dimension)
         {
           GimpEevlQuantity default_unit_factor;
+          gdouble          default_unit_offset;
 
-          eva->unit_resolver_proc (NULL,
-                                   &default_unit_factor,
-                                   eva->data);
+          eva->options.unit_resolver_proc (NULL,
+                                           &default_unit_factor,
+                                           &default_unit_offset,
+                                           eva->options.data);
 
           if (new_term.dimension == 0 &&
               evaluated_terms.dimension == default_unit_factor.dimension)
             {
               new_term.value     /= default_unit_factor.value;
+              new_term.value     += default_unit_offset;
               new_term.dimension  = default_unit_factor.dimension;
             }
           else if (evaluated_terms.dimension == 0 &&
                    new_term.dimension == default_unit_factor.dimension)
             {
               evaluated_terms.value     /= default_unit_factor.value;
+              evaluated_terms.value     += default_unit_offset;
               evaluated_terms.dimension  = default_unit_factor.dimension;
             }
           else
             {
-              gimp_eevl_error (eva, "Dimension missmatch during addition");
+              gimp_eevl_error (eva, "Dimension mismatch during addition");
             }
         }
 
@@ -314,30 +322,61 @@ static GimpEevlQuantity
 gimp_eevl_term (GimpEevl *eva)
 {
   gboolean         division;
-  GimpEevlQuantity evaluated_signed_factors;
+  GimpEevlQuantity evaluated_ratios;
 
-  evaluated_signed_factors = gimp_eevl_signed_factor (eva);
+  evaluated_ratios = gimp_eevl_ratio (eva);
 
   for (division = FALSE;
        gimp_eevl_accept (eva, '*', NULL) ||
        (division = gimp_eevl_accept (eva, '/', NULL));
        division = FALSE)
     {
-      GimpEevlQuantity new_signed_factor = gimp_eevl_signed_factor (eva);
+      GimpEevlQuantity new_ratio = gimp_eevl_ratio (eva);
 
       if (division)
         {
-          evaluated_signed_factors.value     /= new_signed_factor.value;
-          evaluated_signed_factors.dimension -= new_signed_factor.dimension;
-
+          evaluated_ratios.value     /= new_ratio.value;
+          evaluated_ratios.dimension -= new_ratio.dimension;
         }
       else
         {
-          evaluated_signed_factors.value     *= new_signed_factor.value;
-          evaluated_signed_factors.dimension += new_signed_factor.dimension;
+          evaluated_ratios.value     *= new_ratio.value;
+          evaluated_ratios.dimension += new_ratio.dimension;
         }
     }
 
+  return evaluated_ratios;
+}
+
+static GimpEevlQuantity
+gimp_eevl_ratio (GimpEevl *eva)
+{
+  GimpEevlQuantity evaluated_signed_factors;
+
+  if (! eva->options.ratio_expressions)
+    return gimp_eevl_signed_factor (eva);
+
+  evaluated_signed_factors = gimp_eevl_signed_factor (eva);
+
+  while (gimp_eevl_accept (eva, ':', NULL))
+    {
+      GimpEevlQuantity new_signed_factor = gimp_eevl_signed_factor (eva);
+
+      if (eva->options.ratio_invert)
+        {
+          GimpEevlQuantity temp;
+
+          temp                     = evaluated_signed_factors;
+          evaluated_signed_factors = new_signed_factor;
+          new_signed_factor        = temp;
+        }
+
+      evaluated_signed_factors.value     *= eva->options.ratio_quantity.value /
+                                            new_signed_factor.value;
+      evaluated_signed_factors.dimension += eva->options.ratio_quantity.dimension -
+                                            new_signed_factor.dimension;
+    }
+
   return evaluated_signed_factors;
 }
 
@@ -360,18 +399,42 @@ gimp_eevl_signed_factor (GimpEevl *eva)
 static GimpEevlQuantity
 gimp_eevl_factor (GimpEevl *eva)
 {
-  GimpEevlQuantity evaluated_factor = { 0, 0 };
+  GimpEevlQuantity evaluated_factor;
+
+  evaluated_factor = gimp_eevl_quantity (eva);
+
+  if (gimp_eevl_accept (eva, '^', NULL))
+    {
+      GimpEevlQuantity evaluated_exponent;
+
+      evaluated_exponent = gimp_eevl_signed_factor (eva);
+
+      if (evaluated_exponent.dimension != 0)
+        gimp_eevl_error (eva, "Exponent is not a dimensionless quantity");
+
+      evaluated_factor.value      = pow (evaluated_factor.value,
+                                         evaluated_exponent.value);
+      evaluated_factor.dimension *= evaluated_exponent.value;
+    }
+
+  return evaluated_factor;
+}
+
+static GimpEevlQuantity
+gimp_eevl_quantity (GimpEevl *eva)
+{
+  GimpEevlQuantity evaluated_quantity = { 0, 0 };
   GimpEevlToken    consumed_token;
 
   if (gimp_eevl_accept (eva,
                         GIMP_EEVL_TOKEN_NUM,
                         &consumed_token))
     {
-      evaluated_factor.value = consumed_token.value.fl;
+      evaluated_quantity.value = consumed_token.value.fl;
     }
   else if (gimp_eevl_accept (eva, '(', NULL))
     {
-      evaluated_factor = gimp_eevl_expression (eva);
+      evaluated_quantity = gimp_eevl_expression (eva);
       gimp_eevl_expect (eva, ')', 0);
     }
   else
@@ -382,7 +445,8 @@ gimp_eevl_factor (GimpEevl *eva)
   if (eva->current_token.type == GIMP_EEVL_TOKEN_IDENTIFIER)
     {
       gchar            *identifier;
-      GimpEevlQuantity  result;
+      GimpEevlQuantity  factor;
+      gdouble           offset;
 
       gimp_eevl_accept (eva,
                         GIMP_EEVL_TOKEN_ANY,
@@ -393,12 +457,36 @@ gimp_eevl_factor (GimpEevl *eva)
       strncpy (identifier, consumed_token.value.c, consumed_token.value.size);
       identifier[consumed_token.value.size] = '\0';
 
-      if (eva->unit_resolver_proc (identifier,
-                                   &result,
-                                   eva->data))
+      if (eva->options.unit_resolver_proc (identifier,
+                                           &factor,
+                                           &offset,
+                                           eva->options.data))
         {
-          evaluated_factor.value      /= result.value;
-          evaluated_factor.dimension  += result.dimension;
+          if (gimp_eevl_accept (eva, '^', NULL))
+            {
+              GimpEevlQuantity evaluated_exponent;
+
+              evaluated_exponent = gimp_eevl_signed_factor (eva);
+
+              if (evaluated_exponent.dimension != 0)
+                {
+                  gimp_eevl_error (eva,
+                                   "Exponent is not a dimensionless quantity");
+                }
+
+              if (offset != 0.0)
+                {
+                  gimp_eevl_error (eva,
+                                   "Invalid unit exponent");
+                }
+
+              factor.value      = pow (factor.value, evaluated_exponent.value);
+              factor.dimension *= evaluated_exponent.value;
+            }
+
+          evaluated_quantity.value     /= factor.value;
+          evaluated_quantity.value     += offset;
+          evaluated_quantity.dimension += factor.dimension;
         }
       else
         {
@@ -406,7 +494,7 @@ gimp_eevl_factor (GimpEevl *eva)
         }
     }
 
-  return evaluated_factor;
+  return evaluated_quantity;
 }
 
 static gboolean
@@ -447,13 +535,12 @@ gimp_eevl_lex (GimpEevl *eva)
     }
   else if (s[0] == '+' || s[0] == '-')
     {
-      /* Snatch these before the g_strtod() does, othewise they might
+      /* Snatch these before the g_strtod() does, otherwise they might
        * be used in a numeric conversion.
        */
       gimp_eevl_lex_accept_count (eva, 1, s[0]);
     }
   else
-    
     {
       /* Attempt to parse a numeric value */
       gchar  *endptr = NULL;
@@ -557,7 +644,7 @@ gimp_eevl_unit_identifier_size (const gchar *string,
           length++;
         }
     }
-  
+
   return g_utf8_offset_to_pointer (start, length) - start;
 }
 
