@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,10 +29,12 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "gimpwidgetstypes.h"
 
+#include "gimpicons.h"
 #include "gimpnumberpairentry.h"
 
 
@@ -79,8 +81,11 @@ typedef enum
   PARSE_INVALID
 } ParseResult;
 
-typedef struct
+
+struct _GimpNumberPairEntry
 {
+  GtkEntry     parent_instance;
+
   /* The current number pair displayed in the widget. */
   gdouble      left_number;
   gdouble      right_number;
@@ -116,16 +121,13 @@ typedef struct
   /* What range of values considered valid. */
   gdouble      min_valid_value;
   gdouble      max_valid_value;
-} GimpNumberPairEntryPrivate;
-
-#define GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE(obj) \
-  ((GimpNumberPairEntryPrivate *) ((GimpNumberPairEntry *) (obj))->priv)
+};
 
 
 static void         gimp_number_pair_entry_finalize          (GObject             *entry);
 
 static gboolean     gimp_number_pair_entry_valid_separator   (GimpNumberPairEntry *entry,
-                                                              gunichar             canditate);
+                                                              gunichar             candidate);
 static void         gimp_number_pair_entry_ratio_to_fraction (gdouble              ratio,
                                                               gdouble             *numerator,
                                                               gdouble             *denominator);
@@ -139,7 +141,8 @@ static void         gimp_number_pair_entry_get_property      (GObject
                                                               GValue              *value,
                                                               GParamSpec          *pspec);
 static void         gimp_number_pair_entry_changed           (GimpNumberPairEntry *entry);
-static gboolean     gimp_number_pair_entry_events            (GtkWidget           *widgett,
+static void         gimp_number_pair_entry_icon_press        (GimpNumberPairEntry *entry);
+static gboolean     gimp_number_pair_entry_events            (GtkWidget           *widget,
                                                               GdkEvent            *event);
 
 static void         gimp_number_pair_entry_update_text       (GimpNumberPairEntry *entry);
@@ -178,63 +181,65 @@ gimp_number_pair_entry_class_init (GimpNumberPairEntryClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GimpNumberPairEntryPrivate));
-
   entry_signals[NUMBERS_CHANGED] =
     g_signal_new ("numbers-changed",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (GimpNumberPairEntryClass, numbers_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  0,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   entry_signals[RATIO_CHANGED] =
     g_signal_new ("ratio-changed",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (GimpNumberPairEntryClass, ratio_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  0,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
-  klass->numbers_changed = NULL;
-  klass->ratio_changed   = NULL;
-
   object_class->set_property = gimp_number_pair_entry_set_property;
   object_class->get_property = gimp_number_pair_entry_get_property;
   object_class->finalize     = gimp_number_pair_entry_finalize;
 
   g_object_class_install_property (object_class, PROP_LEFT_NUMBER,
                                    g_param_spec_double ("left-number",
-                                                        "Left number", NULL,
+                                                        "Left number",
+                                                        "The left number",
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         100.0,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_RIGHT_NUMBER,
                                    g_param_spec_double ("right-number",
-                                                        "Right number", NULL,
+                                                        "Right number",
+                                                        "The right number",
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         100.0,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_DEFAULT_LEFT_NUMBER,
                                    g_param_spec_double ("default-left-number",
-                                                        "Default left number", NULL,
+                                                        "Default left number",
+                                                        "The default left number",
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         100.0,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_DEFAULT_RIGHT_NUMBER,
                                    g_param_spec_double ("default-right-number",
-                                                        "Default right number", NULL,
+                                                        "Default right number",
+                                                        "The default right number",
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         100.0,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_USER_OVERRIDE,
                                    g_param_spec_boolean ("user-override",
                                                          "User override",
                                                          "Whether the widget is in 'user override' mode",
                                                          FALSE,
                                                          GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_SEPARATORS,
                                    g_param_spec_string ("separators",
                                                         "Separators",
@@ -242,18 +247,21 @@ gimp_number_pair_entry_class_init (GimpNumberPairEntryClass *klass)
                                                         NULL,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
+
   g_object_class_install_property (object_class, PROP_DEFAULT_TEXT,
                                    g_param_spec_string ("default-text",
                                                         "Default text",
                                                         "String to show when in automatic mode",
                                                         NULL,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_ALLOW_SIMPLIFICATION,
                                    g_param_spec_boolean ("allow-simplification",
                                                          "Allow simplification",
                                                          "Whether to allow simplification",
                                                          FALSE,
                                                          GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_MIN_VALID_VALUE,
                                    g_param_spec_double ("min-valid-value",
                                                         "Min valid value",
@@ -261,6 +269,7 @@ gimp_number_pair_entry_class_init (GimpNumberPairEntryClass *klass)
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         G_MINDOUBLE,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_MAX_VALID_VALUE,
                                    g_param_spec_double ("max-valid-value",
                                                         "Max valid value",
@@ -268,15 +277,19 @@ gimp_number_pair_entry_class_init (GimpNumberPairEntryClass *klass)
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         G_MAXDOUBLE,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_RATIO,
                                    g_param_spec_double ("ratio",
-                                                        "Ratio", NULL,
+                                                        "Ratio",
+                                                        "The value as ratio",
                                                         G_MINDOUBLE, G_MAXDOUBLE,
                                                         1.0,
                                                         GIMP_PARAM_READWRITE));
+
   g_object_class_install_property (object_class, PROP_ASPECT,
                                    g_param_spec_enum ("aspect",
-                                                      "Aspect", NULL,
+                                                      "Aspect",
+                                                      "The value as aspect",
                                                       GIMP_TYPE_ASPECT_TYPE,
                                                       GIMP_ASPECT_SQUARE,
                                                       GIMP_PARAM_READWRITE));
@@ -285,30 +298,25 @@ gimp_number_pair_entry_class_init (GimpNumberPairEntryClass *klass)
 static void
 gimp_number_pair_entry_init (GimpNumberPairEntry *entry)
 {
-  GimpNumberPairEntryPrivate *priv;
-
-  entry->priv = G_TYPE_INSTANCE_GET_PRIVATE (entry,
-                                             GIMP_TYPE_NUMBER_PAIR_ENTRY,
-                                             GimpNumberPairEntryPrivate);
-
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  priv->left_number          = 1.0;
-  priv->right_number         = 1.0;
-  priv->default_left_number  = 1.0;
-  priv->default_right_number = 1.0;
-  priv->user_override        = FALSE;
-  priv->font_italic          = FALSE;
-  priv->separators           = NULL;
-  priv->default_text         = NULL;
-  priv->num_separators       = 0;
-  priv->allow_simplification = FALSE;
-  priv->min_valid_value      = G_MINDOUBLE;
-  priv->max_valid_value      = G_MAXDOUBLE;
+  entry->left_number          = 1.0;
+  entry->right_number         = 1.0;
+  entry->default_left_number  = 1.0;
+  entry->default_right_number = 1.0;
+  entry->user_override        = FALSE;
+  entry->font_italic          = FALSE;
+  entry->separators           = NULL;
+  entry->default_text         = NULL;
+  entry->num_separators       = 0;
+  entry->allow_simplification = FALSE;
+  entry->min_valid_value      = G_MINDOUBLE;
+  entry->max_valid_value      = G_MAXDOUBLE;
 
   g_signal_connect (entry, "changed",
                     G_CALLBACK (gimp_number_pair_entry_changed),
                     NULL);
+  g_signal_connect (entry, "icon-press",
+                    G_CALLBACK (gimp_number_pair_entry_icon_press),
+                    NULL);
   g_signal_connect (entry, "focus-out-event",
                     G_CALLBACK (gimp_number_pair_entry_events),
                     NULL);
@@ -317,27 +325,21 @@ gimp_number_pair_entry_init (GimpNumberPairEntry *entry)
                     NULL);
 
   gtk_widget_set_direction (GTK_WIDGET (entry), GTK_TEXT_DIR_LTR);
+
+  gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
+                                     GTK_ENTRY_ICON_SECONDARY,
+                                     GIMP_ICON_EDIT_CLEAR);
 }
 
 static void
 gimp_number_pair_entry_finalize (GObject *object)
 {
-  GimpNumberPairEntryPrivate *priv;
+  GimpNumberPairEntry *entry = GIMP_NUMBER_PAIR_ENTRY (object);
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (object);
+  g_clear_pointer (&entry->separators, g_free);
+  entry->num_separators = 0;
 
-  if (priv->separators)
-    {
-      g_free (priv->separators);
-      priv->separators     = NULL;
-      priv->num_separators = 0;
-    }
-
-  if (priv->default_text)
-    {
-      g_free (priv->default_text);
-      priv->default_text = NULL;
-    }
+  g_clear_pointer (&entry->default_text, g_free);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -367,9 +369,9 @@ gimp_number_pair_entry_finalize (GObject *object)
  * The first separator of @separators is used to display the current
  * value.
  *
- * Return value: The new #GimpNumberPairEntry widget.
+ * Returns: The new #GimpNumberPairEntry widget.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 GtkWidget *
 gimp_number_pair_entry_new (const gchar *separators,
@@ -447,7 +449,7 @@ gimp_number_pair_entry_ratio_to_fraction (gdouble  ratio,
  * An attempt is made to convert the decimal number into a fraction
  * with left_number and right_number < 1000.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_set_ratio (GimpNumberPairEntry *entry,
@@ -471,18 +473,14 @@ gimp_number_pair_entry_set_ratio (GimpNumberPairEntry *entry,
  *
  * Returns: The ratio value.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 gdouble
 gimp_number_pair_entry_get_ratio (GimpNumberPairEntry *entry)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_val_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry), 1.0);
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  return priv->left_number / priv->right_number;
+  return entry->left_number / entry->right_number;
 }
 
 /**
@@ -492,32 +490,29 @@ gimp_number_pair_entry_get_ratio (GimpNumberPairEntry *entry)
  * @right: Right number in the entry.
  *
  * Forces setting the numbers displayed by a #GimpNumberPairEntry,
- * ignoring if the user has set his/her own value. The state of
+ * ignoring if the user has set their own value. The state of
  * user-override will not be changed.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_set_values (GimpNumberPairEntry *entry,
                                    gdouble              left,
                                    gdouble              right)
 {
-  GimpNumberPairEntryPrivate *priv;
-  GimpAspectType              old_aspect;
-  gdouble                     old_ratio;
-  gdouble                     old_left_number;
-  gdouble                     old_right_number;
-  gboolean                    numbers_changed = FALSE;
-  gboolean                    ratio_changed   = FALSE;
+  GimpAspectType old_aspect;
+  gdouble        old_ratio;
+  gdouble        old_left_number;
+  gdouble        old_right_number;
+  gboolean       numbers_changed = FALSE;
+  gboolean       ratio_changed   = FALSE;
 
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
   /* Store current values */
 
-  old_left_number  = priv->left_number;
-  old_right_number = priv->right_number;
+  old_left_number  = entry->left_number;
+  old_right_number = entry->right_number;
   old_ratio        = gimp_number_pair_entry_get_ratio (entry);
   old_aspect       = gimp_number_pair_entry_get_aspect (entry);
 
@@ -529,8 +524,8 @@ gimp_number_pair_entry_set_values (GimpNumberPairEntry *entry,
 
   /* Set the new numbers and update the entry */
 
-  priv->left_number  = left;
-  priv->right_number = right;
+  entry->left_number  = left;
+  entry->right_number = right;
 
   g_object_notify (G_OBJECT (entry), "left-number");
   g_object_notify (G_OBJECT (entry), "right-number");
@@ -550,8 +545,8 @@ gimp_number_pair_entry_set_values (GimpNumberPairEntry *entry,
         g_object_notify (G_OBJECT (entry), "aspect");
     }
 
-  if (old_left_number  != priv->left_number ||
-      old_right_number != priv->right_number)
+  if (old_left_number  != entry->left_number ||
+      old_right_number != entry->right_number)
     {
       numbers_changed = TRUE;
     }
@@ -574,29 +569,25 @@ gimp_number_pair_entry_set_values (GimpNumberPairEntry *entry,
 /**
  * gimp_number_pair_entry_get_values:
  * @entry: A #GimpNumberPairEntry widget.
- * @left:  Pointer of where to store the left number (may be %NULL).
- * @right: Pointer of to store the right number (may be %NULL).
+ * @left:  (out) (optional): Location to store the left number (may be %NULL).
+ * @right: (out) (optional): Location to store the right number (may be %NULL).
  *
  * Gets the numbers displayed by a #GimpNumberPairEntry.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_get_values (GimpNumberPairEntry *entry,
                                    gdouble             *left,
                                    gdouble             *right)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
   if (left != NULL)
-    *left  = priv->left_number;
+    *left  = entry->left_number;
 
   if (right != NULL)
-    *right = priv->right_number;
+    *right = entry->right_number;
 }
 
 /**
@@ -612,20 +603,16 @@ gimp_number_pair_entry_get_values (GimpNumberPairEntry *entry,
  * Set the default string to %NULL to display default values as
  * normal.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 void
 gimp_number_pair_entry_set_default_text (GimpNumberPairEntry *entry,
                                          const gchar         *string)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  g_free (priv->default_text);
-  priv->default_text = g_strdup (string);
+  g_free (entry->default_text);
+  entry->default_text = g_strdup (string);
 
   gimp_number_pair_entry_update_text (entry);
 
@@ -636,21 +623,17 @@ gimp_number_pair_entry_set_default_text (GimpNumberPairEntry *entry,
  * gimp_number_pair_entry_get_default_text:
  * @entry:  A #GimpNumberPairEntry widget.
  *
- * Returns: the string manully set to be shown, or %NULL if values are
- *          shown in a normal fashion.
+ * Returns: (nullable): the string manually set to be shown,
+ *          or %NULL if values are shown in a normal fashion.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 const gchar *
 gimp_number_pair_entry_get_default_text (GimpNumberPairEntry *entry)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_val_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry), NULL);
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  return priv->default_text;
+  return entry->default_text;
 }
 
 /**
@@ -662,34 +645,30 @@ gimp_number_pair_entry_get_default_text (GimpNumberPairEntry *entry)
  * right_number if necessary (or setting them to 1.0 in case that
  * @aspect is %GIMP_ASPECT_SQUARE).
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_set_aspect (GimpNumberPairEntry *entry,
                                    GimpAspectType       aspect)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
   if (gimp_number_pair_entry_get_aspect (entry) == aspect)
     return;
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
   switch (aspect)
     {
     case GIMP_ASPECT_SQUARE:
       gimp_number_pair_entry_set_values (entry,
-                                         priv->left_number,
-                                         priv->left_number);
+                                         entry->left_number,
+                                         entry->left_number);
       break;
 
     case GIMP_ASPECT_LANDSCAPE:
     case GIMP_ASPECT_PORTRAIT:
       gimp_number_pair_entry_set_values (entry,
-                                         priv->right_number,
-                                         priv->left_number);
+                                         entry->right_number,
+                                         entry->left_number);
       break;
     }
 }
@@ -702,22 +681,18 @@ gimp_number_pair_entry_set_aspect (GimpNumberPairEntry *entry,
  *
  * Returns: The entry's current aspect.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 GimpAspectType
 gimp_number_pair_entry_get_aspect (GimpNumberPairEntry *entry)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_val_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry), GIMP_ASPECT_SQUARE);
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  if (priv->left_number > priv->right_number)
+  if (entry->left_number > entry->right_number)
     {
       return GIMP_ASPECT_LANDSCAPE;
     }
-  else if (priv->left_number < priv->right_number)
+  else if (entry->left_number < entry->right_number)
     {
       return GIMP_ASPECT_PORTRAIT;
     }
@@ -731,32 +706,23 @@ static void
 gimp_number_pair_entry_modify_font (GimpNumberPairEntry *entry,
                                     gboolean             italic)
 {
-  GimpNumberPairEntryPrivate *priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-  GtkRcStyle                 *rc_style;
+  GtkStyleContext *style;
 
-  if (priv->font_italic == italic)
+  if (entry->font_italic == italic)
     return;
 
-  rc_style = gtk_widget_get_modifier_style (GTK_WIDGET (entry));
+  style = gtk_widget_get_style_context (GTK_WIDGET (entry));
 
-  if (! rc_style->font_desc)
-    {
-      PangoContext         *context;
-      PangoFontDescription *font_desc;
+  if (italic)
+    gtk_style_context_add_class (style, "italic");
+  else
+    gtk_style_context_remove_class (style, "italic");
 
-      context = gtk_widget_get_pango_context (GTK_WIDGET (entry));
-      font_desc = pango_context_get_font_description (context);
+  gtk_entry_set_icon_sensitive (GTK_ENTRY (entry),
+                                GTK_ENTRY_ICON_SECONDARY,
+                                ! italic);
 
-      rc_style->font_desc = pango_font_description_copy (font_desc);
-    }
-
-  pango_font_description_set_style (rc_style->font_desc,
-                                    italic ?
-                                    PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
-
-  gtk_widget_modify_style (GTK_WIDGET (entry), rc_style);
-
-  priv->font_italic = italic;
+  entry->font_italic = italic;
 }
 
 
@@ -770,25 +736,21 @@ gimp_number_pair_entry_modify_font (GimpNumberPairEntry *entry,
  * change when the default values are changed. When in user overridden
  * mode, setting default values will not affect the active values.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_set_user_override (GimpNumberPairEntry *entry,
                                           gboolean             user_override)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  priv->user_override = user_override;
+  entry->user_override = user_override;
 
   if (! user_override)
     {
       gimp_number_pair_entry_set_default_values (entry,
-                                                 priv->default_left_number,
-                                                 priv->default_right_number);
+                                                 entry->default_left_number,
+                                                 entry->default_right_number);
     }
 
   gimp_number_pair_entry_modify_font (entry, ! user_override);
@@ -800,20 +762,16 @@ gimp_number_pair_entry_set_user_override (GimpNumberPairEntry *entry,
  * gimp_number_pair_entry_get_user_override:
  * @entry: A #GimpNumberPairEntry widget.
  *
- * Returns: Wether or not the the widget is in user overridden mode.
+ * Returns: Whether or not the the widget is in user overridden mode.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 gboolean
 gimp_number_pair_entry_get_user_override (GimpNumberPairEntry *entry)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_val_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry), FALSE);
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  return priv->user_override;
+  return entry->user_override;
 }
 
 static void
@@ -822,16 +780,21 @@ gimp_number_pair_entry_changed (GimpNumberPairEntry *entry)
   gimp_number_pair_entry_modify_font (entry, FALSE);
 }
 
+static void
+gimp_number_pair_entry_icon_press (GimpNumberPairEntry *entry)
+{
+  gimp_number_pair_entry_set_user_override (entry, FALSE);
+
+  gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+}
+
 static gboolean
 gimp_number_pair_entry_events (GtkWidget *widget,
                                GdkEvent  *event)
 {
-  GimpNumberPairEntry        *entry;
-  GimpNumberPairEntryPrivate *priv;
-  gboolean                    force_user_override;
+  GimpNumberPairEntry *entry = GIMP_NUMBER_PAIR_ENTRY (widget);
+  gboolean             force_user_override;
 
-  entry               = GIMP_NUMBER_PAIR_ENTRY (widget);
-  priv                = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
   force_user_override = FALSE;
 
   switch (event->type)
@@ -870,35 +833,28 @@ gimp_number_pair_entry_events (GtkWidget *widget,
           {
           case PARSE_VALID:
             {
-              if (priv->left_number  != left_value  ||
-                  priv->right_number != right_value ||
+              if (entry->left_number  != left_value  ||
+                  entry->right_number != right_value ||
                   force_user_override)
                 {
                   gimp_number_pair_entry_set_values (entry,
                                                      left_value,
                                                      right_value);
 
-                  priv->user_override = TRUE;
-                  g_object_notify (G_OBJECT (entry), "user-override");
+                  gimp_number_pair_entry_set_user_override (entry, TRUE);
                 }
             }
             break;
 
           case PARSE_CLEAR:
-
-            gimp_number_pair_entry_set_values (entry,
-                                               priv->default_left_number,
-                                               priv->default_right_number);
-
-            priv->user_override = FALSE;
-            g_object_notify (G_OBJECT (entry), "user-override");
+            gimp_number_pair_entry_set_user_override (entry, FALSE);
             break;
 
           default:
             break;
           }
 
-        /* Mak sure the entry text is up to date */
+        /* Make sure the entry text is up to date */
 
         gimp_number_pair_entry_update_text (entry);
 
@@ -926,12 +882,11 @@ gimp_number_pair_entry_strdup_number_pair_string (GimpNumberPairEntry *entry,
                                                   gdouble              left_number,
                                                   gdouble              right_number)
 {
-  GimpNumberPairEntryPrivate *priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-  gchar                       sep[8];
-  gint                        len;
+  gchar sep[8];
+  gint  len;
 
-  if (priv->num_separators > 0)
-    len = g_unichar_to_utf8 (priv->separators[0], sep);
+  if (entry->num_separators > 0)
+    len = g_unichar_to_utf8 (entry->separators[0], sep);
   else
     len = g_unichar_to_utf8 (DEFAULT_SEPARATOR, sep);
 
@@ -943,22 +898,21 @@ gimp_number_pair_entry_strdup_number_pair_string (GimpNumberPairEntry *entry,
 static void
 gimp_number_pair_entry_update_text (GimpNumberPairEntry *entry)
 {
-  GimpNumberPairEntryPrivate *priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-  gchar                      *buffer;
+  gchar *buffer;
 
-  if (! priv->user_override &&
-      priv->default_text != NULL)
+  if (! entry->user_override &&
+      entry->default_text != NULL)
     {
       /* Instead of the numbers, show the string explicitly set by a
        * client to show when in automatic mode.
        */
-      buffer = g_strdup (priv->default_text);
+      buffer = g_strdup (entry->default_text);
     }
   else
     {
       buffer = gimp_number_pair_entry_strdup_number_pair_string (entry,
-                                                                 priv->left_number,
-                                                                 priv->right_number);
+                                                                 entry->left_number,
+                                                                 entry->right_number);
     }
 
   g_signal_handlers_block_by_func (entry,
@@ -969,20 +923,20 @@ gimp_number_pair_entry_update_text (GimpNumberPairEntry *entry)
 
   g_signal_handlers_unblock_by_func (entry,
                                      gimp_number_pair_entry_changed, NULL);
+
+  gimp_number_pair_entry_modify_font (entry, ! entry->user_override);
 }
 
 static gboolean
 gimp_number_pair_entry_valid_separator (GimpNumberPairEntry *entry,
                                         gunichar             candidate)
 {
-  GimpNumberPairEntryPrivate *priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  if (priv->num_separators > 0)
+  if (entry->num_separators > 0)
     {
       gint i;
 
-      for (i = 0; i < priv->num_separators; i++)
-        if (priv->separators[i] == candidate)
+      for (i = 0; i < entry->num_separators; i++)
+        if (entry->separators[i] == candidate)
           return TRUE;
     }
   else if (candidate == DEFAULT_SEPARATOR)
@@ -999,18 +953,24 @@ gimp_number_pair_entry_parse_text (GimpNumberPairEntry *entry,
                                    gdouble             *left_value,
                                    gdouble             *right_value)
 {
-  GimpNumberPairEntryPrivate *priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
   gdouble   new_left_number;
   gdouble   new_right_number;
   gboolean  simplify = FALSE;
   gchar    *end;
 
+  /* skip over whitespace */
+  while (g_unichar_isspace (g_utf8_get_char (text)))
+    text = g_utf8_next_char (text);
+
+  /* check if clear */
+  if (! *text)
+    return PARSE_CLEAR;
+
   /* try to parse a number */
   new_left_number = strtod (text, &end);
 
   if (end == text)
-    return PARSE_CLEAR;
+    return PARSE_INVALID;
   else
     text = end;
 
@@ -1039,7 +999,7 @@ gimp_number_pair_entry_parse_text (GimpNumberPairEntry *entry,
   /* check for the simplification char */
   if (g_utf8_get_char (text) == SIMPLIFICATION_CHAR)
     {
-      simplify = priv->allow_simplification;
+      simplify = entry->allow_simplification;
       text = g_utf8_next_char (text);
     }
 
@@ -1079,30 +1039,27 @@ gimp_number_pair_entry_set_property (GObject      *object,
                                      GParamSpec   *pspec)
 {
   GimpNumberPairEntry        *entry = GIMP_NUMBER_PAIR_ENTRY (object);
-  GimpNumberPairEntryPrivate *priv;
-
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
 
   switch (property_id)
     {
     case PROP_LEFT_NUMBER:
       gimp_number_pair_entry_set_values (entry,
                                          g_value_get_double (value),
-                                         priv->right_number);
+                                         entry->right_number);
       break;
     case PROP_RIGHT_NUMBER:
       gimp_number_pair_entry_set_values (entry,
-                                         priv->left_number,
+                                         entry->left_number,
                                          g_value_get_double (value));
       break;
     case PROP_DEFAULT_LEFT_NUMBER:
       gimp_number_pair_entry_set_default_values (entry,
                                                  g_value_get_double (value),
-                                                 priv->default_right_number);
+                                                 entry->default_right_number);
       break;
     case PROP_DEFAULT_RIGHT_NUMBER:
       gimp_number_pair_entry_set_default_values (entry,
-                                                 priv->default_left_number,
+                                                 entry->default_left_number,
                                                  g_value_get_double (value));
       break;
     case PROP_USER_OVERRIDE:
@@ -1110,26 +1067,26 @@ gimp_number_pair_entry_set_property (GObject      *object,
                                                 g_value_get_boolean (value));
       break;
     case PROP_SEPARATORS:
-      g_free (priv->separators);
-      priv->num_separators = 0;
+      g_free (entry->separators);
+      entry->num_separators = 0;
       if (g_value_get_string (value))
-        priv->separators = g_utf8_to_ucs4 (g_value_get_string (value), -1,
-                                           NULL, &priv->num_separators, NULL);
+        entry->separators = g_utf8_to_ucs4 (g_value_get_string (value), -1,
+                                           NULL, &entry->num_separators, NULL);
       else
-        priv->separators = NULL;
+        entry->separators = NULL;
       break;
     case PROP_DEFAULT_TEXT:
       gimp_number_pair_entry_set_default_text (entry,
                                                g_value_get_string (value));
       break;
     case PROP_ALLOW_SIMPLIFICATION:
-      priv->allow_simplification = g_value_get_boolean (value);
+      entry->allow_simplification = g_value_get_boolean (value);
       break;
     case PROP_MIN_VALID_VALUE:
-      priv->min_valid_value = g_value_get_double (value);
+      entry->min_valid_value = g_value_get_double (value);
       break;
     case PROP_MAX_VALID_VALUE:
-      priv->max_valid_value = g_value_get_double (value);
+      entry->max_valid_value = g_value_get_double (value);
       break;
     case PROP_RATIO:
       gimp_number_pair_entry_set_ratio (entry, g_value_get_double (value));
@@ -1149,45 +1106,42 @@ gimp_number_pair_entry_get_property (GObject    *object,
                                      GValue     *value,
                                      GParamSpec *pspec)
 {
-  GimpNumberPairEntry        *entry = GIMP_NUMBER_PAIR_ENTRY (object);
-  GimpNumberPairEntryPrivate *priv;
-
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
+  GimpNumberPairEntry *entry = GIMP_NUMBER_PAIR_ENTRY (object);
 
   switch (property_id)
     {
     case PROP_LEFT_NUMBER:
-      g_value_set_double (value, priv->left_number);
+      g_value_set_double (value, entry->left_number);
       break;
     case PROP_RIGHT_NUMBER:
-      g_value_set_double (value, priv->right_number);
+      g_value_set_double (value, entry->right_number);
       break;
     case PROP_DEFAULT_LEFT_NUMBER:
-      g_value_set_double (value, priv->default_left_number);
+      g_value_set_double (value, entry->default_left_number);
       break;
     case PROP_DEFAULT_RIGHT_NUMBER:
-      g_value_set_double (value, priv->default_right_number);
+      g_value_set_double (value, entry->default_right_number);
       break;
     case PROP_USER_OVERRIDE:
-      g_value_set_boolean (value, priv->user_override);
+      g_value_set_boolean (value, entry->user_override);
       break;
     case PROP_SEPARATORS:
       g_value_take_string (value,
-                           g_ucs4_to_utf8 (priv->separators,
-                                           priv->num_separators,
+                           g_ucs4_to_utf8 (entry->separators,
+                                           entry->num_separators,
                                            NULL, NULL, NULL));
       break;
     case PROP_ALLOW_SIMPLIFICATION:
-      g_value_set_boolean (value, priv->allow_simplification);
+      g_value_set_boolean (value, entry->allow_simplification);
       break;
     case PROP_DEFAULT_TEXT:
-      g_value_set_string (value, priv->default_text);
+      g_value_set_string (value, entry->default_text);
       break;
     case PROP_MIN_VALID_VALUE:
-      g_value_set_double (value, priv->min_valid_value);
+      g_value_set_double (value, entry->min_valid_value);
       break;
     case PROP_MAX_VALID_VALUE:
-      g_value_set_double (value, priv->max_valid_value);
+      g_value_set_double (value, entry->max_valid_value);
       break;
     case PROP_RATIO:
       g_value_set_double (value, gimp_number_pair_entry_get_ratio (entry));
@@ -1207,54 +1161,46 @@ gimp_number_pair_entry_get_property (GObject    *object,
  * @left:  Default left value in the entry.
  * @right: Default right value in the entry.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_set_default_values (GimpNumberPairEntry *entry,
                                            gdouble              left,
                                            gdouble              right)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
+  entry->default_left_number  = left;
+  entry->default_right_number = right;
 
-  priv->default_left_number  = left;
-  priv->default_right_number = right;
-
-  if (! priv->user_override)
+  if (! entry->user_override)
     {
       gimp_number_pair_entry_set_values (entry,
-                                         priv->default_left_number,
-                                         priv->default_right_number);
+                                         entry->default_left_number,
+                                         entry->default_right_number);
     }
 }
 
 /**
  * gimp_number_pair_entry_get_default_values:
  * @entry: A #GimpNumberPairEntry widget.
- * @left:  Pointer of where to put left value.
- * @right: Pointer of where to put right value.
+ * @left:  (out) (optional): Location to put the default left value, or %NULL.
+ * @right: (out) (optional): Location to put the default right value, or %NULL.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  **/
 void
 gimp_number_pair_entry_get_default_values (GimpNumberPairEntry *entry,
                                            gdouble             *left,
                                            gdouble             *right)
 {
-  GimpNumberPairEntryPrivate *priv;
-
   g_return_if_fail (GIMP_IS_NUMBER_PAIR_ENTRY (entry));
 
-  priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
   if (left != NULL)
-    *left = priv->default_left_number;
+    *left = entry->default_left_number;
 
   if (right != NULL)
-    *right = priv->default_right_number;
+    *right = entry->default_right_number;
 }
 
 static gboolean
@@ -1262,10 +1208,8 @@ gimp_number_pair_entry_numbers_in_range (GimpNumberPairEntry *entry,
                                          gdouble              left_number,
                                          gdouble              right_number)
 {
-  GimpNumberPairEntryPrivate *priv = GIMP_NUMBER_PAIR_ENTRY_GET_PRIVATE (entry);
-
-  return (left_number  >= priv->min_valid_value &&
-          left_number  <= priv->max_valid_value &&
-          right_number >= priv->min_valid_value &&
-          right_number <= priv->max_valid_value);
+  return (left_number  >= entry->min_valid_value &&
+          left_number  <= entry->max_valid_value &&
+          right_number >= entry->min_valid_value &&
+          right_number <= entry->max_valid_value);
 }
