@@ -22,12 +22,9 @@
 #ifndef __GIMP_WIDGETS_PRIVATE_H__
 #define __GIMP_WIDGETS_PRIVATE_H__
 
-/* Used to compare similar colors across several widgets. */
-#define GIMP_RGBA_EPSILON 1e-6
 
-
-typedef gboolean (* GimpGetColorFunc)      (GimpRGB *color);
-typedef void     (* GimpEnsureModulesFunc) (void);
+typedef GeglColor * (* GimpGetColorFunc)      (void);
+typedef void        (* GimpEnsureModulesFunc) (void);
 
 
 extern GimpHelpFunc          _gimp_standard_help_func;
