@@ -123,6 +123,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpImage **',
 		     const_type      => 'const GimpImage **',
+		     app_const_type  => 'GimpImage **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpImage) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpImage) (array zero-terminated=1) (transfer container)',
@@ -135,6 +136,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpItem **',
 		     const_type      => 'const GimpItem **',
+		     app_const_type  => 'GimpItem **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpItem) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpItem) (array zero-terminated=1) (transfer container)',
@@ -147,6 +149,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpDrawable **',
 		     const_type      => 'const GimpDrawable **',
+		     app_const_type  => 'GimpDrawable **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpDrawable) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpDrawable) (array zero-terminated=1) (transfer container)',
@@ -159,6 +162,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpLayer **',
 		     const_type      => 'const GimpLayer **',
+		     app_const_type  => 'GimpLayer **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpLayer) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpLayer) (array zero-terminated=1) (transfer container)',
@@ -167,12 +171,13 @@ package Gimp::CodeGen::pdb;
 		     set_value_func  => 'g_value_set_boxed ($value, $var)',
 		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
-    channelarray => { name            => 'CHANNELARRAY',
-		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
-		     type            => 'GimpChannel **',
-		     const_type      => 'const GimpChannel **',
-		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpChannel) (array zero-terminated=1)',
+    channelarray => { name           => 'CHANNELARRAY',
+		      gtype          => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
+		      type           => 'GimpChannel **',
+		      const_type     => 'const GimpChannel **',
+		      app_const_type => 'GimpChannel **',
+		      init_value     => 'NULL',
+		      in_annotate    => '(element-type GimpChannel) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpChannel) (array zero-terminated=1) (transfer container)',
 		     get_value_func  => '$var = g_value_get_boxed ($value)',
 		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
@@ -180,11 +185,12 @@ package Gimp::CodeGen::pdb;
 		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     patharray    => { name            => 'PATHARRAY',
-		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
-		     type            => 'GimpPath **',
-		     const_type      => 'const GimpPath **',
-		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpPath) (array zero-terminated=1)',
+		      gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
+		      type            => 'GimpPath **',
+		      const_type      => 'const GimpPath **',
+		      app_const_type  => 'GimpPath **',
+		      init_value      => 'NULL',
+		      in_annotate     => '(element-type GimpPath) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpPath) (array zero-terminated=1) (transfer container)',
 		     get_value_func  => '$var = g_value_get_boxed ($value)',
 		     dup_value_func  => '$var = g_value_dup_boxed (gimp_value_array_index ($value))',
@@ -195,6 +201,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpDrawableFilter **',
 		     const_type      => 'const GimpDrawableFilter **',
+		     app_const_type  => 'GimpDrawableFilter **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpDrawableFilter) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpDrawableFilter) (array zero-terminated=1) (transfer container)',
@@ -207,6 +214,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpResource **',
 		     const_type      => 'const GimpResource **',
+		     app_const_type  => 'GimpResource **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpResource) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpResource) (array zero-terminated=1) (transfer container)',
@@ -231,6 +239,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpFont **',
 		     const_type      => 'const GimpFont **',
+		     app_const_type  => 'GimpFont **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpFont) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpFont) (array zero-terminated=1) (transfer container)',
@@ -243,6 +252,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpGradient **',
 		     const_type      => 'const GimpGradient **',
+		     app_const_type  => 'GimpGradient **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpGradient) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpGradient) (array zero-terminated=1) (transfer container)',
@@ -255,6 +265,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpPalette **',
 		     const_type      => 'const GimpPalette **',
+		     app_const_type  => 'GimpPalette **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpPalette) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpPalette) (array zero-terminated=1) (transfer container)',
@@ -267,6 +278,7 @@ package Gimp::CodeGen::pdb;
 		     gtype           => 'GIMP_TYPE_CORE_OBJECT_ARRAY',
 		     type            => 'GimpPattern **',
 		     const_type      => 'const GimpPattern **',
+		     app_const_type  => 'GimpPattern **',
 		     init_value      => 'NULL',
 		     in_annotate     => '(element-type GimpPattern) (array zero-terminated=1)',
 		     out_annotate    => '(element-type GimpPattern) (array zero-terminated=1) (transfer container)',
@@ -452,7 +464,7 @@ package Gimp::CodeGen::pdb;
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_DUP_FILE ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
-		     take_value_func => 'g_value_set_object ($value, $var)' },
+		     take_value_func => 'g_value_take_object ($value, $var)' },
 
     parasite    => { name            => 'PARASITE',
 		     gtype           => 'GIMP_TYPE_PARASITE',
