@@ -46,7 +46,7 @@ struct _CheckerboardClass
 
 
 #define CHECKERBOARD_TYPE  (checkerboard_get_type ())
-#define CHECKERBOARD (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CHECKERBOARD_TYPE, Checkerboard))
+#define CHECKERBOARD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CHECKERBOARD_TYPE, Checkerboard))
 
 GType                   checkerboard_get_type         (void) G_GNUC_CONST;
 
@@ -57,9 +57,8 @@ static GimpProcedure  * checkerboard_create_procedure (GimpPlugIn           *plu
 static GimpValueArray * checkerboard_run              (GimpProcedure        *procedure,
                                                        GimpRunMode           run_mode,
                                                        GimpImage            *image,
-                                                       gint                  n_drawables,
                                                        GimpDrawable        **drawables,
-                                                       const GimpValueArray *args,
+                                                       GimpProcedureConfig  *config,
                                                        gpointer              run_data);
 
 static void             do_checkerboard_pattern       (GObject       *config,
@@ -133,23 +132,23 @@ checkerboard_create_procedure (GimpPlugIn  *plug_in,
                                       "Brent Burton & the Edward Blevins",
                                       "1997");
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "psychobily",
-                             _("_Psychobilly"),
-                             _("Render a psychobilly checkerboard"),
-                             FALSE,
-                             G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, "psychobilly",
+                                           _("_Psychobilly"),
+                                           _("Render a psychobilly checkerboard"),
+                                           FALSE,
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "check-size",
-                         _("_Size"),
-                         _("Size of the checks"),
-                         1, GIMP_MAX_IMAGE_SIZE, 10,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "check-size",
+                                       _("_Size"),
+                                       _("Size of the checks"),
+                                       1, GIMP_MAX_IMAGE_SIZE, 10,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_AUX_ARG_UNIT (procedure, "check-size-unit",
-                              _("Check size unit of measure"),
-                              _("Check size unit of measure"),
-                              TRUE, TRUE, GIMP_UNIT_PIXEL,
-                              GIMP_PARAM_READWRITE);
+      gimp_procedure_add_unit_aux_argument (procedure, "check-size-unit",
+                                            _("Check size unit of measure"),
+                                            _("Check size unit of measure"),
+                                            TRUE, TRUE, gimp_unit_pixel (),
+                                            GIMP_PARAM_READWRITE);
     }
 
   return procedure;
@@ -159,17 +158,15 @@ static GimpValueArray *
 checkerboard_run (GimpProcedure        *procedure,
                   GimpRunMode           run_mode,
                   GimpImage            *image,
-                  gint                  n_drawables,
                   GimpDrawable        **drawables,
-                  const GimpValueArray *args,
+                  GimpProcedureConfig  *config,
                   gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
+  GimpDrawable *drawable;
 
   gegl_init (NULL, NULL);
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
@@ -186,27 +183,11 @@ checkerboard_run (GimpProcedure        *procedure,
       drawable = drawables[0];
     }
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-      if (! checkerboard_dialog (procedure, G_OBJECT (config), image,
-                                 drawable))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-      break;
-
-    default:
-      break;
-    }
+  if (run_mode == GIMP_RUN_INTERACTIVE &&
+      ! checkerboard_dialog (procedure, G_OBJECT (config), image, drawable))
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_CANCEL,
+                                             NULL);
 
   if (gimp_drawable_is_rgb  (drawable) ||
       gimp_drawable_is_gray (drawable))
@@ -218,17 +199,11 @@ checkerboard_run (GimpProcedure        *procedure,
     }
   else
     {
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_EXECUTION_ERROR,
                                                NULL);
     }
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -280,7 +255,11 @@ do_checkerboard_pattern (GObject      *config,
                          GimpPreview  *preview)
 {
   CheckerboardParam_t  param;
-  GimpRGB              fg, bg;
+  GeglColor           *color;
+  guchar               fg[4]     = {0, 0, 0, 0};
+  guchar               bg[4]     = {0, 0, 0, 0};
+  guchar               fg_lum[1] = {0};
+  guchar               bg_lum[1] = {0};
   const Babl          *format;
   gint                 bpp;
   gboolean             mode = FALSE;
@@ -288,39 +267,45 @@ do_checkerboard_pattern (GObject      *config,
 
   if (config)
     g_object_get (config,
-                  "check-size", &size,
-                  "psychobily", &mode,
+                  "check-size",  &size,
+                  "psychobilly", &mode,
                   NULL);
 
-  gimp_context_get_background (&bg);
-  gimp_context_get_foreground (&fg);
+  color = gimp_context_get_background ();
+  gegl_color_get_pixel (color, babl_format_with_space ("R'G'B'A u8", NULL), bg);
+  gegl_color_get_pixel (color, babl_format_with_space ("Y' u8", NULL), bg_lum);
+  g_object_unref (color);
+  color = gimp_context_get_foreground ();
+  gegl_color_get_pixel (color, babl_format_with_space ("R'G'B'A u8", NULL), fg);
+  gegl_color_get_pixel (color, babl_format_with_space ("Y' u8", NULL), fg_lum);
+  g_object_unref (color);
 
   if (gimp_drawable_is_gray (drawable))
     {
-      param.bg[0] = gimp_rgb_luminance_uchar (&bg);
-      gimp_rgba_get_uchar (&bg, NULL, NULL, NULL, param.bg + 1);
+      param.bg[0] = bg_lum[0];
+      param.bg[1] = bg[3];
 
-      param.fg[0] = gimp_rgb_luminance_uchar (&fg);
-      gimp_rgba_get_uchar (&fg, NULL, NULL, NULL, param.fg + 3);
-
-      if (gimp_drawable_has_alpha (drawable))
-        format = babl_format ("R'G'B'A u8");
-      else
-        format = babl_format ("R'G'B' u8");
-    }
-  else
-    {
-      gimp_rgba_get_uchar (&bg,
-                           param.bg, param.bg + 1, param.bg + 2, param.bg + 1);
-
-      gimp_rgba_get_uchar (&fg,
-                           param.fg, param.fg + 1, param.fg + 2, param.fg + 3);
+      param.fg[0] = fg_lum[0];
+      param.fg[1] = fg[3];
 
       if (gimp_drawable_has_alpha (drawable))
         format = babl_format ("Y'A u8");
       else
         format = babl_format ("Y' u8");
     }
+  else
+    {
+      for (gint i = 0; i < 4; i++)
+        {
+          param.bg[i] = bg[i];
+          param.fg[i] = fg[i];
+        }
+
+      if (gimp_drawable_has_alpha (drawable))
+        format = babl_format ("R'G'B'A u8");
+      else
+        format = babl_format ("R'G'B' u8");
+    }
 
   bpp = babl_format_get_bytes_per_pixel (format);
 
@@ -506,11 +491,6 @@ checkerboard_dialog (GimpProcedure *procedure,
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
-  preview = gimp_drawable_preview_new_from_drawable (drawable);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      preview, TRUE, TRUE, 0);
-  gtk_widget_show (preview);
-
   /*  Get the image resolution and unit  */
   gimp_image_get_resolution (image, &xres, &yres);
 
@@ -534,10 +514,12 @@ checkerboard_dialog (GimpProcedure *procedure,
                                          1.0, size);
 
   toggle = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
-                                             "psychobily",
+                                             "psychobilly",
                                              GTK_TYPE_CHECK_BUTTON);
   gtk_widget_set_margin_bottom (toggle, 12);
 
+  preview = gimp_procedure_dialog_get_drawable_preview (GIMP_PROCEDURE_DIALOG (dialog),
+                                                        "preview", drawable);
   g_object_set_data (config, "drawable", drawable);
 
   g_signal_connect (preview, "invalidated",
@@ -549,7 +531,7 @@ checkerboard_dialog (GimpProcedure *procedure,
                             preview);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
-                              "check-size", "psychobily",
+                              "preview", "check-size", "psychobilly",
                               NULL);
 
   gtk_widget_show (dialog);
