@@ -98,28 +98,30 @@ struct _GifClass
 
 
 #define GIF_TYPE  (gif_get_type ())
-#define GIF (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIF_TYPE, Gif))
+#define GIF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GIF_TYPE, Gif))
 
 GType                   gif_get_type         (void) G_GNUC_CONST;
 
-static GList          * gif_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * gif_create_procedure (GimpPlugIn           *plug_in,
-                                              const gchar          *name);
+static GList          * gif_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * gif_create_procedure (GimpPlugIn            *plug_in,
+                                              const gchar           *name);
 
-static GimpValueArray * gif_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
-static GimpValueArray * gif_load_thumb       (GimpProcedure        *procedure,
-                                              GFile                *file,
-                                              gint                  size,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
+static GimpValueArray * gif_load             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpMetadataLoadFlags *flags,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
+static GimpValueArray * gif_load_thumb       (GimpProcedure         *procedure,
+                                              GFile                 *file,
+                                              gint                   size,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
 
-static GimpImage      * load_image           (GFile                *file,
-                                              gboolean              thumbnail,
-                                              GError              **error);
+static GimpImage      * load_image           (GFile                 *file,
+                                              gboolean               thumbnail,
+                                              GError               **error);
 
 
 G_DEFINE_TYPE (Gif, gif, GIMP_TYPE_PLUG_IN)
@@ -177,8 +179,7 @@ gif_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_documentation (procedure,
                                         "Loads files of Compuserve GIF "
-                                        "file format",
-                                        "FIXME: write help for gif_load",
+                                        "file format", NULL,
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Spencer Kimball, Peter Mattis, "
@@ -220,11 +221,13 @@ gif_create_procedure (GimpPlugIn  *plug_in,
 
 
 static GimpValueArray *
-gif_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+gif_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -254,24 +257,25 @@ gif_load (GimpProcedure        *procedure,
 #endif
 
   if (! promote_to_rgb)
-    gimp_image_set_colormap (image,
-                             gimp_cmap, highest_used_index);
+    gimp_palette_set_colormap (gimp_image_get_palette (image), babl_format ("R'G'B' u8"),
+                               gimp_cmap, highest_used_index * 3);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
                                                   NULL);
 
+  *flags = GIMP_METADATA_LOAD_ALL;
   GIMP_VALUES_SET_IMAGE (return_vals, 1, image);
 
   return return_vals;
 }
 
 static GimpValueArray *
-gif_load_thumb (GimpProcedure        *procedure,
-                GFile                *file,
-                gint                  size,
-                const GimpValueArray *args,
-                gpointer              run_data)
+gif_load_thumb (GimpProcedure       *procedure,
+                GFile               *file,
+                gint                 size,
+                GimpProcedureConfig *config,
+                gpointer             run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -287,7 +291,7 @@ gif_load_thumb (GimpProcedure        *procedure,
                                              error);
 
   if (! promote_to_rgb)
-    gimp_image_set_colormap (image, gimp_cmap, highest_used_index);
+    gimp_palette_set_colormap (gimp_image_get_palette (image), babl_format ("R'G'B' u8"), gimp_cmap, highest_used_index * 3);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -348,7 +352,8 @@ static struct
   gint delayTime;
   gint inputFlag;
   gint disposal;
-} Gif89 = { -1, -1, -1, 0 };
+  gint num_loops;
+} Gif89 = { -1, -1, -1, 0, -1 };
 
 static void     read_error   (const gchar *error_type,
                               GimpImage   *image,
@@ -395,9 +400,10 @@ load_image (GFile     *file,
   gint       grayScale;
   gboolean   useGlobalColormap;
   gint       bitPixel;
-  gint       imageCount = 0;
-  GimpImage *image      = NULL;
+  gint       imageCount     = 0;
+  GimpImage *image          = NULL;
   gboolean   status;
+  gboolean   saved_parasite = FALSE;
 
   gimp_progress_init_printf (_("Opening '%s'"),
                              gimp_file_get_utf8_name (file));
@@ -461,11 +467,6 @@ load_image (GFile     *file,
         }
     }
 
-  if (GifScreen.AspectRatio != 0 && GifScreen.AspectRatio != 49)
-    {
-      g_message (_("Non-square pixels.  Image might look squashed."));
-    }
-
   highest_used_index = GifScreen.BitPixel;
 
   while (TRUE)
@@ -554,8 +555,29 @@ load_image (GFile     *file,
         }
 
       if (!status)
+        break;
+
+      if (GifScreen.AspectRatio != 0 && GifScreen.AspectRatio != 49)
         {
-          break;
+          /* Byte overflow is allowed per the GIF specification */
+          guint8 par = GifScreen.AspectRatio + 15;
+
+          if (par > 0)
+            {
+              gdouble image_xres;
+              gdouble image_yres;
+              gfloat  ratio = par / 64.0;
+
+              g_message (_("Non-square pixels. Image might look squashed if "
+                           "Dot for Dot mode is enabled."));
+
+              gimp_image_get_resolution (image, &image_xres, &image_yres);
+              if (ratio < 1)
+                image_xres = image_yres * (1 / ratio);
+              else
+                image_yres = image_xres * ratio;
+              gimp_image_set_resolution (image, image_xres, image_yres);
+            }
         }
 
       if (comment_parasite != NULL)
@@ -570,6 +592,25 @@ load_image (GFile     *file,
       /* If we are loading a thumbnail, we stop after the first frame. */
       if (thumbnail)
         break;
+
+      /* If there is more than one frame, we add a parasite so that
+       * we know to export as an animation on overwrite */
+      if (Gif89.num_loops > -1 && ! saved_parasite)
+        {
+          GimpParasite *parasite;
+          gchar        *str;
+
+          str = g_strdup_printf ("%d", Gif89.num_loops);
+          parasite = gimp_parasite_new ("gif/animated",
+                                        GIMP_PARASITE_PERSISTENT,
+                                        strlen (str) + 1,
+                                        (gpointer) str);
+          g_free (str);
+
+          gimp_image_attach_parasite (image, parasite);
+          gimp_parasite_free (parasite);
+          saved_parasite = TRUE;
+        }
     }
 
   fclose (fd);
@@ -670,6 +711,15 @@ DoExtension (FILE *fd,
 #ifdef GIFDEBUG
       str = "Application Extension";
 #endif
+      /* Animation block */
+      if (GetDataBlock (fd, (guchar *) buf))
+        {
+          if (strncmp ((const gchar *) buf, "NETSCAPE2.0", 8) == 0)
+            {
+              if (GetDataBlock (fd, (guchar *) buf))
+                Gif89.num_loops = (buf[0] << 8) | buf[1];
+            }
+        }
       break;
     case 0xfe:                  /* Comment Extension */
 #ifdef GIFDEBUG
@@ -1091,7 +1141,7 @@ ReadImage (FILE        *fd,
           used_cmap[2][i] = gimp_cmap[j++] = cmap[2][i];
         }
 
-      gimp_image_set_colormap (*image, gimp_cmap, ncols);
+      gimp_palette_set_colormap (gimp_image_get_palette (*image), babl_format ("R'G'B' u8"), gimp_cmap, ncols * 3);
 
       if (Gif89.delayTime < 0)
         framename = g_strdup (_("Background"));
