@@ -145,7 +145,7 @@ struct _HotClass
 
 
 #define HOT_TYPE  (hot_get_type ())
-#define HOT (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HOT_TYPE, Hot))
+#define HOT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), HOT_TYPE, Hot))
 
 GType                   hot_get_type         (void) G_GNUC_CONST;
 
@@ -156,9 +156,8 @@ static GimpProcedure  * hot_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * hot_run              (GimpProcedure        *procedure,
                                               GimpRunMode           run_mode,
                                               GimpImage            *image,
-                                              gint                  n_drawables,
                                               GimpDrawable        **drawables,
-                                              const GimpValueArray *args,
+                                              GimpProcedureConfig  *config,
                                               gpointer              run_data);
 
 static gboolean         pluginCore           (GimpImage            *image,
@@ -247,14 +246,14 @@ hot_create_procedure (GimpPlugIn  *plug_in,
                                            GIMP_PROCEDURE_SENSITIVE_DRAWABLE);
 
       gimp_procedure_set_menu_label (procedure, _("_Hot..."));
-      gimp_procedure_add_menu_path (procedure, "<Image>/Colors");
+      gimp_procedure_add_menu_path (procedure, "<Image>/Colors/[Modify]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Find and fix pixels that may "
                                           "be unsafely bright"),
                                         "hot scans an image for pixels that "
                                         "will give unsave values of "
-                                        "chrominance or composite signale "
+                                        "chrominance or composite signal "
                                         "amplitude when encoded into an NTSC "
                                         "or PAL signal. Three actions can be "
                                         "performed on these 'hot' pixels. "
@@ -266,24 +265,30 @@ hot_create_procedure (GimpPlugIn  *plug_in,
                                       "Eric L. Hernes",
                                       "1997");
 
-      GIMP_PROC_ARG_INT (procedure, "mode",
-                         _("Mode"),
-                         "Mode { NTSC (0), PAL (1) }",
-                         0, 1, MODE_NTSC,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "mode",
+                                          _("_Mode"),
+                                          _("Signal mode"),
+                                          gimp_choice_new_with_values ("ntsc", MODE_NTSC, _("NTSC"), NULL,
+                                                                       "pal",  MODE_PAL,  _("PAL"),   NULL,
+                                                                       NULL),
+                                          "ntsc",
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "action",
-                         _("Action"),
-                         "Action { (0) reduce luminance, "
-                         "(1) reduce saturation, or (2) Blacken }",
-                         0, 2, ACT_LREDUX,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "action",
+                                          _("_Action"),
+                                          _("Action"),
+                                          gimp_choice_new_with_values ("reduce-luminance",   ACT_LREDUX, _("Reduce Luminance"),  NULL,
+                                                                       "reduce-saturation",  ACT_SREDUX, _("Reduce Saturation"), NULL,
+                                                                       "blacken",            ACT_FLAG,   _("Blacken"),           NULL,
+                                                                       NULL),
+                                          "reduce-luminance",
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "new-layer",
-                             _("Create _new layer"),
-                             "Create a new layer",
-                             TRUE,
-                             G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, "new-layer",
+                                           _("Create _new layer"),
+                                           _("Create a new layer"),
+                                           TRUE,
+                                           G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -293,17 +298,15 @@ static GimpValueArray *
 hot_run (GimpProcedure        *procedure,
          GimpRunMode           run_mode,
          GimpImage            *image,
-         gint                  n_drawables,
          GimpDrawable        **drawables,
-         const GimpValueArray *args,
+         GimpProcedureConfig  *config,
          gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
+  GimpDrawable *drawable;
 
   gegl_init (NULL, NULL);
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
@@ -320,41 +323,15 @@ hot_run (GimpProcedure        *procedure,
       drawable = drawables[0];
     }
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-
-      if (! plugin_dialog (procedure, G_OBJECT (config)))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-      break;
-
-    case GIMP_RUN_NONINTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      break;
-    }
+  if (run_mode == GIMP_RUN_INTERACTIVE && ! plugin_dialog (procedure, G_OBJECT (config)))
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_CANCEL,
+                                             NULL);
 
   if (! pluginCore (image, drawable, G_OBJECT (config)))
-    {
-      gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-      g_object_unref (config);
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_EXECUTION_ERROR,
-                                               NULL);
-    }
-
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             NULL);
 
   if (run_mode != GIMP_RUN_NONINTERACTIVE)
     gimp_displays_flush ();
@@ -392,10 +369,12 @@ pluginCore (GimpImage    *image,
   gdouble     py;
 
   g_object_get (config,
-                "mode",      &mode,
-                "action",    &action,
                 "new-layer", &new_layer,
                 NULL);
+  mode = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                              "mode");
+  action = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                "action");
 
   width  = gimp_drawable_get_width  (drawable);
   height = gimp_drawable_get_height (drawable);
@@ -668,11 +647,10 @@ static gboolean
 plugin_dialog (GimpProcedure *procedure,
                GObject       *config)
 {
-  GtkWidget    *dlg;
-  GtkWidget    *vbox;
-  GtkWidget    *hbox;
-  GtkListStore *store;
-  gboolean      run;
+  GtkWidget  *dlg;
+  GtkWidget  *vbox;
+  GtkWidget  *hbox;
+  gboolean    run;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
@@ -680,25 +658,10 @@ plugin_dialog (GimpProcedure *procedure,
                                    GIMP_PROCEDURE_CONFIG (config),
                                    _("Hot"));
 
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dlg),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dlg));
-
-  store = gimp_int_store_new (_("N_TSC"), MODE_NTSC,
-                              _("_PAL"),  MODE_PAL,
-                              NULL);
-  gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dlg),
-                                       "mode", GIMP_INT_STORE (store));
-
-  store = gimp_int_store_new (_("Reduce _Luminance"),  ACT_LREDUX,
-                              _("Reduce _Saturation"), ACT_SREDUX,
-                              _("_Blacken"),           ACT_FLAG,
-                              NULL);
-  gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dlg),
-                                       "action", GIMP_INT_STORE (store));
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                    "mode", GIMP_TYPE_INT_RADIO_FRAME);
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dlg),
+                                    "action", GIMP_TYPE_INT_RADIO_FRAME);
 
   vbox = gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dlg),
                                          "hot-left-side",
