@@ -99,7 +99,7 @@ struct _QbistClass
 
 
 #define QBIST_TYPE  (qbist_get_type ())
-#define QBIST (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), QBIST_TYPE, Qbist))
+#define QBIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), QBIST_TYPE, Qbist))
 
 GType                   qbist_get_type         (void) G_GNUC_CONST;
 
@@ -110,12 +110,12 @@ static GimpProcedure  * qbist_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * qbist_run              (GimpProcedure        *procedure,
                                                 GimpRunMode           run_mode,
                                                 GimpImage            *image,
-                                                gint                  n_drawables,
                                                 GimpDrawable        **drawables,
-                                                const GimpValueArray *args,
+                                                GimpProcedureConfig  *config,
                                                 gpointer              run_data);
 
-static gboolean         dialog_run             (void);
+static gboolean         dialog_run             (GimpProcedure        *procedure,
+                                                GimpProcedureConfig  *config);
 static void             dialog_new_variations  (GtkWidget            *widget,
                                                 gpointer              data);
 static void             dialog_update_previews (GtkWidget            *widget,
@@ -189,34 +189,36 @@ qbist_create_procedure (GimpPlugIn  *plug_in,
       gimp_procedure_set_documentation (procedure,
                                         _("Generate a huge variety of "
                                           "abstract patterns"),
-                                        "This Plug-in is based on an article by "
-                                        "Jörn Loviscach (appeared in c't 10/95, "
-                                        "page 326). It generates modern art "
-                                        "pictures from a random genetic "
-                                        "formula.",
+                                        _("This Plug-in is based on an article by "
+                                          "Jörn Loviscach (appeared in c't 10/95, "
+                                          "page 326). It generates modern art "
+                                          "pictures from a random genetic "
+                                          "formula."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Jörn Loviscach, Jens Ch. Restemeier",
                                       "Jörn Loviscach, Jens Ch. Restemeier",
                                       PLUG_IN_VERSION);
 
+      gimp_procedure_add_boolean_argument (procedure, "anti-aliasing",
+                                           _("_Anti-aliasing"),
+                                           _("Enable anti-aliasing using an oversampling "
+                                             "algorithm"),
+                                           TRUE,
+                                           G_PARAM_READWRITE);
+
       /* Saving the pattern as a parasite is a trick allowing to store
        * random binary data.
        */
-      GIMP_PROC_AUX_ARG_PARASITE (procedure, "pattern",
-                                  "Qbist pattern", NULL,
-                                  G_PARAM_READWRITE);
+      gimp_procedure_add_parasite_aux_argument (procedure, "pattern",
+                                                "Qbist pattern", NULL,
+                                                G_PARAM_READWRITE);
 
-      GIMP_PROC_AUX_ARG_INT (procedure, "oversampling",
-                             "Oversampling", NULL,
-                             1, 4, 4,
-                             G_PARAM_READWRITE);
-
-      GIMP_PROC_AUX_ARG_STRING (procedure, "data-path",
-                                "Path of data file",
-                                _("Any file which will be used as source for pattern generation"),
-                                NULL,
-                                G_PARAM_READWRITE);
+     gimp_procedure_add_string_aux_argument (procedure, "data-path",
+                                              "Path of data file",
+                                              _("Any file which will be used as source for pattern generation"),
+                                              NULL,
+                                              G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -226,26 +228,25 @@ static GimpValueArray *
 qbist_run (GimpProcedure        *procedure,
            GimpRunMode           run_mode,
            GimpImage            *image,
-           gint                  n_drawables,
            GimpDrawable        **drawables,
-           const GimpValueArray *args,
+           GimpProcedureConfig  *config,
            gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  gint                 sel_x1, sel_y1, sel_width, sel_height;
-  gint                 img_height, img_width;
-  GeglBuffer          *buffer;
-  GeglBufferIterator  *iter;
-  GimpDrawable        *drawable;
-  GimpParasite        *pattern_parasite;
-  gconstpointer        pattern_data;
-  guint32              pattern_data_length;
-  gint                 total_pixels;
-  gint                 done_pixels;
+  gint                sel_x1, sel_y1, sel_width, sel_height;
+  gint                img_height, img_width;
+  GeglBuffer         *buffer;
+  GeglBufferIterator *iter;
+  GimpDrawable       *drawable;
+  GimpParasite       *pattern_parasite;
+  gconstpointer       pattern_data;
+  guint32             pattern_data_length;
+  gint                total_pixels;
+  gint                done_pixels;
+  gboolean            anti_aliasing = TRUE;
 
   gegl_init (NULL, NULL);
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
@@ -262,9 +263,6 @@ qbist_run (GimpProcedure        *procedure,
       drawable = drawables[0];
     }
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, image, run_mode, args);
-
   img_width  = gimp_drawable_get_width (drawable);
   img_height = gimp_drawable_get_height (drawable);
 
@@ -288,14 +286,12 @@ qbist_run (GimpProcedure        *procedure,
 
   memset (&qbist_info, 0, sizeof (qbist_info));
   create_info (&qbist_info.info);
-  qbist_info.oversampling = 4;
 
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
       g_object_get (config,
                     "data-path",    &qbist_info.path,
-                    "oversampling", &qbist_info.oversampling,
                     "pattern",      &pattern_parasite,
                     NULL);
       if (pattern_parasite)
@@ -307,28 +303,24 @@ qbist_run (GimpProcedure        *procedure,
           pattern_parasite = NULL;
         }
 
-      if (! dialog_run ())
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
+      if (! dialog_run (procedure, config))
+        return gimp_procedure_new_return_values (procedure,
+                                                 GIMP_PDB_CANCEL,
+                                                 NULL);
 
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
+      g_object_get (config, "anti-aliasing", &anti_aliasing, NULL);
 
       pattern_parasite = gimp_parasite_new ("pattern", 0,
                                             sizeof (qbist_info.info),
                                             &qbist_info.info);
       g_object_set (config,
                     "data-path",    qbist_info.path,
-                    "oversampling", qbist_info.oversampling,
                     "pattern",      pattern_parasite,
                     NULL);
       gimp_parasite_free (pattern_parasite);
       break;
 
     case GIMP_RUN_NONINTERACTIVE:
-      gimp_procedure_config_end_run (config, GIMP_PDB_CALLING_ERROR);
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_CALLING_ERROR,
                                                NULL);
@@ -336,9 +328,9 @@ qbist_run (GimpProcedure        *procedure,
 
     case GIMP_RUN_WITH_LAST_VALS:
       g_object_get (config,
-                    "data-path",    &qbist_info.path,
-                    "oversampling", &qbist_info.oversampling,
-                    "pattern",      &pattern_parasite,
+                    "data-path",     &qbist_info.path,
+                    "anti-aliasing", &anti_aliasing,
+                    "pattern",       &pattern_parasite,
                     NULL);
 
       if (pattern_parasite)
@@ -355,6 +347,8 @@ qbist_run (GimpProcedure        *procedure,
   total_pixels = img_width * img_height;
   done_pixels  = 0;
 
+  qbist_info.oversampling = anti_aliasing ? 4 : 1;
+
   buffer = gimp_drawable_get_shadow_buffer (drawable);
 
   iter = gegl_buffer_iterator_new (buffer,
@@ -404,9 +398,6 @@ qbist_run (GimpProcedure        *procedure,
 
   g_rand_free (gr);
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -851,9 +842,9 @@ dialog_load (GtkWidget *widget,
                                         NULL);
 
   gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
+                                            GTK_RESPONSE_OK,
+                                            GTK_RESPONSE_CANCEL,
+                                            -1);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
   if (qbist_info.path)
@@ -893,9 +884,9 @@ dialog_save (GtkWidget *widget,
                                         NULL);
 
   gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
+                                            GTK_RESPONSE_OK,
+                                            GTK_RESPONSE_CANCEL,
+                                            -1);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog),
@@ -915,16 +906,9 @@ dialog_save (GtkWidget *widget,
   gtk_widget_destroy (dialog);
 }
 
-static void
-dialog_toggle_antialaising (GtkWidget *widget,
-                            gpointer   data)
-{
-  qbist_info.oversampling =
-    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) ? 4 : 1;
-}
-
 static gboolean
-dialog_run (void)
+dialog_run (GimpProcedure       *procedure,
+            GimpProcedureConfig *config)
 {
   GtkWidget *dialog;
   GtkWidget *vbox;
@@ -937,33 +921,21 @@ dialog_run (void)
 
   gimp_ui_init (PLUG_IN_BINARY);
 
-  dialog = gimp_dialog_new (_("G-Qbist"), PLUG_IN_ROLE,
-                            NULL, 0,
-                            gimp_standard_help_func, PLUG_IN_PROC,
+  dialog = gimp_procedure_dialog_new (procedure,
+                                      GIMP_PROCEDURE_CONFIG (config),
+                                      _("G-Qbist"));
 
-                            _("_Cancel"), GTK_RESPONSE_CANCEL,
-                            _("_OK"),     GTK_RESPONSE_OK,
-
-                            NULL);
-
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
-
-  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 12);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      vbox, FALSE, FALSE, 0);
-  gtk_widget_show (vbox);
+  vbox = gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
+                                         "qbist-vbox", "anti-aliasing",
+                                         NULL);
 
   grid = gtk_grid_new ();
   gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
   gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_widget_set_margin_bottom (grid, 6);
   gtk_box_pack_start (GTK_BOX (vbox), grid, FALSE, FALSE, 0);
-  gtk_widget_show (grid);
+  gtk_widget_set_visible (grid, TRUE);
+  gtk_box_reorder_child (GTK_BOX (vbox), grid, 0);
 
   info[0] = qbist_info.info;
   dialog_new_variations (NULL, NULL);
@@ -983,7 +955,7 @@ dialog_run (void)
           gtk_grid_attach (GTK_GRID (grid), frame, i % 3, i / 3, 1, 2);
 
           gtk_container_add (GTK_CONTAINER (frame), button);
-          gtk_widget_show (frame);
+          gtk_widget_set_visible (frame, TRUE);
         }
       else if (i > 2)
         {
@@ -994,7 +966,7 @@ dialog_run (void)
           gtk_grid_attach (GTK_GRID (grid), button, i % 3, i / 3, 1, 1);
         }
       gtk_widget_set_valign (button, GTK_ALIGN_END);
-      gtk_widget_show (button);
+      gtk_widget_set_visible (button, TRUE);
 
       g_signal_connect (button, "clicked",
                         G_CALLBACK (dialog_select_preview),
@@ -1003,27 +975,18 @@ dialog_run (void)
       preview[i] = gimp_preview_area_new ();
       gtk_widget_set_size_request (preview[i], preview_size, preview_size);
       gtk_container_add (GTK_CONTAINER (button), preview[i]);
-      gtk_widget_show (preview[i]);
+      gtk_widget_set_visible (preview[i], TRUE);
     }
 
-  button = gtk_check_button_new_with_mnemonic (_("_Antialiasing"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
-                                qbist_info.oversampling > 1);
-  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-  gtk_widget_show (button);
-
-  g_signal_connect (button, "toggled",
-                    G_CALLBACK (dialog_toggle_antialaising),
-                    NULL);
-
   bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
   gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_START);
+  gtk_widget_set_margin_top (bbox, 6);
   gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
-  gtk_widget_show (bbox);
+  gtk_widget_set_visible (bbox, TRUE);
 
   button = gtk_button_new_with_mnemonic (_("_Undo"));
   gtk_container_add (GTK_CONTAINER (bbox), button);
-  gtk_widget_show (button);
+  gtk_widget_set_visible (button, TRUE);
 
   g_signal_connect (button, "clicked",
                     G_CALLBACK (dialog_undo),
@@ -1031,7 +994,7 @@ dialog_run (void)
 
   button = gtk_button_new_with_mnemonic (_("_Open"));
   gtk_container_add (GTK_CONTAINER (bbox), button);
-  gtk_widget_show (button);
+  gtk_widget_set_visible (button, TRUE);
 
   g_signal_connect (button, "clicked",
                     G_CALLBACK (dialog_load),
@@ -1039,16 +1002,18 @@ dialog_run (void)
 
   button = gtk_button_new_with_mnemonic (_("_Save"));
   gtk_container_add (GTK_CONTAINER (bbox), button);
-  gtk_widget_show (button);
+  gtk_widget_set_visible (button, TRUE);
 
   g_signal_connect (button, "clicked",
                     G_CALLBACK (dialog_save),
                     NULL);
 
-  gtk_widget_show (dialog);
+  gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
+                              "qbist-vbox", NULL);
+  gtk_widget_set_visible (dialog, TRUE);
   dialog_update_previews (NULL, preview_size);
 
-  run = (gimp_dialog_run (GIMP_DIALOG (dialog)) == GTK_RESPONSE_OK);
+  run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dialog));
 
   if (run)
     qbist_info.info = info[0];
