@@ -34,7 +34,7 @@ gint                    psd_read               (GInputStream        *input,
                                                 GError             **error);
 
 gboolean                psd_read_len           (GInputStream        *input,
-                                                gsize               *data,
+                                                guint64             *data,
                                                 gint                 psd_version,
                                                 GError            **error);
 
@@ -69,6 +69,7 @@ gint32                  fwrite_pascal_string   (const gchar         *src,
 gchar                 * fread_unicode_string   (gint32              *bytes_read,
                                                 gint32              *bytes_written,
                                                 guint16              mod_len,
+                                                gboolean             ibm_pc_format,
                                                 GInputStream        *input,
                                                 GError             **error);
 
@@ -94,6 +95,8 @@ void                    psd_to_gimp_blend_mode (PSDlayer             *psd_layer,
                                                 LayerModeInfo        *mode_info);
 
 const gchar *           gimp_to_psd_blend_mode (const LayerModeInfo  *mode_info);
+gboolean                convert_psd_mode       (const gchar          *psd_mode,
+                                                GimpLayerMode        *mode);
 
 GimpColorTag            psd_to_gimp_layer_color_tag (guint16          layer_color_tag);
 
