@@ -68,9 +68,8 @@ static GimpProcedure  * gfig_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * gfig_run              (GimpProcedure        *procedure,
                                                GimpRunMode           run_mode,
                                                GimpImage            *image,
-                                               gint                  n_drawables,
                                                GimpDrawable        **drawables,
-                                               const GimpValueArray *args,
+                                               GimpProcedureConfig  *config,
                                                gpointer              run_data);
 
 static void             on_app_activate       (GApplication         *gapp,
@@ -197,9 +196,8 @@ static GimpValueArray *
 gfig_run (GimpProcedure        *procedure,
           GimpRunMode           run_mode,
           GimpImage            *image,
-          gint                  n_drawables,
           GimpDrawable        **drawables,
-          const GimpValueArray *args,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
   GimpDrawable      *drawable;
@@ -207,7 +205,7 @@ gfig_run (GimpProcedure        *procedure,
   gint               pwidth, pheight;
   GimpGfig          *gfig;
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
@@ -232,154 +230,7 @@ gfig_run (GimpProcedure        *procedure,
 #endif
   gfig->success = FALSE;
 
-  gfig->builder = gtk_builder_new_from_string (
-      "<interface>"
-        "<menu id=\"gfig-menubar\">"
-          "<section>"
-            "<submenu>"
-              "<attribute name=\"label\">File</attribute>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Open</attribute>"
-                  "<attribute name=\"action\">app.open</attribute>"
-                "</item>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Save</attribute>"
-                  "<attribute name=\"action\">app.save</attribute>"
-                "</item>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Close</attribute>"
-                  "<attribute name=\"action\">app.close</attribute>"
-                "</item>"
-            "</submenu>"
-            "<submenu>"
-              "<attribute name=\"label\">Edit</attribute>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Undo</attribute>"
-                  "<attribute name=\"action\">app.undo</attribute>"
-                "</item>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Clear</attribute>"
-                  "<attribute name=\"action\">app.clear</attribute>"
-                "</item>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Grid</attribute>"
-                  "<attribute name=\"action\">app.grid</attribute>"
-                "</item>"
-                "<item>"
-                  "<attribute name=\"label\" translatable=\"yes\">Preferences</attribute>"
-                  "<attribute name=\"action\">app.preferences</attribute>"
-                "</item>"
-            "</submenu>"
-          "</section>"
-        "</menu>"
-        "<menu id=\"gfig-toolbar\">"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Line</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">line</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Rectangle</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">rectangle</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Circle</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">circle</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Ellipse</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">ellipse</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Arc</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">arc</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Polygon</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">polygon</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Star</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">star</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Spiral</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">spiral</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Bezier</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">bezier</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Move Object</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">move-obj</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Move Point</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">move-point</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Copy</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">copy</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Delete</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">delete</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Select</attribute>"
-              "<attribute name=\"action\">app.shape</attribute>"
-              "<attribute name=\"target\">select</attribute>"
-            "</item>"
-          "</section>"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Raise</attribute>"
-              "<attribute name=\"action\">app.raise</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Lower</attribute>"
-              "<attribute name=\"action\">app.lower</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Top</attribute>"
-              "<attribute name=\"action\">app.top</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Bottom</attribute>"
-              "<attribute name=\"action\">app.bottom</attribute>"
-            "</item>"
-          "</section>"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Show Previous</attribute>"
-              "<attribute name=\"action\">app.show-prev</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Show Next</attribute>"
-              "<attribute name=\"action\">app.show-next</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Show All</attribute>"
-              "<attribute name=\"action\">app.show-all</attribute>"
-            "</item>"
-          "</section>"
-        "</menu>"
-      "</interface>",
-      -1);
+  gfig->builder = gtk_builder_new_from_resource ("/org/gimp/gfig/gfig-menu.ui");
 
   gfig_context = g_new0 (GFigContext, 1);
 
