@@ -5,9 +5,9 @@
  *
  * Copyright (C) 1998-2004 Maurits Rijk  m.rijk@chello.nl
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -16,14 +16,15 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
 #ifndef _IMAP_ABOUT_H
 #define _IMAP_ABOUT_H
 
-void do_about_dialog(void);
+void do_about_dialog (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       user_data);
 
 #endif /* _IMAP_ABOUT_H */
