@@ -31,25 +31,31 @@ typedef struct
 /* Phong shading */
 /*****************/
 
-static GimpRGB
+static void
 phong_shade (GimpVector3 *pos,
-	     GimpVector3 *viewpoint,
-	     GimpVector3 *normal,
-	     GimpVector3 *light,
-	     GimpRGB      *diff_col,
-	     GimpRGB      *spec_col,
-	     gint         type)
+             GimpVector3 *viewpoint,
+             GimpVector3 *normal,
+             gdouble     *diff_col,
+             gdouble     *spec_col,
+             LightType    type)
 {
-  GimpRGB       ambientcolor, diffusecolor, specularcolor;
-  gdouble      NL, RV, dist;
-  GimpVector3  L, NN, V, N;
+  gdouble       ambientcolor[4], diffusecolor[4], specularcolor[4];
+  gdouble       NL, RV, dist;
+  GimpVector3   L, NN, V, N;
+  GimpVector3  *light;
+
+  light = mapvals.lightsource.type == DIRECTIONAL_LIGHT
+    ? &mapvals.lightsource.direction
+    : &mapvals.lightsource.position,
 
   /* Compute ambient intensity */
   /* ========================= */
 
   N = *normal;
-  ambientcolor = *diff_col;
-  gimp_rgb_multiply (&ambientcolor, mapvals.material.ambient_int);
+  for (gint i = 0; i < 4; i++)
+    ambientcolor[i] = diff_col[i];
+  for (gint i = 0; i < 3; i++)
+    ambientcolor[i] *= mapvals.material.ambient_int;
 
   /* Compute (N*L) term of Phong's equation */
   /* ====================================== */
@@ -82,30 +88,41 @@ phong_shade (GimpVector3 *pos,
       /* Compute diffuse and specular intensity contribution */
       /* =================================================== */
 
-      diffusecolor = *diff_col;
-      gimp_rgb_multiply (&diffusecolor, mapvals.material.diffuse_ref);
-      gimp_rgb_multiply (&diffusecolor, NL);
+     for (gint i = 0; i < 4; i++)
+       diffusecolor[i] = diff_col[i];
+     for (gint i = 0; i < 3; i++)
+       {
+         diffusecolor[i] *= mapvals.material.diffuse_ref;
+         diffusecolor[i] *= NL;
+       }
 
-      specularcolor = *spec_col;
-      gimp_rgb_multiply (&specularcolor, mapvals.material.specular_ref);
-      gimp_rgb_multiply (&specularcolor, RV);
+     for (gint i = 0; i < 4; i++)
+       specularcolor[i] = spec_col[i];
+     for (gint i = 0; i < 3; i++)
+       {
+         specularcolor[i] *= mapvals.material.specular_ref;
+         specularcolor[i] *= RV;
+       }
 
-      gimp_rgb_add (&diffusecolor, &specularcolor);
-      gimp_rgb_multiply (&diffusecolor, mapvals.material.diffuse_int);
-      gimp_rgb_clamp (&diffusecolor);
-
-      gimp_rgb_add (&ambientcolor, &diffusecolor);
+     for (gint i = 0; i < 3; i++)
+       {
+         diffusecolor[i] += specularcolor[i];
+         diffusecolor[i] *= mapvals.material.diffuse_int;
+         diffusecolor[i] = CLAMP (diffusecolor[i], 0.0, 1.0);
+         ambientcolor[i] += diffusecolor[i];
+       }
     }
 
-  return ambientcolor;
+  for (gint i = 0; i < 4; i++)
+    diff_col[i] = ambientcolor[i];
 }
 
 static gint
 plane_intersect (GimpVector3 *dir,
-		 GimpVector3 *viewp,
-		 GimpVector3 *ipos,
-		 gdouble     *u,
-		 gdouble     *v)
+                 GimpVector3 *viewp,
+                 GimpVector3 *ipos,
+                 gdouble     *u,
+                 gdouble     *v)
 {
   static gdouble det, det1, det2, det3, t;
 
@@ -117,11 +134,11 @@ plane_intersect (GimpVector3 *dir,
   /* ============================================================= */
 
   det = (imat[0][0] * imat[1][1] * imat[2][2] +
-	 imat[0][1] * imat[1][2] * imat[2][0] +
-	 imat[0][2] * imat[1][0] * imat[2][1] -
-	 imat[0][2] * imat[1][1] * imat[2][0] -
-	 imat[0][0] * imat[1][2] * imat[2][1] -
-	 imat[2][2] * imat[0][1] * imat[1][0]);
+         imat[0][1] * imat[1][2] * imat[2][0] +
+         imat[0][2] * imat[1][0] * imat[2][1] -
+         imat[0][2] * imat[1][1] * imat[2][0] -
+         imat[0][0] * imat[1][2] * imat[2][1] -
+         imat[2][2] * imat[0][1] * imat[1][0]);
 
   /* If the determinant is non-zero, a intersection point exists */
   /* =========================================================== */
@@ -132,27 +149,27 @@ plane_intersect (GimpVector3 *dir,
       /* ==================================================== */
 
       det1 = (imat[0][3] * imat[1][1] * imat[2][2] +
-	      imat[0][1] * imat[1][2] * imat[2][3] +
-	      imat[0][2] * imat[1][3] * imat[2][1] -
-	      imat[0][2] * imat[1][1] * imat[2][3] -
-	      imat[1][2] * imat[2][1] * imat[0][3] -
-	      imat[2][2] * imat[0][1] * imat[1][3]);
+              imat[0][1] * imat[1][2] * imat[2][3] +
+              imat[0][2] * imat[1][3] * imat[2][1] -
+              imat[0][2] * imat[1][1] * imat[2][3] -
+              imat[1][2] * imat[2][1] * imat[0][3] -
+              imat[2][2] * imat[0][1] * imat[1][3]);
 
       det2 = (imat[0][0] * imat[1][3] * imat[2][2] +
-	      imat[0][3] * imat[1][2] * imat[2][0] +
-	      imat[0][2] * imat[1][0] * imat[2][3] -
-	      imat[0][2] * imat[1][3] * imat[2][0] -
-	      imat[1][2] * imat[2][3] * imat[0][0] -
-	      imat[2][2] * imat[0][3] * imat[1][0]);
+              imat[0][3] * imat[1][2] * imat[2][0] +
+              imat[0][2] * imat[1][0] * imat[2][3] -
+              imat[0][2] * imat[1][3] * imat[2][0] -
+              imat[1][2] * imat[2][3] * imat[0][0] -
+              imat[2][2] * imat[0][3] * imat[1][0]);
 
       det3 = (imat[0][0] * imat[1][1] * imat[2][3] +
-	      imat[0][1] * imat[1][3] * imat[2][0] +
-	      imat[0][3] * imat[1][0] * imat[2][1] -
-	      imat[0][3] * imat[1][1] * imat[2][0] -
-	      imat[1][3] * imat[2][1] * imat[0][0] -
-	      imat[2][3] * imat[0][1] * imat[1][0]);
+              imat[0][1] * imat[1][3] * imat[2][0] +
+              imat[0][3] * imat[1][0] * imat[2][1] -
+              imat[0][3] * imat[1][1] * imat[2][0] -
+              imat[1][3] * imat[2][1] * imat[0][0] -
+              imat[2][3] * imat[0][1] * imat[1][0]);
 
-      /* Now we have the simultanous solutions. Lets compute the unknowns */
+      /* Now we have the simultaneous solutions. Lets compute the unknowns */
       /* (skip u&v if t is <0, this means the intersection is behind us)  */
       /* ================================================================ */
 
@@ -163,7 +180,7 @@ plane_intersect (GimpVector3 *dir,
           *u = 1.0 + ((det2 / det) - 0.5);
           *v = 1.0 + ((det3 / det) - 0.5);
 
-      	  ipos->x = viewp->x + t * dir->x;
+                ipos->x = viewp->x + t * dir->x;
           ipos->y = viewp->y + t * dir->y;
           ipos->z = viewp->z + t * dir->z;
 
@@ -179,11 +196,10 @@ plane_intersect (GimpVector3 *dir,
  * of the plane at a given point
  *****************************************************************************/
 
-GimpRGB
-get_ray_color_plane (GimpVector3 *pos)
+void
+get_ray_color_plane (GimpVector3 *pos,
+                     gdouble     *color)
 {
-  GimpRGB color = background;
-
   static gint         inside = FALSE;
   static GimpVector3  ray, spos;
   static gdouble      vx, vy;
@@ -191,6 +207,9 @@ get_ray_color_plane (GimpVector3 *pos)
   /* Construct a line from our VP to the point */
   /* ========================================= */
 
+  for (gint i = 0; i < 4; i++)
+    color[i] = background[i];
+
   gimp_vector3_sub (&ray, pos, &mapvals.viewpoint);
   gimp_vector3_normalize (&ray);
 
@@ -199,33 +218,28 @@ get_ray_color_plane (GimpVector3 *pos)
 
   if (plane_intersect (&ray, &mapvals.viewpoint, &spos, &vx, &vy) == TRUE)
     {
-      color = get_image_color (vx, vy, &inside);
+      get_image_color (vx, vy, &inside, color);
 
-      if (color.a != 0.0 && inside == TRUE &&
-	  mapvals.lightsource.type != NO_LIGHT)
+      if (color[3] != 0.0 && inside == TRUE &&
+          mapvals.lightsource.type != NO_LIGHT)
         {
           /* Compute shading at this point */
           /* ============================= */
 
-          color = phong_shade (&spos,
-			       &mapvals.viewpoint,
-			       &mapvals.normal,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&spos,
+                       &mapvals.viewpoint,
+                       &mapvals.normal,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
     }
 
-  if (mapvals.transparent_background == FALSE && color.a < 1.0)
-    {
-      gimp_rgb_composite (&color, &background,
-			  GIMP_RGB_COMPOSITE_BEHIND);
-    }
-
-  return color;
+  if (mapvals.transparent_background == FALSE && color[3] < 1.0)
+    composite (color, background, COMPOSITE_BEHIND);
 }
 
 /***********************************************************************/
@@ -235,8 +249,8 @@ get_ray_color_plane (GimpVector3 *pos)
 
 static void
 sphere_to_image (GimpVector3 *normal,
-		 gdouble     *u,
-		 gdouble     *v)
+                 gdouble     *u,
+                 gdouble     *v)
 {
   static gdouble      alpha, fac;
   static GimpVector3  cross_prod;
@@ -252,7 +266,7 @@ sphere_to_image (GimpVector3 *normal,
   else
     {
       fac = (gimp_vector3_inner_product (&mapvals.firstaxis, normal) /
-	     sin (alpha));
+             sin (alpha));
 
       /* Make sure that we map to -1.0..1.0 (take care of rounding errors) */
       /* ================================================================= */
@@ -262,7 +276,7 @@ sphere_to_image (GimpVector3 *normal,
       *u = acos (fac) / (2.0 * G_PI);
 
       cross_prod = gimp_vector3_cross_product (&mapvals.secondaxis,
-					       &mapvals.firstaxis);
+                                               &mapvals.firstaxis);
 
       if (gimp_vector3_inner_product (&cross_prod, normal) < 0.0)
         *u = 1.0 - *u;
@@ -275,9 +289,9 @@ sphere_to_image (GimpVector3 *normal,
 
 static gint
 sphere_intersect (GimpVector3 *dir,
-		  GimpVector3 *viewp,
-		  GimpVector3 *spos1,
-		  GimpVector3 *spos2)
+                  GimpVector3 *viewp,
+                  GimpVector3 *spos1,
+                  GimpVector3 *spos2)
 {
   static gdouble      alpha, beta, tau, s1, s2, tmp;
   static GimpVector3  t;
@@ -320,21 +334,22 @@ sphere_intersect (GimpVector3 *dir,
  * of the sphere at a given point
  *****************************************************************************/
 
-GimpRGB
-get_ray_color_sphere (GimpVector3 *pos)
+void
+get_ray_color_sphere (GimpVector3 *pos,
+                      gdouble     *color)
 {
-  GimpRGB color = background;
-
-  static GimpRGB      color2;
+  static gdouble      color2[4];
   static gint         inside = FALSE;
   static GimpVector3  normal, ray, spos1, spos2;
   static gdouble      vx, vy;
 
+  for (gint i = 0; i < 4; i++)
+    color[i] = background[i];
   /* Check if ray is within the bounding box */
   /* ======================================= */
 
   if (pos->x<bx1 || pos->x>bx2 || pos->y<by1 || pos->y>by2)
-    return color;
+    return;
 
   /* Construct a line from our VP to the point */
   /* ========================================= */
@@ -353,79 +368,75 @@ get_ray_color_sphere (GimpVector3 *pos)
       gimp_vector3_sub (&normal, &spos1, &mapvals.position);
       gimp_vector3_normalize (&normal);
       sphere_to_image (&normal, &vx, &vy);
-      color = get_image_color (vx, vy, &inside);
+      get_image_color (vx, vy, &inside, color);
 
       /* Check for total transparency... */
       /* =============================== */
 
-      if (color.a < 1.0)
+      if (color[3] < 1.0)
         {
           /* Hey, we can see  through here!      */
           /* Lets see what's on the other side.. */
           /* =================================== */
 
-          color = phong_shade (&spos1,
-			       &mapvals.viewpoint,
-			       &normal,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&spos1,
+                       &mapvals.viewpoint,
+                       &normal,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
 
           gimp_vector3_sub (&normal, &spos2, &mapvals.position);
           gimp_vector3_normalize (&normal);
           sphere_to_image (&normal, &vx, &vy);
-          color2 = get_image_color (vx, vy, &inside);
+          get_image_color (vx, vy, &inside, color2);
 
           /* Make the normal point inwards */
           /* ============================= */
 
           gimp_vector3_mul (&normal, -1.0);
 
-          color2 = phong_shade (&spos2,
-				&mapvals.viewpoint,
-				&normal,
-				&mapvals.lightsource.position,
-				&color2,
-				&mapvals.lightsource.color,
-				mapvals.lightsource.type);
+          phong_shade (&spos2,
+                       &mapvals.viewpoint,
+                       &normal,
+                       color2,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color2);
+          for (gint i = 0; i < 4; i++)
+            color2[i] = CLAMP (color2[i], 0.0, 1.0);
 
           /* Compute a mix of the first and second colors */
           /* ============================================ */
 
-	  gimp_rgb_composite (&color, &color2, GIMP_RGB_COMPOSITE_NORMAL);
-          gimp_rgb_clamp (&color);
+          composite (color, color2, COMPOSITE_NORMAL);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
-      else if (color.a != 0.0 &&
-	       inside == TRUE &&
-	       mapvals.lightsource.type != NO_LIGHT)
+      else if (color[3] != 0.0 &&
+               inside == TRUE &&
+               mapvals.lightsource.type != NO_LIGHT)
         {
           /* Compute shading at this point */
           /* ============================= */
 
-          color = phong_shade (&spos1,
-			       &mapvals.viewpoint,
-			       &normal,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&spos1,
+                       &mapvals.viewpoint,
+                       &normal,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
     }
 
-  if (mapvals.transparent_background == FALSE && color.a < 1.0)
-    {
-      gimp_rgb_composite (&color, &background,
-			  GIMP_RGB_COMPOSITE_BEHIND);
-    }
-
-  return color;
+  if (mapvals.transparent_background == FALSE && color[3] < 1.0)
+    composite (color, background, COMPOSITE_BEHIND);
 }
 
 /***************************************************/
@@ -482,8 +493,8 @@ compute_bounding_box (void)
 
 void
 vecmulmat (GimpVector3 *u,
-	   GimpVector3 *v,
-	   gfloat       m[16])
+           GimpVector3 *v,
+           gfloat       m[16])
 {
   gfloat v0=v->x, v1=v->y, v2=v->z;
 #define M(row,col)  m[col*4+row]
@@ -495,8 +506,8 @@ vecmulmat (GimpVector3 *u,
 
 void
 rotatemat (gfloat       angle,
-	   GimpVector3 *v,
-	   gfloat       m[16])
+           GimpVector3 *v,
+           gfloat       m[16])
 {
   /* This function contributed by Erich Boleyn (erich@uruk.org) */
   gfloat mag, s, c;
@@ -591,8 +602,8 @@ transpose_mat (gfloat m[16])
 
 void
 matmul (gfloat a[16],
-	gfloat b[16],
-	gfloat c[16])
+        gfloat b[16],
+        gfloat c[16])
 {
   gint   i, j, k;
   gfloat value;
@@ -605,10 +616,10 @@ matmul (gfloat a[16],
     {
       for (j = 0; j < 4; j++)
         {
-	  value = 0.0;
+          value = 0.0;
 
-	  for (k = 0; k < 4; k++)
-	    value += A(i,k) * B(k,j);
+          for (k = 0; k < 4; k++)
+            value += A(i,k) * B(k,j);
 
           C(i,j) = value;
         }
@@ -642,11 +653,11 @@ ident_mat (gfloat m[16])
 
 static gboolean
 intersect_rect (gdouble            u,
-		gdouble            v,
-		gdouble            w,
-		GimpVector3        viewp,
-		GimpVector3        dir,
-		FaceIntersectInfo *face_info)
+                gdouble            v,
+                gdouble            w,
+                GimpVector3        viewp,
+                GimpVector3        dir,
+                FaceIntersectInfo *face_info)
 {
   gboolean result = FALSE;
   gdouble  u2, v2;
@@ -675,9 +686,9 @@ intersect_rect (gdouble            u,
 
 static gboolean
 intersect_box (GimpVector3        scale,
-	       GimpVector3        viewp,
-	       GimpVector3        dir,
-	       FaceIntersectInfo *face_intersect)
+               GimpVector3        viewp,
+               GimpVector3        dir,
+               FaceIntersectInfo *face_intersect)
 {
   GimpVector3        v, d, tmp, axis[3];
   FaceIntersectInfo  face_tmp;
@@ -693,7 +704,7 @@ intersect_box (GimpVector3        scale,
   /* ========== */
 
   if (intersect_rect (scale.x, scale.y, scale.z / 2.0,
-		      viewp, dir, &face_intersect[i]) == TRUE)
+                      viewp, dir, &face_intersect[i]) == TRUE)
     {
       face_intersect[i].face = 0;
       gimp_vector3_set (&face_intersect[i++].n, 0.0, 0.0, 1.0);
@@ -704,7 +715,7 @@ intersect_box (GimpVector3        scale,
   /* ========= */
 
   if (intersect_rect (scale.x, scale.y, -scale.z / 2.0,
-		      viewp, dir, &face_intersect[i]) == TRUE)
+                      viewp, dir, &face_intersect[i]) == TRUE)
     {
       face_intersect[i].face = 1;
       face_intersect[i].u = 1.0 - face_intersect[i].u;
@@ -725,7 +736,7 @@ intersect_box (GimpVector3        scale,
       vecmulmat (&d, &dir, m);
 
       if (intersect_rect (scale.x, scale.z, scale.y / 2.0,
-			  v, d, &face_intersect[i]) == TRUE)
+                          v, d, &face_intersect[i]) == TRUE)
         {
           face_intersect[i].face = 2;
 
@@ -751,7 +762,7 @@ intersect_box (GimpVector3        scale,
       vecmulmat (&d, &dir, m);
 
       if (intersect_rect (scale.x, scale.z, -scale.y / 2.0,
-			  v, d, &face_intersect[i]) == TRUE)
+                          v, d, &face_intersect[i]) == TRUE)
         {
           face_intersect[i].face = 3;
 
@@ -781,7 +792,7 @@ intersect_box (GimpVector3        scale,
       vecmulmat (&d, &dir, m);
 
       if (intersect_rect (scale.z, scale.y, scale.x / 2.0,
-			  v, d, &face_intersect[i]) == TRUE)
+                          v, d, &face_intersect[i]) == TRUE)
         {
           face_intersect[i].face = 4;
 
@@ -807,7 +818,7 @@ intersect_box (GimpVector3        scale,
       vecmulmat (&d, &dir, m);
 
       if (intersect_rect (scale.z, scale.y, -scale.x / 2.0,
-			  v, d, &face_intersect[i]) == TRUE)
+                          v, d, &face_intersect[i]) == TRUE)
         {
           face_intersect[i].face = 5;
 
@@ -834,16 +845,19 @@ intersect_box (GimpVector3        scale,
   return result;
 }
 
-GimpRGB
-get_ray_color_box (GimpVector3 *pos)
+void
+get_ray_color_box (GimpVector3 *pos,
+                   gdouble     *color)
 {
   GimpVector3        lvp, ldir, vp, p, dir, ns, nn;
-  GimpRGB             color, color2;
+  gdouble            color2[4];
   gfloat             m[16];
   gint               i;
   FaceIntersectInfo  face_intersect[2];
 
-  color = background;
+  for (i = 0; i < 4; i++)
+    color[i] = background[i];
+
   vp = mapvals.viewpoint;
   p = *pos;
 
@@ -904,87 +918,85 @@ get_ray_color_box (GimpVector3 *pos)
           face_intersect[i].n = nn;
         }
 
-      color = get_box_image_color (face_intersect[0].face,
-				   face_intersect[0].u,
-				   face_intersect[0].v);
+      get_box_image_color (face_intersect[0].face,
+                           face_intersect[0].u,
+                           face_intersect[0].v,
+                           color);
 
       /* Check for total transparency... */
       /* =============================== */
 
-      if (color.a < 1.0)
+      if (color[3] < 1.0)
         {
           /* Hey, we can see  through here!      */
           /* Lets see what's on the other side.. */
           /* =================================== */
 
-          color = phong_shade (&face_intersect[0].s,
-			       &mapvals.viewpoint,
-			       &face_intersect[0].n,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&face_intersect[0].s,
+                       &mapvals.viewpoint,
+                       &face_intersect[0].n,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
 
-          color2 = get_box_image_color (face_intersect[1].face,
-					face_intersect[1].u,
-					face_intersect[1].v);
+          get_box_image_color (face_intersect[1].face,
+                               face_intersect[1].u,
+                               face_intersect[1].v,
+                               color2);
 
           /* Make the normal point inwards */
           /* ============================= */
 
           gimp_vector3_mul (&face_intersect[1].n, -1.0);
 
-          color2 = phong_shade (&face_intersect[1].s,
-				&mapvals.viewpoint,
-				&face_intersect[1].n,
-				&mapvals.lightsource.position,
-				&color2,
-				&mapvals.lightsource.color,
-				mapvals.lightsource.type);
+          phong_shade (&face_intersect[1].s,
+                       &mapvals.viewpoint,
+                       &face_intersect[1].n,
+                       color2,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color2);
+          for (gint i = 0; i < 4; i++)
+            color2[i] = CLAMP (color2[i], 0.0, 1.0);
 
-          if (mapvals.transparent_background == FALSE && color2.a < 1.0)
-            {
-	      gimp_rgb_composite (&color2, &background,
-				  GIMP_RGB_COMPOSITE_BEHIND);
-	    }
+          if (mapvals.transparent_background == FALSE && color2[3] < 1.0)
+            composite (color2, background, COMPOSITE_BEHIND);
 
           /* Compute a mix of the first and second colors */
           /* ============================================ */
 
-	  gimp_rgb_composite (&color, &color2, GIMP_RGB_COMPOSITE_NORMAL);
-          gimp_rgb_clamp (&color);
+          composite (color, color2, COMPOSITE_NORMAL);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
-      else if (color.a != 0.0 && mapvals.lightsource.type != NO_LIGHT)
+      else if (color[3] != 0.0 && mapvals.lightsource.type != NO_LIGHT)
         {
-	  color = phong_shade (&face_intersect[0].s,
-			       &mapvals.viewpoint,
-			       &face_intersect[0].n,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&face_intersect[0].s,
+                       &mapvals.viewpoint,
+                       &face_intersect[0].n,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-	  gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
     }
   else
     {
       if (mapvals.transparent_background == TRUE)
-	gimp_rgb_set_alpha (&color, 0.0);
+        color[3] = 0.0;
     }
-
-  return color;
 }
 
 static gboolean
 intersect_circle (GimpVector3        vp,
-		  GimpVector3        dir,
-		  gdouble            w,
-		  FaceIntersectInfo *face_info)
+                  GimpVector3        dir,
+                  gdouble            w,
+                  FaceIntersectInfo *face_info)
 {
   gboolean result = FALSE;
   gdouble  r, d;
@@ -1016,8 +1028,8 @@ intersect_circle (GimpVector3        vp,
 
 static gboolean
 intersect_cylinder (GimpVector3        vp,
-		    GimpVector3        dir,
-		    FaceIntersectInfo *face_intersect)
+                    GimpVector3        dir,
+                    FaceIntersectInfo *face_intersect)
 {
   gdouble  a, b, c, d, e, f, tmp, l;
   gboolean result = FALSE;
@@ -1109,32 +1121,32 @@ intersect_cylinder (GimpVector3        vp,
   return result;
 }
 
-static GimpRGB
-get_cylinder_color (gint    face,
-		    gdouble u,
-		    gdouble v)
+static void
+get_cylinder_color (gint     face,
+                    gdouble  u,
+                    gdouble  v,
+                    gdouble *color)
 {
-  GimpRGB  color;
-  gint    inside;
+  gint inside;
 
   if (face == 0)
-    color = get_image_color (u, v, &inside);
+    get_image_color (u, v, &inside, color);
   else
-    color = get_cylinder_image_color (face - 1, u, v);
-
-  return color;
+    get_cylinder_image_color (face - 1, u, v, color);
 }
 
-GimpRGB
-get_ray_color_cylinder (GimpVector3 *pos)
+void
+get_ray_color_cylinder (GimpVector3 *pos,
+                        gdouble     *color)
 {
   GimpVector3       lvp, ldir, vp, p, dir, ns, nn;
-  GimpRGB            color, color2;
+  gdouble           color2[4];
   gfloat            m[16];
   gint              i;
   FaceIntersectInfo face_intersect[2];
 
-  color = background;
+  for (i = 0; i < 4; i++)
+    color[i] = background[i];
   vp = mapvals.viewpoint;
   p = *pos;
 
@@ -1183,78 +1195,117 @@ get_ray_color_cylinder (GimpVector3 *pos)
           face_intersect[i].n = nn;
         }
 
-      color = get_cylinder_color (face_intersect[0].face,
-				  face_intersect[0].u,
-				  face_intersect[0].v);
+      get_cylinder_color (face_intersect[0].face,
+                          face_intersect[0].u,
+                          face_intersect[0].v,
+                          color);
 
       /* Check for transparency... */
       /* ========================= */
 
-      if (color.a < 1.0)
+      if (color[3] < 1.0)
         {
           /* Hey, we can see  through here!      */
           /* Lets see what's on the other side.. */
           /* =================================== */
 
-          color = phong_shade (&face_intersect[0].s,
-			       &mapvals.viewpoint,
-			       &face_intersect[0].n,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&face_intersect[0].s,
+                       &mapvals.viewpoint,
+                       &face_intersect[0].n,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
 
-          color2 = get_cylinder_color (face_intersect[1].face,
-				       face_intersect[1].u,
-				       face_intersect[1].v);
+          get_cylinder_color (face_intersect[1].face,
+                              face_intersect[1].u,
+                              face_intersect[1].v,
+                              color2);
 
           /* Make the normal point inwards */
           /* ============================= */
 
           gimp_vector3_mul (&face_intersect[1].n, -1.0);
 
-          color2 = phong_shade (&face_intersect[1].s,
-				&mapvals.viewpoint,
-				&face_intersect[1].n,
-				&mapvals.lightsource.position,
-				&color2,
-				&mapvals.lightsource.color,
-				mapvals.lightsource.type);
+          phong_shade (&face_intersect[1].s,
+                       &mapvals.viewpoint,
+                       &face_intersect[1].n,
+                       color2,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-          gimp_rgb_clamp (&color2);
+          for (gint i = 0; i < 4; i++)
+            color2[i] = CLAMP (color2[i], 0.0, 1.0);
 
-          if (mapvals.transparent_background == FALSE && color2.a < 1.0)
-            {
-	      gimp_rgb_composite (&color2, &background,
-				  GIMP_RGB_COMPOSITE_BEHIND);
-            }
+          if (mapvals.transparent_background == FALSE && color2[3] < 1.0)
+            composite (color2, background, COMPOSITE_BEHIND);
 
           /* Compute a mix of the first and second colors */
           /* ============================================ */
 
-	  gimp_rgb_composite (&color, &color2, GIMP_RGB_COMPOSITE_NORMAL);
-          gimp_rgb_clamp (&color);
+          composite (color, color2, COMPOSITE_NORMAL);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
-      else if (color.a != 0.0 && mapvals.lightsource.type != NO_LIGHT)
+      else if (color[3] != 0.0 && mapvals.lightsource.type != NO_LIGHT)
         {
-	  color = phong_shade (&face_intersect[0].s,
-			       &mapvals.viewpoint,
-			       &face_intersect[0].n,
-			       &mapvals.lightsource.position,
-			       &color,
-			       &mapvals.lightsource.color,
-			       mapvals.lightsource.type);
+          phong_shade (&face_intersect[0].s,
+                       &mapvals.viewpoint,
+                       &face_intersect[0].n,
+                       color,
+                       mapvals.lightsource.color,
+                       mapvals.lightsource.type);
 
-	  gimp_rgb_clamp (&color);
+          for (gint i = 0; i < 4; i++)
+            color[i] = CLAMP (color[i], 0.0, 1.0);
         }
     }
   else
     {
       if (mapvals.transparent_background == TRUE)
-	gimp_rgb_set_alpha (&color, 0.0);
+        color[3] = 0.0;
+    }
+}
+
+void
+composite (gdouble              *color1,
+           gdouble              *color2,
+           CompositeType         composite)
+{
+  g_return_if_fail (color1 != NULL);
+  g_return_if_fail (color2 != NULL);
+
+  if (composite == COMPOSITE_NORMAL)
+    {
+      /*  put color2 on top of color1  */
+      if (color2[3] == 1.0)
+        {
+          for (gint i = 0; i < 4; i++)
+            color1[i] = color2[i];
+        }
+      else
+        {
+          gdouble factor = color1[3] * (1.0 - color2[3]);
+
+          color1[0] = color1[0] * factor + color2[0] * color2[3];
+          color1[1] = color1[1] * factor + color2[1] * color2[3];
+          color1[2] = color1[2] * factor + color2[2] * color2[3];
+          color1[3] = factor + color2[3];
+        }
+    }
+  else if (composite == COMPOSITE_BEHIND)
+    {
+      /*  put color2 below color1  */
+      if (color1[3] < 1.0)
+        {
+          gdouble factor = color2[3] * (1.0 - color1[3]);
+
+          color1[0] = color2[0] * factor + color1[0] * color1[3];
+          color1[1] = color2[1] * factor + color1[1] * color1[3];
+          color1[2] = color2[2] * factor + color1[2] * color1[3];
+          color1[3] = factor + color1[3];
+        }
     }
-
-  return color;
 }
