@@ -59,7 +59,7 @@ struct _PrintClass
 
 
 #define PRINT_TYPE  (print_get_type ())
-#define PRINT (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PRINT_TYPE, Print))
+#define PRINT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PRINT_TYPE, Print))
 
 GType                     print_get_type         (void) G_GNUC_CONST;
 
@@ -70,9 +70,8 @@ static GimpProcedure    * print_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray   * print_run              (GimpProcedure        *procedure,
                                                   GimpRunMode           run_mode,
                                                   GimpImage            *image,
-                                                  gint                  n_drawables,
                                                   GimpDrawable        **drawables,
-                                                  const GimpValueArray *args,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static GimpPDBStatusType  print_image            (GimpImage         *image,
@@ -166,7 +165,7 @@ print_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("_Print..."));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_DOCUMENT_PRINT);
-      gimp_procedure_add_menu_path (procedure, "<Image>/File");
+      gimp_procedure_add_menu_path (procedure, "<Image>/File/[Send]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Print the image"),
@@ -190,7 +189,7 @@ print_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("Page Set_up..."));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_DOCUMENT_PAGE_SETUP);
-      gimp_procedure_add_menu_path (procedure, "<Image>/File");
+      gimp_procedure_add_menu_path (procedure, "<Image>/File/[Send]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Adjust page size and orientation "
@@ -213,9 +212,8 @@ static GimpValueArray *
 print_run (GimpProcedure        *procedure,
            GimpRunMode           run_mode,
            GimpImage            *image,
-           gint                  n_drawables,
            GimpDrawable        **drawables,
-           const GimpValueArray *args,
+           GimpProcedureConfig  *config,
            gpointer              run_data)
 {
   GimpPDBStatusType  status;
@@ -289,6 +287,7 @@ print_image (GimpImage *image,
   data.use_full_page   = FALSE;
   data.draw_crop_marks = FALSE;
   data.operation       = operation;
+  data.has_layout_gui  = FALSE;
 
   gimp_image_get_resolution (image, &data.xres, &data.yres);
 
@@ -309,7 +308,7 @@ print_image (GimpImage *image,
 #ifndef EMBED_PAGE_SETUP
   print_operation = operation;
   temp_proc = print_temp_proc_install (image);
-  gimp_plug_in_extension_enable (gimp_get_plug_in ());
+  gimp_plug_in_persistent_enable (gimp_get_plug_in ());
 #endif
 
   if (interactive)
@@ -378,9 +377,10 @@ print_image (GimpImage *image,
 static GimpPDBStatusType
 page_setup (GimpImage *image)
 {
-  GtkPrintOperation  *operation;
-  GimpValueArray     *return_vals;
-  gchar              *name;
+  GtkPrintOperation *operation;
+  GimpProcedure     *procedure;
+  GimpValueArray    *return_vals;
+  gchar             *name;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
@@ -401,11 +401,13 @@ page_setup (GimpImage *image)
   gimp_plug_in_set_pdb_error_handler (gimp_get_plug_in (),
                                       GIMP_PDB_ERROR_HANDLER_PLUGIN);
 
-  return_vals = gimp_pdb_run_procedure (gimp_get_pdb (),
-                                        name,
-                                        GIMP_TYPE_IMAGE, image,
-                                        G_TYPE_NONE);
-  gimp_value_array_unref (return_vals);
+  /* Notify the Print plug-in if Page Setup was called from there */
+  procedure   = gimp_pdb_lookup_procedure (gimp_get_pdb (), name);
+  if (procedure)
+    {
+      return_vals = gimp_procedure_run (procedure, "image", image, NULL);
+      gimp_value_array_unref (return_vals);
+    }
 
   g_free (name);
 
@@ -447,6 +449,26 @@ begin_print (GtkPrintOperation *operation,
              GtkPrintContext   *context,
              PrintData         *data)
 {
+  if (! data->has_layout_gui)
+    {
+      GtkWidget *dialog, *layout_gui;
+      layout_gui = print_page_layout_gui (data, PRINT_PROC_NAME);
+      dialog     = gtk_dialog_new_with_buttons (_("Image Settings"), NULL, 0,
+                                                _("_Print"), GTK_RESPONSE_ACCEPT,
+                                                _("_Cancel"), GTK_RESPONSE_REJECT, NULL);
+      gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+                          layout_gui, TRUE, TRUE, 0);
+      gtk_widget_show_all (dialog);
+      if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_ACCEPT)
+        {
+          gtk_print_operation_cancel (operation);
+          gtk_widget_destroy (dialog);
+          return;
+        }
+
+      gtk_widget_destroy (dialog);
+    }
+
   gtk_print_operation_set_use_full_page (operation, data->use_full_page);
 
   gimp_progress_init (_("Printing"));
@@ -504,14 +526,18 @@ create_custom_widget (GtkPrintOperation *operation,
 #ifndef EMBED_PAGE_SETUP
 static GimpValueArray *
 print_temp_proc_run (GimpProcedure        *procedure,
-                     const GimpValueArray *args,
+                     GimpProcedureConfig  *config,
                      gpointer              run_data)
 {
-  GimpImage *image = GIMP_VALUES_GET_IMAGE (args, 0);
+  GimpImage *image;
+
+  g_object_get (config, "image", &image, NULL);
 
   if (print_operation)
     print_page_setup_load (print_operation, image);
 
+  g_object_unref (image);
+
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -544,11 +570,11 @@ print_temp_proc_install (GimpImage *image)
                                   "Sven Neumann",
                                   "2008");
 
-  GIMP_PROC_ARG_IMAGE (procedure, "image",
-                       "Image",
-                       "The image to notify about",
-                       FALSE,
-                       G_PARAM_READWRITE);
+  gimp_procedure_add_image_argument (procedure, "image",
+                                     "Image",
+                                     "The image to notify about",
+                                     FALSE,
+                                     G_PARAM_READWRITE);
 
   gimp_plug_in_add_temp_procedure (plug_in, procedure);
   g_object_unref (procedure);
