@@ -31,14 +31,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* Necessary in order to have SetProcessDPIAware() defined.
- * This value of _WIN32_WINNT corresponds to Windows 7, which is our
- * minimum supported platform.
- */
-#ifdef _WIN32_WINNT
-#undef _WIN32_WINNT
-#endif
-#define _WIN32_WINNT 0x0601
 #include <windows.h>
 
 #include <libgimp/gimp.h>
@@ -160,37 +152,40 @@ screenshot_win32_get_capabilities (void)
 {
   return (SCREENSHOT_CAN_SHOOT_DECORATIONS |
           SCREENSHOT_CAN_SHOOT_WINDOW      |
-          SCREENSHOT_CAN_SHOOT_POINTER);
+          SCREENSHOT_CAN_SHOOT_POINTER     |
+          SCREENSHOT_CAN_DELAY_WINDOW_SHOT);
 }
 
 GimpPDBStatusType
-screenshot_win32_shoot (ScreenshotValues  *shootvals,
-                        GdkMonitor        *monitor,
-                        GimpImage        **_image,
-                        GError           **error)
+screenshot_win32_shoot (ShootType    shoot_type,
+                        guint        screenshot_delay,
+                        gboolean     show_cursor,
+                        GdkMonitor  *monitor,
+                        GimpImage  **_image,
+                        GError     **error)
 {
   GimpPDBStatusType status = GIMP_PDB_EXECUTION_ERROR;
 
-  /* leave "shootvals->monitor" alone until somebody patches the code
+  /* leave "monitor" alone until somebody patches the code
    * to be able to get a monitor's color profile
    */
 
   image = _image;
 
-  winsnapvals.delay = shootvals->screenshot_delay;
-  capturePointer = shootvals->show_cursor;
+  winsnapvals.delay = screenshot_delay;
+  capturePointer = show_cursor;
 
-  if (shootvals->shoot_type == SHOOT_ROOT)
+  if (shoot_type == SHOOT_ROOT)
     {
       doCapture (0);
 
       status = GIMP_PDB_SUCCESS;
     }
-  else if (shootvals->shoot_type == SHOOT_WINDOW)
+  else if (shoot_type == SHOOT_WINDOW)
     {
       status = 0 == doWindowCapture () ? GIMP_PDB_CANCEL : GIMP_PDB_SUCCESS;
     }
-  else if (shootvals->shoot_type == SHOOT_REGION)
+  else if (shoot_type == SHOOT_REGION)
     {
       /* FIXME */
     }
