@@ -21,9 +21,15 @@
 GList         *script_fu_interpreter_list_defined_proc_names (
                                             GimpPlugIn  *plug_in,
                                             const gchar *path_to_this_plugin);
-GimpProcedure *script_fu_interpreter_create_proc_at_path (
+GimpProcedure *script_fu_interpreter_create_proc (
                                             GimpPlugIn  *plug_in,
                                             const gchar *proc_name,
-                                            const gchar *path_to_this_script);
+                                            const gchar *path_to_script);
+void           script_fu_interpreter_get_i18n (
+                                            GimpPlugIn  *plug_in,
+                                            const gchar *proc_name,
+                                            const gchar *path_to_this_script,
+                                            gchar **i18n_domain,
+                                            gchar **i18n_catalog);
 
 #endif /*  __SCRIPT_FU_INTERPRETER_H__  */
