@@ -30,11 +30,13 @@
 
 #include "scheme-wrapper.h"
 
+#include "script-fu-lib.h"
 #include "script-fu-types.h"
 
 #include "script-fu-interface.h"
 #include "script-fu-scripts.h"
 #include "script-fu-script.h"
+#include "script-fu-arg.h"
 
 #include "script-fu-intl.h"
 
@@ -53,13 +55,7 @@ typedef struct
   GtkWidget  *grid;
   GtkWidget **widgets;
 
-  GtkWidget  *progress_label;
-  GtkWidget  *progress_bar;
-
   gchar      *title;
-  gchar      *last_command;
-  gint        command_count;
-  gint        consec_command_count;
 
   gboolean    running;
 } SFInterface;
@@ -81,14 +77,16 @@ static void   script_fu_file_callback       (GtkWidget     *widget,
                                              SFFilename    *file);
 static void   script_fu_combo_callback      (GtkWidget     *widget,
                                              SFOption      *option);
+static void   script_fu_color_button_update (GimpColorButton *button,
+                                             SFColorType      color);
 
 static void   script_fu_resource_set_handler (gpointer       data,
                                               gpointer       resource,
                                               gboolean       closing);
 
-static GtkWidget * script_fu_resource_widget (const gchar   *title,
-                                              gchar        **id,
-                                              GType          resource_type);
+static GtkWidget * script_fu_resource_widget (const gchar    *title,
+                                              SFArg          *arg,
+                                              GType           resource_type);
 
 static void   script_fu_flush_events         (void);
 static void   script_fu_activate_main_dialog (void);
@@ -115,63 +113,21 @@ script_fu_interface_is_active (void)
   return (sf_interface != NULL);
 }
 
-void
-script_fu_interface_report_cc (const gchar *command)
-{
-  if (sf_interface == NULL)
-    return;
-
-  if (sf_interface->last_command &&
-      strcmp (sf_interface->last_command, command) == 0)
-    {
-      sf_interface->command_count++;
-
-      if (! g_str_has_prefix (command, "gimp-progress-"))
-        {
-          gchar *new_command;
-
-          new_command = g_strdup_printf ("%s <%d>",
-                                         command, sf_interface->command_count);
-          gtk_label_set_text (GTK_LABEL (sf_interface->progress_label),
-                              new_command);
-          g_free (new_command);
-        }
-    }
-  else
-    {
-      sf_interface->command_count = 1;
-
-      g_free (sf_interface->last_command);
-      sf_interface->last_command = g_strdup (command);
-
-      if (! g_str_has_prefix (command, "gimp-progress-"))
-        {
-          gtk_label_set_text (GTK_LABEL (sf_interface->progress_label),
-                              command);
-        }
-      else
-        {
-          gtk_label_set_text (GTK_LABEL (sf_interface->progress_label), "");
-        }
-    }
-
-  while (gtk_events_pending ())
-    gtk_main_iteration ();
-}
 
 GimpPDBStatusType
-script_fu_interface (SFScript  *script,
-                     gint       start_arg)
+script_fu_interface_dialog (SFScript  *script,
+                            gint       start_arg)
 {
   GtkWidget    *dialog;
   GtkWidget    *vbox;
-  GtkWidget    *vbox2;
   GtkSizeGroup *group;
   GSList       *list;
   gchar        *title;
   gint          i;
 
-  static gboolean gtk_initted = FALSE;
+  /* Requires gimp_ui_init called previously. */
+
+  g_debug ("%s", G_STRFUNC);
 
   /* Simply return if there is already an interface. This is an
    * ugly workaround for the fact that we can not process two
@@ -193,13 +149,6 @@ script_fu_interface (SFScript  *script,
 
   g_return_val_if_fail (script != NULL, FALSE);
 
-  if (!gtk_initted)
-    {
-      gimp_ui_init ("script-fu");
-
-      gtk_initted = TRUE;
-    }
-
   sf_status = GIMP_PDB_SUCCESS;
 
   sf_interface = g_slice_new0 (SFInterface);
@@ -207,7 +156,7 @@ script_fu_interface (SFScript  *script,
   sf_interface->widgets = g_new0 (GtkWidget *, script->n_args);
   sf_interface->title   = script_fu_script_get_title (script);
 
-  title = g_strdup_printf (_("Script-Fu: %s"), sf_interface->title);
+  title = g_strdup_printf ("%s", sf_interface->title);
 
   sf_interface->dialog = dialog =
     gimp_dialog_new (title, "gimp-script-fu",
@@ -302,7 +251,7 @@ script_fu_interface (SFScript  *script,
               break;
 
             case SF_VECTORS:
-              widget = gimp_vectors_combo_box_new (NULL, NULL, NULL);
+              widget = gimp_path_combo_box_new (NULL, NULL, NULL);
               ID_ptr = &arg->value.sfa_vectors;
               break;
 
@@ -318,13 +267,12 @@ script_fu_interface (SFScript  *script,
         case SF_COLOR:
           {
             GimpColorConfig *config;
+            GeglColor       *color = sf_color_get_gegl_color (arg->value.sfa_color);
 
             left_align = TRUE;
             widget = gimp_color_button_new (_("Script-Fu Color Selection"),
-                                            COLOR_SAMPLE_WIDTH,
-                                            COLOR_SAMPLE_HEIGHT,
-                                            &arg->value.sfa_color,
-                                            GIMP_COLOR_AREA_FLAT);
+                                            COLOR_SAMPLE_WIDTH, COLOR_SAMPLE_HEIGHT,
+                                            color, GIMP_COLOR_AREA_FLAT);
 
             gimp_color_button_set_update (GIMP_COLOR_BUTTON (widget), TRUE);
 
@@ -332,10 +280,11 @@ script_fu_interface (SFScript  *script,
             gimp_color_button_set_color_config (GIMP_COLOR_BUTTON (widget),
                                                 config);
             g_object_unref (config);
+            g_object_unref (color);
 
             g_signal_connect (widget, "color-changed",
-                              G_CALLBACK (gimp_color_button_get_color),
-                              &arg->value.sfa_color);
+                              G_CALLBACK (script_fu_color_button_update),
+                              arg->value.sfa_color);
           }
           break;
 
@@ -351,7 +300,6 @@ script_fu_interface (SFScript  *script,
                             &arg->value.sfa_toggle);
           break;
 
-        case SF_VALUE:
         case SF_STRING:
           widget = gtk_entry_new ();
           gtk_widget_set_size_request (widget, TEXT_WIDTH, -1);
@@ -401,6 +349,13 @@ script_fu_interface (SFScript  *script,
                                                  arg->default_value.sfa_adjustment.lower,
                                                  arg->default_value.sfa_adjustment.upper,
                                                  arg->default_value.sfa_adjustment.digits);
+
+                  /* #12157 We should not need to set value, since we just passed it.
+                   * But seems to be a flaw in the widget.
+                   * The comments in gimp_label_spin_init, says it inits to bogus.
+                   */
+                  gimp_label_spin_set_value ((GimpLabelSpin*)widget, arg->value.sfa_adjustment.value);
+
                   gimp_label_spin_set_increments (GIMP_LABEL_SPIN (widget),
                                                   arg->default_value.sfa_adjustment.step,
                                                   arg->default_value.sfa_adjustment.page);
@@ -463,35 +418,35 @@ script_fu_interface (SFScript  *script,
           break;
 
         case SF_FONT:
-          widget = script_fu_resource_widget (_("Script-Fu Font Selection"),
-                                              &arg->value.sfa_font,
+          widget = script_fu_resource_widget (label_text,
+                                              arg,
                                               GIMP_TYPE_FONT);
           break;
 
         case SF_PALETTE:
-          widget = script_fu_resource_widget (_("Script-Fu Palette Selection"),
-                                              &arg->value.sfa_palette,
+          widget = script_fu_resource_widget (label_text,
+                                              arg,
                                               GIMP_TYPE_PALETTE);
           break;
 
         case SF_PATTERN:
           left_align = TRUE;
-          widget = script_fu_resource_widget (_("Script-Fu Pattern Selection"),
-                                              &arg->value.sfa_pattern,
+          widget = script_fu_resource_widget (label_text,
+                                              arg,
                                               GIMP_TYPE_PATTERN);
           break;
 
         case SF_GRADIENT:
           left_align = TRUE;
-          widget = script_fu_resource_widget (_("Script-Fu Gradient Selection"),
-                                              &arg->value.sfa_gradient,
+          widget = script_fu_resource_widget (label_text,
+                                              arg,
                                               GIMP_TYPE_GRADIENT);
           break;
 
         case SF_BRUSH:
           left_align = TRUE;
-          widget = script_fu_resource_widget (_("Script-Fu Brush Selection"),
-                                              &arg->value.sfa_brush,
+          widget = script_fu_resource_widget (label_text,
+                                              arg,
                                               GIMP_TYPE_BRUSH);
           break;
 
@@ -554,26 +509,6 @@ script_fu_interface (SFScript  *script,
 
   g_object_unref (group);
 
-  /* the script progress bar */
-  vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_box_pack_end (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
-  gtk_widget_show (vbox2);
-
-  sf_interface->progress_bar = gimp_progress_bar_new ();
-  gtk_box_pack_start (GTK_BOX (vbox2), sf_interface->progress_bar,
-                      FALSE, FALSE, 0);
-  gtk_widget_show (sf_interface->progress_bar);
-
-  sf_interface->progress_label = gtk_label_new (NULL);
-  gtk_label_set_xalign (GTK_LABEL (sf_interface->progress_label), 0.0);
-  gtk_label_set_ellipsize (GTK_LABEL (sf_interface->progress_label),
-                           PANGO_ELLIPSIZE_MIDDLE);
-  gimp_label_set_attributes (GTK_LABEL (sf_interface->progress_label),
-                             PANGO_ATTR_STYLE, PANGO_STYLE_ITALIC,
-                             -1);
-  gtk_box_pack_start (GTK_BOX (vbox2), sf_interface->progress_label,
-                      FALSE, FALSE, 0);
-  gtk_widget_show (sf_interface->progress_label);
 #ifdef G_OS_WIN32
     {
       HWND foreground = GetForegroundWindow ();
@@ -582,6 +517,12 @@ script_fu_interface (SFScript  *script,
   gtk_widget_show (dialog);
 
   gtk_main ();
+  /* The script ran, or was canceled, and called gtk_main_quit */
+
+#ifdef GDK_WINDOWING_QUARTZ
+  /* Make dock icon go away. */
+  [NSApp setActivationPolicy:NSApplicationActivationPolicyAccessory];
+#endif
 
 #ifdef G_OS_WIN32
       if (! GetForegroundWindow ())
@@ -592,37 +533,54 @@ script_fu_interface (SFScript  *script,
 }
 
 /* Return a widget for choosing a resource.
- * Generic on type.
+ * Dispatch on resource type.
+ *
+ * On first show, widget will show initial choice : declared by script.
+ * Thereafter, the choice from prior use of filter.
+ *
+ * Widget will update model if user touches widget.
  */
-GtkWidget *
-script_fu_resource_widget (const gchar   *title,
-                           gchar        **id_handle,
-                           GType          resource_type)
+static GtkWidget *
+script_fu_resource_widget (const gchar    *title,
+                           SFArg          *arg,
+                           GType           resource_type)
 {
   GtkWidget    *result_widget = NULL;
-  GimpResource *resource;
+  GimpResource *initial_value;
 
-  resource = gimp_resource_get_by_name (resource_type, *id_handle);
+  g_debug ("%s type: %" G_GSIZE_FORMAT, G_STRFUNC, resource_type);
 
+  /* Init the arg's value (ID).
+   * On first run, initialize to default.
+   * On second run, init to prior choice.
+   * Can't do it at registration time, resources are not loaded.
+   */
+  sf_resource_arg_init_current_value (arg);
+
+  /* Passing empty string for outer widget label,
+   * since this old-style interface makes the label.
+   */
+  /* Create a widget with initial value of the arg. */
+  initial_value = sf_resource_arg_get_value (arg);
   if (g_type_is_a (resource_type, GIMP_TYPE_FONT))
     {
-      result_widget = gimp_font_select_button_new (title, resource);
+      result_widget = gimp_font_chooser_new (title, "", GIMP_FONT (initial_value));
     }
   else if (g_type_is_a (resource_type, GIMP_TYPE_BRUSH))
     {
-      result_widget = gimp_brush_select_button_new (title, resource);
+      result_widget = gimp_brush_chooser_new (title, "", GIMP_BRUSH (initial_value));
     }
   else if (g_type_is_a (resource_type, GIMP_TYPE_GRADIENT))
     {
-      result_widget = gimp_gradient_select_button_new (title, resource);
+      result_widget = gimp_gradient_chooser_new (title, "", GIMP_GRADIENT (initial_value));
     }
   else if (g_type_is_a (resource_type, GIMP_TYPE_PALETTE))
     {
-      result_widget = gimp_palette_select_button_new (title, resource);
+      result_widget = gimp_palette_chooser_new (title, "", GIMP_PALETTE (initial_value));
     }
   else if (g_type_is_a (resource_type, GIMP_TYPE_PATTERN))
     {
-      result_widget = gimp_pattern_select_button_new (title, resource);
+      result_widget = gimp_pattern_chooser_new (title, "", GIMP_PATTERN (initial_value));
     }
   else
     {
@@ -632,12 +590,13 @@ script_fu_resource_widget (const gchar   *title,
   if (result_widget != NULL)
   {
     /* All resource widgets emit signal resource-set
-     * Connect to our handler which will be passed the id_handle,
-     * the place to store the ID of the new choice of resource.
+     * Connect to our handler which will be passed
+     * a Resource* and int* to the arg's value,
+     * the model of the new choice of resource.
      */
     g_signal_connect_swapped (result_widget, "resource-set",
                               G_CALLBACK (script_fu_resource_set_handler),
-                              id_handle);
+                              &arg->value.sfa_resource.history);  /* data */
   }
 
   return result_widget;
@@ -646,31 +605,11 @@ script_fu_resource_widget (const gchar   *title,
 static void
 script_fu_interface_quit (SFScript *script)
 {
-  gint i;
-
   g_return_if_fail (script != NULL);
   g_return_if_fail (sf_interface != NULL);
 
   g_free (sf_interface->title);
-
-  for (i = 0; i < script->n_args; i++)
-    switch (script->args[i].type)
-      {
-      case SF_FONT:
-      case SF_PALETTE:
-      case SF_PATTERN:
-      case SF_GRADIENT:
-      case SF_BRUSH:
-        gimp_resource_select_button_close_popup
-          (GIMP_RESOURCE_SELECT_BUTTON (sf_interface->widgets[i]));
-        break;
-
-      default:
-        break;
-      }
-
   g_free (sf_interface->widgets);
-  g_free (sf_interface->last_command);
 
   g_slice_free (SFInterface, sf_interface);
   sf_interface = NULL;
@@ -701,26 +640,33 @@ script_fu_combo_callback (GtkWidget *widget,
   option->history = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
 }
 
+static void
+script_fu_color_button_update (GimpColorButton *button,
+                               SFColorType      arg_value)
+{
+  GeglColor *color = gimp_color_button_get_color (button);
+
+  sf_color_set_from_gegl_color (&arg_value, color);
+
+  g_object_unref (color);
+}
+
 /* Handle resource-set signal.
- * Store id of newly chosen resource in SF local cache of args.
+ * Store id of newly chosen resource in SF local cache of args,
+ * at the integer location passed by pointer in "data"
  *
  * Note the callback is the same for all resource types.
- * The callback passes only the resource, and no attributes of the resource,
- * except it's ID.
+ * The callback passes the resource, and no attributes of the resource.
  */
 static void
 script_fu_resource_set_handler (gpointer      data,  /* callback "data" */
                                 gpointer      resource,
                                 gboolean      closing)
 {
-  gchar **id_handle = data;
+  gint32 *id_pointer = data;
 
-  /* Free any existing copy of the id string. */
-  if (*id_handle)
-    g_free (*id_handle);
-
-  /* We don't own the resource, nor its string.  Copy the string. */
-  *id_handle = g_strdup (gimp_resource_get_name (resource));
+  /* Store integer ID, not pointer to Resource.  We bind to integer ID. */
+  *id_pointer = gimp_resource_get_id (resource);
 
   if (closing) script_fu_activate_main_dialog ();
 }
@@ -790,31 +736,22 @@ script_fu_response (GtkWidget *widget,
   script_fu_flush_events ();
 }
 
+/* Update model for all widgets that
+ * don't have callbacks that update the model.
+ * Only the text and string widgets don't.
+ *
+ * The model is the arg.value field.
+ */
 static void
-script_fu_ok (SFScript *script)
+script_fu_update_models (SFScript *script)
 {
-  GString *output;
-  gchar   *command;
-  gint     i;
-
-  for (i = 0; i < script->n_args; i++)
+  for (gint i = 0; i < script->n_args; i++)
     {
       SFArgValue *arg_value = &script->args[i].value;
       GtkWidget  *widget    = sf_interface->widgets[i];
 
       switch (script->args[i].type)
         {
-        case SF_IMAGE:
-        case SF_DRAWABLE:
-        case SF_LAYER:
-        case SF_CHANNEL:
-        case SF_VECTORS:
-        case SF_DISPLAY:
-        case SF_COLOR:
-        case SF_TOGGLE:
-          break;
-
-        case SF_VALUE:
         case SF_STRING:
           g_free (arg_value->sfa_value);
           arg_value->sfa_value =
@@ -840,56 +777,71 @@ script_fu_ok (SFScript *script)
           }
           break;
 
-        case SF_ADJUSTMENT:
-        case SF_FILENAME:
-        case SF_DIRNAME:
-        case SF_FONT:
-        case SF_PALETTE:
-        case SF_PATTERN:
-        case SF_GRADIENT:
-        case SF_BRUSH:
-        case SF_OPTION:
-        case SF_ENUM:
+        default:
+          /* Silence warnings about unhandled enumeration values */
           break;
         }
     }
+}
+
+
+/* Handler for event: OK button clicked.
+ *
+ * Run the scripts with values from the dialog.
+ *
+ * Sets a global status of the PDB call to this plugin,
+ * which is returned later by interface_dialog.
+ */
+static void
+script_fu_ok (SFScript *script)
+{
+  gchar   *command;
+
+  script_fu_update_models (script);
 
   command = script_fu_script_get_command (script);
 
   /*  run the command through the interpreter  */
-  output = g_string_new (NULL);
-  ts_register_output_func (ts_gstring_output_func, output);
 
   gimp_plug_in_set_pdb_error_handler (gimp_get_plug_in (),
                                       GIMP_PDB_ERROR_HANDLER_PLUGIN);
 
+  script_fu_redirect_output_to_stdout ();
+
+  /* Returns non-zero error code on failure. */
   if (ts_interpret_string (command))
     {
-      gchar *message = g_strdup_printf (_("Error while executing %s:"),
-                                        script->name);
+      gchar *message;
 
-      g_message ("%s\n\n%s", message, output->str);
+      /* Log to stdout.  Later to Gimp. */
+      message = g_strdup_printf (_("Error while executing %s:"), script->name);
+      g_message ("%s\n", message);
       g_free (message);
+
+      /* Set global to be returned by script-fu-interface-dialog. */
+      sf_status = GIMP_PDB_EXECUTION_ERROR;
     }
 
   gimp_plug_in_set_pdb_error_handler (gimp_get_plug_in (),
                                       GIMP_PDB_ERROR_HANDLER_INTERNAL);
 
-  g_string_free (output, TRUE);
-
   g_free (command);
 }
 
+/* On user clicking "Reset" button. */
 static void
 script_fu_reset (SFScript *script)
 {
   gint i;
 
+  /* Reset the model to defaults. */
   script_fu_script_reset (script, FALSE);
 
+  /* Reset the view to the model values. */
   for (i = 0; i < script->n_args; i++)
     {
-      SFArgValue *value  = &script->args[i].value;
+      SFArg      *arg    = &script->args[i];
+      SFArgValue *value  = &arg->value;
       GtkWidget  *widget = sf_interface->widgets[i];
 
       switch (script->args[i].type)
@@ -903,8 +855,12 @@ script_fu_reset (SFScript *script)
           break;
 
         case SF_COLOR:
-          gimp_color_button_set_color (GIMP_COLOR_BUTTON (widget),
-                                       &value->sfa_color);
+          {
+            GeglColor *color = sf_color_get_gegl_color (value->sfa_color);
+
+            gimp_color_button_set_color (GIMP_COLOR_BUTTON (widget), color);
+            g_object_unref (color);
+          }
           break;
 
         case SF_TOGGLE:
@@ -912,7 +868,6 @@ script_fu_reset (SFScript *script)
                                         value->sfa_toggle);
           break;
 
-        case SF_VALUE:
         case SF_STRING:
           gtk_entry_set_text (GTK_ENTRY (widget), value->sfa_value);
           break;
@@ -965,23 +920,13 @@ script_fu_reset (SFScript *script)
                                          value->sfa_file.filename);
           break;
 
-        /* Reset should get the initial/default value that the plugin author
-         * declared by name in the SF- declaration.
-         * Said name is a string stored in the default_value.sfa_font
-         *
-         * But this code goes away when ScriptFu uses GimpProcedureDialog.
-         * Rather than make this temporary code do the correct thing,
-         * instead simply pass NULL to reset to the value from context.
-         * It is extremely unlikely that any user depends on or will notice
-         * this temporaryily slightly changed behavior.
-         */
         case SF_FONT:
         case SF_PALETTE:
         case SF_PATTERN:
         case SF_GRADIENT:
         case SF_BRUSH:
-          gimp_resource_select_button_set_resource (GIMP_RESOURCE_SELECT_BUTTON (widget),
-                                                    NULL);
+          gimp_resource_chooser_set_resource (GIMP_RESOURCE_CHOOSER (widget),
+                                              sf_resource_arg_get_default (arg));
           break;
 
         case SF_OPTION:
@@ -1065,9 +1010,9 @@ script_fu_activate_main_dialog (void)
 {
   /* Ensure the main dialog of the script-fu extension process is not obscured. */
 #ifdef GDK_WINDOWING_QUARTZ
-  /* In Objective-C, get instance of NSApplication
-   * and send it a "activateIgnoringOtherApps" message, i.e. bring to front.
-   */
+  /* Make Dock icon appear. */
+  [NSApp setActivationPolicy:NSApplicationActivationPolicyRegular];
+  /* Bring to front. */
   [NSApp activateIgnoringOtherApps: YES];
 #else
   /* empty function, optimized out. */
