@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 srcdir="$1"
 output="$2"
@@ -7,7 +7,7 @@ echo "Creating ${output} based on git log"
 
 gitdir="${srcdir}/.git"
 
-if [[ ! -d "${gitdir}" ]]; then
+if [ ! -d "${gitdir}" ]; then
     echo "A git checkout and git-log is required to write changelog in ${output}." \
     | tee ${output} >&2
     exit 1
@@ -17,9 +17,10 @@ fi
 CHANGELOG_START=74424325abb54620b370f2595445b2b2a19fe5e7
 
 ( \
-    git log "${CHANGELOG_START}^.." --stat "${srcdir}" | fmt --split-only \
-        > "${output}.tmp" \
-    && [ ${PIPESTATUS[0]} -eq 0 ] \
+    git log "${CHANGELOG_START}^.." --stat "${srcdir}" > temp_log.tmp
+    status=$?
+    cat temp_log.tmp | fmt --split-only > "${output}.tmp" | rm temp_log.tmp \
+    && [ "$status" -eq 0 ] \
     && mv "${output}.tmp" "${output}" -f \
     && echo "Appending ChangeLog.pre-git" \
     && cat "${srcdir}/ChangeLog.pre-git" >> "${output}" \
